@@ -105,10 +105,10 @@ namespace Amesos2 {
     template <typename LO, typename GO, typename GS, typename Node>
     const Teuchos::RCP<const Tpetra::Map<LO,GO,Node> >
     getDistributionMap(EDistribution distribution,
-		       GS num_global_elements,
-		       const Teuchos::RCP<const Teuchos::Comm<int> >& comm,
-		       GO indexBase = 0);
-    
+                       GS num_global_elements,
+                       const Teuchos::RCP<const Teuchos::Comm<int> >& comm,
+                       GO indexBase = 0);
+
 
 #ifdef HAVE_TPETRA_INST_INT_INT
 #ifdef HAVE_AMESOS2_EPETRA
@@ -144,7 +144,7 @@ namespace Amesos2 {
      * \ingroup amesos2_utils
      */
     const RCP<const Epetra_Comm> to_epetra_comm(RCP<const Teuchos::Comm<int> > c);
-#endif	// HAVE_AMESOS2_EPETRA
+#endif  // HAVE_AMESOS2_EPETRA
 #endif // HAVE_TPETRA_INST_INT_INT
 
     /**
@@ -153,14 +153,14 @@ namespace Amesos2 {
      * \ingroup amesos2_utils
      */
     template <typename Scalar,
-	      typename GlobalOrdinal,
-	      typename GlobalSizeT>
+              typename GlobalOrdinal,
+              typename GlobalSizeT>
     void transpose(ArrayView<Scalar> vals,
-		   ArrayView<GlobalOrdinal> indices,
-		   ArrayView<GlobalSizeT> ptr,
-		   ArrayView<Scalar> trans_vals,
-		   ArrayView<GlobalOrdinal> trans_indices,
-		   ArrayView<GlobalSizeT> trans_ptr);
+                   ArrayView<GlobalOrdinal> indices,
+                   ArrayView<GlobalSizeT> ptr,
+                   ArrayView<Scalar> trans_vals,
+                   ArrayView<GlobalOrdinal> trans_indices,
+                   ArrayView<GlobalSizeT> trans_ptr);
 
     /**
      * \brief Scales a 1-D representation of a multivector.
@@ -177,7 +177,7 @@ namespace Amesos2 {
      */
     template <typename Scalar1, typename Scalar2>
     void scale(ArrayView<Scalar1> vals, size_t l,
-	       size_t ld, ArrayView<Scalar2> s);
+               size_t ld, ArrayView<Scalar2> s);
 
     /**
      * \brief Scales a 1-D representation of a multivector.
@@ -199,7 +199,7 @@ namespace Amesos2 {
      */
     template <typename Scalar1, typename Scalar2, class BinaryOp>
     void scale(ArrayView<Scalar1> vals, size_t l,
-	       size_t ld, ArrayView<Scalar2> s, BinaryOp binary_op);
+               size_t ld, ArrayView<Scalar2> s, BinaryOp binary_op);
 
 
     /// Prints a line of 70 "-"s on std::cout.
@@ -209,7 +209,7 @@ namespace Amesos2 {
     //////////////////////////////////
     // Matrix/MultiVector Utilities //
     //////////////////////////////////
-    
+
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
     /*
      * The following represents a general way of getting a CRS or CCS
@@ -237,17 +237,17 @@ namespace Amesos2 {
     struct same_gs_helper
     {
       static void do_get(const Teuchos::Ptr<const M> mat,
-			 const ArrayView<typename M::scalar_t> nzvals,
-			 const ArrayView<typename M::global_ordinal_t> indices,
-			 const ArrayView<GS> pointers,
-			 GS& nnz,
-			 const Teuchos::Ptr<
-			   const Tpetra::Map<typename M::local_ordinal_t,
+                         const ArrayView<typename M::scalar_t> nzvals,
+                         const ArrayView<typename M::global_ordinal_t> indices,
+                         const ArrayView<GS> pointers,
+                         GS& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
                                              typename M::global_ordinal_t,
-			                     typename M::node_t> > map,
-			 EStorage_Ordering ordering)
+                                             typename M::node_t> > map,
+                         EStorage_Ordering ordering)
       {
-	Op::apply(mat, nzvals, indices, pointers, nnz, map, ordering);
+        Op::apply(mat, nzvals, indices, pointers, nnz, map, ordering);
       }
     };
 
@@ -255,27 +255,27 @@ namespace Amesos2 {
     struct diff_gs_helper
     {
       static void do_get(const Teuchos::Ptr<const M> mat,
-			 const ArrayView<typename M::scalar_t> nzvals,
-			 const ArrayView<typename M::global_ordinal_t> indices,
-			 const ArrayView<GS> pointers,
-			 GS& nnz,
-			 const Teuchos::Ptr<
-			   const Tpetra::Map<typename M::local_ordinal_t,
+                         const ArrayView<typename M::scalar_t> nzvals,
+                         const ArrayView<typename M::global_ordinal_t> indices,
+                         const ArrayView<GS> pointers,
+                         GS& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
                                              typename M::global_ordinal_t,
-			                     typename M::node_t> > map,
-			 EStorage_Ordering ordering)
+                                             typename M::node_t> > map,
+                         EStorage_Ordering ordering)
       {
-	typedef typename M::global_size_t mat_gs_t;
-	typename ArrayView<GS>::size_type i, size = pointers.size();
-	Teuchos::Array<mat_gs_t> pointers_tmp(size);
-	mat_gs_t nnz_tmp = 0;
+        typedef typename M::global_size_t mat_gs_t;
+        typename ArrayView<GS>::size_type i, size = pointers.size();
+        Teuchos::Array<mat_gs_t> pointers_tmp(size);
+        mat_gs_t nnz_tmp = 0;
 
-	Op::apply(mat, nzvals, indices, pointers_tmp, nnz_tmp, map, ordering);
+        Op::apply(mat, nzvals, indices, pointers_tmp, nnz_tmp, map, ordering);
 
-	for (i = 0; i < size; ++i){
-	  pointers[i] = Teuchos::as<GS>(pointers_tmp[i]);
-	}
-	nnz = Teuchos::as<GS>(nnz_tmp);
+        for (i = 0; i < size; ++i){
+          pointers[i] = Teuchos::as<GS>(pointers_tmp[i]);
+        }
+        nnz = Teuchos::as<GS>(nnz_tmp);
       }
     };
 
@@ -283,22 +283,22 @@ namespace Amesos2 {
     struct same_go_helper
     {
       static void do_get(const Teuchos::Ptr<const M> mat,
-			 const ArrayView<typename M::scalar_t> nzvals,
-			 const ArrayView<GO> indices,
-			 const ArrayView<GS> pointers,
-			 GS& nnz,
-			 const Teuchos::Ptr<
-			   const Tpetra::Map<typename M::local_ordinal_t,
+                         const ArrayView<typename M::scalar_t> nzvals,
+                         const ArrayView<GO> indices,
+                         const ArrayView<GS> pointers,
+                         GS& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
                                              typename M::global_ordinal_t,
-			                     typename M::node_t> > map,
-			 EStorage_Ordering ordering)
+                                             typename M::node_t> > map,
+                         EStorage_Ordering ordering)
       {
-	typedef typename M::global_size_t mat_gs_t;
-	if_then_else<is_same<GS,mat_gs_t>::value,
-	  same_gs_helper<M,S,GO,GS,Op>,
-	  diff_gs_helper<M,S,GO,GS,Op> >::type::do_get(mat, nzvals, indices,
-						       pointers, nnz, map,
-						       ordering);
+        typedef typename M::global_size_t mat_gs_t;
+        if_then_else<is_same<GS,mat_gs_t>::value,
+          same_gs_helper<M,S,GO,GS,Op>,
+          diff_gs_helper<M,S,GO,GS,Op> >::type::do_get(mat, nzvals, indices,
+                                                       pointers, nnz, map,
+                                                       ordering);
       }
     };
 
@@ -306,30 +306,30 @@ namespace Amesos2 {
     struct diff_go_helper
     {
       static void do_get(const Teuchos::Ptr<const M> mat,
-			 const ArrayView<typename M::scalar_t> nzvals,
-			 const ArrayView<GO> indices,
-			 const ArrayView<GS> pointers,
-			 GS& nnz,
-			 const Teuchos::Ptr<
-			   const Tpetra::Map<typename M::local_ordinal_t,
+                         const ArrayView<typename M::scalar_t> nzvals,
+                         const ArrayView<GO> indices,
+                         const ArrayView<GS> pointers,
+                         GS& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
                                              typename M::global_ordinal_t,
-			                     typename M::node_t> > map,
-			 EStorage_Ordering ordering)
+                                             typename M::node_t> > map,
+                         EStorage_Ordering ordering)
       {
-	typedef typename M::global_ordinal_t mat_go_t;
-	typedef typename M::global_size_t mat_gs_t;
-	typename ArrayView<GO>::size_type i, size = indices.size();
-	Teuchos::Array<mat_go_t> indices_tmp(size);
+        typedef typename M::global_ordinal_t mat_go_t;
+        typedef typename M::global_size_t mat_gs_t;
+        typename ArrayView<GO>::size_type i, size = indices.size();
+        Teuchos::Array<mat_go_t> indices_tmp(size);
 
-	if_then_else<is_same<GS,mat_gs_t>::value,
-	  same_gs_helper<M,S,GO,GS,Op>,
-	  diff_gs_helper<M,S,GO,GS,Op> >::type::do_get(mat, nzvals, indices_tmp,
-						       pointers, nnz, map,
-						       ordering);
+        if_then_else<is_same<GS,mat_gs_t>::value,
+          same_gs_helper<M,S,GO,GS,Op>,
+          diff_gs_helper<M,S,GO,GS,Op> >::type::do_get(mat, nzvals, indices_tmp,
+                                                       pointers, nnz, map,
+                                                       ordering);
 
-	for (i = 0; i < size; ++i){
-	  indices[i] = Teuchos::as<GO>(indices_tmp[i]);
-	}
+        for (i = 0; i < size; ++i){
+          indices[i] = Teuchos::as<GO>(indices_tmp[i]);
+        }
       }
     };
 
@@ -337,22 +337,22 @@ namespace Amesos2 {
     struct same_scalar_helper
     {
       static void do_get(const Teuchos::Ptr<const M> mat,
-			 const ArrayView<S> nzvals,
-			 const ArrayView<GO> indices,
-			 const ArrayView<GS> pointers,
-			 GS& nnz,
-			 const Teuchos::Ptr<
-			   const Tpetra::Map<typename M::local_ordinal_t,
+                         const ArrayView<S> nzvals,
+                         const ArrayView<GO> indices,
+                         const ArrayView<GS> pointers,
+                         GS& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
                                              typename M::global_ordinal_t,
-			                     typename M::node_t> > map,
-			 EStorage_Ordering ordering)
+                                             typename M::node_t> > map,
+                         EStorage_Ordering ordering)
       {
-	typedef typename M::global_ordinal_t mat_go_t;
-	if_then_else<is_same<GO,mat_go_t>::value,
-	  same_go_helper<M,S,GO,GS,Op>,
-	  diff_go_helper<M,S,GO,GS,Op> >::type::do_get(mat, nzvals, indices,
-						       pointers, nnz, map,
-						       ordering);
+        typedef typename M::global_ordinal_t mat_go_t;
+        if_then_else<is_same<GO,mat_go_t>::value,
+          same_go_helper<M,S,GO,GS,Op>,
+          diff_go_helper<M,S,GO,GS,Op> >::type::do_get(mat, nzvals, indices,
+                                                       pointers, nnz, map,
+                                                       ordering);
       }
     };
 
@@ -360,33 +360,33 @@ namespace Amesos2 {
     struct diff_scalar_helper
     {
       static void do_get(const Teuchos::Ptr<const M> mat,
-			 const ArrayView<S> nzvals,
-			 const ArrayView<GO> indices,
-			 const ArrayView<GS> pointers,
-			 GS& nnz,
-			 const Teuchos::Ptr<
-			   const Tpetra::Map<typename M::local_ordinal_t,
+                         const ArrayView<S> nzvals,
+                         const ArrayView<GO> indices,
+                         const ArrayView<GS> pointers,
+                         GS& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename M::local_ordinal_t,
                                              typename M::global_ordinal_t,
-			                     typename M::node_t> > map,
-			 EStorage_Ordering ordering)
+                                             typename M::node_t> > map,
+                         EStorage_Ordering ordering)
       {
-	typedef typename M::scalar_t mat_scalar_t;
-	typedef typename M::global_ordinal_t mat_go_t;
-	typename ArrayView<S>::size_type i, size = nzvals.size();
-	Teuchos::Array<mat_scalar_t> nzvals_tmp(size);
+        typedef typename M::scalar_t mat_scalar_t;
+        typedef typename M::global_ordinal_t mat_go_t;
+        typename ArrayView<S>::size_type i, size = nzvals.size();
+        Teuchos::Array<mat_scalar_t> nzvals_tmp(size);
 
-	if_then_else<is_same<GO,mat_go_t>::value,
-	  same_go_helper<M,S,GO,GS,Op>,
-	  diff_go_helper<M,S,GO,GS,Op> >::type::do_get(mat, nzvals_tmp, indices,
-						       pointers, nnz, map,
-						       ordering);
+        if_then_else<is_same<GO,mat_go_t>::value,
+          same_go_helper<M,S,GO,GS,Op>,
+          diff_go_helper<M,S,GO,GS,Op> >::type::do_get(mat, nzvals_tmp, indices,
+                                                       pointers, nnz, map,
+                                                       ordering);
 
-	for (i = 0; i < size; ++i){
-	  nzvals[i] = Teuchos::as<S>(nzvals_tmp[i]);
-	}
+        for (i = 0; i < size; ++i){
+          nzvals[i] = Teuchos::as<S>(nzvals_tmp[i]);
+        }
       }
     };
-#endif	// DOXYGEN_SHOULD_SKIP_THIS
+#endif  // DOXYGEN_SHOULD_SKIP_THIS
 
     /**
      * \brief A generic base class for the CRS and CCS helpers.
@@ -404,23 +404,23 @@ namespace Amesos2 {
     struct get_cxs_helper
     {
       static void do_get(const Teuchos::Ptr<const Matrix> mat,
-			 const ArrayView<S> nzvals,
-			 const ArrayView<GO> indices,
-			 const ArrayView<GS> pointers,
-			 GS& nnz, EDistribution distribution,
-			 EStorage_Ordering ordering=ARBITRARY,
-			 GO indexBase = 0)
+                         const ArrayView<S> nzvals,
+                         const ArrayView<GO> indices,
+                         const ArrayView<GS> pointers,
+                         GS& nnz, EDistribution distribution,
+                         EStorage_Ordering ordering=ARBITRARY,
+                         GO indexBase = 0)
       {
-	typedef typename Matrix::local_ordinal_t lo_t;
-	typedef typename Matrix::global_ordinal_t go_t;
-	typedef typename Matrix::global_size_t gs_t;
-	typedef typename Matrix::node_t node_t;
-	
-	const Teuchos::RCP<const Tpetra::Map<lo_t,go_t,node_t> > map
-	  = getDistributionMap<lo_t,go_t,gs_t,node_t>(distribution,
-						      Op::get_dimension(mat),
-						      mat->getComm(), indexBase);
-	do_get(mat, nzvals, indices, pointers, nnz, Teuchos::ptrInArg(*map), ordering);
+        typedef typename Matrix::local_ordinal_t lo_t;
+        typedef typename Matrix::global_ordinal_t go_t;
+        typedef typename Matrix::global_size_t gs_t;
+        typedef typename Matrix::node_t node_t;
+
+        const Teuchos::RCP<const Tpetra::Map<lo_t,go_t,node_t> > map
+          = getDistributionMap<lo_t,go_t,gs_t,node_t>(distribution,
+                                                      Op::get_dimension(mat),
+                                                      mat->getComm(), indexBase);
+        do_get(mat, nzvals, indices, pointers, nnz, Teuchos::ptrInArg(*map), ordering);
       }
 
       /**
@@ -428,16 +428,16 @@ namespace Amesos2 {
        * returned by Op::getMap().
        */
       static void do_get(const Teuchos::Ptr<const Matrix> mat,
-			 const ArrayView<S> nzvals,
-			 const ArrayView<GO> indices,
-			 const ArrayView<GS> pointers,
-			 GS& nnz, EStorage_Ordering ordering=ARBITRARY)
+                         const ArrayView<S> nzvals,
+                         const ArrayView<GO> indices,
+                         const ArrayView<GS> pointers,
+                         GS& nnz, EStorage_Ordering ordering=ARBITRARY)
       {
-	const Teuchos::RCP<const Tpetra::Map<typename Matrix::local_ordinal_t,
-	                                     typename Matrix::global_ordinal_t,
-	                                     typename Matrix::node_t> > map
-	  = Op::getMap(mat);
-	do_get(mat, nzvals, indices, pointers, nnz, Teuchos::ptrInArg(*map), ordering);
+        const Teuchos::RCP<const Tpetra::Map<typename Matrix::local_ordinal_t,
+                                             typename Matrix::global_ordinal_t,
+                                             typename Matrix::node_t> > map
+          = Op::getMap(mat);
+        do_get(mat, nzvals, indices, pointers, nnz, Teuchos::ptrInArg(*map), ordering);
       }
 
       /**
@@ -445,23 +445,23 @@ namespace Amesos2 {
        * representation's distribution.
        */
       static void do_get(const Teuchos::Ptr<const Matrix> mat,
-			 const ArrayView<S> nzvals,
-			 const ArrayView<GO> indices,
-			 const ArrayView<GS> pointers,
-			 GS& nnz,
-			 const Teuchos::Ptr<
-			   const Tpetra::Map<typename Matrix::local_ordinal_t,
+                         const ArrayView<S> nzvals,
+                         const ArrayView<GO> indices,
+                         const ArrayView<GS> pointers,
+                         GS& nnz,
+                         const Teuchos::Ptr<
+                           const Tpetra::Map<typename Matrix::local_ordinal_t,
                                              typename Matrix::global_ordinal_t,
-			                     typename Matrix::node_t> > map,
-			 EStorage_Ordering ordering=ARBITRARY)
+                                             typename Matrix::node_t> > map,
+                         EStorage_Ordering ordering=ARBITRARY)
       {
-	typedef typename Matrix::scalar_t mat_scalar;
-	if_then_else<is_same<mat_scalar,S>::value,
-	  same_scalar_helper<Matrix,S,GO,GS,Op>,
-	  diff_scalar_helper<Matrix,S,GO,GS,Op> >::type::do_get(mat,
-								nzvals, indices,
-								pointers, nnz,
-								map, ordering);
+        typedef typename Matrix::scalar_t mat_scalar;
+        if_then_else<is_same<mat_scalar,S>::value,
+          same_scalar_helper<Matrix,S,GO,GS,Op>,
+          diff_scalar_helper<Matrix,S,GO,GS,Op> >::type::do_get(mat,
+                                                                nzvals, indices,
+                                                                pointers, nnz,
+                                                                map, ordering);
       }
     };
 
@@ -474,33 +474,33 @@ namespace Amesos2 {
     struct get_ccs_func
     {
       static void apply(const Teuchos::Ptr<const Matrix> mat,
-			const ArrayView<typename Matrix::scalar_t> nzvals,
-			const ArrayView<typename Matrix::global_ordinal_t> rowind,
-			const ArrayView<typename Matrix::global_size_t> colptr,
-			typename Matrix::global_size_t& nnz,
-			const Teuchos::Ptr<
-			  const Tpetra::Map<typename Matrix::local_ordinal_t,
+                        const ArrayView<typename Matrix::scalar_t> nzvals,
+                        const ArrayView<typename Matrix::global_ordinal_t> rowind,
+                        const ArrayView<typename Matrix::global_size_t> colptr,
+                        typename Matrix::global_size_t& nnz,
+                        const Teuchos::Ptr<
+                          const Tpetra::Map<typename Matrix::local_ordinal_t,
                                             typename Matrix::global_ordinal_t,
-			                    typename Matrix::node_t> > map,
-			EStorage_Ordering ordering)
+                                            typename Matrix::node_t> > map,
+                        EStorage_Ordering ordering)
       {
-	mat->getCcs(nzvals, rowind, colptr, nnz, map, ordering);
+        mat->getCcs(nzvals, rowind, colptr, nnz, map, ordering);
       }
 
       static
       const Teuchos::RCP<const Tpetra::Map<typename Matrix::local_ordinal_t,
-					   typename Matrix::global_ordinal_t,
-					   typename Matrix::node_t> >
+                                           typename Matrix::global_ordinal_t,
+                                           typename Matrix::node_t> >
       getMap(const Teuchos::Ptr<const Matrix> mat)
       {
-	return mat->getColMap();
+        return mat->getColMap();
       }
 
       static
       typename Matrix::global_size_t
       get_dimension(const Teuchos::Ptr<const Matrix> mat)
       {
-	return mat->getGlobalNumCols();
+        return mat->getGlobalNumCols();
       }
     };
 
@@ -508,36 +508,36 @@ namespace Amesos2 {
     struct get_crs_func
     {
       static void apply(const Teuchos::Ptr<const Matrix> mat,
-			const ArrayView<typename Matrix::scalar_t> nzvals,
-			const ArrayView<typename Matrix::global_ordinal_t> colind,
-			const ArrayView<typename Matrix::global_size_t> rowptr,
-			typename Matrix::global_size_t& nnz,
-			const Teuchos::Ptr<
-			  const Tpetra::Map<typename Matrix::local_ordinal_t,
-			                    typename Matrix::global_ordinal_t,
-			                    typename Matrix::node_t> > map,
-			EStorage_Ordering ordering)
+                        const ArrayView<typename Matrix::scalar_t> nzvals,
+                        const ArrayView<typename Matrix::global_ordinal_t> colind,
+                        const ArrayView<typename Matrix::global_size_t> rowptr,
+                        typename Matrix::global_size_t& nnz,
+                        const Teuchos::Ptr<
+                          const Tpetra::Map<typename Matrix::local_ordinal_t,
+                                            typename Matrix::global_ordinal_t,
+                                            typename Matrix::node_t> > map,
+                        EStorage_Ordering ordering)
       {
-	mat->getCrs(nzvals, colind, rowptr, nnz, map, ordering);
+        mat->getCrs(nzvals, colind, rowptr, nnz, map, ordering);
       }
 
       static
       const Teuchos::RCP<const Tpetra::Map<typename Matrix::local_ordinal_t,
-					   typename Matrix::global_ordinal_t,
-					   typename Matrix::node_t> >
+                                           typename Matrix::global_ordinal_t,
+                                           typename Matrix::node_t> >
       getMap(const Teuchos::Ptr<const Matrix> mat)
       {
-	return mat->getRowMap();
+        return mat->getRowMap();
       }
 
       static
       typename Matrix::global_size_t
       get_dimension(const Teuchos::Ptr<const Matrix> mat)
       {
-	return mat->getGlobalNumRows();
+        return mat->getGlobalNumRows();
       }
     };
-#endif	// DOXYGEN_SHOULD_SKIP_THIS
+#endif  // DOXYGEN_SHOULD_SKIP_THIS
 
     /**
      * \brief A generic helper class for getting a CCS representation
@@ -572,7 +572,7 @@ namespace Amesos2 {
      * Array<int> colptr(numcols+1);
      * get_ccs_helper<mat_t,solver_complex,int,int>::do_get(mat,nzvals,rowind,rowptr,nnz,Rooted,Arbitrary);
      * \endcode
-     * 
+     *
      * \sa \ref get_crs_helper
      * \ingroup amesos2_util
      */
@@ -584,7 +584,7 @@ namespace Amesos2 {
      * \brief Similar to get_ccs_helper , but used to get a CRS
      * representation of the given matrix.
      *
-     * \sa \ref get_ccs_helper 
+     * \sa \ref get_ccs_helper
      * \ingroup amesos2_util
      */
     template<class Matrix, typename S, typename GO, typename GS>
@@ -601,35 +601,31 @@ namespace Amesos2 {
     template <typename LO, typename GO, typename GS, typename Node>
     const Teuchos::RCP<const Tpetra::Map<LO,GO,Node> >
     getDistributionMap(EDistribution distribution,
-		       GS num_global_elements,
-		       const Teuchos::RCP<const Teuchos::Comm<int> >& comm,
-		       GO indexBase)
+                       GS num_global_elements,
+                       const Teuchos::RCP<const Teuchos::Comm<int> >& comm,
+                       GO indexBase)
     {
         // TODO: Need to add indexBase to cases other than ROOTED
         //  We do not support these maps in any solver now.
       switch( distribution ){
       case DISTRIBUTED:
       case DISTRIBUTED_NO_OVERLAP:
-	return Tpetra::createUniformContigMapWithNode<LO,GO, Node>(num_global_elements, comm);
-	break;
+        return Tpetra::createUniformContigMapWithNode<LO,GO, Node>(num_global_elements, comm);
       case GLOBALLY_REPLICATED:
-	return Tpetra::createLocalMapWithNode<LO,GO, Node>(num_global_elements, comm);
-	break;
+        return Tpetra::createLocalMapWithNode<LO,GO, Node>(num_global_elements, comm);
       case ROOTED:
-	{
-	  int rank = Teuchos::rank(*comm);
-	  size_t my_num_elems = Teuchos::OrdinalTraits<size_t>::zero();
-	  if( rank == 0 ) my_num_elems = num_global_elements;
-	  return rcp(new Tpetra::Map<LO,GO, Node>(num_global_elements,
-						my_num_elems, indexBase, comm));
-	  break;
-	}
+        {
+          int rank = Teuchos::rank(*comm);
+          size_t my_num_elems = Teuchos::OrdinalTraits<size_t>::zero();
+          if( rank == 0 ) my_num_elems = num_global_elements;
+          return rcp(new Tpetra::Map<LO,GO, Node>(num_global_elements,
+                                                my_num_elems, indexBase, comm));
+        }
       default:
-	TEUCHOS_TEST_FOR_EXCEPTION( true,
-			    std::logic_error,
-			    "Control should never reach this point.  "
-			    "Please contact the Amesos2 developers." );
-	break;
+        TEUCHOS_TEST_FOR_EXCEPTION( true,
+                            std::logic_error,
+                            "Control should never reach this point.  "
+                            "Please contact the Amesos2 developers." );
       }
     }
 
@@ -653,9 +649,9 @@ namespace Amesos2 {
 
       typedef Tpetra::Map<LO,GO,Node> map_t;
       RCP<map_t> tmap = rcp(new map_t(Teuchos::OrdinalTraits<GS>::invalid(),
-				      my_global_elements(),
-				      as<GO>(map.IndexBase()),
-				      to_teuchos_comm(Teuchos::rcpFromRef(map.Comm()))));
+                                      my_global_elements(),
+                                      as<GO>(map.IndexBase()),
+                                      to_teuchos_comm(Teuchos::rcpFromRef(map.Comm()))));
       return tmap;
     }
 
@@ -664,35 +660,35 @@ namespace Amesos2 {
     tpetra_map_to_epetra_map(const Tpetra::Map<LO,GO,Node>& map)
     {
       using Teuchos::as;
-      
+
       Teuchos::Array<GO> elements_tmp;
       elements_tmp = map.getNodeElementList();
       int num_my_elements = elements_tmp.size();
       Teuchos::Array<int> my_global_elements(num_my_elements);
       for (int i = 0; i < num_my_elements; ++i){
-	my_global_elements[i] = as<int>(elements_tmp[i]);
+        my_global_elements[i] = as<int>(elements_tmp[i]);
       }
 
       using Teuchos::rcp;
       RCP<Epetra_Map> emap = rcp(new Epetra_Map(-1,
-						num_my_elements,
-						my_global_elements.getRawPtr(),
-						as<GO>(map.getIndexBase()),
-						*to_epetra_comm(map.getComm())));
+                                                num_my_elements,
+                                                my_global_elements.getRawPtr(),
+                                                as<GO>(map.getIndexBase()),
+                                                *to_epetra_comm(map.getComm())));
       return emap;
     }
-#endif	// HAVE_AMESOS2_EPETRA
+#endif  // HAVE_AMESOS2_EPETRA
 #endif  // HAVE_TPETRA_INST_INT_INT
 
     template <typename Scalar,
-	      typename GlobalOrdinal,
-	      typename GlobalSizeT>
+              typename GlobalOrdinal,
+              typename GlobalSizeT>
     void transpose(Teuchos::ArrayView<Scalar> vals,
-		   Teuchos::ArrayView<GlobalOrdinal> indices,
-		   Teuchos::ArrayView<GlobalSizeT> ptr,
-		   Teuchos::ArrayView<Scalar> trans_vals,
-		   Teuchos::ArrayView<GlobalOrdinal> trans_indices,
-		   Teuchos::ArrayView<GlobalSizeT> trans_ptr)
+                   Teuchos::ArrayView<GlobalOrdinal> indices,
+                   Teuchos::ArrayView<GlobalSizeT> ptr,
+                   Teuchos::ArrayView<Scalar> trans_vals,
+                   Teuchos::ArrayView<GlobalOrdinal> trans_indices,
+                   Teuchos::ArrayView<GlobalSizeT> trans_ptr)
     {
       /* We have a compressed-row storage format of this matrix.  We
        * transform this into a compressed-column format using a
@@ -706,14 +702,14 @@ namespace Amesos2 {
       ind_end = indices.end();
       size_t min_trans_ptr_size = *std::max_element(ind_begin, ind_end) + 1;
       TEUCHOS_TEST_FOR_EXCEPTION( Teuchos::as<size_t>(trans_ptr.size()) < min_trans_ptr_size,
-			  std::invalid_argument,
-			  "Transpose pointer size not large enough." );
+                          std::invalid_argument,
+                          "Transpose pointer size not large enough." );
       TEUCHOS_TEST_FOR_EXCEPTION( trans_vals.size() < vals.size(),
-			  std::invalid_argument,
-			  "Transpose values array not large enough." );
+                          std::invalid_argument,
+                          "Transpose values array not large enough." );
       TEUCHOS_TEST_FOR_EXCEPTION( trans_indices.size() < indices.size(),
-			  std::invalid_argument,
-			  "Transpose indices array not large enough." );
+                          std::invalid_argument,
+                          "Transpose indices array not large enough." );
 #else
       typename Teuchos::ArrayView<GlobalOrdinal>::iterator ind_it, ind_end;
 #endif
@@ -722,14 +718,14 @@ namespace Amesos2 {
       Teuchos::Array<GlobalSizeT> count(trans_ptr.size(), 0);
       ind_end = indices.end();
       for( ind_it = indices.begin(); ind_it != ind_end; ++ind_it ){
-	++(count[(*ind_it) + 1]);
+        ++(count[(*ind_it) + 1]);
       }
 
       // Accumulate
       typename Teuchos::Array<GlobalSizeT>::iterator cnt_it, cnt_end;
       cnt_end = count.end();
       for( cnt_it = count.begin() + 1; cnt_it != cnt_end; ++cnt_it ){
-	*cnt_it = *cnt_it + *(cnt_it - 1);
+        *cnt_it = *cnt_it + *(cnt_it - 1);
       }
       // This becomes the array of column pointers
       trans_ptr.assign(count);
@@ -748,14 +744,14 @@ namespace Amesos2 {
 
       GlobalSizeT size = ptr.size();
       for( GlobalSizeT i = 0; i < size - 1; ++i ){
-	GlobalOrdinal u = ptr[i];
-	GlobalOrdinal v = ptr[i + 1];
-	for( GlobalOrdinal j = u; j < v; ++j ){
-	  GlobalOrdinal k = count[indices[j]];
-	  trans_vals[k] = vals[j];
-	  trans_indices[k] = i;
-	  ++(count[indices[j]]);
-	}
+        GlobalOrdinal u = ptr[i];
+        GlobalOrdinal v = ptr[i + 1];
+        for( GlobalOrdinal j = u; j < v; ++j ){
+          GlobalOrdinal k = count[indices[j]];
+          trans_vals[k] = vals[j];
+          trans_indices[k] = i;
+          ++(count[indices[j]]);
+        }
       }
     }
 
@@ -763,47 +759,47 @@ namespace Amesos2 {
     template <typename Scalar1, typename Scalar2>
     void
     scale(Teuchos::ArrayView<Scalar1> vals, size_t l,
-	  size_t ld, Teuchos::ArrayView<Scalar2> s)
+          size_t ld, Teuchos::ArrayView<Scalar2> s)
     {
       size_t vals_size = vals.size();
 #ifdef HAVE_AMESOS2_DEBUG
       size_t s_size = s.size();
       TEUCHOS_TEST_FOR_EXCEPTION( s_size < l,
-			  std::invalid_argument,
-			  "Scale vector must have length at least that of the vector" );
+                          std::invalid_argument,
+                          "Scale vector must have length at least that of the vector" );
 #endif
       size_t i, s_i;
       for( i = 0, s_i = 0; i < vals_size; ++i, ++s_i ){
-	if( s_i == l ){
-	  // bring i to the next multiple of ld
-	  i += ld - s_i;
-	  s_i = 0;
-	}
-	vals[i] *= s[s_i];
+        if( s_i == l ){
+          // bring i to the next multiple of ld
+          i += ld - s_i;
+          s_i = 0;
+        }
+        vals[i] *= s[s_i];
       }
     }
 
     template <typename Scalar1, typename Scalar2, class BinaryOp>
     void
     scale(Teuchos::ArrayView<Scalar1> vals, size_t l,
-	  size_t ld, Teuchos::ArrayView<Scalar2> s,
-	  BinaryOp binary_op)
+          size_t ld, Teuchos::ArrayView<Scalar2> s,
+          BinaryOp binary_op)
     {
       size_t vals_size = vals.size();
 #ifdef HAVE_AMESOS2_DEBUG
       size_t s_size = s.size();
       TEUCHOS_TEST_FOR_EXCEPTION( s_size < l,
-			  std::invalid_argument,
-			  "Scale vector must have length at least that of the vector" );
+                          std::invalid_argument,
+                          "Scale vector must have length at least that of the vector" );
 #endif
       size_t i, s_i;
       for( i = 0, s_i = 0; i < vals_size; ++i, ++s_i ){
-	if( s_i == l ){
-	  // bring i to the next multiple of ld
-	  i += ld - s_i;
-	  s_i = 0;
-	}
-	vals[i] = binary_op(vals[i], s[s_i]);
+        if( s_i == l ){
+          // bring i to the next multiple of ld
+          i += ld - s_i;
+          s_i = 0;
+        }
+        vals[i] = binary_op(vals[i], s[s_i]);
       }
     }
 
@@ -813,4 +809,4 @@ namespace Amesos2 {
 
 } // end namespace Amesos2
 
-#endif	// #ifndef AMESOS2_UTIL_HPP
+#endif  // #ifndef AMESOS2_UTIL_HPP
