@@ -148,7 +148,7 @@ int main (int argc, char *argv[]) {
 
     // Create an Epetra_Matrix
 
-    Teuchos::RCP<Epetra_CrsMatrix> A = Teuchos::rcp( new Epetra_CrsMatrix(Copy, Map, &NumNz[0]) );
+    Teuchos::RCP<Epetra_CrsMatrix> A = Teuchos::rcp( new Epetra_CrsMatrix(Epetra_DataAccess::Copy, Map, &NumNz[0]) );
 
     if (MyPID == 0) {
       cout << "Filling matrix" << endl;
