@@ -50,6 +50,7 @@
 #include "AnasaziOperatorTraits.hpp"
 #include "AnasaziOutputManager.hpp"
 
+#include "Teuchos_MatrixMarket_SetScientific.hpp"
 #include "Teuchos_RCP.hpp"
 #include "Teuchos_as.hpp"
 
@@ -66,6 +67,7 @@ namespace Anasazi {
                 const Teuchos::RCP<const MV> &A ) {
 
     using std::endl;
+    using Teuchos::MatrixMarket::details::SetScientific;
 
     /* MVT Contract:
 
@@ -148,6 +150,7 @@ namespace Anasazi {
     const ScalarType one      = SCT::one();
     const ScalarType zero     = SCT::zero();
     const MagType    zero_mag = Teuchos::ScalarTraits<MagType>::zero();
+    const MagType    tol = SCT::eps()*100;
 
     // Don't change these two without checking the initialization of ind below
     const int numvecs   = 10;
@@ -200,7 +203,8 @@ namespace Anasazi {
     *********************************************************************/
     {
       Teuchos::RCP<MV> B = MVT::Clone(*A,numvecs);
-      std::vector<MagType> norms(numvecs);
+      std::vector<MagType> norms(2*numvecs);
+      bool ResizeWarning = false;
       if ( MVT::GetNumberVecs(*B) != numvecs ) {
         om->stream(Warnings)
           << "*** ERROR *** MultiVecTraits::Clone()." << endl
@@ -214,10 +218,11 @@ namespace Anasazi {
         return false;
       }
       MVT::MvNorm(*B, norms);
-      if ( (int)norms.size() != numvecs ) {
+      if ( (int)norms.size() != 2*numvecs && (ResizeWarning == false) ) {
         om->stream(Warnings)
-          << "*** ERROR *** MultiVecTraits::MvNorm()." << endl
+          << "*** WARNING *** MultiVecTraits::MvNorm()." << endl
           << "Method resized the output vector." << endl;
+        ResizeWarning = true;
       }
       for (int i=0; i<numvecs; i++) {
         if ( norms[i] < zero_mag ) {
@@ -420,21 +425,26 @@ namespace Anasazi {
         return false;
       }
       for (int i=0; i<numvecs_2; i++) {
-        if ( norms2[i] != norms[ind[i]] ) {
+        if ( SCT::magnitude( norms2[i] - norms[ind[i]] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::CloneCopy(ind)." << endl
             << "Copied vectors do not agree:" 
-            << norms2[i] << " != " << norms[ind[i]] << endl;
+            << norms2[i] << " != " << norms[ind[i]] << endl
+            << "Difference " << SCT::magnitude (norms2[i] - norms[ind[i]])
+            << " exceeds the tolerance 100*eps = " << tol << endl;
+
           return false;
         }
       }
       MVT::MvInit(*B,zero);
       MVT::MvNorm(*C, norms2); 
       for (int i=0; i<numvecs_2; i++) {
-        if ( norms2[i] != norms2[i] ) {
+        if ( SCT::magnitude( norms2[i] - norms2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::CloneCopy(ind)." << endl
-            << "Copied vectors were not independent." << endl;
+            << "Copied vectors were not independent." << endl
+            << "Difference " << SCT::magnitude (norms2[i] - norms[i])
+            << " exceeds the tolerance 100*eps = " << tol << endl;
           return false;
         }
       }
@@ -462,20 +472,24 @@ namespace Anasazi {
         return false;
       }
       for (int i=0; i<numvecs; i++) {
-        if ( norms2[i] != norms[i] ) {
+        if ( SCT::magnitude( norms2[i] - norms[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::CloneCopy()." << endl
-            << "Copied vectors do not agree." << endl;
+            << "Copied vectors do not agree." << endl
+            << "Difference " << SCT::magnitude (norms2[i] - norms[i])
+            << " exceeds the tolerance 100*eps = " << tol << endl;
           return false;
         }
       }
       MVT::MvInit(*B,zero);
       MVT::MvNorm(*C, norms); 
       for (int i=0; i<numvecs; i++) {
-        if ( norms2[i] != norms[i] ) {
+        if ( SCT::magnitude( norms2[i] - norms[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::CloneCopy()." << endl
-            << "Copied vectors were not independent." << endl;
+            << "Copied vectors were not independent." << endl
+            << "Difference " << SCT::magnitude (norms2[i] - norms[i])
+            << " exceeds the tolerance 100*eps = " << tol << endl;
           return false;
         }
       }
@@ -504,7 +518,7 @@ namespace Anasazi {
         return false;
       }
       for (int i=0; i<numvecs_2; i++) {
-        if ( norms2[i] != norms[ind[i]] ) {
+        if ( SCT::magnitude( norms2[i] - norms[ind[i]] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::CloneView(ind)." << endl
             << "Viewed vectors do not agree." << endl;
@@ -541,7 +555,7 @@ namespace Anasazi {
         return false;
       }
       for (int i=0; i<numvecs_2; i++) {
-        if ( normsC[i] != normsB[ind[i]] ) {
+        if ( SCT::magnitude( normsC[i] - normsB[ind[i]] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** const MultiVecTraits::CloneView(ind)." << endl
             << "Viewed vectors do not agree." << endl;
@@ -585,7 +599,7 @@ namespace Anasazi {
 
       // check that C was not changed by SetBlock
       for (int i=0; i<numvecs_2; i++) {
-        if ( normsC1[i] != normsC2[i] ) {
+        if ( SCT::magnitude( normsC1[i] - normsC2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::SetBlock()." << endl
             << "Operation modified source vectors." << endl;
@@ -597,16 +611,18 @@ namespace Anasazi {
       for (int i=0; i<numvecs; i++) {
         if (i % 2 == 0) {
           // should be a vector from C
-          if ( normsB2[i] != normsC1[i/2] ) {
+          if ( SCT::magnitude(normsB2[i]-normsC1[i/2]) > tol ) {
             om->stream(Warnings)
               << "*** ERROR *** MultiVecTraits::SetBlock()." << endl
-              << "Copied vectors do not agree." << endl;
+              << "Copied vectors do not agree." << endl
+              << "Difference " << SCT::magnitude (normsB2[i] - normsC1[i/2])
+              << " exceeds the tolerance 100*eps = " << tol << endl;
             return false;
           }
         }
         else {
           // should be an original vector
-          if ( normsB1[i] != normsB2[i] ) {
+          if ( SCT::magnitude(normsB1[i]-normsB2[i]) > tol ) {
             om->stream(Warnings)
               << "*** ERROR *** MultiVecTraits::SetBlock()." << endl
               << "Incorrect vectors were modified." << endl;
@@ -618,7 +634,7 @@ namespace Anasazi {
       MVT::MvNorm(*B,normsB1);
       // verify that we copied and didn't reference
       for (int i=0; i<numvecs; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude(normsB1[i]-normsB2[i]) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::SetBlock()." << endl
             << "Copied vectors were not independent." << endl;
@@ -767,8 +783,9 @@ namespace Anasazi {
     *********************************************************************/
     {
       const int p = 7;
+      const int q = 9;
       Teuchos::RCP<MV> B, C;
-      std::vector<ScalarType> iprods(p);
+      std::vector<ScalarType> iprods(q);
       std::vector<MagType> normsB(p), normsC(p);
 
       B = MVT::Clone(*A,p);
@@ -779,7 +796,7 @@ namespace Anasazi {
       MVT::MvNorm(*B,normsB);
       MVT::MvNorm(*C,normsC);
       MVT::MvDot( *B, *C, iprods );
-      if ( (int)iprods.size() != p ) {
+      if ( (int)iprods.size() != q ) {
         om->stream(Warnings)
           << "*** ERROR *** MultiVecTraits::MvDot." << endl
           << "Routine resized results vector." << endl;
@@ -850,19 +867,19 @@ namespace Anasazi {
       MVT::MvNorm(*C,normsC2);
       MVT::MvNorm(*D,normsD1);
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv()." << endl
             << "Input arguments were modified." << endl;
           return false;
         }
-        else if ( normsC1[i] != normsC2[i] ) {
+        else if ( SCT::magnitude( normsC1[i] - normsC2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv()." << endl
             << "Input arguments were modified." << endl;
           return false;
         }
-        else if ( normsC1[i] != normsD1[i] ) {
+        else if ( SCT::magnitude(normsC1[i]-normsD1[i]) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv()." << endl
             << "Assignment did not work." << endl;
@@ -876,19 +893,19 @@ namespace Anasazi {
       MVT::MvNorm(*C,normsC2);
       MVT::MvNorm(*D,normsD1);
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv()." << endl
             << "Input arguments were modified." << endl;
           return false;
         }
-        else if ( normsC1[i] != normsC2[i] ) {
+        else if ( SCT::magnitude( normsC1[i] - normsC2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv()." << endl
             << "Input arguments were modified." << endl;
           return false;
         }
-        else if ( normsB1[i] != normsD1[i] ) {
+        else if ( SCT::magnitude( normsB1[i] - normsD1[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv()." << endl
             << "Assignment did not work." << endl;
@@ -905,13 +922,13 @@ namespace Anasazi {
       MVT::MvNorm(*D,normsD1);
       // check that input args are not modified
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv()." << endl
             << "Input arguments were modified." << endl;
           return false;
         }
-        else if ( normsC1[i] != normsC2[i] ) {
+        else if ( SCT::magnitude( normsC1[i] - normsC2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv()." << endl
             << "Input arguments were modified." << endl;
@@ -927,19 +944,19 @@ namespace Anasazi {
       // check that input args are not modified and that D is the same
       // as the above test
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv()." << endl
             << "Input arguments were modified." << endl;
           return false;
         }
-        else if ( normsC1[i] != normsC2[i] ) {
+        else if ( SCT::magnitude( normsC1[i] - normsC2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv()." << endl
             << "Input arguments were modified." << endl;
           return false;
         }
-        else if ( normsD1[i] != normsD2[i] ) {
+        else if ( SCT::magnitude( normsD1[i] - normsD2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv()." << endl
             << "Results varies depending on initial state of dest vectors." << endl;
@@ -986,7 +1003,7 @@ namespace Anasazi {
       MVT::MvAddMv(zero,*B,one,*C,*D);
       MVT::MvNorm(*D,normsD);
       for (int i=0; i<p; i++) {
-        if ( normsB[i] != normsD[i] ) {
+        if ( SCT::magnitude( normsB[i] - normsD[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv() #2" << endl
             << "Assignment did not work." << endl;
@@ -998,7 +1015,7 @@ namespace Anasazi {
       MVT::MvAddMv(one,*B,zero,*C,*D);
       MVT::MvNorm(*D,normsD);
       for (int i=0; i<p; i++) {
-        if ( normsB[i] != normsD[i] ) {
+        if ( SCT::magnitude( normsB[i] - normsD[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvAddMv() #2" << endl
             << "Assignment did not work." << endl;
@@ -1038,7 +1055,7 @@ namespace Anasazi {
       MVT::MvNorm(*B,normsB2);
       MVT::MvNorm(*C,normsC2);
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Input vectors were modified." << endl;
@@ -1046,7 +1063,7 @@ namespace Anasazi {
         }
       }
       for (int i=0; i<q; i++) {
-        if ( normsC1[i] != normsC2[i] ) {
+        if ( SCT::magnitude( normsC1[i] - normsC2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Arithmetic test 1 failed." << endl;
@@ -1064,7 +1081,7 @@ namespace Anasazi {
       MVT::MvNorm(*B,normsB2);
       MVT::MvNorm(*C,normsC2);
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Input vectors were modified." << endl;
@@ -1098,7 +1115,7 @@ namespace Anasazi {
       MVT::MvNorm(*B,normsB2);
       MVT::MvNorm(*C,normsC2);
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Input vectors were modified." << endl;
@@ -1106,7 +1123,7 @@ namespace Anasazi {
         }
       }
       for (int i=0; i<q; i++) {
-        if ( normsB1[i] != normsC2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsC2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Arithmetic test 3 failed: "
@@ -1128,7 +1145,7 @@ namespace Anasazi {
       MVT::MvNorm(*B,normsB2);
       MVT::MvNorm(*C,normsC2);
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Input vectors were modified." << endl;
@@ -1136,7 +1153,7 @@ namespace Anasazi {
         }
       }
       for (int i=0; i<q; i++) {
-        if ( normsC1[i] != normsC2[i] ) {
+        if ( SCT::magnitude( normsC1[i] - normsC2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Arithmetic test 4 failed." << endl;
@@ -1174,7 +1191,7 @@ namespace Anasazi {
       MVT::MvNorm(*B,normsB2);
       MVT::MvNorm(*C,normsC2);
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Input vectors were modified." << endl;
@@ -1182,7 +1199,7 @@ namespace Anasazi {
         }
       }
       for (int i=0; i<q; i++) {
-        if ( normsC1[i] != normsC2[i] ) {
+        if ( SCT::magnitude( normsC1[i] - normsC2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Arithmetic test 5 failed." << endl;
@@ -1200,7 +1217,7 @@ namespace Anasazi {
       MVT::MvNorm(*B,normsB2);
       MVT::MvNorm(*C,normsC2);
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Input vectors were modified." << endl;
@@ -1233,7 +1250,7 @@ namespace Anasazi {
       MVT::MvNorm(*B,normsB2);
       MVT::MvNorm(*C,normsC2);
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Input vectors were modified." << endl;
@@ -1241,7 +1258,7 @@ namespace Anasazi {
         }
       }
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsC2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsC2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Arithmetic test 7 failed." << endl;
@@ -1267,7 +1284,7 @@ namespace Anasazi {
       MVT::MvNorm(*B,normsB2);
       MVT::MvNorm(*C,normsC2);
       for (int i=0; i<p; i++) {
-        if ( normsB1[i] != normsB2[i] ) {
+        if ( SCT::magnitude( normsB1[i] - normsB2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Input vectors were modified." << endl;
@@ -1275,7 +1292,7 @@ namespace Anasazi {
         }
       }
       for (int i=0; i<q; i++) {
-        if ( normsC1[i] != normsC2[i] ) {
+        if ( SCT::magnitude( normsC1[i] - normsC2[i] ) > tol ) {
           om->stream(Warnings)
             << "*** ERROR *** MultiVecTraits::MvTimesMatAddMv()." << endl
             << "Arithmetic test 8 failed." << endl;
@@ -1315,6 +1332,8 @@ namespace Anasazi {
     typedef OperatorTraits<ScalarType, MV, OP> OPT;
     typedef typename SCT::magnitudeType        MagType;
 
+    const MagType    tol = SCT::eps()*100;
+
     const int numvecs = 10;
 
     Teuchos::RCP<MV> B = MVT::Clone(*A,numvecs), 
@@ -1322,7 +1341,6 @@ namespace Anasazi {
 
     std::vector<MagType> normsB1(numvecs), normsB2(numvecs),
                          normsC1(numvecs), normsC2(numvecs);
-    bool NonDeterministicWarning;
 
     /*********** Apply() *************************************************
         Verify:
@@ -1338,7 +1356,7 @@ namespace Anasazi {
     MVT::MvNorm(*B,normsB2);
     MVT::MvNorm(*C,normsC2);
     for (int i=0; i<numvecs; i++) {
-      if (normsB2[i] != normsB1[i]) {
+      if ( SCT::magnitude( normsB2[i] - normsB1[i] ) > tol ) {
         om->stream(Warnings)
           << "*** ERROR *** OperatorTraits::Apply() [1]" << endl
           << "Apply() modified the input vectors." << endl;
@@ -1360,7 +1378,7 @@ namespace Anasazi {
     MVT::MvNorm(*C,normsC2);
     bool ZeroWarning = false;
     for (int i=0; i<numvecs; i++) {
-      if (normsB2[i] != normsB1[i]) {
+      if ( SCT::magnitude( normsB2[i] - normsB1[i] ) > tol ) {
         om->stream(Warnings)
           << "*** ERROR *** OperatorTraits::Apply() [2]" << endl
           << "Apply() modified the input vectors." << endl;
@@ -1382,7 +1400,7 @@ namespace Anasazi {
     MVT::MvNorm(*B,normsB2);
     MVT::MvNorm(*C,normsC1);
     for (int i=0; i<numvecs; i++) {
-      if (normsB2[i] != normsB1[i]) {
+      if ( SCT::magnitude( normsB2[i] - normsB1[i] ) > tol ) {
         om->stream(Warnings)
           << "*** ERROR *** OperatorTraits::Apply() [3]" << endl
           << "Apply() modified the input vectors." << endl;
@@ -1399,20 +1417,18 @@ namespace Anasazi {
     OPT::Apply(*M,*B,*C);
     MVT::MvNorm(*B,normsB2);
     MVT::MvNorm(*C,normsC2);
-    NonDeterministicWarning = false;
     for (int i=0; i<numvecs; i++) {
-      if (normsB2[i] != normsB1[i]) {
+      if ( SCT::magnitude( normsB2[i] - normsB1[i] ) > tol ) {
         om->stream(Warnings)
           << "*** ERROR *** OperatorTraits::Apply() [4]" << endl
           << "Apply() modified the input vectors." << endl;
         return false;
       }
-      if (normsC1[i] != normsC2[i] && !NonDeterministicWarning) {
+      if ( SCT::magnitude( normsC1[i] - normsC2[i]) > tol ) {
         om->stream(Warnings)
           << endl
           << "*** WARNING *** OperatorTraits::Apply() [4]" << endl
           << "Apply() returned two different results." << endl << endl;
-        NonDeterministicWarning = true;
       }
     }
 
