@@ -59,10 +59,7 @@
 #ifdef HAVE_BELOS_TSQR
 #  include "BelosTsqrOrthoManager.hpp"
 #endif // HAVE_BELOS_TSQR
-#include "BelosStatusTestMaxIters.hpp"
-#include "BelosStatusTestGenResNorm.hpp"
-#include "BelosStatusTestImpResNorm.hpp"
-#include "BelosStatusTestCombo.hpp"
+#include "BelosStatusTestFactory.hpp"
 #include "BelosStatusTestOutputFactory.hpp"
 #include "BelosOutputManager.hpp"
 #include "Teuchos_BLAS.hpp"
@@ -251,6 +248,12 @@ namespace Belos {
      *   right-hand sides, this controls whether output shows residual
      *   norms for all the right-hand sides, or just the current
      *   maximum residual norm over all right-hand sides.
+     *		\param pl [in] ParameterList with construction information
+     *			\htmlonly
+     *			<iframe src="belos_PseudoBlockGmres.xml" width=100% scrolling="no" frameborder="0">
+     *			</iframe>
+     *			<hr />
+     *			\endhtmlonly
      */
     PseudoBlockGmresSolMgr( const Teuchos::RCP<LinearProblem<ScalarType,MV,OP> > &problem,
                             const Teuchos::RCP<Teuchos::ParameterList> &pl );
@@ -377,7 +380,9 @@ namespace Belos {
     /// (short-circuiting OR, like the || operator in C++) after
     /// Pseudoblock GMRES' standard convergence test.
     virtual void setUserConvStatusTest(
-      const Teuchos::RCP<StatusTest<ScalarType,MV,OP> > &userConvStatusTest
+      const Teuchos::RCP<StatusTest<ScalarType,MV,OP> > &userConvStatusTest,
+      const typename StatusTestCombo<ScalarType,MV,OP>::ComboType &comboType =
+          StatusTestCombo<ScalarType,MV,OP>::SEQ
       );
 
     /// \brief Set a debug status test.
@@ -476,6 +481,8 @@ namespace Belos {
     Teuchos::RCP<StatusTest<ScalarType,MV,OP> > convTest_;
     Teuchos::RCP<StatusTestResNorm<ScalarType,MV,OP> > impConvTest_, expConvTest_;
     Teuchos::RCP<StatusTestOutput<ScalarType,MV,OP> > outputTest_;
+    typename StatusTestCombo<ScalarType,MV,OP>::ComboType comboType_;
+    Teuchos::RCP<std::map<std::string, Teuchos::RCP<StatusTest<ScalarType, MV, OP> > > > taggedTests_;
 
     // Orthogonalization manager.
     Teuchos::RCP<MatOrthoManager<ScalarType,MV,OP> > ortho_;
@@ -497,6 +504,7 @@ namespace Belos {
     static const int defQuorum_default_;
     static const std::string impResScale_default_;
     static const std::string expResScale_default_;
+    static const MagnitudeType resScaleFactor_default_;
     static const std::string label_default_;
     static const std::string orthoType_default_;
     static const Teuchos::RCP<std::ostream> outputStream_default_;
@@ -508,6 +516,7 @@ namespace Belos {
     bool showMaxResNormOnly_;
     std::string orthoType_;
     std::string impResScale_, expResScale_;
+    MagnitudeType resScaleFactor_;
 
     // Timers.
     std::string label_;
@@ -559,6 +568,9 @@ const std::string PseudoBlockGmresSolMgr<ScalarType,MV,OP>::impResScale_default_
 template<class ScalarType, class MV, class OP>
 const std::string PseudoBlockGmresSolMgr<ScalarType,MV,OP>::expResScale_default_ = "Norm of Initial Residual";
 
+template<class ScalarType, class MV, class OP>
+const typename PseudoBlockGmresSolMgr<ScalarType,MV,OP>::MagnitudeType PseudoBlockGmresSolMgr<ScalarType,MV,OP>::resScaleFactor_default_ = 1.0;
+
 template<class ScalarType, class MV, class OP>
 const std::string PseudoBlockGmresSolMgr<ScalarType,MV,OP>::label_default_ = "Belos";
 
@@ -568,11 +580,11 @@ const std::string PseudoBlockGmresSolMgr<ScalarType,MV,OP>::orthoType_default_ =
 template<class ScalarType, class MV, class OP>
 const Teuchos::RCP<std::ostream> PseudoBlockGmresSolMgr<ScalarType,MV,OP>::outputStream_default_ = Teuchos::rcp(&std::cout,false);
 
-
 // Empty Constructor
 template<class ScalarType, class MV, class OP>
 PseudoBlockGmresSolMgr<ScalarType,MV,OP>::PseudoBlockGmresSolMgr() :
   outputStream_(outputStream_default_),
+  taggedTests_(Teuchos::null),
   convtol_(convtol_default_),
   orthoKappa_(orthoKappa_default_),
   achievedTol_(Teuchos::ScalarTraits<typename Teuchos::ScalarTraits<ScalarType>::magnitudeType>::zero()),
@@ -589,6 +601,7 @@ PseudoBlockGmresSolMgr<ScalarType,MV,OP>::PseudoBlockGmresSolMgr() :
   orthoType_(orthoType_default_),
   impResScale_(impResScale_default_),
   expResScale_(expResScale_default_),
+  resScaleFactor_(resScaleFactor_default_),
   label_(label_default_),
   isSet_(false),
   isSTSet_(false),
@@ -603,6 +616,7 @@ PseudoBlockGmresSolMgr (const Teuchos::RCP<LinearProblem<ScalarType,MV,OP> > &pr
                         const Teuchos::RCP<Teuchos::ParameterList> &pl) :
   problem_(problem),
   outputStream_(outputStream_default_),
+  taggedTests_(Teuchos::null),
   convtol_(convtol_default_),
   orthoKappa_(orthoKappa_default_),
   achievedTol_(Teuchos::ScalarTraits<typename Teuchos::ScalarTraits<ScalarType>::magnitudeType>::zero()),
@@ -619,6 +633,7 @@ PseudoBlockGmresSolMgr (const Teuchos::RCP<LinearProblem<ScalarType,MV,OP> > &pr
   orthoType_(orthoType_default_),
   impResScale_(impResScale_default_),
   expResScale_(expResScale_default_),
+  resScaleFactor_(resScaleFactor_default_),
   label_(label_default_),
   isSet_(false),
   isSTSet_(false),
@@ -648,7 +663,12 @@ setParameters (const Teuchos::RCP<Teuchos::ParameterList>& params)
   if (params_ == Teuchos::null) {
     params_ = parameterList (*getValidParameters ());
   } else {
-    params->validateParameters (*getValidParameters ());
+    // TAW: 3/8/2016: do not validate sub parameter lists as they
+    //                might not have a pre-defined structure
+    //                e.g. user-specified status tests
+    // The Belos Pseudo Block GMRES parameters on the first level are
+    // not affected and verified.
+    params->validateParameters (*getValidParameters (), 0);
   }
 
   // Check for maximum number of restarts
@@ -808,6 +828,17 @@ setParameters (const Teuchos::RCP<Teuchos::ParameterList>& params)
     if (! outputTest_.is_null ()) {
       isSTSet_ = false;
     }
+
+    // Check if user has specified his own status tests
+    if (params->isSublist ("User Status Tests")) {
+      Teuchos::ParameterList userStatusTestsList = params->sublist("User Status Tests",true);
+      if ( userStatusTestsList.numParams() > 0 ) {
+	std::string userCombo_string = params->get<std::string>("User Status Tests Combo Type", "SEQ");
+	Teuchos::RCP<StatusTestFactory<ScalarType,MV,OP> > testFactory = Teuchos::rcp(new StatusTestFactory<ScalarType,MV,OP>());
+	setUserConvStatusTest( testFactory->buildStatusTests(userStatusTestsList), testFactory->stringToComboType(userCombo_string) );
+	taggedTests_ = testFactory->getTaggedTests();
+      }
+    }
   }
 
   // output stream
@@ -855,6 +886,43 @@ setParameters (const Teuchos::RCP<Teuchos::ParameterList>& params)
     }
   }
 
+  // Grab the user defined residual scaling
+  bool userDefinedResidualScalingUpdated = false;
+  if (params->isParameter ("User Defined Residual Scaling")) {
+    const MagnitudeType tempResScaleFactor =
+      Teuchos::getParameter<MagnitudeType> (*params, "User Defined Residual Scaling");
+
+    // Only update the scaling if it's different.
+    if (resScaleFactor_ != tempResScaleFactor) {
+      resScaleFactor_ = tempResScaleFactor;
+      userDefinedResidualScalingUpdated = true;
+    }
+
+    if(userDefinedResidualScalingUpdated)
+    {
+      if (! params->isParameter ("Implicit Residual Scaling") && ! impConvTest_.is_null ()) {
+        try {
+          if(impResScale_ == "User Provided")
+            impConvTest_->defineScaleForm (Belos::UserProvided, Belos::TwoNorm, resScaleFactor_);
+        }
+        catch (std::exception& e) {
+          // Make sure the convergence test gets constructed again.
+          isSTSet_ = false;
+        }
+      }
+      if (! params->isParameter ("Explicit Residual Scaling") && ! expConvTest_.is_null ()) {
+        try {
+          if(expResScale_ == "User Provided")
+            expConvTest_->defineScaleForm (Belos::UserProvided, Belos::TwoNorm, resScaleFactor_);
+        }
+        catch (std::exception& e) {
+          // Make sure the convergence test gets constructed again.
+          isSTSet_ = false;
+        }
+      }
+    }
+  }
+
   // Check for a change in scaling, if so we need to build new residual tests.
   if (params->isParameter ("Implicit Residual Scaling")) {
     const std::string tempImpResScale =
@@ -869,7 +937,24 @@ setParameters (const Teuchos::RCP<Teuchos::ParameterList>& params)
       params_->set ("Implicit Residual Scaling", impResScale_);
       if (! impConvTest_.is_null ()) {
         try {
-          impConvTest_->defineScaleForm (impResScaleType, Belos::TwoNorm);
+          if(impResScale_ == "User Provided")
+            impConvTest_->defineScaleForm (impResScaleType, Belos::TwoNorm, resScaleFactor_);
+          else
+            impConvTest_->defineScaleForm (impResScaleType, Belos::TwoNorm);
+        }
+        catch (std::exception& e) {
+          // Make sure the convergence test gets constructed again.
+          isSTSet_ = false;
+        }
+      }
+    }
+    else if (userDefinedResidualScalingUpdated) {
+      Belos::ScaleType impResScaleType = convertStringToScaleType (impResScale_);
+
+      if (! impConvTest_.is_null ()) {
+        try {
+          if(impResScale_ == "User Provided")
+            impConvTest_->defineScaleForm (impResScaleType, Belos::TwoNorm, resScaleFactor_);
         }
         catch (std::exception& e) {
           // Make sure the convergence test gets constructed again.
@@ -892,7 +977,24 @@ setParameters (const Teuchos::RCP<Teuchos::ParameterList>& params)
       params_->set ("Explicit Residual Scaling", expResScale_);
       if (! expConvTest_.is_null ()) {
         try {
-          expConvTest_->defineScaleForm (expResScaleType, Belos::TwoNorm);
+          if(expResScale_ == "User Provided")
+            expConvTest_->defineScaleForm (expResScaleType, Belos::TwoNorm, resScaleFactor_);
+          else
+            expConvTest_->defineScaleForm (expResScaleType, Belos::TwoNorm);
+        }
+        catch (std::exception& e) {
+          // Make sure the convergence test gets constructed again.
+          isSTSet_ = false;
+        }
+      }
+    }
+    else if (userDefinedResidualScalingUpdated) {
+      Belos::ScaleType expResScaleType = convertStringToScaleType (expResScale_);
+
+      if (! expConvTest_.is_null ()) {
+        try {
+          if(expResScale_ == "User Provided")
+            expConvTest_->defineScaleForm (expResScaleType, Belos::TwoNorm, resScaleFactor_);
         }
         catch (std::exception& e) {
           // Make sure the convergence test gets constructed again.
@@ -997,10 +1099,12 @@ setParameters (const Teuchos::RCP<Teuchos::ParameterList>& params)
 template<class ScalarType, class MV, class OP>
 void
 PseudoBlockGmresSolMgr<ScalarType,MV,OP>::setUserConvStatusTest(
-  const Teuchos::RCP<StatusTest<ScalarType,MV,OP> > &userConvStatusTest
+  const Teuchos::RCP<StatusTest<ScalarType,MV,OP> > &userConvStatusTest,
+  const typename StatusTestCombo<ScalarType,MV,OP>::ComboType &comboType
   )
 {
   userConvStatusTest_ = userConvStatusTest;
+  comboType_ = comboType;
 }
 
 template<class ScalarType, class MV, class OP>
@@ -1067,6 +1171,10 @@ PseudoBlockGmresSolMgr<ScalarType,MV,OP>::getValidParameters() const
     pl->set("Orthogonalization Constant",orthoKappa_default_,
       "The constant used by DGKS orthogonalization to determine\n"
       "whether another step of classical Gram-Schmidt is necessary.");
+    pl->sublist("User Status Tests");
+    pl->set("User Status Tests Combo Type", "SEQ",
+        "Type of logical combination operation of user-defined\n"
+        "and/or solver-specific status tests.");
     validPL = pl;
   }
   return validPL;
@@ -1094,7 +1202,11 @@ bool PseudoBlockGmresSolMgr<ScalarType,MV,OP>::checkStatusTest() {
     // Implicit residual test, using the native residual to determine if convergence was achieved.
     Teuchos::RCP<StatusTestGenResNorm_t> tmpImpConvTest =
       Teuchos::rcp( new StatusTestGenResNorm_t( convtol_, defQuorum_ ) );
-    tmpImpConvTest->defineScaleForm( convertStringToScaleType(impResScale_), Belos::TwoNorm );
+    if(impResScale_ == "User Provided")
+      tmpImpConvTest->defineScaleForm( convertStringToScaleType(impResScale_), Belos::TwoNorm, resScaleFactor_ );
+    else
+      tmpImpConvTest->defineScaleForm( convertStringToScaleType(impResScale_), Belos::TwoNorm );
+
     tmpImpConvTest->setShowMaxResNormOnly( showMaxResNormOnly_ );
     impConvTest_ = tmpImpConvTest;
 
@@ -1102,7 +1214,10 @@ bool PseudoBlockGmresSolMgr<ScalarType,MV,OP>::checkStatusTest() {
     Teuchos::RCP<StatusTestGenResNorm_t> tmpExpConvTest =
       Teuchos::rcp( new StatusTestGenResNorm_t( convtol_, defQuorum_ ) );
     tmpExpConvTest->defineResForm( StatusTestGenResNorm_t::Explicit, Belos::TwoNorm );
-    tmpExpConvTest->defineScaleForm( convertStringToScaleType(expResScale_), Belos::TwoNorm );
+    if(expResScale_ == "User Provided")
+      tmpExpConvTest->defineScaleForm( convertStringToScaleType(expResScale_), Belos::TwoNorm, resScaleFactor_ );
+    else
+      tmpExpConvTest->defineScaleForm( convertStringToScaleType(expResScale_), Belos::TwoNorm );
     tmpExpConvTest->setShowMaxResNormOnly( showMaxResNormOnly_ );
     expConvTest_ = tmpExpConvTest;
 
@@ -1115,7 +1230,10 @@ bool PseudoBlockGmresSolMgr<ScalarType,MV,OP>::checkStatusTest() {
     // Use test that checks for loss of accuracy.
     Teuchos::RCP<StatusTestImpResNorm_t> tmpImpConvTest =
       Teuchos::rcp( new StatusTestImpResNorm_t( convtol_, defQuorum_ ) );
-    tmpImpConvTest->defineScaleForm( convertStringToScaleType(impResScale_), Belos::TwoNorm );
+    if(impResScale_ == "User Provided")
+      tmpImpConvTest->defineScaleForm( convertStringToScaleType(impResScale_), Belos::TwoNorm, resScaleFactor_ );
+    else
+      tmpImpConvTest->defineScaleForm( convertStringToScaleType(impResScale_), Belos::TwoNorm );
     tmpImpConvTest->setShowMaxResNormOnly( showMaxResNormOnly_ );
     impConvTest_ = tmpImpConvTest;
 
@@ -1133,16 +1251,22 @@ bool PseudoBlockGmresSolMgr<ScalarType,MV,OP>::checkStatusTest() {
   if (nonnull(userConvStatusTest_) ) {
     // Override the overall convergence test with the users convergence test
     convTest_ = Teuchos::rcp(
-      new StatusTestCombo_t( StatusTestCombo_t::SEQ, convTest_, userConvStatusTest_ ) );
+      new StatusTestCombo_t( comboType_, convTest_, userConvStatusTest_ ) );
+    // brief output style not compatible with more general combinations
+    //outputStyle_ = Belos::General;
     // NOTE: Above, you have to run the other convergence tests also because
     // the logic in this class depends on it.  This is very unfortunate.
   }
 
   sTest_ = Teuchos::rcp( new StatusTestCombo_t( StatusTestCombo_t::OR, maxIterTest_, convTest_ ) );
 
+  /*if(taggedTests_ != Teuchos::null) {
+    std::cout << "# tagged tests " << taggedTests_->size() << std::endl;
+  }*/
+
   // Create the status test output class.
   // This class manages and formats the output from the status test.
-  StatusTestOutputFactory<ScalarType,MV,OP> stoFactory( outputStyle_ );
+  StatusTestOutputFactory<ScalarType,MV,OP> stoFactory( outputStyle_, taggedTests_ );
   outputTest_ = stoFactory.create( printer_, sTest_, outputFreq_, Passed+Failed+Undefined );
 
   // Set the solver string for the output test
