@@ -292,9 +292,9 @@ class StatusTestGenResNorm: public StatusTestResNorm<ScalarType,MV,OP> {
       else {
         oss << "(";
         oss << ((scalenormtype_==OneNorm) ? "1-Norm" : (resnormtype_==TwoNorm) ? "2-Norm" : "Inf-Norm");
-        if (scaletype_==NormOfInitRes)
+        if (scaletype_==NormOfInitRes || scaletype_==NormOfFullInitRes || scaletype_==NormOfFullScaledInitRes)
           oss << " Res0";
-        else if (scaletype_==NormOfPrecInitRes)
+        else if (scaletype_==NormOfPrecInitRes || scaletype_==NormOfFullPrecInitRes || scaletype_==NormOfFullScaledPrecInitRes)
           oss << " Prec Res0";
         else
           oss << " RHS ";
@@ -657,13 +657,13 @@ StatusType StatusTestGenResNorm<ScalarType,MV,OP>::firstCallCheckStatusSetup( It
       scalevector_.resize( numrhs_ );
       MVT::MvNorm( *rhs, scalevector_, scalenormtype_ );
     }
-    else if (scaletype_==NormOfInitRes) {
+    else if (scaletype_==NormOfInitRes || scaletype_==NormOfFullInitRes || scaletype_==NormOfFullScaledInitRes) {
       Teuchos::RCP<const MV> init_res = lp.getInitResVec();
       numrhs_ = MVT::GetNumberVecs( *init_res );
       scalevector_.resize( numrhs_ );
       MVT::MvNorm( *init_res, scalevector_, scalenormtype_ );
     }
-    else if (scaletype_==NormOfPrecInitRes) {
+    else if (scaletype_==NormOfPrecInitRes || scaletype_==NormOfFullPrecInitRes || scaletype_==NormOfFullScaledPrecInitRes) {
       Teuchos::RCP<const MV> init_res = lp.getInitPrecResVec();
       numrhs_ = MVT::GetNumberVecs( *init_res );
       scalevector_.resize( numrhs_ );
