@@ -47,10 +47,6 @@
 #include <Teuchos_ScalarTraits.hpp>
 #include <Tpetra_ConfigDefs.hpp>
 
-#if defined(HAVE_TPETRA_DEBUG) && ! defined(HAVE_IFPACK2_DEBUG)
-#  define HAVE_IFPACK2_DEBUG 1
-#endif
-
 //The sgn function isn't well defined for complex.
 //Is it correct to operate on the real part of x as is done below?
 template<class Scalar>
@@ -95,6 +91,12 @@ namespace Ifpack2 {
   */
   namespace Experimental {
   }
+
+  /// \namespace DeprecatedAndMayDisappearAtAnyTime
+  /// \brief Ifpack2 features that have been DEPRECATED and may
+  ///   DISAPPEAR AT ANY TIME.  USE AT YOUR OWN RISK.
+  namespace DeprecatedAndMayDisappearAtAnyTime {}
+
 } // namespace Ifpack2
 
 #endif /*_IFPACK2_CONFIGDEFS_HPP_*/
