@@ -45,6 +45,7 @@
 
 #include "Ifpack2_LocalFilter.hpp"
 #include "Tpetra_CrsMatrix.hpp"
+#include "Ifpack2_LocalSparseTriangularSolver.hpp"
 
 #ifdef IFPACK2_ILUK_EXPERIMENTAL
 #include <shylubasker_def.hpp>
@@ -112,8 +113,21 @@ RILUK<MatrixType>::setMatrix (const Teuchos::RCP<const row_matrix_type>& A)
     isAllocated_ = false;
     isInitialized_ = false;
     isComputed_ = false;
-    A_local_crs_ = Teuchos::null;
+    A_local_ = Teuchos::null;
     Graph_ = Teuchos::null;
+
+    // The sparse triangular solvers get a triangular factor as their
+    // input matrix.  The triangular factors L_ and U_ are getting
+    // reset, so we reset the solvers' matrices to null.  Do that
+    // before setting L_ and U_ to null, so that latter step actually
+    // frees the factors.
+    if (! L_solver_.is_null ()) {
+      L_solver_->setMatrix (Teuchos::null);
+    }
+    if (! U_solver_.is_null ()) {
+      U_solver_->setMatrix (Teuchos::null);
+    }
+
     L_ = Teuchos::null;
     U_ = Teuchos::null;
     D_ = Teuchos::null;
@@ -363,8 +377,8 @@ setParameters (const Teuchos::ParameterList& params)
     // Accept the default value.
   }
 
-  //Experimental 
-  //Note: just following RILUK original style. 
+  //Experimental
+  //Note: just following RILUK original style.
   //Do not think catch is the method for this. JDB
 #ifdef IFPACK2_ILUK_EXPERIMENTAL
   try {
@@ -376,7 +390,7 @@ setParameters (const Teuchos::ParameterList& params)
   catch (InvalidParameterName&) {
     //Use default
   }
- 
+
   try {
     basker_threads = params.get<int> ("fact: iluk experimental basker threads");
   }
@@ -411,8 +425,8 @@ setParameters (const Teuchos::ParameterList& params)
       hts_nthreads_ = params.get<int> (name);
   }
 # endif
-#endif 
-  
+#endif
+
   // "Commit" the values only after validating all of them.  This
   // ensures that there are no side effects if this routine throws an
   // exception.
@@ -523,9 +537,9 @@ void RILUK<MatrixType>::initialize ()
     isComputed_ = false;
     Graph_ = Teuchos::null;
 
-    RCP<const row_matrix_type> A_local = makeLocalFilter (A_);
+    A_local_ = makeLocalFilter (A_);
     TEUCHOS_TEST_FOR_EXCEPTION(
-      A_local.is_null (), std::logic_error, "Ifpack2::RILUK::initialize: "
+      A_local_.is_null (), std::logic_error, "Ifpack2::RILUK::initialize: "
       "makeLocalFilter returned null; it failed to compute A_local.  "
       "Please report this bug to the Ifpack2 developers.");
 
@@ -536,15 +550,15 @@ void RILUK<MatrixType>::initialize ()
     // we just copy the input matrix if it's not a CrsMatrix.
     {
       RCP<const crs_matrix_type> A_local_crs =
-        rcp_dynamic_cast<const crs_matrix_type> (A_local);
+        rcp_dynamic_cast<const crs_matrix_type> (A_local_);
       if (A_local_crs.is_null ()) {
         // FIXME (mfh 24 Jan 2014) It would be smarter to count up the
         // number of elements in each row of A_local, so that we can
         // create A_local_crs_nc using static profile.  The code below is
         // correct but potentially slow.
         RCP<crs_matrix_type> A_local_crs_nc =
-          rcp (new crs_matrix_type (A_local->getRowMap (),
-                                    A_local->getColMap (), 0));
+          rcp (new crs_matrix_type (A_local_->getRowMap (),
+                                    A_local_->getColMap (), 0));
         // FIXME (mfh 24 Jan 2014) This Import approach will only work
         // if A_ has a one-to-one row Map.  This is generally the case
         // with matrices given to Ifpack2.
@@ -553,72 +567,77 @@ void RILUK<MatrixType>::initialize ()
         // That way, the Import just implements a copy.
         typedef Tpetra::Import<local_ordinal_type, global_ordinal_type,
           node_type> import_type;
-        import_type import (A_local->getRowMap (), A_local->getRowMap ());
-        A_local_crs_nc->doImport (*A_local, import, Tpetra::REPLACE);
-        A_local_crs_nc->fillComplete (A_local->getDomainMap (), A_local->getRangeMap ());
+        import_type import (A_local_->getRowMap (), A_local_->getRowMap ());
+        A_local_crs_nc->doImport (*A_local_, import, Tpetra::REPLACE);
+        A_local_crs_nc->fillComplete (A_local_->getDomainMap (), A_local_->getRangeMap ());
         A_local_crs = rcp_const_cast<const crs_matrix_type> (A_local_crs_nc);
       }
-      A_local_crs_ = A_local_crs;
       Graph_ = rcp (new Ifpack2::IlukGraph<crs_graph_type> (A_local_crs->getCrsGraph (),
                                                             LevelOfFill_, 0));
+#ifdef IFPACK2_ILUK_EXPERIMENTAL
+      if (isExperimental_) A_local_crs_ = A_local_crs;
+#endif
     }
 
     if(!isExperimental_)
       {
-	Graph_->initialize ();
-	allocate_L_and_U ();
-	initAllValues (*A_local_crs_);
+        Graph_->initialize ();
+        allocate_L_and_U ();
+        checkOrderingConsistency (*A_local_);
+        // Do not call initAllValues. compute() always calls initAllValues to
+        // fill L and U with possibly new numbers. initialize() is concerned
+        // only with the nonzero pattern.
       }
     else
       {
 #ifdef IFPACK2_ILUK_EXPERIMENTAL
-	typedef typename node_type::device_type    kokkos_device;
-	typedef typename kokkos_device::execution_space kokkos_exe;
-	typedef typename crs_matrix_type::local_matrix_type kokkos_csr_matrix;
+        typedef typename node_type::device_type    kokkos_device;
+        typedef typename kokkos_device::execution_space kokkos_exe;
+        typedef typename crs_matrix_type::local_matrix_type kokkos_csr_matrix;
 
-	static_assert( std::is_same< kokkos_exe, 
-		       Kokkos::OpenMP>::value, 
-		       "Kokkos node type not supported by experimental thread basker RILUK");
+        static_assert( std::is_same< kokkos_exe,
+                       Kokkos::OpenMP>::value,
+                       "Kokkos node type not supported by experimental thread basker RILUK");
 
 
-	myBasker = rcp( new BaskerNS::Basker<local_ordinal_type, scalar_type, Kokkos::OpenMP>);
-	myBasker->Options.no_pivot     = true;
-	myBasker->Options.transpose    = true; //CRS not CCS
-	myBasker->Options.symmetric    = false;
-	myBasker->Options.realloc      = true;
-	myBasker->Options.btf          = false;
-	myBasker->Options.incomplete   = true;
-	myBasker->Options.inc_lvl      = LevelOfFill_;
-	myBasker->Options.user_fill    = basker_user_fill;
-	myBasker->Options.same_pattern = false;
-	myBasker->SetThreads(basker_threads);
-	basker_reuse = false;
-	
-	kokkos_csr_matrix kcsr = A_local_crs_->getLocalMatrix();
+        myBasker = rcp( new BaskerNS::Basker<local_ordinal_type, scalar_type, Kokkos::OpenMP>);
+        myBasker->Options.no_pivot     = true;
+        myBasker->Options.transpose    = true; //CRS not CCS
+        myBasker->Options.symmetric    = false;
+        myBasker->Options.realloc      = true;
+        myBasker->Options.btf          = false;
+        myBasker->Options.incomplete   = true;
+        myBasker->Options.inc_lvl      = LevelOfFill_;
+        myBasker->Options.user_fill    = basker_user_fill;
+        myBasker->Options.same_pattern = false;
+        myBasker->SetThreads(basker_threads);
+        basker_reuse = false;
 
-	local_ordinal_type* r_ptr;
-	r_ptr = new local_ordinal_type[(local_ordinal_type)A_local_crs_->getNodeNumRows()+1];
+        kokkos_csr_matrix kcsr = A_local_crs_->getLocalMatrix();
 
-	//Still need to convert
-	//Lost on why Trilinos uses such odd types for row_ptrs
-	for(local_ordinal_type bi = 0; 
-	    bi < A_local_crs_->getNodeNumRows()+1; bi++)
-	  {
-	    r_ptr[bi] = (local_ordinal_type)kcsr.graph.row_map(bi);
-	  }
+        local_ordinal_type* r_ptr;
+        r_ptr = new local_ordinal_type[(local_ordinal_type)A_local_crs_->getNodeNumRows()+1];
 
-	int basker_error = 
-	myBasker->Symbolic(
-	 ((local_ordinal_type)A_local_crs_->getNodeNumRows()),
-	 ((local_ordinal_type)A_local_crs_->getNodeNumCols()), 
-	 ((local_ordinal_type)A_local_crs_->getNodeNumEntries()),
-	 r_ptr,
-	 &(kcsr.graph.entries(0)),
-	 &(kcsr.values(0)));
+        //Still need to convert
+        //Lost on why Trilinos uses such odd types for row_ptrs
+        for(local_ordinal_type bi = 0;
+            bi < A_local_crs_->getNodeNumRows()+1; bi++)
+          {
+            r_ptr[bi] = (local_ordinal_type)kcsr.graph.row_map(bi);
+          }
 
-	TEUCHOS_TEST_FOR_EXCEPTION(
+        int basker_error =
+        myBasker->Symbolic(
+         ((local_ordinal_type)A_local_crs_->getNodeNumRows()),
+         ((local_ordinal_type)A_local_crs_->getNodeNumCols()),
+         ((local_ordinal_type)A_local_crs_->getNodeNumEntries()),
+         r_ptr,
+         &(kcsr.graph.entries(0)),
+         &(kcsr.values(0)));
+
+        TEUCHOS_TEST_FOR_EXCEPTION(
         basker_error != 0, std::logic_error, "Ifpack2::RILUK::initialize:"
-	 "Error in basker Symbolic");
+         "Error in basker Symbolic");
 
 
 #else
@@ -628,7 +647,7 @@ void RILUK<MatrixType>::initialize ()
       "Try again with -DIFPACK2_ILUK_EXPERIMENAL.");
 #endif
       }
-    
+
   } // Stop timing
 
   isInitialized_ = true;
@@ -636,25 +655,46 @@ void RILUK<MatrixType>::initialize ()
   initializeTime_ += timer.totalElapsedTime ();
 }
 
+#ifdef IFPACK2_ILUK_EXPERIMENTAL
+// Basker needs to refresh numbers using A_local_crs_, not just A_local_.
+template<class MatrixType>
+void
+RILUK<MatrixType>::
+initLocalCrs ()
+{
+  using Teuchos::RCP;
+  using Teuchos::rcp;
+  using Teuchos::rcp_dynamic_cast;
+  using Teuchos::rcp_implicit_cast;
+  using Teuchos::rcp_const_cast;
+
+  RCP<crs_matrix_type> A_local_crs_nc = rcp_const_cast<crs_matrix_type> (A_local_crs_);
+  A_local_crs_ = rcp_dynamic_cast<const crs_matrix_type> (A_local_);
+  if (A_local_crs_.is_null ()) {
+    Teuchos::Array<local_ordinal_type> lclColInds;
+    Teuchos::Array<scalar_type> vals;
+    A_local_crs_nc->resumeFill();
+    for (size_t i = 0; i < A_local_crs_nc->getNodeNumRows(); ++i) {
+      size_t numEnt;
+      const auto nc = A_local_->getNumEntriesInLocalRow(i);
+      if (static_cast<size_t>(lclColInds.size()) < nc) {
+        lclColInds.resize(nc);
+        vals.resize(nc);
+      }
+      A_local_->getLocalRowCopy(i, lclColInds(), vals(), numEnt);
+      A_local_crs_nc->replaceLocalValues(i, lclColInds.view(0, numEnt), vals.view(0, numEnt));
+    }
+    A_local_crs_nc->fillComplete();
+  }
+  A_local_crs_ = rcp_const_cast<const crs_matrix_type> (A_local_crs_nc);
+}
+#endif
 
 template<class MatrixType>
 void
 RILUK<MatrixType>::
-initAllValues (const row_matrix_type& A)
+checkOrderingConsistency (const row_matrix_type& A)
 {
-  using Teuchos::ArrayRCP;
-  using Teuchos::Comm;
-  using Teuchos::ptr;
-  using Teuchos::RCP;
-  using Teuchos::REDUCE_SUM;
-  using Teuchos::reduceAll;
-  typedef Tpetra::Map<local_ordinal_type,global_ordinal_type,node_type> map_type;
-
-  size_t NumIn = 0, NumL = 0, NumU = 0;
-  bool DiagFound = false;
-  size_t NumNonzeroDiags = 0;
-  size_t MaxNumEntries = A.getGlobalMaxNumRowEntries();
-
   // First check that the local row map ordering is the same as the local portion of the column map.
   // The extraction of the strictly lower/upper parts of A, as well as the factorization,
   // implicitly assume that this is the case.
@@ -673,7 +713,26 @@ initAllValues (const row_matrix_type& A)
                              "The ordering of the local GIDs in the row and column maps is not the same"
                              << std::endl << "at index " << indexOfInconsistentGID
                              << ".  Consistency is required, as all calculations are done with"
-                             << std::endl << "local indexing.");
+                             << std::endl << "local indexing.");  
+}
+
+template<class MatrixType>
+void
+RILUK<MatrixType>::
+initAllValues (const row_matrix_type& A)
+{
+  using Teuchos::ArrayRCP;
+  using Teuchos::Comm;
+  using Teuchos::ptr;
+  using Teuchos::RCP;
+  using Teuchos::REDUCE_SUM;
+  using Teuchos::reduceAll;
+  typedef Tpetra::Map<local_ordinal_type,global_ordinal_type,node_type> map_type;
+
+  size_t NumIn = 0, NumL = 0, NumU = 0;
+  bool DiagFound = false;
+  size_t NumNonzeroDiags = 0;
+  size_t MaxNumEntries = A.getGlobalMaxNumRowEntries();
 
   // Allocate temporary space for extracting the strictly
   // lower and upper parts of the matrix A.
@@ -779,14 +838,6 @@ initAllValues (const row_matrix_type& A)
     }
   }
 
-  // The domain of L and the range of U are exactly their own row maps
-  // (there is no communication).  The domain of U and the range of L
-  // must be the same as those of the original matrix, However if the
-  // original matrix is a VbrMatrix, these two latter maps are
-  // translation from a block map to a point map.
-  L_->fillComplete (L_->getColMap (), A_local_crs_->getRangeMap ());
-  U_->fillComplete (A_local_crs_->getDomainMap (), U_->getRowMap ());
-
   // At this point L and U have the values of A in the structure of L
   // and U, and diagonal vector D
 
@@ -797,6 +848,7 @@ initAllValues (const row_matrix_type& A)
 template<class MatrixType>
 void RILUK<MatrixType>::compute ()
 {
+  using Teuchos::rcp;
   const char prefix[] = "Ifpack2::RILUK::compute: ";
 
   // initialize() checks this too, but it's easier for users if the
@@ -817,12 +869,15 @@ void RILUK<MatrixType>::compute ()
   }
 
   Teuchos::Time timer ("RILUK::compute");
-  { //Start timing
-    if(! isExperimental_) {
+  if ( ! isExperimental_) {
+    // Start timing
+    Teuchos::TimeMonitor timeMon (timer);
+
     isComputed_ = false;
 
-    L_->resumeFill ();
-    U_->resumeFill ();
+    // Fill L and U with numbers. This supports nonzero pattern reuse by calling
+    // initialize() once and then compute() multiple times.
+    initAllValues (*A_local_);
 
     // MinMachNum should be officially defined, for now pick something a little
     // bigger than IEEE underflow value
@@ -948,121 +1003,128 @@ void RILUK<MatrixType>::compute ()
       }
     }
 
+    // The domain of L and the range of U are exactly their own row maps
+    // (there is no communication).  The domain of U and the range of L
+    // must be the same as those of the original matrix, However if the
+    // original matrix is a VbrMatrix, these two latter maps are
+    // translation from a block map to a point map.
     // FIXME (mfh 23 Dec 2013) Do we know that the column Map of L_ is
     // always one-to-one?
-    L_->fillComplete (L_->getColMap (), A_local_crs_->getRangeMap ());
-    U_->fillComplete (A_local_crs_->getDomainMap (), U_->getRowMap ());
+    L_->fillComplete (L_->getColMap (), A_local_->getRangeMap ());
+    U_->fillComplete (A_local_->getDomainMap (), U_->getRowMap ());
 
     // Validate that the L and U factors are actually lower and upper triangular
+
+    //18-Aug-2016 The following two Teuchos tests-for-exceptions were changed by Massimiliano Lupo Pasini
     TEUCHOS_TEST_FOR_EXCEPTION(
+      0 < L_->getNodeNumRows() &&
       ! L_->isLowerTriangular (), std::runtime_error,
       "Ifpack2::RILUK::compute: L isn't lower triangular.");
     TEUCHOS_TEST_FOR_EXCEPTION(
+      0 < U_->getNodeNumRows() &&
       ! U_->isUpperTriangular (), std::runtime_error,
       "Ifpack2::RILUK::compute: U isn't lower triangular.");
-  } 
-  else{
+
+    L_solver_ = Teuchos::rcp (new LocalSparseTriangularSolver<row_matrix_type> (L_));
+    L_solver_->initialize ();
+    L_solver_->compute ();
+
+    U_solver_ = Teuchos::rcp (new LocalSparseTriangularSolver<row_matrix_type> (U_));
+    U_solver_->initialize ();
+    U_solver_->compute ();
+  }
+  else {
 #ifdef IFPACK2_ILUK_EXPERIMENTAL
     Teuchos::Time timerbasker ("RILUK::basercompute");
     {
+      // Start timing
+      Teuchos::TimeMonitor timeMon (timer);
       //
       if(basker_reuse == false)
-	{
-	  //We don't have to reimport Matrix because same
-	  {
-	    //Teuchos::TimeMonitor timeMon(timerbasker);
-	    myBasker->Factor_Inc(0);
-	    basker_reuse = true;
-	  }
-	}
+        {
+          //We don't have to reimport Matrix because same
+          {
+            //Teuchos::TimeMonitor timeMon(timerbasker);
+            myBasker->Factor_Inc(0);
+            basker_reuse = true;
+          }
+        }
       else
-	{
-	  //Do we need to import Matrix with file again?
-	  myBasker->Options.same_pattern = true;
-	  
-	  typedef typename crs_matrix_type::local_matrix_type kokkos_csr_matrix;
-	  kokkos_csr_matrix kcsr = A_local_crs_->getLocalMatrix();
+        {
+          //Do we need to import Matrix with file again?
+          myBasker->Options.same_pattern = true;
 
-	  
-	 local_ordinal_type* r_ptr;
-	r_ptr = new local_ordinal_type[(local_ordinal_type)A_local_crs_->getNodeNumRows()+1];
+          typedef typename crs_matrix_type::local_matrix_type kokkos_csr_matrix;
+          kokkos_csr_matrix kcsr = A_local_crs_->getLocalMatrix();
 
-	//Still need to convert
-	//Lost on why Trilinos uses such odd types for row_ptrs
-	for(local_ordinal_type bi = 0; bi < A_local_crs_->getNodeNumRows()+1; bi++)
-	  {
-	    r_ptr[bi] = (local_ordinal_type)kcsr.graph.row_map(bi);
-	  }
 
-	int basker_error = 
-	myBasker->Factor(
-	 ((local_ordinal_type)A_local_crs_->getNodeNumRows()),
-	 ((local_ordinal_type)A_local_crs_->getNodeNumCols()), 
-	 ((local_ordinal_type)A_local_crs_->getNodeNumEntries()),
-	 r_ptr,
-	 &(kcsr.graph.entries(0)),
-	 &(kcsr.values(0)));
-	
-	//myBasker->DEBUG_PRINT();
+         local_ordinal_type* r_ptr;
+        r_ptr = new local_ordinal_type[(local_ordinal_type)A_local_crs_->getNodeNumRows()+1];
 
-	TEUCHOS_TEST_FOR_EXCEPTION(
+        //Still need to convert
+        for(local_ordinal_type bi = 0; bi < A_local_crs_->getNodeNumRows()+1; bi++)
+          {
+            r_ptr[bi] = (local_ordinal_type)kcsr.graph.row_map(bi);
+          }
+
+        int basker_error =
+        myBasker->Factor(
+         ((local_ordinal_type)A_local_crs_->getNodeNumRows()),
+         ((local_ordinal_type)A_local_crs_->getNodeNumCols()),
+         ((local_ordinal_type)A_local_crs_->getNodeNumEntries()),
+         r_ptr,
+         &(kcsr.graph.entries(0)),
+         &(kcsr.values(0)));
+
+        //myBasker->DEBUG_PRINT();
+
+        TEUCHOS_TEST_FOR_EXCEPTION(
         basker_error != 0, std::logic_error, "Ifpack2::RILUK::initialize:"
-	 "Error in basker compute");
-	 
-	
+         "Error in basker compute");
+
+
         }
     }
 # ifdef IFPACK2_HTS_EXPERIMENTAL
     //TODO Reuse symbolic information.
     if (use_hts_) {
-
       Teuchos::Time basker_getL("basker_getL");
       Teuchos::Time hts_buildL ("hts_buildL");
       Teuchos::Time basker_getU("basker_getU");
       Teuchos::Time hts_buildU ("hts_buildU");
-      
 
       hts_mgr_ = Teuchos::rcp(new HTSManager());
       local_ordinal_type* p, * q;
       local_ordinal_type nnz;
 
-     
-      
       myBasker->GetPerm(&p, &q);
-      
       {
-	
         HTSData d;
         myBasker->GetL(d.n, nnz, &d.ir, &d.jc, &d.v);
         d.sort();
         typename HTST::CrsMatrix* T = HTST::make_CrsMatrix(d.n, d.ir, d.jc, d.v, true);
         hts_mgr_->Limpl = HTST::preprocess(T, 1, hts_nthreads_, true, p, 0);
+        HTST::delete_CrsMatrix(T);
         delete[] p;
-
       }
-
       {
         HTSData d;
         myBasker->GetU(d.n, nnz, &d.ir, &d.jc, &d.v);
         d.sort();
-       
         typename HTST::CrsMatrix* T = HTST::make_CrsMatrix(d.n, d.ir, d.jc, d.v, true);
         hts_mgr_->Uimpl = HTST::preprocess(T, 1, hts_nthreads_, true, 0, q);
-
-
+        HTST::delete_CrsMatrix(T);
+        delete[] q;
       }
-
-
     }
 # endif
-  
+
 #else
     TEUCHOS_TEST_FOR_EXCEPTION(
-       0==1, std::runtime_error,
+       false, std::runtime_error,
        "Ifpack2::RILUK::compute: experimental not enabled");
 #endif
   }//end -- if experimental
-  }//end timing
 
   isComputed_ = true;
   ++numCompute_;
@@ -1127,31 +1189,31 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
     if (alpha == one && beta == zero) {
       if (mode == Teuchos::NO_TRANS) { // Solve L (D (U Y)) = X for Y.
         // Start by solving L C = X for C.  C must have the same Map
-        // as D.  We have to use a temp multivector, since
-        // localSolve() does not allow its input and output to alias
-        // one another.
+        // as D.  We have to use a temp multivector, since our
+        // implementation of triangular solves does not allow its
+        // input and output to alias one another.
         //
         // FIXME (mfh 24 Jan 2014) Cache this temp multivector.
         MV C (D_->getMap (), X.getNumVectors ());
-        L_->localSolve (X, C, mode);
+        L_solver_->apply (X, C, mode);
 
         // Solve D Y_tmp = C.  Y_tmp must have the same Map as C, and
         // the operation lets us do this in place in C, so we can
         // write "solve D C = C for C."
         C.elementWiseMultiply (one, *D_, C, zero);
 
-        U_->localSolve (C, Y, mode); // Solve U Y = C.
+        U_solver_->apply (C, Y, mode); // Solve U Y = C.
       }
       else { // Solve U^P (D^P (U^P Y)) = X for Y (where P is * or T).
 
         // Start by solving U^P C = X for C.  C must have the same Map
-        // as D.  We have to use a temp multivector, since
-        // localSolve() does not allow its input and output to alias
-        // one another.
+        // as D.  We have to use a temp multivector, since our
+        // implementation of triangular solves does not allow its
+        // input and output to alias one another.
         //
         // FIXME (mfh 24 Jan 2014) Cache this temp multivector.
         MV C (D_->getMap (), X.getNumVectors ());
-        U_->localSolve (X, C, mode);
+        U_solver_->apply (X, C, mode);
 
         // Solve D^P Y_tmp = C.  Y_tmp must have the same Map as C,
         // and the operation lets us do this in place in C, so we can
@@ -1162,7 +1224,7 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
         // D_, not just with D_ itself.
         C.elementWiseMultiply (one, *D_, C, zero);
 
-        L_->localSolve (C, Y, mode); // Solve L^P Y = C.
+        L_solver_->apply (C, Y, mode); // Solve L^P Y = C.
       }
     }
     else { // alpha != 1 or beta != 0
@@ -1178,7 +1240,7 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
         Y.update (alpha, Y_tmp, beta);
       }
     }
-  } 
+  }
   else
     {
 #ifdef IFPACK2_ILUK_EXPERIMENTAL
@@ -1186,7 +1248,7 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
       Teuchos::ArrayRCP<const scalar_type> YY;
       XX = X.get1dView();
       YY = Y.get1dView();
-      
+
 # ifdef IFPACK2_HTS_EXPERIMENTAL
       if (use_hts_) {
         HTST::solve_omp(hts_mgr_->Limpl, const_cast<scalar_type*>(XX.getRawPtr()),
@@ -1196,13 +1258,13 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
       } else
 # endif
       {
-        myBasker->Solve(((local_ordinal_type)X.getNumVectors()), 
+        myBasker->Solve(((local_ordinal_type)X.getNumVectors()),
                         (const_cast<scalar_type *>(XX.getRawPtr())),
                         (const_cast<scalar_type *>(YY.getRawPtr())));
       }
 #else
       TEUCHOS_TEST_FOR_EXCEPTION(
-      0==1, std::runtime_error, 
+      0==1, std::runtime_error,
       "Ifpack2::RILUK::apply: Experimental no enabled");
 #endif
     }//end isExperimental
