@@ -1,13 +1,13 @@
 /*
 //@HEADER
 // ************************************************************************
-// 
+//
 //                        Kokkos v. 2.0
 //              Copyright (2014) Sandia Corporation
-// 
+//
 // Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 // the U.S. Government retains certain rights in this software.
-// 
+//
 // Redistribution and use in source and binary forms, with or without
 // modification, are permitted provided that the following conditions are
 // met:
@@ -36,7 +36,7 @@
 // SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
 // Questions? Contact  H. Carter Edwards (hcedwar@sandia.gov)
-// 
+//
 // ************************************************************************
 //@HEADER
 */
@@ -50,8 +50,9 @@ namespace Kokkos {
 // Cuda native CAS supports int, unsigned int, and unsigned long long int (non-standard type).
 // Must cast-away 'volatile' for the CAS call.
 
-#if defined( KOKKOS_ATOMICS_USE_CUDA )
+#if defined( KOKKOS_HAVE_CUDA )
 
+#if defined(__CUDA_ARCH__) || defined(KOKKOS_CUDA_CLANG_WORKAROUND)
 __inline__ __device__
 int atomic_compare_exchange( volatile int * const dest, const int compare, const int val)
 { return atomicCAS((int*)dest,compare,val); }
@@ -96,31 +97,37 @@ T atomic_compare_exchange( volatile T * const dest , const T & compare ,
 {
   T return_val;
   // This is a way to (hopefully) avoid dead lock in a warp
-  int done = 1;
-  while ( done>0 ) {
-    done++;
-    if( Impl::lock_address_cuda_space( (void*) dest ) ) {
-      return_val = *dest;
-      if( return_val == compare )
-        *dest = val;
-      Impl::unlock_address_cuda_space( (void*) dest );
-      done = 0;
+  int done = 0;
+  unsigned int active = __ballot(1);
+  unsigned int done_active = 0;
+  while (active!=done_active) {
+    if(!done) {
+      if( Impl::lock_address_cuda_space( (void*) dest ) ) {
+        return_val = *dest;
+        if( return_val == compare )
+          *dest = val;
+        Impl::unlock_address_cuda_space( (void*) dest );
+        done = 1;
+      }
     }
+    done_active = __ballot(done);
   }
   return return_val;
 }
+#endif
+#endif
 
 //----------------------------------------------------------------------------
 // GCC native CAS supports int, long, unsigned int, unsigned long.
 // Intel native CAS support int and long with the same interface as GCC.
+#if !defined(__CUDA_ARCH__) || defined(KOKKOS_CUDA_CLANG_WORKAROUND)
+#if defined(KOKKOS_ATOMICS_USE_GCC) || defined(KOKKOS_ATOMICS_USE_INTEL)
 
-#elif defined(KOKKOS_ATOMICS_USE_GCC) || defined(KOKKOS_ATOMICS_USE_INTEL)
-
-KOKKOS_INLINE_FUNCTION
+inline
 int atomic_compare_exchange( volatile int * const dest, const int compare, const int val)
 { return __sync_val_compare_and_swap(dest,compare,val); }
 
-KOKKOS_INLINE_FUNCTION
+inline
 long atomic_compare_exchange( volatile long * const dest, const long compare, const long val )
 { return __sync_val_compare_and_swap(dest,compare,val); }
 
@@ -128,11 +135,11 @@ long atomic_compare_exchange( volatile long * const dest, const long compare, co
 
 // GCC supports unsigned
 
-KOKKOS_INLINE_FUNCTION
+inline
 unsigned int atomic_compare_exchange( volatile unsigned int * const dest, const unsigned int compare, const unsigned int val )
 { return __sync_val_compare_and_swap(dest,compare,val); }
 
-KOKKOS_INLINE_FUNCTION
+inline
 unsigned long atomic_compare_exchange( volatile unsigned long * const dest ,
                                        const unsigned long compare ,
                                        const unsigned long val )
@@ -141,7 +148,7 @@ unsigned long atomic_compare_exchange( volatile unsigned long * const dest ,
 #endif
 
 template < typename T >
-KOKKOS_INLINE_FUNCTION
+inline
 T atomic_compare_exchange( volatile T * const dest, const T & compare,
   typename Kokkos::Impl::enable_if< sizeof(T) == sizeof(int) , const T & >::type val )
 {
@@ -163,7 +170,7 @@ T atomic_compare_exchange( volatile T * const dest, const T & compare,
 }
 
 template < typename T >
-KOKKOS_INLINE_FUNCTION
+inline
 T atomic_compare_exchange( volatile T * const dest, const T & compare,
   typename Kokkos::Impl::enable_if< sizeof(T) != sizeof(int) &&
                                     sizeof(T) == sizeof(long) , const T & >::type val )
@@ -187,7 +194,7 @@ T atomic_compare_exchange( volatile T * const dest, const T & compare,
 
 #if defined( KOKKOS_ENABLE_ASM) && defined ( KOKKOS_USE_ISA_X86_64 )
 template < typename T >
-KOKKOS_INLINE_FUNCTION
+inline
 T atomic_compare_exchange( volatile T * const dest, const T & compare,
   typename Kokkos::Impl::enable_if< sizeof(T) != sizeof(int) &&
                                     sizeof(T) != sizeof(long) &&
@@ -218,7 +225,17 @@ T atomic_compare_exchange( volatile T * const dest , const T compare ,
   while( !Impl::lock_address_host_space( (void*) dest ) );
   T return_val = *dest;
   if( return_val == compare ) {
-    const T tmp = *dest = val;
+    // Don't use the following line of code here:
+    //
+    //const T tmp = *dest = val;
+    //
+    // Instead, put each assignment in its own statement.  This is
+    // because the overload of T::operator= for volatile *this should
+    // return void, not volatile T&.  See Kokkos #177:
+    //
+    // https://github.com/kokkos/kokkos/issues/177
+    *dest = val;
+    const T tmp = *dest;
     #ifndef KOKKOS_COMPILER_CLANG
     (void) tmp;
     #endif
@@ -239,11 +256,12 @@ T atomic_compare_exchange( volatile T * const dest, const T compare, const T val
   {
     retval = dest[0];
     if ( retval == compare )
-  	dest[0] = val;
+        dest[0] = val;
   }
   return retval;
 }
 
+#endif
 #endif
 
 template <typename T>
@@ -252,7 +270,6 @@ bool atomic_compare_exchange_strong(volatile T* const dest, const T compare, con
 {
   return compare == atomic_compare_exchange(dest, compare, val);
 }
-
 //----------------------------------------------------------------------------
 
 } // namespace Kokkos
