@@ -1,13 +1,13 @@
 /*
 //@HEADER
 // ************************************************************************
-// 
+//
 //                        Kokkos v. 2.0
 //              Copyright (2014) Sandia Corporation
-// 
+//
 // Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 // the U.S. Government retains certain rights in this software.
-// 
+//
 // Redistribution and use in source and binary forms, with or without
 // modification, are permitted provided that the following conditions are
 // met:
@@ -36,7 +36,7 @@
 // SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
 // Questions? Contact  H. Carter Edwards (hcedwar@sandia.gov)
-// 
+//
 // ************************************************************************
 //@HEADER
 */
@@ -48,7 +48,8 @@ namespace Kokkos {
 
 //----------------------------------------------------------------------------
 
-#if defined( KOKKOS_ATOMICS_USE_CUDA )
+#if defined( KOKKOS_HAVE_CUDA )
+#if defined(__CUDA_ARCH__) || defined(KOKKOS_CUDA_CLANG_WORKAROUND)
 
 __inline__ __device__
 int atomic_exchange( volatile int * const dest , const int val )
@@ -106,15 +107,19 @@ T atomic_exchange( volatile T * const dest ,
 {
   T return_val;
   // This is a way to (hopefully) avoid dead lock in a warp
-  int done = 1;
-  while ( done > 0 ) {
-    done++;
-    if( Impl::lock_address_cuda_space( (void*) dest ) ) {
-      return_val = *dest;
-      *dest = val;
-      Impl::unlock_address_cuda_space( (void*) dest );
-      done = 0;
+  int done = 0;
+  unsigned int active = __ballot(1);
+  unsigned int done_active = 0;
+  while (active!=done_active) {
+    if(!done) {
+      if( Impl::lock_address_cuda_space( (void*) dest ) ) {
+        return_val = *dest;
+        *dest = val;
+        Impl::unlock_address_cuda_space( (void*) dest );
+        done = 1;
+      }
     }
+    done_active = __ballot(done);
   }
   return return_val;
 }
@@ -152,12 +157,16 @@ void atomic_assign(
   (void) atomic_exchange(dest,val);
 }
 
+#endif
+#endif
+
 //----------------------------------------------------------------------------
 
-#elif defined(KOKKOS_ATOMICS_USE_GCC) || defined(KOKKOS_ATOMICS_USE_INTEL)
+#if !defined(__CUDA_ARCH__) || defined(KOKKOS_CUDA_CLANG_WORKAROUND)
+#if defined(KOKKOS_ATOMICS_USE_GCC) || defined(KOKKOS_ATOMICS_USE_INTEL)
 
 template< typename T >
-KOKKOS_INLINE_FUNCTION
+inline
 T atomic_exchange( volatile T * const dest ,
   typename Kokkos::Impl::enable_if< sizeof(T) == sizeof(int) || sizeof(T) == sizeof(long)
                                   , const T & >::type val )
@@ -172,7 +181,7 @@ T atomic_exchange( volatile T * const dest ,
   union U {
     T val_T ;
     type val_type ;
-    KOKKOS_INLINE_FUNCTION U() {};
+    inline U() {};
   } old ;
 #else
   union { T val_T ; type val_type ; } old ;
@@ -190,7 +199,7 @@ T atomic_exchange( volatile T * const dest ,
 
 #if defined(KOKKOS_ENABLE_ASM) && defined ( KOKKOS_USE_ISA_X86_64 )
 template< typename T >
-KOKKOS_INLINE_FUNCTION
+inline
 T atomic_exchange( volatile T * const dest ,
   typename Kokkos::Impl::enable_if< sizeof(T) == sizeof(Impl::cas128_t)
                                   , const T & >::type val )
@@ -198,7 +207,7 @@ T atomic_exchange( volatile T * const dest ,
   union U {
     Impl::cas128_t i ;
     T t ;
-    KOKKOS_INLINE_FUNCTION U() {};
+    inline U() {};
   } assume , oldval , newval ;
 
   oldval.t = *dest ;
@@ -228,7 +237,17 @@ T atomic_exchange( volatile T * const dest ,
 {
   while( !Impl::lock_address_host_space( (void*) dest ) );
   T return_val = *dest;
-  const T tmp = *dest = val;
+  // Don't use the following line of code here:
+  //
+  //const T tmp = *dest = val;
+  //
+  // Instead, put each assignment in its own statement.  This is
+  // because the overload of T::operator= for volatile *this should
+  // return void, not volatile T&.  See Kokkos #177:
+  //
+  // https://github.com/kokkos/kokkos/issues/177
+  *dest = val;
+  const T tmp = *dest;
   #ifndef KOKKOS_COMPILER_CLANG
   (void) tmp;
   #endif
@@ -237,7 +256,7 @@ T atomic_exchange( volatile T * const dest ,
 }
 
 template< typename T >
-KOKKOS_INLINE_FUNCTION
+inline
 void atomic_assign( volatile T * const dest ,
   typename Kokkos::Impl::enable_if< sizeof(T) == sizeof(int) || sizeof(T) == sizeof(long)
                                   , const T & >::type val )
@@ -252,7 +271,7 @@ void atomic_assign( volatile T * const dest ,
   union U {
     T val_T ;
     type val_type ;
-    KOKKOS_INLINE_FUNCTION U() {};
+    inline U() {};
   } old ;
 #else
   union { T val_T ; type val_type ; } old ;
@@ -268,7 +287,7 @@ void atomic_assign( volatile T * const dest ,
 
 #if defined( KOKKOS_ENABLE_ASM ) && defined ( KOKKOS_USE_ISA_X86_64 )
 template< typename T >
-KOKKOS_INLINE_FUNCTION
+inline
 void atomic_assign( volatile T * const dest ,
   typename Kokkos::Impl::enable_if< sizeof(T) == sizeof(Impl::cas128_t)
                                   , const T & >::type val )
@@ -276,7 +295,7 @@ void atomic_assign( volatile T * const dest ,
   union U {
     Impl::cas128_t i ;
     T t ;
-    KOKKOS_INLINE_FUNCTION U() {};
+    inline U() {};
   } assume , oldval , newval ;
 
   oldval.t = *dest ;
@@ -305,7 +324,9 @@ void atomic_assign( volatile T * const dest ,
   // member.  The volatile return value implicitly defines a
   // dereference that some compilers (gcc 4.7.2) warn is being ignored.
   // Suppress warning by casting return to void.
-  (void)( *dest = val );
+  //(void)( *dest = val );
+  *dest = val;
+
   Impl::unlock_address_host_space( (void*) dest );
 }
 //----------------------------------------------------------------------------
@@ -313,7 +334,7 @@ void atomic_assign( volatile T * const dest ,
 #elif defined( KOKKOS_ATOMICS_USE_OMP31 )
 
 template < typename T >
-KOKKOS_INLINE_FUNCTION
+inline
 T atomic_exchange( volatile T * const dest , const T val )
 {
   T retval;
@@ -327,7 +348,7 @@ T atomic_exchange( volatile T * const dest , const T val )
 }
 
 template < typename T >
-KOKKOS_INLINE_FUNCTION
+inline
 void atomic_assign( volatile T * const dest , const T val )
 {
 //#pragma omp atomic
@@ -338,7 +359,7 @@ void atomic_assign( volatile T * const dest , const T val )
 }
 
 #endif
-
+#endif
 } // namespace Kokkos
 
 #endif
