@@ -55,21 +55,28 @@
 //#define TESTMEMORYPOOL_PRINT
 //#define TESTMEMORYPOOL_PRINT_STATUS
 
-#ifdef KOKKOS_HAVE_CUDA
-#define STRIDE 32
-#else
 #define STRIDE 1
+#ifdef KOKKOS_HAVE_CUDA
+#define STRIDE_ALLOC 32
+#else
+#define STRIDE_ALLOC 1
 #endif
 
 namespace TestMemoryPool {
 
 struct pointer_obj {
   uint64_t *  ptr;
+
+  KOKKOS_INLINE_FUNCTION
+  pointer_obj() : ptr( 0 ) {}
 };
 
 struct pointer_obj2 {
   void *  ptr;
   size_t  size;
+
+  KOKKOS_INLINE_FUNCTION
+  pointer_obj2() : ptr( 0 ), size( 0 ) {}
 };
 
 template < typename PointerView, typename Allocator >
@@ -86,14 +93,14 @@ struct allocate_memory {
     : m_pointers( ptrs ), m_chunk_size( cs ), m_mempool( m )
   {
     // Initialize the view with the out degree of each vertex.
-    Kokkos::parallel_for( num_ptrs * STRIDE, *this );
+    Kokkos::parallel_for( num_ptrs * STRIDE_ALLOC, *this );
   }
 
   KOKKOS_INLINE_FUNCTION
   void operator()( size_type i ) const
   {
-    if ( i % STRIDE == 0 ) {
-      m_pointers[i / STRIDE].ptr =
+    if ( i % STRIDE_ALLOC == 0 ) {
+      m_pointers[i / STRIDE_ALLOC].ptr =
         static_cast< uint64_t * >( m_mempool.allocate( m_chunk_size ) );
     }
   }
@@ -231,14 +238,14 @@ struct allocate_deallocate_memory {
       m_mempool( m )
   {
     // Initialize the view with the out degree of each vertex.
-    Kokkos::parallel_for( work_size * STRIDE, *this );
+    Kokkos::parallel_for( work_size * STRIDE_ALLOC, *this );
   }
 
   KOKKOS_INLINE_FUNCTION
   void operator()( size_type i ) const
   {
-    if ( i % STRIDE == 0 ) {
-      unsigned my_work = m_work[i / STRIDE];
+    if ( i % STRIDE_ALLOC == 0 ) {
+      unsigned my_work = m_work[i / STRIDE_ALLOC];
 
       if ( ( my_work & 1 ) == 0 ) {
         // Allocation.
@@ -303,7 +310,7 @@ bool test_mempool( size_t chunk_size, size_t total_size )
   typedef Kokkos::View< pointer_obj *, device_type >       pointer_view;
   typedef Kokkos::Experimental::MemoryPool< device_type >  pool_memory_space;
 
-  uint64_t result;
+  uint64_t result = 0;
   size_t num_chunks = total_size / chunk_size;
   bool return_val = true;
 
@@ -334,10 +341,6 @@ bool test_mempool( size_t chunk_size, size_t total_size )
   timer.reset();
 #endif
 
-  // Tests:
-  //   test for correct behvior when out of memory
-  //   test for correct behvior when interleaving allocate() and deallocate()
-
   {
     allocate_memory< pointer_view, pool_memory_space >
       am( pointers, num_chunks, chunk_size, mempool );
@@ -647,8 +650,8 @@ void test_mempool2( unsigned base_chunk_size, size_t num_chunk_sizes,
 #ifdef TESTMEMORYPOOL_PRINT
   printf( "\n*** test_mempool2() ***\n" );
   printf( "       num_chunk_sizes: %12zu\n", num_chunk_sizes );
-  printf( "       base_chunk_size: %12zu\n", base_chunk_size );
-  printf( "  ceil_base_chunk_size: %12zu\n", ceil_base_chunk_size );
+  printf( "       base_chunk_size: %12u\n", base_chunk_size );
+  printf( "  ceil_base_chunk_size: %12u\n", ceil_base_chunk_size );
   printf( "           phase1_size: %12zu\n", phase1_size );
   printf( "           phase2_size: %12zu\n", phase2_size );
   printf( "           phase3_size: %12zu\n", phase3_size );
@@ -727,18 +730,91 @@ void test_mempool2( unsigned base_chunk_size, size_t num_chunk_sizes,
 #endif
 }
 
-}
+// Tests for correct behavior when the allocator is out of memory.
+template < class Device >
+void test_memory_exhaustion()
+{
+#ifdef TESTMEMORYPOOL_PRINT
+  typedef typename Device::execution_space                 execution_space;
+#endif
+  typedef typename Device::memory_space                    memory_space;
+  typedef Device                                           device_type;
+  typedef Kokkos::View< pointer_obj *, device_type >       pointer_view;
+  typedef Kokkos::Experimental::MemoryPool< device_type >  pool_memory_space;
+
+  // The allocator will have a single superblock, and allocations will all be
+  // of the same chunk size.  The allocation loop will attempt to allocate
+  // twice the number of chunks as are available in the allocator.  The
+  // deallocation loop will only free the successfully allocated chunks.
+
+  size_t chunk_size = 128;
+  size_t num_chunks = 128;
+  size_t half_num_chunks = num_chunks / 2;
+  size_t superblock_size = chunk_size * half_num_chunks;
+  size_t lg_superblock_size =
+    Kokkos::Impl::integral_power_of_two( superblock_size );
+
+  pointer_view pointers( "pointers", num_chunks );
 
 #ifdef TESTMEMORYPOOL_PRINT
-#undef TESTMEMORYPOOL_PRINT
+  std::cout << "\n*** test_memory_exhaustion() ***" << std::endl;
+
+  double elapsed_time = 0;
+  Kokkos::Timer timer;
 #endif
 
+  pool_memory_space mempool( memory_space(), superblock_size,
+                             lg_superblock_size );
+
+#ifdef TESTMEMORYPOOL_PRINT
+  execution_space::fence();
+  elapsed_time = timer.seconds();
+  print_results( "initialize mempool: ", elapsed_time );
 #ifdef TESTMEMORYPOOL_PRINT_STATUS
+  mempool.print_status();
+#endif
+  timer.reset();
+#endif
+
+  {
+    allocate_memory< pointer_view, pool_memory_space >
+      am( pointers, num_chunks, chunk_size, mempool );
+  }
+
+#ifdef TESTMEMORYPOOL_PRINT
+  execution_space::fence();
+  elapsed_time = timer.seconds();
+  print_results( "allocate chunks: ", elapsed_time );
+#ifdef TESTMEMORYPOOL_PRINT_STATUS
+  mempool.print_status();
+#endif
+  timer.reset();
+#endif
+
+  {
+    // In parallel, the allocations that succeeded were not put contiguously
+    // into the pointers View.  The whole View can still be looped over and
+    // have deallocate called because deallocate will just do nothing for NULL
+    // pointers.
+    deallocate_memory< pointer_view, pool_memory_space >
+      dm( pointers, num_chunks, chunk_size, mempool );
+  }
+
+#ifdef TESTMEMORYPOOL_PRINT
+  execution_space::fence();
+  elapsed_time = timer.seconds();
+  print_results( "deallocate chunks: ", elapsed_time );
+#ifdef TESTMEMORYPOOL_PRINT_STATUS
+  mempool.print_status();
+#endif
+#endif
+}
+
+}
+
+#undef TESTMEMORYPOOL_PRINT
 #undef TESTMEMORYPOOL_PRINT_STATUS
-#endif
-
-#ifdef STRIDE
 #undef STRIDE
-#endif
+#undef STRIDE_ALLOC
 
 #endif
