@@ -58,6 +58,10 @@ public:
   }
 private:
   void test_compile_time_parameters() {
+    {
+      Kokkos::Impl::expand_variadic();
+      Kokkos::Impl::expand_variadic(1,2,3);
+    }
     {
       typedef Kokkos::RangePolicy<> policy_t;
       typedef typename policy_t::execution_space execution_space;
@@ -421,68 +425,68 @@ private:
     ASSERT_EQ  (p1.league_size() , league_size);
     ASSERT_EQ  (p1.team_size()   , team_size);
     ASSERT_TRUE(p1.chunk_size()  > 0);
-    ASSERT_EQ  (p1.scratch_size(), 0);
+    ASSERT_EQ  (p1.scratch_size(0), 0);
 
     policy_t p2 = p1.set_chunk_size(chunk_size);
     ASSERT_EQ  (p1.league_size() , league_size);
     ASSERT_EQ  (p1.team_size()   , team_size);
     ASSERT_TRUE(p1.chunk_size()  > 0);
-    ASSERT_EQ  (p1.scratch_size(), 0);
+    ASSERT_EQ  (p1.scratch_size(0), 0);
 
     ASSERT_EQ  (p2.league_size() , league_size);
     ASSERT_EQ  (p2.team_size()   , team_size);
     ASSERT_EQ  (p2.chunk_size()  , chunk_size);
-    ASSERT_EQ  (p2.scratch_size(), 0);
+    ASSERT_EQ  (p2.scratch_size(0), 0);
 
     policy_t p3 = p2.set_scratch_size(0,Kokkos::PerTeam(per_team_scratch));
     ASSERT_EQ  (p2.league_size() , league_size);
     ASSERT_EQ  (p2.team_size()   , team_size);
     ASSERT_EQ  (p2.chunk_size()  , chunk_size);
-    ASSERT_EQ  (p2.scratch_size(), 0);
+    ASSERT_EQ  (p2.scratch_size(0), 0);
     ASSERT_EQ  (p3.league_size() , league_size);
     ASSERT_EQ  (p3.team_size()   , team_size);
     ASSERT_EQ  (p3.chunk_size()  , chunk_size);
-    ASSERT_EQ  (p3.scratch_size(), per_team_scratch);
+    ASSERT_EQ  (p3.scratch_size(0), per_team_scratch);
 
     policy_t p4 = p2.set_scratch_size(0,Kokkos::PerThread(per_thread_scratch));
     ASSERT_EQ  (p2.league_size() , league_size);
     ASSERT_EQ  (p2.team_size()   , team_size);
     ASSERT_EQ  (p2.chunk_size()  , chunk_size);
-    ASSERT_EQ  (p2.scratch_size(), 0);
+    ASSERT_EQ  (p2.scratch_size(0), 0);
     ASSERT_EQ  (p4.league_size() , league_size);
     ASSERT_EQ  (p4.team_size()   , team_size);
     ASSERT_EQ  (p4.chunk_size()  , chunk_size);
-    ASSERT_EQ  (p4.scratch_size(), per_thread_scratch*team_size);
+    ASSERT_EQ  (p4.scratch_size(0), per_thread_scratch*team_size);
 
     policy_t p5 = p2.set_scratch_size(0,Kokkos::PerThread(per_thread_scratch),Kokkos::PerTeam(per_team_scratch));
     ASSERT_EQ  (p2.league_size() , league_size);
     ASSERT_EQ  (p2.team_size()   , team_size);
     ASSERT_EQ  (p2.chunk_size()  , chunk_size);
-    ASSERT_EQ  (p2.scratch_size(), 0);
+    ASSERT_EQ  (p2.scratch_size(0), 0);
     ASSERT_EQ  (p5.league_size() , league_size);
     ASSERT_EQ  (p5.team_size()   , team_size);
     ASSERT_EQ  (p5.chunk_size()  , chunk_size);
-    ASSERT_EQ  (p5.scratch_size(), scratch_size);
+    ASSERT_EQ  (p5.scratch_size(0), scratch_size);
 
     policy_t p6 = p2.set_scratch_size(0,Kokkos::PerTeam(per_team_scratch),Kokkos::PerThread(per_thread_scratch));
     ASSERT_EQ  (p2.league_size() , league_size);
     ASSERT_EQ  (p2.team_size()   , team_size);
     ASSERT_EQ  (p2.chunk_size()  , chunk_size);
-    ASSERT_EQ  (p2.scratch_size(), 0);
+    ASSERT_EQ  (p2.scratch_size(0), 0);
     ASSERT_EQ  (p6.league_size() , league_size);
     ASSERT_EQ  (p6.team_size()   , team_size);
     ASSERT_EQ  (p6.chunk_size()  , chunk_size);
-    ASSERT_EQ  (p6.scratch_size(), scratch_size);
+    ASSERT_EQ  (p6.scratch_size(0), scratch_size);
 
     policy_t p7 = p3.set_scratch_size(0,Kokkos::PerTeam(per_team_scratch),Kokkos::PerThread(per_thread_scratch));
     ASSERT_EQ  (p3.league_size() , league_size);
     ASSERT_EQ  (p3.team_size()   , team_size);
     ASSERT_EQ  (p3.chunk_size()  , chunk_size);
-    ASSERT_EQ  (p3.scratch_size(), per_team_scratch);
+    ASSERT_EQ  (p3.scratch_size(0), per_team_scratch);
     ASSERT_EQ  (p7.league_size() , league_size);
     ASSERT_EQ  (p7.team_size()   , team_size);
     ASSERT_EQ  (p7.chunk_size()  , chunk_size);
-    ASSERT_EQ  (p7.scratch_size(), scratch_size);
+    ASSERT_EQ  (p7.scratch_size(0), scratch_size);
 }
   void test_run_time_parameters() {
     test_run_time_parameters_type<Kokkos::TeamPolicy<ExecutionSpace> >();
