@@ -1,13 +1,13 @@
 /*
 //@HEADER
 // ************************************************************************
-// 
+//
 //                        Kokkos v. 2.0
 //              Copyright (2014) Sandia Corporation
-// 
+//
 // Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 // the U.S. Government retains certain rights in this software.
-// 
+//
 // Redistribution and use in source and binary forms, with or without
 // modification, are permitted provided that the following conditions are
 // met:
@@ -36,7 +36,7 @@
 // SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 //
 // Questions? Contact  H. Carter Edwards (hcedwar@sandia.gov)
-// 
+//
 // ************************************************************************
 //@HEADER
 */
@@ -46,11 +46,8 @@
 #include <Kokkos_Core.hpp>
 #include <Kokkos_Qthread.hpp>
 
-#include <Qthread/Kokkos_Qthread_TaskPolicy.hpp>
-
 //----------------------------------------------------------------------------
 
-#include <TestViewImpl.hpp>
 #include <TestAtomic.hpp>
 
 #include <TestViewAPI.hpp>
@@ -249,6 +246,10 @@ TEST_F( qthread, team_shared ) {
   TestSharedTeam< Kokkos::Qthread , Kokkos::Schedule<Kokkos::Static> >();
 }
 
+TEST_F( qthread, shmem_size) {
+  TestShmemSize< Kokkos::Qthread >();
+}
+
 TEST_F( qthread , team_scan )
 {
   TestScanTeam< Kokkos::Qthread , Kokkos::Schedule<Kokkos::Static> >( 10 );
