@@ -185,7 +185,7 @@ struct TestRange {
       },error);
       ASSERT_EQ(error,0);
 
-      if(ExecSpace::concurrency()>1) {
+      if( ( ExecSpace::concurrency()>(int)1) && (N>static_cast<size_t>(4*ExecSpace::concurrency())) ) {
         size_t min = N;
         size_t max = 0;
         for(int t=0; t<ExecSpace::concurrency(); t++) {
@@ -196,6 +196,7 @@ struct TestRange {
         //if(ExecSpace::concurrency()>2)
         //  ASSERT_TRUE(2*min<max);
       }
+      
     }
 
     {
@@ -218,7 +219,7 @@ struct TestRange {
       },error);
       ASSERT_EQ(error,0);
 
-      if(ExecSpace::concurrency()>1) {
+      if( ( ExecSpace::concurrency()>(int)1) && (N>static_cast<size_t>(4*ExecSpace::concurrency())) ) {
         size_t min = N;
         size_t max = 0;
         for(int t=0; t<ExecSpace::concurrency(); t++) {
