@@ -53,9 +53,11 @@
 
 namespace Test {
 
-template< class ExecSpace >
+template< class Space >
 void test_view_mapping()
 {
+  typedef typename Space::execution_space ExecSpace ;
+
   typedef Kokkos::Experimental::Impl::ViewDimension<>  dim_0 ;
   typedef Kokkos::Experimental::Impl::ViewDimension<2> dim_s2 ;
   typedef Kokkos::Experimental::Impl::ViewDimension<2,3> dim_s2_s3 ;
@@ -97,47 +99,67 @@ void test_view_mapping()
   ASSERT_LE( sizeof(dim_s0_s0_s0_s0_s0_s0_s0) , 8 * sizeof(unsigned) );
   ASSERT_EQ( sizeof(dim_s0_s0_s0_s0_s0_s0_s0_s0) , 8 * sizeof(unsigned) );
 
-  ASSERT_EQ( int(dim_0::rank) , int(0) );
-  ASSERT_EQ( int(dim_0::rank_dynamic) , int(0) );
+  static_assert( int(dim_0::rank) == int(0) , "" );
+  static_assert( int(dim_0::rank_dynamic) == int(0) , "" );
+  static_assert( int(dim_0::ArgN0) == 1 , "" );
+  static_assert( int(dim_0::ArgN1) == 1 , "" );
+  static_assert( int(dim_0::ArgN2) == 1 , "" );
 
-  ASSERT_EQ( int(dim_s2::rank) , int(1) );
-  ASSERT_EQ( int(dim_s2::rank_dynamic) , int(0) );
+  static_assert( int(dim_s2::rank) == int(1) , "" );
+  static_assert( int(dim_s2::rank_dynamic) == int(0) , "" );
+  static_assert( int(dim_s2::ArgN0) == 2 , "" );
+  static_assert( int(dim_s2::ArgN1) == 1 , "" );
 
-  ASSERT_EQ( int(dim_s2_s3::rank) , int(2) );
-  ASSERT_EQ( int(dim_s2_s3::rank_dynamic) , int(0) );
+  static_assert( int(dim_s2_s3::rank) == int(2) , "" );
+  static_assert( int(dim_s2_s3::rank_dynamic) == int(0) , "" );
+  static_assert( int(dim_s2_s3::ArgN0) == 2 , "" );
+  static_assert( int(dim_s2_s3::ArgN1) == 3 , "" );
+  static_assert( int(dim_s2_s3::ArgN2) == 1 , "" );
 
-  ASSERT_EQ( int(dim_s2_s3_s4::rank) , int(3) );
-  ASSERT_EQ( int(dim_s2_s3_s4::rank_dynamic) , int(0) );
+  static_assert( int(dim_s2_s3_s4::rank) == int(3) , "" );
+  static_assert( int(dim_s2_s3_s4::rank_dynamic) == int(0) , "" );
+  static_assert( int(dim_s2_s3_s4::ArgN0) == 2 , "" );
+  static_assert( int(dim_s2_s3_s4::ArgN1) == 3 , "" );
+  static_assert( int(dim_s2_s3_s4::ArgN2) == 4 , "" );
+  static_assert( int(dim_s2_s3_s4::ArgN3) == 1 , "" );
 
-  ASSERT_EQ( int(dim_s0::rank) , int(1) );
-  ASSERT_EQ( int(dim_s0::rank_dynamic) , int(1) );
+  static_assert( int(dim_s0::rank) == int(1) , "" );
+  static_assert( int(dim_s0::rank_dynamic) == int(1) , "" );
 
-  ASSERT_EQ( int(dim_s0_s3::rank) , int(2) );
-  ASSERT_EQ( int(dim_s0_s3::rank_dynamic) , int(1) );
+  static_assert( int(dim_s0_s3::rank) == int(2) , "" );
+  static_assert( int(dim_s0_s3::rank_dynamic) == int(1) , "" );
+  static_assert( int(dim_s0_s3::ArgN0) == 0 , "" );
+  static_assert( int(dim_s0_s3::ArgN1) == 3 , "" );
 
-  ASSERT_EQ( int(dim_s0_s3_s4::rank) , int(3) );
-  ASSERT_EQ( int(dim_s0_s3_s4::rank_dynamic) , int(1) );
+  static_assert( int(dim_s0_s3_s4::rank) == int(3) , "" );
+  static_assert( int(dim_s0_s3_s4::rank_dynamic) == int(1) , "" );
+  static_assert( int(dim_s0_s3_s4::ArgN0) == 0 , "" );
+  static_assert( int(dim_s0_s3_s4::ArgN1) == 3 , "" );
+  static_assert( int(dim_s0_s3_s4::ArgN2) == 4 , "" );
 
-  ASSERT_EQ( int(dim_s0_s0_s4::rank) , int(3) );
-  ASSERT_EQ( int(dim_s0_s0_s4::rank_dynamic) , int(2) );
+  static_assert( int(dim_s0_s0_s4::rank) == int(3) , "" );
+  static_assert( int(dim_s0_s0_s4::rank_dynamic) == int(2) , "" );
+  static_assert( int(dim_s0_s0_s4::ArgN0) == 0 , "" );
+  static_assert( int(dim_s0_s0_s4::ArgN1) == 0 , "" );
+  static_assert( int(dim_s0_s0_s4::ArgN2) == 4 , "" );
 
-  ASSERT_EQ( int(dim_s0_s0_s0::rank) , int(3) );
-  ASSERT_EQ( int(dim_s0_s0_s0::rank_dynamic) , int(3) );
+  static_assert( int(dim_s0_s0_s0::rank) == int(3) , "" );
+  static_assert( int(dim_s0_s0_s0::rank_dynamic) == int(3) , "" );
 
-  ASSERT_EQ( int(dim_s0_s0_s0_s0::rank) , int(4) );
-  ASSERT_EQ( int(dim_s0_s0_s0_s0::rank_dynamic) , int(4) );
+  static_assert( int(dim_s0_s0_s0_s0::rank) == int(4) , "" );
+  static_assert( int(dim_s0_s0_s0_s0::rank_dynamic) == int(4) , "" );
 
-  ASSERT_EQ( int(dim_s0_s0_s0_s0_s0::rank) , int(5) );
-  ASSERT_EQ( int(dim_s0_s0_s0_s0_s0::rank_dynamic) , int(5) );
+  static_assert( int(dim_s0_s0_s0_s0_s0::rank) == int(5) , "" );
+  static_assert( int(dim_s0_s0_s0_s0_s0::rank_dynamic) == int(5) , "" );
 
-  ASSERT_EQ( int(dim_s0_s0_s0_s0_s0_s0::rank) , int(6) );
-  ASSERT_EQ( int(dim_s0_s0_s0_s0_s0_s0::rank_dynamic) , int(6) );
+  static_assert( int(dim_s0_s0_s0_s0_s0_s0::rank) == int(6) , "" );
+  static_assert( int(dim_s0_s0_s0_s0_s0_s0::rank_dynamic) == int(6) , "" );
 
-  ASSERT_EQ( int(dim_s0_s0_s0_s0_s0_s0_s0::rank) , int(7) );
-  ASSERT_EQ( int(dim_s0_s0_s0_s0_s0_s0_s0::rank_dynamic) , int(7) );
+  static_assert( int(dim_s0_s0_s0_s0_s0_s0_s0::rank) == int(7) , "" );
+  static_assert( int(dim_s0_s0_s0_s0_s0_s0_s0::rank_dynamic) == int(7) , "" );
 
-  ASSERT_EQ( int(dim_s0_s0_s0_s0_s0_s0_s0_s0::rank) , int(8) );
-  ASSERT_EQ( int(dim_s0_s0_s0_s0_s0_s0_s0_s0::rank_dynamic) , int(8) );
+  static_assert( int(dim_s0_s0_s0_s0_s0_s0_s0_s0::rank) == int(8) , "" );
+  static_assert( int(dim_s0_s0_s0_s0_s0_s0_s0_s0::rank_dynamic) == int(8) , "" );
 
   dim_s0          d1( 2, 3, 4, 5, 6, 7, 8, 9 ); 
   dim_s0_s0       d2( 2, 3, 4, 5, 6, 7, 8, 9 );
@@ -204,7 +226,8 @@ void test_view_mapping()
   {
     typedef Kokkos::Experimental::Impl::ViewOffset< dim_s0_s0_s4 , Kokkos::LayoutLeft > left_s0_s0_s4 ;
 
-    left_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>(), 2, 3, 0, 0, 0, 0, 0, 0 );
+    left_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>()
+                          , Kokkos::LayoutLeft( 2, 3, 0, 0, 0, 0, 0, 0 ) );
 
     stride_s0_s0_s0  stride3( dyn_off3 );
 
@@ -215,6 +238,17 @@ void test_view_mapping()
     ASSERT_EQ( dyn_off3.m_dim.N3 , 1 );
     ASSERT_EQ( dyn_off3.size() , 2 * 3 * 4 );
 
+    const Kokkos::LayoutLeft layout = dyn_off3.layout();
+
+    ASSERT_EQ( layout.dimension[0] , 2 );
+    ASSERT_EQ( layout.dimension[1] , 3 );
+    ASSERT_EQ( layout.dimension[2] , 4 );
+    ASSERT_EQ( layout.dimension[3] , 1 );
+    ASSERT_EQ( layout.dimension[4] , 1 );
+    ASSERT_EQ( layout.dimension[5] , 1 );
+    ASSERT_EQ( layout.dimension[6] , 1 );
+    ASSERT_EQ( layout.dimension[7] , 1 );
+
     ASSERT_EQ( stride3.m_dim.rank , 3 );
     ASSERT_EQ( stride3.m_dim.N0 , 2 );
     ASSERT_EQ( stride3.m_dim.N1 , 3 );
@@ -242,7 +276,8 @@ void test_view_mapping()
 
     typedef Kokkos::Experimental::Impl::ViewOffset< dim_s0_s0_s4 , Kokkos::LayoutLeft > left_s0_s0_s4 ;
 
-    left_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>(), N0, N1, 0, 0, 0, 0, 0, 0 );
+    left_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>()
+                          , Kokkos::LayoutLeft( N0, N1, 0, 0, 0, 0, 0, 0 ) );
 
     stride_s0_s0_s0  stride3( dyn_off3 );
 
@@ -314,7 +349,8 @@ void test_view_mapping()
   {
     typedef Kokkos::Experimental::Impl::ViewOffset< dim_s0_s0_s4 , Kokkos::LayoutRight > right_s0_s0_s4 ;
 
-    right_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>(), 2, 3, 0, 0, 0, 0, 0, 0 );
+    right_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>()
+                           , Kokkos::LayoutRight( 2, 3, 0, 0, 0, 0, 0, 0 ) );
 
     stride_s0_s0_s0  stride3( dyn_off3 );
 
@@ -352,7 +388,8 @@ void test_view_mapping()
 
     typedef Kokkos::Experimental::Impl::ViewOffset< dim_s0_s0_s4 , Kokkos::LayoutRight > right_s0_s0_s4 ;
 
-    right_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>(), N0, N1, 0, 0, 0, 0, 0, 0 );
+    right_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>()
+                           , Kokkos::LayoutRight( N0, N1, 0, 0, 0, 0, 0, 0 ) );
 
     stride_s0_s0_s0  stride3( dyn_off3 );
 
@@ -428,7 +465,8 @@ void test_view_mapping()
 
     typedef Kokkos::Experimental::Impl::ViewOffset< dim_s0_s0_s4 , Kokkos::LayoutLeft > left_s0_s0_s4 ;
 
-    left_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>(), N0, N1, 0, 0, 0, 0, 0, 0 );
+    left_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>()
+                          , Kokkos::LayoutLeft( N0, N1, 0, 0, 0, 0, 0, 0 ) );
 
     Kokkos::Experimental::Impl::SubviewExtents< 3 , 3 >
       sub( dyn_off3.m_dim
@@ -466,7 +504,8 @@ void test_view_mapping()
 
     typedef Kokkos::Experimental::Impl::ViewOffset< dim_s0_s0_s4 , Kokkos::LayoutRight > right_s0_s0_s4 ;
 
-    right_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>(), N0, N1, 0, 0, 0, 0, 0, 0 );
+    right_s0_s0_s4 dyn_off3( std::integral_constant<unsigned,sizeof(int)>()
+                           , Kokkos::LayoutRight( N0, N1, 0, 0, 0, 0, 0, 0 ) );
 
     Kokkos::Experimental::Impl::SubviewExtents< 3 , 3 >
       sub( dyn_off3.m_dim
@@ -515,6 +554,13 @@ void test_view_mapping()
 
     static_assert( a_int_r1::dimension::rank == 1 , "" );
     static_assert( a_int_r1::dimension::rank_dynamic == 1 , "" );
+    static_assert( a_int_r5::dimension::ArgN0 == 0 , "" );
+    static_assert( a_int_r5::dimension::ArgN1 == 0 , "" );
+    static_assert( a_int_r5::dimension::ArgN2 == 4 , "" );
+    static_assert( a_int_r5::dimension::ArgN3 == 5 , "" );
+    static_assert( a_int_r5::dimension::ArgN4 == 6 , "" );
+    static_assert( a_int_r5::dimension::ArgN5 == 1 , "" );
+
     static_assert( std::is_same< typename a_int_r1::dimension , ViewDimension<0> >::value , "" );
     static_assert( std::is_same< typename a_int_r1::non_const_value_type , int >::value , "" );
 
@@ -526,8 +572,14 @@ void test_view_mapping()
     static_assert( a_const_int_r5::dimension::rank == 5 , "" );
     static_assert( a_const_int_r5::dimension::rank_dynamic == 2 , "" );
 
-    static_assert( std::is_same< typename a_const_int_r5::dimension , ViewDimension<0,0,4,5,6> >::value , "" );
+    static_assert( a_const_int_r5::dimension::ArgN0 == 0 , "" );
+    static_assert( a_const_int_r5::dimension::ArgN1 == 0 , "" );
+    static_assert( a_const_int_r5::dimension::ArgN2 == 4 , "" );
+    static_assert( a_const_int_r5::dimension::ArgN3 == 5 , "" );
+    static_assert( a_const_int_r5::dimension::ArgN4 == 6 , "" );
+    static_assert( a_const_int_r5::dimension::ArgN5 == 1 , "" );
 
+    static_assert( std::is_same< typename a_const_int_r5::dimension , ViewDimension<0,0,4,5,6> >::value , "" );
     static_assert( std::is_same< typename a_const_int_r5::non_const_value_type , int >::value , "" );
 
     static_assert( a_int_r5::dimension::rank == 5 , "" );
@@ -565,10 +617,10 @@ void test_view_mapping()
     static_assert( std::is_same< typename a_const_int_r1::type , const int * >::value , "" );
     static_assert( std::is_same< typename a_const_int_r1::value_type , const int >::value , "" );
 
-    static_assert( std::is_same< typename a_const_int_r1::array_scalar_type , const int * >::value , "" );
+    static_assert( std::is_same< typename a_const_int_r1::scalar_array_type , const int * >::value , "" );
     static_assert( std::is_same< typename a_const_int_r1::const_type , const int * >::value , "" );
     static_assert( std::is_same< typename a_const_int_r1::const_value_type , const int >::value , "" );
-    static_assert( std::is_same< typename a_const_int_r1::const_array_scalar_type , const int * >::value , "" );
+    static_assert( std::is_same< typename a_const_int_r1::const_scalar_array_type , const int * >::value , "" );
     static_assert( std::is_same< typename a_const_int_r1::non_const_type , int * >::value , "" );
     static_assert( std::is_same< typename a_const_int_r1::non_const_value_type , int >::value , "" );
 
@@ -580,13 +632,13 @@ void test_view_mapping()
 
     static_assert( std::is_same< typename a_const_int_r3::type , const int**[4] >::value , "" );
     static_assert( std::is_same< typename a_const_int_r3::value_type , const int >::value , "" );
-    static_assert( std::is_same< typename a_const_int_r3::array_scalar_type , const int**[4] >::value , "" );
+    static_assert( std::is_same< typename a_const_int_r3::scalar_array_type , const int**[4] >::value , "" );
     static_assert( std::is_same< typename a_const_int_r3::const_type , const int**[4] >::value , "" );
     static_assert( std::is_same< typename a_const_int_r3::const_value_type , const int >::value , "" );
-    static_assert( std::is_same< typename a_const_int_r3::const_array_scalar_type , const int**[4] >::value , "" );
+    static_assert( std::is_same< typename a_const_int_r3::const_scalar_array_type , const int**[4] >::value , "" );
     static_assert( std::is_same< typename a_const_int_r3::non_const_type , int**[4] >::value , "" );
     static_assert( std::is_same< typename a_const_int_r3::non_const_value_type , int >::value , "" );
-    static_assert( std::is_same< typename a_const_int_r3::non_const_array_scalar_type , int**[4] >::value , "" );
+    static_assert( std::is_same< typename a_const_int_r3::non_const_scalar_array_type , int**[4] >::value , "" );
 
 
     // std::cout << "typeid(const int**[4]).name() = " << typeid(const int**[4]).name() << std::endl ;
@@ -597,8 +649,8 @@ void test_view_mapping()
   {
     constexpr int N = 10 ;
 
-    typedef Kokkos::Experimental::View<int*,ExecSpace>        T ;
-    typedef Kokkos::Experimental::View<const int*,ExecSpace>  C ;
+    typedef Kokkos::View<int*,Space>        T ;
+    typedef Kokkos::View<const int*,Space>  C ;
 
     int data[N] ;
 
@@ -618,15 +670,15 @@ void test_view_mapping()
     ASSERT_TRUE( ( std::is_same< typename T::const_data_type     , const int* >::value ) );
     ASSERT_TRUE( ( std::is_same< typename T::non_const_data_type , int* >::value ) );
 
-    ASSERT_TRUE( ( std::is_same< typename T::array_scalar_type           , int* >::value ) );
-    ASSERT_TRUE( ( std::is_same< typename T::const_array_scalar_type     , const int* >::value ) );
-    ASSERT_TRUE( ( std::is_same< typename T::non_const_array_scalar_type , int* >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename T::scalar_array_type           , int* >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename T::const_scalar_array_type     , const int* >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename T::non_const_scalar_array_type , int* >::value ) );
 
     ASSERT_TRUE( ( std::is_same< typename T::value_type           , int >::value ) );
     ASSERT_TRUE( ( std::is_same< typename T::const_value_type     , const int >::value ) );
     ASSERT_TRUE( ( std::is_same< typename T::non_const_value_type , int >::value ) );
 
-    ASSERT_TRUE( ( std::is_same< typename T::memory_space , typename ExecSpace::memory_space >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename T::memory_space , typename Space::memory_space >::value ) );
     ASSERT_TRUE( ( std::is_same< typename T::reference_type , int & >::value ) );
 
     ASSERT_EQ( T::Rank , 1 );
@@ -635,22 +687,22 @@ void test_view_mapping()
     ASSERT_TRUE( ( std::is_same< typename C::const_data_type     , const int* >::value ) );
     ASSERT_TRUE( ( std::is_same< typename C::non_const_data_type , int* >::value ) );
 
-    ASSERT_TRUE( ( std::is_same< typename C::array_scalar_type           , const int* >::value ) );
-    ASSERT_TRUE( ( std::is_same< typename C::const_array_scalar_type     , const int* >::value ) );
-    ASSERT_TRUE( ( std::is_same< typename C::non_const_array_scalar_type , int* >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename C::scalar_array_type           , const int* >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename C::const_scalar_array_type     , const int* >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename C::non_const_scalar_array_type , int* >::value ) );
 
     ASSERT_TRUE( ( std::is_same< typename C::value_type           , const int >::value ) );
     ASSERT_TRUE( ( std::is_same< typename C::const_value_type     , const int >::value ) );
     ASSERT_TRUE( ( std::is_same< typename C::non_const_value_type , int >::value ) );
 
-    ASSERT_TRUE( ( std::is_same< typename C::memory_space , typename ExecSpace::memory_space >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename C::memory_space , typename Space::memory_space >::value ) );
     ASSERT_TRUE( ( std::is_same< typename C::reference_type , const int & >::value ) );
 
     ASSERT_EQ( C::Rank , 1 );
 
     ASSERT_EQ( vr1.dimension_0() , N );
 
-    if ( Kokkos::Impl::VerifyExecutionCanAccessMemorySpace< typename ExecSpace::memory_space , Kokkos::HostSpace >::value ) {
+    if ( Kokkos::Impl::VerifyExecutionCanAccessMemorySpace< typename Space::memory_space , Kokkos::HostSpace >::value ) {
       for ( int i = 0 ; i < N ; ++i ) data[i] = i + 1 ;
       for ( int i = 0 ; i < N ; ++i ) ASSERT_EQ( vr1[i] , i + 1 );
       for ( int i = 0 ; i < N ; ++i ) ASSERT_EQ( cr1[i] , i + 1 );
@@ -666,10 +718,11 @@ void test_view_mapping()
     }
   }
 
+
   {
     constexpr int N = 10 ;
-    typedef Kokkos::Experimental::View<int*,ExecSpace>        T ;
-    typedef Kokkos::Experimental::View<const int*,ExecSpace>  C ;
+    typedef Kokkos::View<int*,Space>        T ;
+    typedef Kokkos::View<const int*,Space>  C ;
 
     T vr1("vr1",N);
     C cr1(vr1);
@@ -678,21 +731,21 @@ void test_view_mapping()
     ASSERT_TRUE( ( std::is_same< typename T::const_data_type     , const int* >::value ) );
     ASSERT_TRUE( ( std::is_same< typename T::non_const_data_type , int* >::value ) );
 
-    ASSERT_TRUE( ( std::is_same< typename T::array_scalar_type           , int* >::value ) );
-    ASSERT_TRUE( ( std::is_same< typename T::const_array_scalar_type     , const int* >::value ) );
-    ASSERT_TRUE( ( std::is_same< typename T::non_const_array_scalar_type , int* >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename T::scalar_array_type           , int* >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename T::const_scalar_array_type     , const int* >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename T::non_const_scalar_array_type , int* >::value ) );
 
     ASSERT_TRUE( ( std::is_same< typename T::value_type           , int >::value ) );
     ASSERT_TRUE( ( std::is_same< typename T::const_value_type     , const int >::value ) );
     ASSERT_TRUE( ( std::is_same< typename T::non_const_value_type , int >::value ) );
 
-    ASSERT_TRUE( ( std::is_same< typename T::memory_space , typename ExecSpace::memory_space >::value ) );
+    ASSERT_TRUE( ( std::is_same< typename T::memory_space , typename Space::memory_space >::value ) );
     ASSERT_TRUE( ( std::is_same< typename T::reference_type , int & >::value ) );
     ASSERT_EQ( T::Rank , 1 );
  
     ASSERT_EQ( vr1.dimension_0() , N );
 
-    if ( Kokkos::Impl::VerifyExecutionCanAccessMemorySpace< typename ExecSpace::memory_space , Kokkos::HostSpace >::value ) {
+    if ( Kokkos::Impl::VerifyExecutionCanAccessMemorySpace< typename Space::memory_space , Kokkos::HostSpace >::value ) {
       for ( int i = 0 ; i < N ; ++i ) vr1(i) = i + 1 ;
       for ( int i = 0 ; i < N ; ++i ) ASSERT_EQ( vr1[i] , i + 1 );
       for ( int i = 0 ; i < N ; ++i ) ASSERT_EQ( cr1[i] , i + 1 );
@@ -708,6 +761,20 @@ void test_view_mapping()
     }
   }
 
+  // Testing proper handling of zero-length allocations
+  {
+    constexpr int N = 0 ;
+    typedef Kokkos::View<int*,Space>        T ;
+    typedef Kokkos::View<const int*,Space>  C ;
+
+    T vr1("vr1",N);
+    C cr1(vr1);
+
+    ASSERT_EQ( vr1.dimension_0() , 0 );
+    ASSERT_EQ( cr1.dimension_0() , 0 );
+  }
+
+
   // Testing using space instance for allocation.
   // The execution space of the memory space must be available for view data initialization
 
@@ -737,7 +804,7 @@ void test_view_mapping()
   }
 
   {
-    typedef Kokkos::Experimental::ViewTraits<int***,Kokkos::LayoutStride,ExecSpace>  traits_t ;
+    typedef Kokkos::ViewTraits<int***,Kokkos::LayoutStride,ExecSpace>  traits_t ;
     typedef Kokkos::Experimental::Impl::ViewDimension<0,0,0>                         dims_t ;
     typedef Kokkos::Experimental::Impl::ViewOffset< dims_t , Kokkos::LayoutStride >  offset_t ;
 
@@ -750,7 +817,7 @@ void test_view_mapping()
     stride.stride[1] = 1 ;
     stride.stride[2] = 12 ;
 
-    const offset_t offset( stride );
+    const offset_t offset( std::integral_constant<unsigned,0>() , stride );
 
     ASSERT_EQ( offset.dimension_0() , 3 );
     ASSERT_EQ( offset.dimension_1() , 4 );
@@ -763,12 +830,14 @@ void test_view_mapping()
     ASSERT_EQ( offset.span() , 60 );
     ASSERT_TRUE( offset.span_is_contiguous() );
 
-    Kokkos::Experimental::Impl::ViewMapping< traits_t , void >  v( (int*) 0 , std::false_type() , stride );
+    Kokkos::Experimental::Impl::ViewMapping< traits_t , void >
+      v( Kokkos::Experimental::Impl::ViewCtorProp<int*>((int*)0), stride );
   }
 
   {
-    typedef Kokkos::Experimental::View<int**,ExecSpace>  V ;
+    typedef Kokkos::View<int**,Space>  V ;
     typedef typename V::HostMirror  M ;
+    typedef typename Kokkos::View<int**,Space>::array_layout layout_type;
 
     constexpr int N0 = 10 ;
     constexpr int N1 = 11 ;
@@ -790,6 +859,14 @@ void test_view_mapping()
       ASSERT_EQ( b(i0,i1) , c(i0,i1) );
 
     Kokkos::Experimental::resize( b , 5 , 6 );
+
+    for ( int i0 = 0 ; i0 < 5 ; ++i0 )
+    for ( int i1 = 0 ; i1 < 6 ; ++i1 ) {
+      int val = 1 + i0 + i1 * N0;
+      ASSERT_EQ( b(i0,i1) , c(i0,i1) );
+      ASSERT_EQ( b(i0,i1) , val );
+    }
+
     Kokkos::Experimental::realloc( c , 5 , 6 );
     Kokkos::Experimental::realloc( d , 5 , 6 );
 
@@ -799,13 +876,89 @@ void test_view_mapping()
     ASSERT_EQ( c.dimension_1() , 6 );
     ASSERT_EQ( d.dimension_0() , 5 );
     ASSERT_EQ( d.dimension_1() , 6 );
+
+    layout_type layout(7,8);
+    Kokkos::Experimental::resize( b , layout );
+    for ( int i0 = 0 ; i0 < 7 ; ++i0 )
+    for ( int i1 = 6 ; i1 < 8 ; ++i1 )
+      b(i0,i1) = 1 + i0 + i1 * N0 ;
+
+    for ( int i0 = 5 ; i0 < 7 ; ++i0 )
+    for ( int i1 = 0 ; i1 < 8 ; ++i1 )
+      b(i0,i1) = 1 + i0 + i1 * N0 ;
+
+    for ( int i0 = 0 ; i0 < 7 ; ++i0 )
+    for ( int i1 = 0 ; i1 < 8 ; ++i1 ) {
+       int val = 1 + i0 + i1 * N0;
+       ASSERT_EQ( b(i0,i1) , val );
+    }
+
+    Kokkos::Experimental::realloc( c , layout );
+    Kokkos::Experimental::realloc( d , layout );
+
+    ASSERT_EQ( b.dimension_0() , 7 );
+    ASSERT_EQ( b.dimension_1() , 8 );
+    ASSERT_EQ( c.dimension_0() , 7 );
+    ASSERT_EQ( c.dimension_1() , 8 );
+    ASSERT_EQ( d.dimension_0() , 7 );
+    ASSERT_EQ( d.dimension_1() , 8 );
+
   }
 
-#if defined( KOKKOS_USING_EXPERIMENTAL_VIEW )
-  /* Only works when experimental view is activated */
   {
-    typedef Kokkos::Experimental::View<int*,ExecSpace> V ;
-    typedef Kokkos::Experimental::View<int*,ExecSpace,Kokkos::MemoryUnmanaged> U ;
+    typedef Kokkos::View<int**,Kokkos::LayoutStride,Space>  V ;
+    typedef typename V::HostMirror  M ;
+    typedef typename Kokkos::View<int**,Kokkos::LayoutStride,Space>::array_layout layout_type;
+
+    constexpr int N0 = 10 ;
+    constexpr int N1 = 11 ;
+
+    const int dimensions[] = {N0,N1};
+    const int order[] = {1,0};
+
+    V a("a",Kokkos::LayoutStride::order_dimensions(2,order,dimensions));
+    M b = Kokkos::Experimental::create_mirror(a);
+    M c = Kokkos::Experimental::create_mirror_view(a);
+    M d ;
+
+    for ( int i0 = 0 ; i0 < N0 ; ++i0 )
+    for ( int i1 = 0 ; i1 < N1 ; ++i1 )
+      b(i0,i1) = 1 + i0 + i1 * N0 ;
+
+    Kokkos::Experimental::deep_copy( a , b );
+    Kokkos::Experimental::deep_copy( c , a );
+
+    for ( int i0 = 0 ; i0 < N0 ; ++i0 )
+    for ( int i1 = 0 ; i1 < N1 ; ++i1 )
+      ASSERT_EQ( b(i0,i1) , c(i0,i1) );
+
+    const int dimensions2[] = {7,8};
+    const int order2[] = {1,0};
+    layout_type layout = layout_type::order_dimensions(2,order2,dimensions2);
+    Kokkos::Experimental::resize( b , layout );
+
+    for ( int i0 = 0 ; i0 < 7 ; ++i0 )
+    for ( int i1 = 0 ; i1 < 8 ; ++i1 ) {
+       int val = 1 + i0 + i1 * N0;
+       ASSERT_EQ( b(i0,i1) , c(i0,i1) );
+       ASSERT_EQ( b(i0,i1) , val );
+    }
+
+    Kokkos::Experimental::realloc( c , layout );
+    Kokkos::Experimental::realloc( d , layout );
+
+    ASSERT_EQ( b.dimension_0() , 7 );
+    ASSERT_EQ( b.dimension_1() , 8 );
+    ASSERT_EQ( c.dimension_0() , 7 );
+    ASSERT_EQ( c.dimension_1() , 8 );
+    ASSERT_EQ( d.dimension_0() , 7 );
+    ASSERT_EQ( d.dimension_1() , 8 );
+
+  }
+
+  {
+    typedef Kokkos::View<int*,Space> V ;
+    typedef Kokkos::View<int*,Space,Kokkos::MemoryUnmanaged> U ;
 
 
     V a("a",10);
@@ -840,9 +993,15 @@ void test_view_mapping()
     ASSERT_EQ( a.use_count() , 1 );
     ASSERT_EQ( b.use_count() , 0 );
 
+#if ! defined ( KOKKOS_CUDA_USE_LAMBDA )
+    /* Cannot launch host lambda when CUDA lambda is enabled */
+
+    typedef typename Kokkos::Impl::HostMirror< Space >::Space::execution_space
+      host_exec_space ;
+
     Kokkos::parallel_for(
-      Kokkos::RangePolicy< Kokkos::DefaultHostExecutionSpace >(0,10) ,
-      [=]( int i ){
+      Kokkos::RangePolicy< host_exec_space >(0,10) ,
+      KOKKOS_LAMBDA( int i ){
         // 'a' is captured by copy and the capture mechanism
         // converts 'a' to an unmanaged copy.
         // When the parallel dispatch accepts a move for the lambda
@@ -852,41 +1011,44 @@ void test_view_mapping()
         ASSERT_EQ( a.use_count() , 2 );
         ASSERT_EQ( x.use_count() , 2 );
       });
+#endif /* #if ! defined ( KOKKOS_CUDA_USE_LAMBDA ) */
   }
-#endif /* #if defined( KOKKOS_USING_EXPERIMENTAL_VIEW ) */
 }
 
-template< class ExecSpace >
-struct TestViewMappingSubview {
+template< class Space >
+struct TestViewMappingSubview
+{
+  typedef typename Space::execution_space ExecSpace ;
+  typedef typename Space::memory_space    MemSpace ;
 
   typedef Kokkos::pair<int,int> range ;
 
   enum { AN = 10 };
-  typedef Kokkos::Experimental::View<int*,ExecSpace>  AT ;
-  typedef Kokkos::Experimental::View<const int*,ExecSpace>  ACT ;
-  typedef Kokkos::Experimental::Subview< AT , range >  AS ;
+  typedef Kokkos::View<int*,ExecSpace>  AT ;
+  typedef Kokkos::View<const int*,ExecSpace>  ACT ;
+  typedef Kokkos::Subview< AT , range >  AS ;
 
   enum { BN0 = 10 , BN1 = 11 , BN2 = 12 };
-  typedef Kokkos::Experimental::View<int***,ExecSpace>  BT ;
-  typedef Kokkos::Experimental::Subview< BT , range , range , range >  BS ;
+  typedef Kokkos::View<int***,ExecSpace>  BT ;
+  typedef Kokkos::Subview< BT , range , range , range >  BS ;
 
   enum { CN0 = 10 , CN1 = 11 , CN2 = 12 };
-  typedef Kokkos::Experimental::View<int***[13][14],ExecSpace>  CT ;
-  typedef Kokkos::Experimental::Subview< CT , range , range , range , int , int >  CS ;
+  typedef Kokkos::View<int***[13][14],ExecSpace>  CT ;
+  typedef Kokkos::Subview< CT , range , range , range , int , int >  CS ;
 
   enum { DN0 = 10 , DN1 = 11 , DN2 = 12 , DN3 = 13 , DN4 = 14 };
-  typedef Kokkos::Experimental::View<int***[DN3][DN4],ExecSpace>  DT ;
-  typedef Kokkos::Experimental::Subview< DT , int , range , range , range , int >  DS ;
+  typedef Kokkos::View<int***[DN3][DN4],ExecSpace>  DT ;
+  typedef Kokkos::Subview< DT , int , range , range , range , int >  DS ;
 
 
-  typedef Kokkos::Experimental::View<int***[13][14],Kokkos::LayoutLeft,ExecSpace>  DLT ;
-  typedef Kokkos::Experimental::Subview< DLT , range , int , int , int , int >  DLS1 ;
+  typedef Kokkos::View<int***[13][14],Kokkos::LayoutLeft,ExecSpace>  DLT ;
+  typedef Kokkos::Subview< DLT , range , int , int , int , int >  DLS1 ;
 
   static_assert( DLS1::rank == 1 && std::is_same< typename DLS1::array_layout , Kokkos::LayoutLeft >::value
                , "Subview layout error for rank 1 subview of left-most range of LayoutLeft" );
 
-  typedef Kokkos::Experimental::View<int***[13][14],Kokkos::LayoutRight,ExecSpace>  DRT ;
-  typedef Kokkos::Experimental::Subview< DRT , int , int , int , int , range >  DRS1 ;
+  typedef Kokkos::View<int***[13][14],Kokkos::LayoutRight,ExecSpace>  DRT ;
+  typedef Kokkos::Subview< DRT , int , int , int , int , range >  DRS1 ;
 
   static_assert( DRS1::rank == 1 && std::is_same< typename DRS1::array_layout , Kokkos::LayoutRight >::value
                , "Subview layout error for rank 1 subview of right-most range of LayoutRight" );
@@ -1003,9 +1165,11 @@ struct TestViewMappingSubview {
 
 };
 
-template< class ExecSpace >
+template< class Space >
 void test_view_mapping_subview()
 {
+  typedef typename Space::execution_space ExecSpace ;
+
   TestViewMappingSubview< ExecSpace >::run();
 }
 
@@ -1130,36 +1294,41 @@ struct TestViewMapOperator {
 };
 
 
-template< class ExecSpace >
+template< class Space >
 void test_view_mapping_operator()
 {
-  TestViewMapOperator< Kokkos::Experimental::View<int,Kokkos::LayoutLeft,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int*,Kokkos::LayoutLeft,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int**,Kokkos::LayoutLeft,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int***,Kokkos::LayoutLeft,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int****,Kokkos::LayoutLeft,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int*****,Kokkos::LayoutLeft,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int******,Kokkos::LayoutLeft,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int*******,Kokkos::LayoutLeft,ExecSpace> >::run();
+  typedef typename Space::execution_space ExecSpace ;
 
-  TestViewMapOperator< Kokkos::Experimental::View<int,Kokkos::LayoutRight,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int*,Kokkos::LayoutRight,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int**,Kokkos::LayoutRight,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int***,Kokkos::LayoutRight,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int****,Kokkos::LayoutRight,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int*****,Kokkos::LayoutRight,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int******,Kokkos::LayoutRight,ExecSpace> >::run();
-  TestViewMapOperator< Kokkos::Experimental::View<int*******,Kokkos::LayoutRight,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int,Kokkos::LayoutLeft,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int*,Kokkos::LayoutLeft,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int**,Kokkos::LayoutLeft,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int***,Kokkos::LayoutLeft,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int****,Kokkos::LayoutLeft,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int*****,Kokkos::LayoutLeft,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int******,Kokkos::LayoutLeft,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int*******,Kokkos::LayoutLeft,ExecSpace> >::run();
+
+  TestViewMapOperator< Kokkos::View<int,Kokkos::LayoutRight,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int*,Kokkos::LayoutRight,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int**,Kokkos::LayoutRight,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int***,Kokkos::LayoutRight,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int****,Kokkos::LayoutRight,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int*****,Kokkos::LayoutRight,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int******,Kokkos::LayoutRight,ExecSpace> >::run();
+  TestViewMapOperator< Kokkos::View<int*******,Kokkos::LayoutRight,ExecSpace> >::run();
 }
 
 /*--------------------------------------------------------------------------*/
 
-template< class ExecSpace >
+template< class Space >
 struct TestViewMappingAtomic {
+  typedef typename Space::execution_space ExecSpace ;
+  typedef typename Space::memory_space    MemSpace ;
+
   typedef Kokkos::MemoryTraits< Kokkos::Atomic >  mem_trait ;
 
-  typedef Kokkos::Experimental::View< int * , ExecSpace > T ;
-  typedef Kokkos::Experimental::View< int * , ExecSpace , mem_trait >  T_atom ;
+  typedef Kokkos::View< int * , ExecSpace > T ;
+  typedef Kokkos::View< int * , ExecSpace , mem_trait >  T_atom ;
 
   T      x ;
   T_atom x_atom ;
@@ -1206,8 +1375,10 @@ struct TestViewMappingAtomic {
 
 /*--------------------------------------------------------------------------*/
 
-template< class ExecSpace >
+template< class Space >
 struct TestViewMappingClassValue {
+  typedef typename Space::execution_space ExecSpace ;
+  typedef typename Space::memory_space    MemSpace ;
 
   struct ValueType {
     KOKKOS_INLINE_FUNCTION
