@@ -67,6 +67,8 @@
 #include <MueLu_ExplicitInstantiation.hpp>
 #endif
 
+#include <MueLu_CreateXpetraPreconditioner.hpp>
+
 // This example demonstrates how to reuse some parts of a classical SA multigrid setup between runs.
 //
 // In this example, we suppose that the pattern of the matrix does not change between runs so that:
@@ -77,133 +79,6 @@
 // The resulting preconditioners are identical to multigrid preconditioners built without recycling the parts described above.
 // This can be verified by using the --no-recycling option.
 
-#ifdef HAVE_MUELU_TPETRA
-#include <MueLu_CreateTpetraPreconditioner.hpp>
-#endif
-#ifdef HAVE_MUELU_EPETRA
-#include <MueLu_CreateEpetraPreconditioner.hpp>
-#endif
-
-template<class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-Teuchos::RCP<MueLu::Hierarchy<Scalar,LocalOrdinal,GlobalOrdinal,Node> >
-CreateHierarchy(Teuchos::RCP<Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > A, Teuchos::ParameterList& paramList, Teuchos::RCP<Xpetra::MultiVector<Scalar,LocalOrdinal,GlobalOrdinal,Node> > coords = Teuchos::null) {
-#include <MueLu_UseShortNames.hpp>
-  using Teuchos::RCP;
-
-  Xpetra::UnderlyingLib lib = A->getRowMap()->lib();
-
-  if (lib == Xpetra::UseTpetra) {
-#ifdef HAVE_MUELU_TPETRA
-    RCP<Tpetra::CrsMatrix<SC, LO, GO, NO> >     tA = Utilities::Op2NonConstTpetraCrs(A);
-    RCP<MueLu::TpetraOperator<SC, LO, GO, NO> > tH = MueLu::CreateTpetraPreconditioner(tA, paramList, Utilities::MV2NonConstTpetraMV(coords));
-
-    return tH->GetHierarchy();
-#else
-    throw MueLu::Exceptions::RuntimeError("Tpetra is not available");
-#endif // HAVE_MUELU_TPETRA
-  }
-
-  XPETRA_FACTORY_ERROR_IF_EPETRA(lib);
-  XPETRA_FACTORY_END;
-}
-
-#ifdef HAVE_MUELU_EPETRA
-template<>
-Teuchos::RCP<MueLu::Hierarchy<double,int,int,Xpetra::EpetraNode> >
-CreateHierarchy(Teuchos::RCP<Xpetra::Matrix<double,int,int,Xpetra::EpetraNode> > A, Teuchos::ParameterList& paramList, Teuchos::RCP<Xpetra::MultiVector<double,int,int,Xpetra::EpetraNode> > coords) {
-  typedef double                Scalar;
-  typedef int                   LocalOrdinal;
-  typedef int                   GlobalOrdinal;
-  typedef Xpetra::EpetraNode    Node;
-#include <MueLu_UseShortNames.hpp>
-  using Teuchos::RCP;
-
-  Xpetra::UnderlyingLib lib = A->getRowMap()->lib();
-
-  if (lib == Xpetra::UseTpetra) {
-#ifdef HAVE_MUELU_TPETRA
-    RCP<Tpetra::CrsMatrix<SC, LO, GO, NO> >     tA = Utilities::Op2NonConstTpetraCrs(A);
-    RCP<MueLu::TpetraOperator<SC, LO, GO, NO> > tH = MueLu::CreateTpetraPreconditioner(tA, paramList, Utilities::MV2NonConstTpetraMV(coords));
-
-    return tH->GetHierarchy();
-#else
-    throw MueLu::Exceptions::RuntimeError("Tpetra is not available");
-#endif
-  }
-
-  if (lib == Xpetra::UseEpetra) {
-#ifdef HAVE_MUELU_EPETRA
-    RCP<Epetra_CrsMatrix>      eA = Utilities::Op2NonConstEpetraCrs(A);
-    RCP<MueLu::EpetraOperator> eH = MueLu::CreateEpetraPreconditioner(eA, paramList, Utilities::MV2NonConstEpetraMV(coords));
-    return eH->GetHierarchy();
-#else
-    throw MueLu::Exceptions::RuntimeError("Epetra is not available");
-#endif
-  }
-
-  XPETRA_FACTORY_END;
-}
-#endif // HAVE_MUELU_EPETRA
-
-template<class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
-void
-ReuseHierarchy(Teuchos::RCP<Xpetra::Matrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > A, MueLu::Hierarchy<Scalar,LocalOrdinal,GlobalOrdinal,Node>& H) {
-#include <MueLu_UseShortNames.hpp>
-  using Teuchos::RCP;
-
-  Xpetra::UnderlyingLib lib = A->getRowMap()->lib();
-
-  if (lib == Xpetra::UseTpetra) {
-#ifdef HAVE_MUELU_TPETRA
-    MueLu::TpetraOperator<SC, LO, GO, NO> tH(Teuchos::rcpFromRef(H));
-    MueLu::ReuseTpetraPreconditioner(Utilities::Op2NonConstTpetraCrs(A), tH);
-    return;
-#else
-    throw MueLu::Exceptions::RuntimeError("Tpetra is not available");
-#endif
-  }
-
-  XPETRA_FACTORY_ERROR_IF_EPETRA(lib);
-  XPETRA_FACTORY_END;
-}
-
-#ifdef HAVE_MUELU_EPETRA
-template<>
-void
-ReuseHierarchy(Teuchos::RCP<Xpetra::Matrix<double,int,int,Xpetra::EpetraNode> > A, MueLu::Hierarchy<double,int,int,Xpetra::EpetraNode>& H) {
-  typedef double                Scalar;
-  typedef int                   LocalOrdinal;
-  typedef int                   GlobalOrdinal;
-  typedef Xpetra::EpetraNode    Node;
-#include <MueLu_UseShortNames.hpp>
-  using Teuchos::RCP;
-
-  Xpetra::UnderlyingLib lib = A->getRowMap()->lib();
-
-  if (lib == Xpetra::UseTpetra) {
-#ifdef HAVE_MUELU_TPETRA
-    MueLu::TpetraOperator<SC, LO, GO, NO> tH(Teuchos::rcpFromRef(H));
-    MueLu::ReuseTpetraPreconditioner(Utilities::Op2NonConstTpetraCrs(A), tH);
-    return;
-#else
-    throw MueLu::Exceptions::RuntimeError("Tpetra is not available");
-#endif
-  }
-
-  if (lib == Xpetra::UseEpetra) {
-#ifdef HAVE_MUELU_EPETRA
-    MueLu::EpetraOperator eH(Teuchos::rcpFromRef(H));
-    MueLu::ReuseEpetraPreconditioner(Utilities::Op2NonConstEpetraCrs(A), eH);
-    return;
-#else
-    throw MueLu::Exceptions::RuntimeError("Epetra is not available");
-#endif
-  }
-
-  XPETRA_FACTORY_END;
-}
-#endif
-
 template<class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
 void ConstructData(const std::string& matrixType, Teuchos::ParameterList& galeriList,
                    Xpetra::UnderlyingLib lib, Teuchos::RCP<const Teuchos::Comm<int> >& comm,
@@ -317,11 +192,6 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc, char *argv[]) {
   }
   Xpetra::UnderlyingLib lib = xpetraParameters.GetLib();
 
-  ParameterList paramList;
-  paramList.set("verbosity", "none");
-  if (xmlFileName != "")
-    Teuchos::updateParametersFromXmlFileAndBroadcast(xmlFileName, Teuchos::Ptr<ParameterList>(&paramList), *comm);
-
   // Retrieve matrix parameters (they may have been changed on the command line)
   // [for instance, if we changed matrix type from 2D to 3D we need to update nz]
   ParameterList galeriList = galeriParameters.GetParameterList();
@@ -362,6 +232,13 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc, char *argv[]) {
   std::string thickSeparator = "=============================================================";
   std::string thinSeparator  = "-------------------------------------------------------------";
 
+  ParameterList paramList;
+  paramList.set("verbosity", "none");
+  if (xmlFileName != "")
+    Teuchos::updateParametersFromXmlFileAndBroadcast(xmlFileName, Teuchos::Ptr<ParameterList>(&paramList), *comm);
+
+  out << "Parameter list:" << std::endl << paramList << std::endl;
+
   // =========================================================================
   // Setup #1 (no reuse)
   // =========================================================================
@@ -378,7 +255,7 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc, char *argv[]) {
         tm->start();
 
       A->SetMaxEigenvalueEstimate(-one);
-      H = CreateHierarchy(A, paramList, coordinates);
+      H = MueLu::CreateXpetraPreconditioner(A, paramList, coordinates);
 
       // Stop timing
       if (!(numRebuilds && i == 0)) {
@@ -393,7 +270,7 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc, char *argv[]) {
 
     // Run a build for matrix B to record its convergence
     B->SetMaxEigenvalueEstimate(-one);
-    H = CreateHierarchy(B, paramList, coordinates);
+    H = MueLu::CreateXpetraPreconditioner(B, paramList, coordinates);
 
     X->putScalar(zero);
     H->Iterate(*Y, *X, nIts);
@@ -404,9 +281,10 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc, char *argv[]) {
   // Setup #2-inf (reuse)
   // =========================================================================
   std::vector<std::string> reuseTypes, reuseNames;
-  reuseTypes.push_back("S");  reuseNames.push_back("smoothers");
-  reuseTypes.push_back("tP"); reuseNames.push_back("tentative P");
-  reuseTypes.push_back("RP"); reuseNames.push_back("smoothed P and R");
+  reuseTypes.push_back("none"); reuseNames.push_back("none");
+  reuseTypes.push_back("S");    reuseNames.push_back("smoothers");
+  reuseTypes.push_back("tP");   reuseNames.push_back("tentative P");
+  reuseTypes.push_back("RP");   reuseNames.push_back("smoothed P and R");
 
   for (size_t k = 0; k < reuseTypes.size(); k++) {
     out << thickSeparator << " " << reuseTypes[k] << " " << thickSeparator << std::endl;
@@ -415,7 +293,7 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc, char *argv[]) {
     paramList.set("reuse: type", reuseTypes[k]);
 
     out << thinSeparator << " " << reuseTypes[k] << " (initial) " << thinSeparator << std::endl;
-    RCP<Hierarchy> H = CreateHierarchy(A, paramList, coordinates);
+    RCP<Hierarchy> H = MueLu::CreateXpetraPreconditioner(A, paramList, coordinates);
 
     X->putScalar(zero);
     H->Iterate(*Y, *X, nIts);
@@ -433,7 +311,7 @@ int main_(Teuchos::CommandLineProcessor &clp, int argc, char *argv[]) {
         tm->start();
 
       B->SetMaxEigenvalueEstimate(-one);
-      ReuseHierarchy(B, *H);
+      MueLu::ReuseXpetraPreconditioner(B, H);
 
       // Stop timing
       if (!(numRebuilds && i == 0)) {
