@@ -99,16 +99,6 @@ namespace MueLu {
     Kokkos::parallel_reduce("MueLu:LWGraph:LWGraph:maxnonzeros", graph_.numRows(), maxNumRowEntriesFunctor, maxNumRowEntries_);
   }
 
-  template<class LocalOrdinal, class GlobalOrdinal, class DeviceType>
-  typename LWGraph_kokkos<LocalOrdinal,GlobalOrdinal,Kokkos::Compat::KokkosDeviceWrapperNode<DeviceType>>::row_type
-  LWGraph_kokkos<LocalOrdinal,GlobalOrdinal,Kokkos::Compat::KokkosDeviceWrapperNode<DeviceType>>::
-  getNeighborVertices(LocalOrdinal i) const {
-    auto rowPointers = graph_.row_map;
-    auto colIndices  = graph_.entries;
-
-    return Kokkos::subview(colIndices, std::pair<size_t,size_t>(rowPointers(i), rowPointers(i+1)));
-  }
-
 }
 
 #endif // MUELU_LWGRAPH_KOKKOS_DEF_HPP
