@@ -65,7 +65,7 @@
 
 #include "MueLu_FactoryBase_fwd.hpp"
 #include "MueLu_FactoryManager.hpp" // no fwd declaration because constructor of FactoryManager is used as a default parameter of Setup()
-#include "MueLu_HierarchyHelpers_fwd.hpp"
+#include "MueLu_HierarchyUtils_fwd.hpp"
 #include "MueLu_KeepType.hpp"
 #include "MueLu_Level_fwd.hpp"
 #include "MueLu_MasterList.hpp"
@@ -264,7 +264,7 @@ namespace MueLu {
       Default behavior is to print system and transfer matrices from the entire hierarchy.
       Files are named "A_0.m", "P_1.m", "R_1.m", etc, and are in matrix market coordinate format.
     */
-    void Write(const LO &start=-1, const LO &end=-1);
+    void Write(const LO &start=-1, const LO &end=-1, const std::string &suffix="");
 
     //@}
 
