@@ -71,7 +71,7 @@ namespace MueLu {
     }
     return problemSpecificList_;
   }
-
+ 
    std::string MasterList::interpretParameterName(const std::string& name, const std::string& value) {
 
     // used to concatenate the return string
@@ -119,22 +119,22 @@ namespace MueLu {
     // put in auto-generated code here
 
 
-    if (name == "number of equations") { ss << "<Parameter name=\"number of equations\" type=\"int\" value=" << value << "/>"; return ss.str(); }
-    if (name == "max levels") { ss << "<Parameter name=\"max levels\" type=\"int\" value=" << value << "/>"; return ss.str(); }
-    if (name == "problem: symmetric") { ss << "<Parameter name=\"problem: symmetric\" type=\"bool\" value=" << value << "/>"; return ss.str(); }
-    if (name == "aggregation: drop tol") { ss << "<Parameter name=\"aggregation: drop tol\" type=\"double\" value=" << value << "/>"; return ss.str(); }
-    if (name == "print initial parameters") { ss << "<Parameter name=\"print initial parameters\" type=\"bool\" value=" << value << "/>"; return ss.str(); }
-    if (name == "print unused parameters") { ss << "<Parameter name=\"print unused parameters\" type=\"bool\" value=" << value << "/>"; return ss.str(); }
-    if (name == "sa: damping factor") { ss << "<Parameter name=\"sa: damping factor\" type=\"double\" value=" << value << "/>"; return ss.str(); }
-    if (name == "sa: eigenvalue estimate num iterations") { ss << "<Parameter name=\"sa: eigenvalue estimate num iterations\" type=\"int\" value=" << value << "/>"; return ss.str(); }
-    if (name == "repartition: enable") { ss << "<Parameter name=\"repartition: enable\" type=\"bool\" value=" << value << "/>"; return ss.str(); }
-    if (name == "repartition: start level") { ss << "<Parameter name=\"repartition: start level\" type=\"int\" value=" << value << "/>"; return ss.str(); }
-    if (name == "repartition: min rows per proc") { ss << "<Parameter name=\"repartition: min rows per proc\" type=\"int\" value=" << value << "/>"; return ss.str(); }
-    if (name == "repartition: max imbalance") { ss << "<Parameter name=\"repartition: max imbalance\" type=\"double\" value=" << value << "/>"; return ss.str(); }
-    if (name == "use external multigrid package") { ss << "<Parameter name=\"use external multigrid package\" type=\"string\" value=" << value << "/>"; return ss.str(); }
+    if (name == "number of equations") { ss << "<Parameter name=\"number of equations\" type=\"int\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "max levels") { ss << "<Parameter name=\"max levels\" type=\"int\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "problem: symmetric") { ss << "<Parameter name=\"problem: symmetric\" type=\"bool\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "aggregation: drop tol") { ss << "<Parameter name=\"aggregation: drop tol\" type=\"double\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "print initial parameters") { ss << "<Parameter name=\"print initial parameters\" type=\"bool\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "print unused parameters") { ss << "<Parameter name=\"print unused parameters\" type=\"bool\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "sa: damping factor") { ss << "<Parameter name=\"sa: damping factor\" type=\"double\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "sa: eigenvalue estimate num iterations") { ss << "<Parameter name=\"sa: eigenvalue estimate num iterations\" type=\"int\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "repartition: enable") { ss << "<Parameter name=\"repartition: enable\" type=\"bool\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "repartition: start level") { ss << "<Parameter name=\"repartition: start level\" type=\"int\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "repartition: min rows per proc") { ss << "<Parameter name=\"repartition: min rows per proc\" type=\"int\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "repartition: max imbalance") { ss << "<Parameter name=\"repartition: max imbalance\" type=\"double\" value=" << value << "/>"; return ss.str(); }      
+    if (name == "use external multigrid package") { ss << "<Parameter name=\"use external multigrid package\" type=\"string\" value=" << value << "/>"; return ss.str(); }      
     return "";
   }
-
+ 
   Teuchos::RCP<Teuchos::ParameterList> MasterList::masterList_ = Teuchos::null;
   Teuchos::RCP<Teuchos::ParameterList> MasterList::problemSpecificList_ = Teuchos::null;
   std::string                          MasterList::problemType_ = "unknown";
@@ -177,6 +177,7 @@ namespace MueLu {
   "<Parameter name=\"aggregation: enable phase 2a\" type=\"bool\" value=\"true\"/>"
   "<Parameter name=\"aggregation: enable phase 2b\" type=\"bool\" value=\"true\"/>"
   "<Parameter name=\"aggregation: enable phase 3\" type=\"bool\" value=\"true\"/>"
+  "<Parameter name=\"aggregation: allow empty prolongator columns\" type=\"bool\" value=\"false\"/>"
   "<Parameter name=\"aggregation: preserve Dirichlet points\" type=\"bool\" value=\"false\"/>"
   "<Parameter name=\"aggregation: allow user-specified singletons\" type=\"bool\" value=\"false\"/>"
   "<Parameter name=\"aggregation: export visualization data\" type=\"bool\" value=\"false\"/>"
@@ -234,284 +235,286 @@ namespace MueLu {
 ("Poisson-2D",
 
     "<ParameterList name=\"MueLu\">"
-
+    
             "<Parameter name=\"number of equations\" type=\"int\" value=\"1\"/>"
-
+          
             "<Parameter name=\"smoother: type\" type=\"string\" value=\"CHEBYSHEV\"/>"
-
+          
             "<Parameter name=\"multigrid algorithm\" type=\"string\" value=\"sa\"/>"
-
+          
     "</ParameterList>"
   )
 ("Poisson-3D",
 
     "<ParameterList name=\"MueLu\">"
-
+    
             "<Parameter name=\"number of equations\" type=\"int\" value=\"1\"/>"
-
+          
             "<Parameter name=\"smoother: type\" type=\"string\" value=\"CHEBYSHEV\"/>"
-
+          
             "<Parameter name=\"multigrid algorithm\" type=\"string\" value=\"sa\"/>"
-
+          
     "</ParameterList>"
   )
 ("Elasticity-2D",
 
     "<ParameterList name=\"MueLu\">"
-
+    
             "<Parameter name=\"number of equations\" type=\"int\" value=\"2\"/>"
-
+          
             "<Parameter name=\"smoother: type\" type=\"string\" value=\"CHEBYSHEV\"/>"
-
+          
             "<Parameter name=\"multigrid algorithm\" type=\"string\" value=\"sa\"/>"
-
+          
     "</ParameterList>"
   )
 ("Elasticity-3D",
 
     "<ParameterList name=\"MueLu\">"
-
+    
             "<Parameter name=\"number of equations\" type=\"int\" value=\"3\"/>"
-
+          
             "<Parameter name=\"smoother: type\" type=\"string\" value=\"CHEBYSHEV\"/>"
-
+          
             "<Parameter name=\"multigrid algorithm\" type=\"string\" value=\"sa\"/>"
-
+          
     "</ParameterList>"
   )
 ("MHD",
 
     "<ParameterList name=\"MueLu\">"
-
+    
             "<Parameter name=\"smoother: type\" type=\"string\" value=\"SCHWARZ\"/>"
-
+          
     "<ParameterList name=\"smoother: params\">"
-
+    
         "<Parameter name=\"schwarz: overlap level\" type=\"int\" value=\"1\"/>"
-
+        
         "<Parameter name=\"schwarz: combine mode\" type=\"string\" value=\"Zero\"/>"
-
+        
         "<Parameter name=\"schwarz: use reordering\" type=\"bool\" value=\"false\"/>"
-
+        
         "<Parameter name=\"subdomain solver name\" type=\"string\" value=\"RILUK\"/>"
-
+        
     "<ParameterList name=\"subdomain solver parameters\">"
-
+    
         "<Parameter name=\"fact: iluk level-of-fill\" type=\"int\" value=\"0\"/>"
-
+        
         "<Parameter name=\"fact: absolute threshold\" type=\"double\" value=\"0.\"/>"
-
+        
         "<Parameter name=\"fact: relative threshold\" type=\"double\" value=\"1.\"/>"
-
+        
         "<Parameter name=\"fact: relax value\" type=\"double\" value=\"0.\"/>"
-
+        
     "</ParameterList>"
-
+  
     "</ParameterList>"
-
+  
             "<Parameter name=\"transpose: use implicit\" type=\"bool\" value=\"true\"/>"
-
+          
             "<Parameter name=\"multigrid algorithm\" type=\"string\" value=\"unsmoothed\"/>"
-
+          
     "</ParameterList>"
   )
 ("ConvectionDiffusion",
 
     "<ParameterList name=\"MueLu\">"
-
+    
             "<Parameter name=\"problem: symmetric\" type=\"bool\" value=\"false\"/>"
-
+          
             "<Parameter name=\"smoother: type\" type=\"string\" value=\"RELAXATION\"/>"
-
+          
     "<ParameterList name=\"smoother: params\">"
-
+    
         "<Parameter name=\"relaxation: type\" type=\"string\" value=\"Gauss-Seidel\"/>"
-
+        
     "</ParameterList>"
-
+  
             "<Parameter name=\"multigrid algorithm\" type=\"string\" value=\"pg\"/>"
-
+          
             "<Parameter name=\"sa: use filtered matrix\" type=\"bool\" value=\"true\"/>"
-
+          
     "</ParameterList>"
   )
 ;
   std::map<std::string,std::string> MasterList::ML2MueLuLists_ = DefaultProblemStrings<std::string,std::string>
 
          ("default values","problem: type")
-
+      
          ("ML output","verbosity")
-
+      
          ("PDE equations","number of equations")
-
+      
          ("max levels","max levels")
-
+      
          ("prec type","cycle type")
-
+      
          ("problem: symmetric","problem: symmetric")
-
+      
          ("xml parameter file","xml parameter file")
-
+      
          ("parameterlist: syntax","parameterlist: syntax")
-
+      
          ("smoother: pre or post","smoother: pre or post")
-
+      
          ("smoother: type","smoother: type")
-
+      
          ("smoother: pre type","smoother: pre type")
-
+      
          ("smoother: post type","smoother: post type")
-
+      
          ("smoother: params","smoother: params")
-
+      
          ("smoother: pre params","smoother: pre params")
-
+      
          ("smoother: post params","smoother: post params")
-
+      
          ("smoother: overlap","smoother: overlap")
-
+      
          ("smoother: pre overlap","smoother: pre overlap")
-
+      
          ("smoother: post overlap","smoother: post overlap")
-
+      
          ("max size","coarse: max size")
-
+      
          ("coarse: type","coarse: type")
-
+      
          ("coarse: params","coarse: params")
-
+      
          ("coarse: overlap","coarse: overlap")
-
+      
          ("aggregation: type","aggregation: type")
-
+      
          ("aggregation: ordering","aggregation: ordering")
-
+      
          ("aggregation: drop scheme","aggregation: drop scheme")
-
+      
          ("aggregation: threshold","aggregation: drop tol")
-
+      
          ("aggregation: min agg size","aggregation: min agg size")
-
+      
          ("aggregation: max agg size","aggregation: max agg size")
-
+      
          ("aggregation: brick x size","aggregation: brick x size")
-
+      
          ("aggregation: brick y size","aggregation: brick y size")
-
+      
          ("aggregation: brick z size","aggregation: brick z size")
-
+      
          ("aggregation: max selected neighbors","aggregation: max selected neighbors")
-
+      
          ("aggregation: Dirichlet threshold","aggregation: Dirichlet threshold")
-
+      
          ("aggregation: enable phase 1","aggregation: enable phase 1")
-
+      
          ("aggregation: enable phase 2a","aggregation: enable phase 2a")
-
+      
          ("aggregation: enable phase 2b","aggregation: enable phase 2b")
-
+      
          ("aggregation: enable phase 3","aggregation: enable phase 3")
-
+      
+         ("aggregation: allow empty prolongator columns","aggregation: allow empty prolongator columns")
+      
          ("aggregation: preserve Dirichlet points","aggregation: preserve Dirichlet points")
-
+      
          ("aggregation: allow user-specified singletons","aggregation: allow user-specified singletons")
-
+      
          ("aggregation: export visualization data","aggregation: export visualization data")
-
+      
          ("aggregation: output filename","aggregation: output filename")
-
+      
          ("aggregation: output file: time step","aggregation: output file: time step")
-
+      
          ("aggregation: output file: iter","aggregation: output file: iter")
-
+      
          ("aggregation: output file: agg style","aggregation: output file: agg style")
-
+      
          ("aggregation: output file: fine graph edges","aggregation: output file: fine graph edges")
-
+      
          ("aggregation: output file: coarse graph edges","aggregation: output file: coarse graph edges")
-
+      
          ("aggregation: output file: build colormap","aggregation: output file: build colormap")
-
+      
          ("aggregation: params","aggregation: params")
-
+      
          ("strength-of-connection: params","strength-of-connection: params")
-
+      
          ("export data","export data")
-
+      
          ("ML print initial list","print initial parameters")
-
+      
          ("print unused","print unused parameters")
-
+      
          ("transpose: use implicit","transpose: use implicit")
-
+      
          ("use kokkos refactor","use kokkos refactor")
-
+      
          ("energy minimization: enable","multigrid algorithm")
-
+      
          ("toggle: mode","toggle: mode")
-
+      
          ("semicoarsen: coarsen rate","semicoarsen: coarsen rate")
-
+      
          ("semicoarsen: number of levels","semicoarsen: number of levels")
-
+      
          ("linedetection: orientation","linedetection: orientation")
-
+      
          ("linedetection: num layers","linedetection: num layers")
-
+      
          ("aggregation: damping factor","sa: damping factor")
-
+      
          ("sa: use filtered matrix","sa: use filtered matrix")
-
+      
          ("sa: calculate eigenvalue estimate","sa: calculate eigenvalue estimate")
-
+      
          ("eigen-analysis: iterations","sa: eigenvalue estimate num iterations")
-
+      
          ("transfer: params","transfer: params")
-
+      
          ("filtered matrix: use lumping","filtered matrix: use lumping")
-
+      
          ("filtered matrix: reuse eigenvalue","filtered matrix: reuse eigenvalue")
-
+      
          ("filtered matrix: reuse graph","filtered matrix: reuse graph")
-
+      
          ("emin: iterative method","emin: iterative method")
-
+      
          ("emin: num iterations","emin: num iterations")
-
+      
          ("emin: num reuse iterations","emin: num reuse iterations")
-
+      
          ("emin: pattern","emin: pattern")
-
+      
          ("emin: pattern order","emin: pattern order")
-
+      
          ("repartition: enable","repartition: enable")
-
+      
          ("repartition: partitioner","repartition: partitioner")
-
+      
          ("repartition: params","repartition: params")
-
+      
          ("repartition: start level","repartition: start level")
-
+      
          ("repartition: min per proc","repartition: min rows per proc")
-
+      
          ("repartition: max min ratio","repartition: max imbalance")
-
+      
          ("repartition: remap parts","repartition: remap parts")
-
+      
          ("repartition: remap num values","repartition: remap num values")
-
+      
          ("repartition: print partition distribution","repartition: print partition distribution")
-
+      
          ("repartition: rebalance P and R","repartition: rebalance P and R")
-
+      
          ("repartition: use subcommunicators","repartition: use subcommunicators")
-
+      
          ("reuse: type","reuse: type")
-
+      
          ("use external multigrid package","use external multigrid package")
-
+      
          ("amgx:params","amgx:params")
-
+      
          ("debug: graph level","debug: graph level")
       ;
 
