@@ -66,12 +66,48 @@
 
 namespace MueLu {
 
+
   /*!
     @class Zoltan2Interface
     @brief Interface to Zoltan2 library.
+    @ingroup Rebalancing
 
     This interface provides access to partitioning methods in Zoltan2.
-    Currently, it only supports the MultiJagged and RCB algorithms.
+    Currently, it supports the RCB algorithm only.
+
+    ## Input/output of Zoltan2Interface ##
+
+    ### User parameters of Zoltan2Interface ###
+    Parameter | type | default | master.xml | validated | requested | description
+    ----------|------|---------|:----------:|:---------:|:---------:|------------
+    | A                                      | Factory | null  |   | * | * | Generating factory of the matrix A used during the prolongator smoothing process |
+    | Coordinates                            | Factory | null  |   | * | * | Factory generating coordinates vector used for rebalancing (RCB algorithm)
+    | rowWeight                              | int     | 0     |   | * |   | Default weight to rows (total weight = nnz + rowWeight")
+    | ParameterList                          | ParamterList | null |  | * |  | Zoltan2 parameters
+    | number of partitions                   | GO      | - |  |  |  | Short-cut parameter set by RepartitionFactory. Avoid repartitioning algorithms if only one partition is necessary (see details below)
+
+    The * in the @c master.xml column denotes that the parameter is defined in the @c master.xml file.<br>
+    The * in the @c validated column means that the parameter is declared in the list of valid input parameters (see Zoltan2Interface::GetValidParameters).<br>
+    The * in the @c requested column states that the data is requested as input with all dependencies (see Zoltan2Interface::DeclareInput).
+
+    ### Variables provided by Zoltan2Interface ###
+
+    After Zoltan2Interface::Build the following data is available (if requested)
+
+    Parameter | generated by | description
+    ----------|--------------|------------
+    | Partition       | Zoltan2Interface   | GOVector based on the Row map of A (DOF-based) containing the process id the DOF should be living in after rebalancing/repartitioning
+
+    The "Partition" vector is used as input for the RepartitionFactory class.
+    If Re-partitioning/rebalancing is necessary it uses the "Partition" variable to create the corresponding Xpetra::Import object which then is used
+    by the RebalanceFactory classes (e.g., RebalanceAcFactory, RebalanceTransferFactory,...) to rebalance the coarse level operators.
+
+    The RepartitionHeuristicFactory calculates how many partitions are to be built when performing rebalancing.
+    It stores the result in the "number of partitions" variable on the current level (type = GO).
+    If it is "number of partitions=1" we skip the Zoltan2 call and just create an dummy "Partition" vector containing zeros only.
+    If no repartitioning is necessary (i.e., just keep the current partitioning) we return "Partition = Teuchos::null".
+    If "number of partitions" > 1, the algorithm tries to find the requested number of partitions.
+
   */
 
   //FIXME: this class should not be templated
@@ -145,12 +181,14 @@ namespace MueLu {
 
       validParamList->set< RCP<const FactoryBase> >   ("A",             Teuchos::null, "Factory of the matrix A");
       validParamList->set< RCP<const FactoryBase> >   ("Coordinates",   Teuchos::null, "Factory of the coordinates");
+      validParamList->set< RCP<const FactoryBase> >   ("number of partitions", Teuchos::null, "Instance of RepartitionHeuristicFactory.");
       validParamList->set< RCP<const ParameterList> > ("ParameterList", Teuchos::null, "Zoltan2 parameters");
 
       return validParamList;
     };
     void DeclareInput(Level& currentLevel) const {
       Input(currentLevel, "A");
+      Input(currentLevel, "number of partitions");
       Input(currentLevel, "Coordinates");
     };
     void Build(Level& currentLevel) const {
