@@ -142,6 +142,9 @@ namespace MueLu {
     //! pointer to Amesos2 solver object
     RCP<Amesos2::Solver<Tpetra_CrsMatrix, Tpetra_MultiVector> > prec_;
 
+    bool useTransformation_;
+    RCP<MultiVector> X_, B_;
+
   }; // class Amesos2Smoother
 
 #ifdef HAVE_MUELU_EPETRA
@@ -166,7 +169,9 @@ namespace MueLu {
     void DeclareInput(Level& currentLevel) const {};
     void Setup(Level& currentLevel) {};
     void Apply(MultiVector& X, const MultiVector& B, bool InitialGuessIsZero = false) const {};
+
     RCP<SmootherPrototype> Copy() const { return Teuchos::null; };
+
     std::string description() const { return std::string(""); };
     void print(Teuchos::FancyOStream& out, const VerbLevel verbLevel = Default) const {};
   };
