@@ -147,8 +147,6 @@ namespace MueLu {
 
     const size_t numRows   = rowMap->getNodeNumElements();
 
-    RCP<const Teuchos::Comm<int> > comm = rowMap->getComm();
-
     typedef Teuchos::ScalarTraits<SC> STS;
     typedef typename STS::magnitudeType Magnitude;
     const SC     zero      = STS::zero();
@@ -180,11 +178,6 @@ namespace MueLu {
                             << "using GO->LO conversion with performance penalty" << std::endl;
     }
 
-    // Find largest aggregate size
-    LO maxAggSize = 0;
-    for (GO i = 0; i < numAggs; i++)
-      maxAggSize = std::max(maxAggSize, aggStart[i+1] - aggStart[i]);
-
     coarseNullspace = MultiVectorFactory::Build(coarseMap, NSDim);
 
     // Pull out the nullspace vectors so that we can have random access.
@@ -378,8 +371,6 @@ namespace MueLu {
   void TentativePFactory<Scalar, LocalOrdinal, GlobalOrdinal, Node>::
   BuildPcoupled(RCP<Matrix> A, RCP<Aggregates> aggregates, RCP<AmalgamationInfo> amalgInfo, RCP<MultiVector> fineNullspace,
                 RCP<const Map> coarseMap, RCP<Matrix>& Ptentative, RCP<MultiVector>& coarseNullspace) const {
-    RCP<const Teuchos::Comm<int> > comm = A->getRowMap()->getComm();
-
     typedef Teuchos::ScalarTraits<SC> STS;
     typedef typename STS::magnitudeType Magnitude;
     const SC     zero      = STS::zero();
