@@ -395,7 +395,7 @@ namespace MueLu {
   template<class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
   void AggregationExportFactory<Scalar, LocalOrdinal, GlobalOrdinal, Node>::doAlphaHulls2D_(std::vector<int>& vertices, std::vector<int>& geomSizes) const
   {
-    const double ALPHA_VAL = 2; //Make configurable?
+    //const double ALPHA_VAL = 2; //Make configurable?
     using namespace std;
     //CGAL setup
     typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
@@ -408,6 +408,7 @@ namespace MueLu {
     typedef CGAL::Delaunay_triangulation_2<K,Tds> Triangulation_2;
     typedef CGAL::Alpha_shape_2<Triangulation_2> Alpha_shape_2;
     typedef Alpha_shape_2::Alpha_shape_edges_iterator Alpha_shape_edges_iterator;
+#if 0 // taw: does not compile with CGAL 4.8
     for(int i = 0; i < numAggs_; i++)
     {
       //Populate a list of Point_2 for this aggregate
@@ -424,7 +425,7 @@ namespace MueLu {
       }
       Alpha_shape_2 hull(aggPoints.begin(), aggPoints.end(), FT(ALPHA_VAL), Alpha_shape_2::GENERAL);
       vector<Segment> segments;
-      alpha_edges(hull, back_inserter(segments));
+      CGAL::alpha_edges(hull, back_inserter(segments));
       vertices.reserve(vertices.size() + 2 * segments.size());
       geomSizes.reserve(geomSizes.size() + segments.size());
       for(size_t j = 0; j < segments.size(); j++)
@@ -448,12 +449,14 @@ namespace MueLu {
         geomSizes.push_back(2); //all cells are line segments
       }
     }
+#endif // if 0
   }
 
   template<class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
   void AggregationExportFactory<Scalar, LocalOrdinal, GlobalOrdinal, Node>::doAlphaHulls3D_(std::vector<int>& vertices, std::vector<int>& geomSizes) const
   {
     typedef CGAL::Exact_predicates_inexact_constructions_kernel Gt;
+#if 0 // does not compile with CGAL 4-8
     typedef CGAL::Alpha_shape_cell_base_3<Gt> Fb;
     typedef CGAL::Triangulation_data_structure_3<Vb,Fb> Tds;
     typedef CGAL::Delaunay_triangulation_3<Gt,Tds> Triangulation_3;
@@ -467,6 +470,7 @@ namespace MueLu {
     typedef Gt::Bare_point Bare_point;
     const double ALPHA_VAL = 2; //Make configurable?
     using namespace std;
+
     for(int i = 0; i < numAggs_; i++)
     {
       list<Point> aggPoints;
@@ -539,6 +543,7 @@ namespace MueLu {
 
       }
     }
+#endif // if 0
   }
 #endif
 
