@@ -127,13 +127,13 @@
 
 
 int main(int argc, char *argv[]) {
-#if defined(HAVE_MUELU_SERIAL) && defined(HAVE_MUELU_EPETRA)
+#if defined(HAVE_MUELU_EPETRA) && defined(HAVE_MUELU_EPETRAEXT)
   typedef double Scalar;
   typedef int LocalOrdinal;
   typedef int GlobalOrdinal;
   typedef LocalOrdinal LO;
   typedef GlobalOrdinal GO;
-  typedef Kokkos::Compat::KokkosSerialWrapperNode Node;
+  typedef Xpetra::EpetraNode Node;
 #include "MueLu_UseShortNames.hpp"
 
   using Teuchos::RCP;
@@ -153,10 +153,6 @@ int main(int argc, char *argv[]) {
   Time myTime("global");
   TimeMonitor MM(myTime);
 
-#ifndef HAVE_XPETRA_INT_LONG_LONG
-  *out << "Warning: scaling test was not compiled with long long int support" << std::endl;
-#endif
-
   // custom parameters
   LocalOrdinal maxLevels = 3;
 
@@ -198,8 +194,6 @@ int main(int argc, char *argv[]) {
 
   EpetraExt::MatrixMarketFileToCrsMatrix("A_re1000_5932.txt",*fullmap,*fullmap,*fullmap,ptrA);
   EpetraExt::MatrixMarketFileToVector("b_re1000_5932.txt",*fullmap,ptrf);
-  //EpetraExt::MatrixMarketFileToCrsMatrix("/home/tobias/promotion/trilinos/fc17-dyn/packages/muelu/test/navierstokes/A_re1000_5932.txt",*fullmap,*fullmap,*fullmap,ptrA);
-  //EpetraExt::MatrixMarketFileToVector("/home/tobias/promotion/trilinos/fc17-dyn/packages/muelu/test/navierstokes/b_re1000_5932.txt",*fullmap,ptrf);
 
   RCP<Epetra_CrsMatrix> epA = rcp(ptrA);
   RCP<Epetra_Vector> epv = rcp(ptrf);
@@ -238,10 +232,10 @@ int main(int argc, char *argv[]) {
   /////////////////////////////////////// build blocked transfer operator
   // using the map extractor
   RCP<Xpetra::BlockedCrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node> > bOp = rcp(new Xpetra::BlockedCrsMatrix<Scalar,LocalOrdinal,GlobalOrdinal,Node>(map_extractor,map_extractor,10));
-  bOp->setMatrix(0,0,xA11);
-  bOp->setMatrix(0,1,xA12);
-  bOp->setMatrix(1,0,xA21);
-  bOp->setMatrix(1,1,xA22);
+  bOp->setMatrix(0,0,Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LocalOrdinal,GlobalOrdinal,Node>(xA11)));
+  bOp->setMatrix(0,1,Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LocalOrdinal,GlobalOrdinal,Node>(xA12)));
+  bOp->setMatrix(1,0,Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LocalOrdinal,GlobalOrdinal,Node>(xA21)));
+  bOp->setMatrix(1,1,Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LocalOrdinal,GlobalOrdinal,Node>(xA22)));
 
   bOp->fillComplete();
 
@@ -422,16 +416,6 @@ int main(int argc, char *argv[]) {
   AcFact->SetFactory("P", PFact);
   AcFact->SetFactory("R", RFact);
 
-  /* TODO: not available yet for BlockedRAPFactory. Need some inheritence.
-  // register aggregation export factory in RAPFactory
-  RCP<MueLu::AggregationExportFactory<Scalar,LocalOrdinal,GlobalOrdinal,Node> > aggExpFact = rcp(new MueLu::AggregationExportFactory<Scalar,LocalOrdinal,GlobalOrdinal,Node>());
-  aggExpFact->SetParameter("Output filename","aggs_level%LEVELID_proc%PROCID.out");
-  aggExpFact->SetFactory("Aggregates", CoupledAggFact11);
-  aggExpFact->SetFactory("DofsPerNode", dropFact11);
-
-  AcFact->AddTransferFactory(aggExpFact);
-  */
-
   *out << "Creating Braess-Sarazin Smoother" << std::endl;
 
   //////////////////////////////////////////////////////////////////////
@@ -474,8 +458,6 @@ int main(int argc, char *argv[]) {
   smootherPrototype->AddFactoryManager(MB,0);
   coarseSolverPrototype->AddFactoryManager(MB,0);
 
-
-
   // main factory manager
   FactoryManager M;
   M.SetFactory("A",            AcFact);
@@ -551,7 +533,7 @@ int main(int argc, char *argv[]) {
 
   return EXIT_SUCCESS;
 #else
-  std::cout << "Epetra needs Serial node. Please recompile MueLu with the Serial node enabled." << std::endl;
+  std::cout << "Epetra (and/or EpetraExt) not enabled. Skip test." << std::endl;
   return EXIT_SUCCESS;
 #endif // #if defined(HAVE_MUELU_SERIAL) && defined(HAVE_MUELU_EPETRA)
 }
