@@ -123,13 +123,13 @@
 
 
 int main(int argc, char *argv[]) {
-#if defined(HAVE_MUELU_SERIAL) && defined(HAVE_MUELU_EPETRA)
+#if defined(HAVE_MUELU_EPETRA) && defined(HAVE_MUELU_EPETRAEXT)
   typedef double Scalar;
   typedef int LocalOrdinal;
   typedef int GlobalOrdinal;
   typedef LocalOrdinal LO;
   typedef GlobalOrdinal GO;
-  typedef Kokkos::Compat::KokkosSerialWrapperNode Node;
+  typedef Xpetra::EpetraNode Node;
 #include "MueLu_UseShortNames.hpp"
 
   using Teuchos::RCP;
@@ -168,10 +168,6 @@ int main(int argc, char *argv[]) {
     Time myTime("global");
     TimeMonitor MM(myTime);
 
-#ifndef HAVE_XPETRA_INT_LONG_LONG
-    *out << "Warning: scaling test was not compiled with long long int support" << std::endl;
-#endif
-
     // custom parameters
     LO maxLevels = 2;   // TODO: singular system if MaxLevels > 2?
 
@@ -248,10 +244,10 @@ int main(int argc, char *argv[]) {
     /////////////////////////////////////// build blocked transfer operator
     // using the map extractor
     RCP<Xpetra::BlockedCrsMatrix<Scalar,LO,GO,Node> > bOp = rcp(new Xpetra::BlockedCrsMatrix<Scalar,LO,GO,Node>(map_extractor,map_extractor,10));
-    bOp->setMatrix(0,0,xA11);
-    bOp->setMatrix(0,1,xA12);
-    bOp->setMatrix(1,0,xA21);
-    bOp->setMatrix(1,1,xA22);
+    bOp->setMatrix(0,0,Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LocalOrdinal,GlobalOrdinal,Node>(xA11)));
+    bOp->setMatrix(0,1,Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LocalOrdinal,GlobalOrdinal,Node>(xA12)));
+    bOp->setMatrix(1,0,Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LocalOrdinal,GlobalOrdinal,Node>(xA21)));
+    bOp->setMatrix(1,1,Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LocalOrdinal,GlobalOrdinal,Node>(xA22)));
 
     bOp->fillComplete();
 
@@ -288,7 +284,6 @@ int main(int argc, char *argv[]) {
     RCP<SmootherPrototype> smoProto11     = rcp( new TrilinosSmoother(ifpack11Type, ifpack11List, 0) );
     smoProto11->SetFactory("A", A11Fact);
     RCP<SmootherFactory> Smoo11Fact = rcp( new SmootherFactory(smoProto11) );
-    Smoo11Fact->SetFactory("A",A11Fact);
 
     ////////////////////////////////////////// prepare null space for A11
     RCP<MultiVector> nullspace11 = MultiVectorFactory::Build(xstridedvelmap, 2);  // this is a 2D standard null space
@@ -580,7 +575,7 @@ int main(int argc, char *argv[]) {
 
   return ( success ? EXIT_SUCCESS : EXIT_FAILURE );
 #else
-  std::cout << "Epetra needs Serial node. Please recompile MueLu with the Serial node enabled." << std::endl;
+  std::cout << "Epetra (and/or EpetraExt) are not available. Skip test." << std::endl;
   return EXIT_SUCCESS;
 #endif // #if defined(HAVE_MUELU_SERIAL) && defined(HAVE_MUELU_EPETRA)
 }
