@@ -100,6 +100,7 @@
 #if defined(HAVE_MPI) && defined(HAVE_MUELU_ZOLTAN) && defined(HAVE_MUELU_ISORROPIA)
 #include "MueLu_IsorropiaInterface.hpp"
 #include "MueLu_RepartitionInterface.hpp"
+#include "MueLu_RepartitionHeuristicFactory.hpp"
 #include "MueLu_RepartitionFactory.hpp"
 #include "MueLu_RebalanceTransferFactory.hpp"
 #include "MueLu_RebalanceAcFactory.hpp"
@@ -339,25 +340,32 @@ int main(int argc, char *argv[]) {
       RCP<AmalgamationFactory> rebAmalgFact = rcp(new AmalgamationFactory());
       rebAmalgFact->SetFactory("A", Acfact);
 
+      // Repartitioning (decides how many partitions are built)
+      RCP<Factory> RepartitionHeuristicFact = rcp(new RepartitionHeuristicFactory());
+      {
+        Teuchos::ParameterList paramList;
+        paramList.set("repartition: min rows per proc", optMinRowsPerProc);
+        paramList.set("repartition: max imbalance", optNnzImbalance);
+        RepartitionHeuristicFact->SetParameterList(paramList);
+      }
+      RepartitionHeuristicFact->SetFactory("A", Acfact);
+
       // create amalgamated "Partition"
       RCP<MueLu::IsorropiaInterface<LO, GO, NO> > isoInterface = rcp(new MueLu::IsorropiaInterface<LO, GO, NO>());
       isoInterface->SetFactory("A", Acfact);
+      isoInterface->SetFactory("number of partitions", RepartitionHeuristicFact);
       isoInterface->SetFactory("UnAmalgamationInfo", rebAmalgFact);
 
       // create "Partition" by unamalgamtion
       RCP<MueLu::RepartitionInterface<LO, GO, NO> > repInterface = rcp(new MueLu::RepartitionInterface<LO, GO, NO>());
       repInterface->SetFactory("A", Acfact);
+      repInterface->SetFactory("number of partitions", RepartitionHeuristicFact);
       repInterface->SetFactory("AmalgamatedPartition", isoInterface);
 
       // Repartitioning (creates "Importer" from "Partition")
       RCP<Factory> RepartitionFact = rcp(new RepartitionFactory());
-      {
-        Teuchos::ParameterList paramList;
-        paramList.set("repartition: min rows per proc", optMinRowsPerProc);
-        paramList.set("repartition: max imbalance", optNnzImbalance);
-        RepartitionFact->SetParameterList(paramList);
-      }
       RepartitionFact->SetFactory("A", Acfact);
+      RepartitionFact->SetFactory("number of partitions", RepartitionHeuristicFact);
       RepartitionFact->SetFactory("Partition", repInterface);
 
       // Reordering of the transfer operators
