@@ -43,112 +43,132 @@
 // ***********************************************************************
 //
 // @HEADER
-#include "Teuchos_UnitTestHarness.hpp"
-#include "MueLu_TestHelpers.hpp"
-#include "MueLu_Version.hpp"
+#include <Teuchos_UnitTestHarness.hpp>
+#include <Teuchos_TestingHelpers.hpp>
+#include <Teuchos_ScalarTraits.hpp>
+#include <MueLu_TestHelpers.hpp>
+#include <MueLu_Version.hpp>
 
-#include "MueLu_FactoryManagerBase.hpp"
-#include "MueLu_CoalesceDropFactory.hpp"
-#include "MueLu_CoupledAggregationFactory.hpp"
-#include "MueLu_UncoupledAggregationFactory.hpp"
-#include "MueLu_AmalgamationFactory.hpp"
-#include "MueLu_AmalgamationInfo.hpp"
-#include "MueLu_Aggregates.hpp"
-
-#include "MueLu_UseDefaultTypes.hpp"
+#include <MueLu_FactoryManagerBase.hpp>
+#include <MueLu_CoalesceDropFactory.hpp>
+#include <MueLu_CoupledAggregationFactory.hpp>
+#include <MueLu_UncoupledAggregationFactory.hpp>
+#include <MueLu_AmalgamationFactory.hpp>
+#include <MueLu_AmalgamationInfo.hpp>
+#include <MueLu_Aggregates.hpp>
 
 namespace MueLuTests {
 
-#include "MueLu_UseShortNames.hpp"
+template
+<class Scalar, class LocalOrdinal, class GlobalOrdinal, class Node>
+class AggregateGenerator {
+  //typedef MueLu::AmalgamationInfo<LocalOrdinal,GlobalOrdinal,Node> amalgamation_info_type;
+  //typedef MueLu::Aggregates<LocalOrdinal,GlobalOrdinal,Node> aggregates_type;
+  //typedef Xpetra::Matrix<Scalar, LocalOrdinal, GlobalOrdinal, Node> xpetra_matrix_type;
+#   include <MueLu_UseShortNames.hpp>
 
-  // Little utility to generate uncoupled aggregates.
-  RCP<Aggregates> gimmeUncoupledAggregates(const RCP<Matrix> & A, RCP<AmalgamationInfo> & amalgInfo, bool bPhase1 = true, bool bPhase2a = true, bool bPhase2b = true, bool bPhase3 = true)
+    public:
+
+    // Little utility to generate uncoupled aggregates.
+    RCP<Aggregates>
+    static gimmeUncoupledAggregates(const RCP<Matrix> & A, RCP<AmalgamationInfo> & amalgInfo, bool bPhase1 = true, bool bPhase2a = true, bool bPhase2b = true, bool bPhase3 = true)
+  //  RCP<MueLu::Aggregates> gimmeUncoupledAggregates(const RCP<xpetra_matrix_type> & A, RCP<AmalgamationInfo> & amalgInfo, bool bPhase1 = true, bool bPhase2a = true, bool bPhase2b = true, bool bPhase3 = true)
+    {
+      Level level;
+      TestHelpers::TestFactory<SC,LO,GO,NO>::createSingleLevelHierarchy(level);
+      level.Set("A", A);
+
+      RCP<AmalgamationFactory> amalgFact = rcp(new AmalgamationFactory());
+      RCP<CoalesceDropFactory> dropFact = rcp(new CoalesceDropFactory());
+      dropFact->SetFactory("UnAmalgamationInfo", amalgFact);
+
+      std::cout << "Phase2a=" << bPhase2a << std::endl;
+
+      // Setup aggregation factory (use default factory for graph)
+      RCP<UncoupledAggregationFactory> aggFact = rcp(new UncoupledAggregationFactory());
+      aggFact->SetFactory("Graph", dropFact);
+      aggFact->SetParameter("aggregation: max agg size",Teuchos::ParameterEntry(3));
+      aggFact->SetParameter("aggregation: min agg size",Teuchos::ParameterEntry(3));
+      aggFact->SetParameter("aggregation: max selected neighbors",Teuchos::ParameterEntry(0));
+      aggFact->SetParameter("aggregation: ordering",Teuchos::ParameterEntry(std::string("natural")));
+      aggFact->SetParameter("aggregation: enable phase 1",  Teuchos::ParameterEntry(bPhase1));
+      aggFact->SetParameter("aggregation: enable phase 2a", Teuchos::ParameterEntry(bPhase2a));
+      aggFact->SetParameter("aggregation: enable phase 2b", Teuchos::ParameterEntry(bPhase2b));
+      aggFact->SetParameter("aggregation: enable phase 3",  Teuchos::ParameterEntry(bPhase3));
+
+      level.Request("Aggregates", aggFact.get());
+      level.Request("UnAmalgamationInfo", amalgFact.get());
+
+      level.Request(*aggFact);
+      aggFact->Build(level);
+      RCP<Aggregates> aggregates = level.Get<RCP<Aggregates> >("Aggregates",aggFact.get()); // fix me
+      amalgInfo = level.Get<RCP<AmalgamationInfo> >("UnAmalgamationInfo",amalgFact.get()); // fix me
+      level.Release("UnAmalgamationInfo", amalgFact.get());
+      level.Release("Aggregates", aggFact.get());
+      return aggregates;
+    }  // gimmeUncoupledAggregates
+
+    // Little utility to generate coupled aggregates.
+    static RCP<Aggregates>
+    gimmeCoupledAggregates(const RCP<Matrix> & A, RCP<AmalgamationInfo> & amalgInfo)
+    //  RCP<Aggregates> gimmeCoupledAggregates(const RCP<xpetra_matrix_type> & A, RCP<AmalgamationInfo> & amalgInfo)
+    {
+      Level level;
+      TestHelpers::TestFactory<SC,LO,GO,NO>::createSingleLevelHierarchy(level);
+      level.Set("A", A);
+
+      RCP<AmalgamationFactory> amalgFact = rcp(new AmalgamationFactory());
+      RCP<CoalesceDropFactory> dropFact = rcp(new CoalesceDropFactory());
+      dropFact->SetFactory("UnAmalgamationInfo", amalgFact);
+
+      // Setup aggregation factory (use default factory for graph)
+      RCP<CoupledAggregationFactory> aggFact = rcp(new CoupledAggregationFactory());
+      aggFact->SetFactory("Graph", dropFact);
+      aggFact->SetMinNodesPerAggregate(3);
+      aggFact->SetMaxNeighAlreadySelected(0);
+      aggFact->SetOrdering("natural");
+      aggFact->SetPhase3AggCreation(0.5);
+
+      level.Request("Aggregates", aggFact.get());
+      level.Request("UnAmalgamationInfo", amalgFact.get());
+
+      level.Request(*aggFact);
+      aggFact->Build(level);
+      RCP<Aggregates> aggregates = level.Get<RCP<Aggregates> >("Aggregates",aggFact.get()); // fix me
+      amalgInfo = level.Get<RCP<AmalgamationInfo> >("UnAmalgamationInfo",amalgFact.get()); // fix me
+      level.Release("UnAmalgamationInfo", amalgFact.get());
+      level.Release("Aggregates", aggFact.get());
+      return aggregates;
+    }  // gimmeCoupledAggregates
+};
+
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL(Aggregates, JustAggregation, Scalar, LocalOrdinal, GlobalOrdinal, Node)
   {
-    Level level;
-    TestHelpers::TestFactory<SC,LO,GO,NO>::createSingleLevelHierarchy(level);
-    level.Set("A", A);
+#   include <MueLu_UseShortNames.hpp>
+    MUELU_TESTING_SET_OSTREAM;
+    MUELU_TESTING_LIMIT_SCOPE(Scalar,GlobalOrdinal,Node);
 
-    RCP<AmalgamationFactory> amalgFact = rcp(new AmalgamationFactory());
-    RCP<CoalesceDropFactory> dropFact = rcp(new CoalesceDropFactory());
-    dropFact->SetFactory("UnAmalgamationInfo", amalgFact);
-
-    std::cout << "Phase2a=" << bPhase2a << std::endl;
-
-    // Setup aggregation factory (use default factory for graph)
-    RCP<UncoupledAggregationFactory> aggFact = rcp(new UncoupledAggregationFactory());
-    aggFact->SetFactory("Graph", dropFact);
-    aggFact->SetParameter("aggregation: max agg size",Teuchos::ParameterEntry(3));
-    aggFact->SetParameter("aggregation: min agg size",Teuchos::ParameterEntry(3));
-    aggFact->SetParameter("aggregation: max selected neighbors",Teuchos::ParameterEntry(0));
-    aggFact->SetParameter("aggregation: ordering",Teuchos::ParameterEntry(std::string("natural")));
-    aggFact->SetParameter("aggregation: enable phase 1",  Teuchos::ParameterEntry(bPhase1));
-    aggFact->SetParameter("aggregation: enable phase 2a", Teuchos::ParameterEntry(bPhase2a));
-    aggFact->SetParameter("aggregation: enable phase 2b", Teuchos::ParameterEntry(bPhase2b));
-    aggFact->SetParameter("aggregation: enable phase 3",  Teuchos::ParameterEntry(bPhase3));
-
-    level.Request("Aggregates", aggFact.get());
-    level.Request("UnAmalgamationInfo", amalgFact.get());
-
-    level.Request(*aggFact);
-    aggFact->Build(level);
-    RCP<Aggregates> aggregates = level.Get<RCP<Aggregates> >("Aggregates",aggFact.get()); // fix me
-    amalgInfo = level.Get<RCP<AmalgamationInfo> >("UnAmalgamationInfo",amalgFact.get()); // fix me
-    level.Release("UnAmalgamationInfo", amalgFact.get());
-    level.Release("Aggregates", aggFact.get());
-    return aggregates;
-  }  // gimmeUncoupledAggregates
-
-  // Little utility to generate coupled aggregates.
-  RCP<Aggregates> gimmeCoupledAggregates(const RCP<Matrix> & A, RCP<AmalgamationInfo> & amalgInfo)
-  {
-    Level level;
-    TestHelpers::TestFactory<SC,LO,GO,NO>::createSingleLevelHierarchy(level);
-    level.Set("A", A);
-
-    RCP<AmalgamationFactory> amalgFact = rcp(new AmalgamationFactory());
-    RCP<CoalesceDropFactory> dropFact = rcp(new CoalesceDropFactory());
-    dropFact->SetFactory("UnAmalgamationInfo", amalgFact);
-
-    // Setup aggregation factory (use default factory for graph)
-    RCP<CoupledAggregationFactory> aggFact = rcp(new CoupledAggregationFactory());
-    aggFact->SetFactory("Graph", dropFact);
-    aggFact->SetMinNodesPerAggregate(3);
-    aggFact->SetMaxNeighAlreadySelected(0);
-    aggFact->SetOrdering("natural");
-    aggFact->SetPhase3AggCreation(0.5);
-
-    level.Request("Aggregates", aggFact.get());
-    level.Request("UnAmalgamationInfo", amalgFact.get());
-
-    level.Request(*aggFact);
-    aggFact->Build(level);
-    RCP<Aggregates> aggregates = level.Get<RCP<Aggregates> >("Aggregates",aggFact.get()); // fix me
-    amalgInfo = level.Get<RCP<AmalgamationInfo> >("UnAmalgamationInfo",amalgFact.get()); // fix me
-    level.Release("UnAmalgamationInfo", amalgFact.get());
-    level.Release("Aggregates", aggFact.get());
-    return aggregates;
-  }  // gimmeCoupledAggregates
-
-  TEUCHOS_UNIT_TEST(Aggregates, JustAggregation)
-  {
     out << "version: " << MueLu::Version() << std::endl;
     RCP<Matrix> A = TestHelpers::TestFactory<SC, LO, GO, NO>::Build1DPoisson(15);
     RCP<AmalgamationInfo> amalgInfo;
-    RCP<Aggregates> aggregates = gimmeCoupledAggregates(A, amalgInfo);
+    RCP<Aggregates> aggregates = AggregateGenerator<SC,LO,GO,NO>::gimmeCoupledAggregates(A, amalgInfo);
     TEST_EQUALITY(aggregates != Teuchos::null, true);
     TEST_EQUALITY(aggregates->AggregatesCrossProcessors(),true);
   }
 
   ///////////////////////////////////////////////////////////////////////////
 
-  TEUCHOS_UNIT_TEST(Aggregates, GetNumAggregates)
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL(Aggregates, GetNumAggregates, Scalar, LocalOrdinal, GlobalOrdinal, Node)
   {
+#   include <MueLu_UseShortNames.hpp>
+    MUELU_TESTING_SET_OSTREAM;
+    MUELU_TESTING_LIMIT_SCOPE(Scalar,GlobalOrdinal,Node);
     out << "version: " << MueLu::Version() << std::endl;
 
     RCP<Matrix> A = TestHelpers::TestFactory<SC, LO, GO, NO>::Build1DPoisson(36);
     RCP<const Map> rowmap = A->getRowMap();
     RCP<AmalgamationInfo> amalgInfo;
-    RCP<Aggregates> aggregates = gimmeCoupledAggregates(A, amalgInfo);
+    RCP<Aggregates> aggregates = AggregateGenerator<SC,LO,GO,NO>::gimmeCoupledAggregates(A, amalgInfo);
     GO numAggs = aggregates->GetNumAggregates();
     RCP<const Teuchos::Comm<int> > comm = TestHelpers::Parameters::getDefaultComm();
 
@@ -217,26 +237,32 @@ namespace MueLuTests {
 
   } //GetNumAggregates
 
-  TEUCHOS_UNIT_TEST(Aggregates, JustUncoupledAggregation)
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL(Aggregates, JustUncoupledAggregation, Scalar, LocalOrdinal, GlobalOrdinal, Node)
   {
+#   include <MueLu_UseShortNames.hpp>
+    MUELU_TESTING_SET_OSTREAM;
+    MUELU_TESTING_LIMIT_SCOPE(Scalar,GlobalOrdinal,Node);
     out << "version: " << MueLu::Version() << std::endl;
     RCP<Matrix> A = TestHelpers::TestFactory<SC, LO, GO, NO>::Build1DPoisson(15);
     RCP<AmalgamationInfo> amalgInfo;
-    RCP<Aggregates> aggregates = gimmeUncoupledAggregates(A, amalgInfo);
+    RCP<Aggregates> aggregates = AggregateGenerator<SC,LO,GO,NO>::gimmeUncoupledAggregates(A, amalgInfo);
     TEST_EQUALITY(aggregates != Teuchos::null, true);
     TEST_EQUALITY(aggregates->AggregatesCrossProcessors(),false);
   }
 
   ///////////////////////////////////////////////////////////////////////////
 
-  TEUCHOS_UNIT_TEST(Aggregates, GetNumUncoupledAggregates)
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL(Aggregates, GetNumUncoupledAggregates, Scalar, LocalOrdinal, GlobalOrdinal, Node)
   {
+#   include <MueLu_UseShortNames.hpp>
+    MUELU_TESTING_SET_OSTREAM;
+    MUELU_TESTING_LIMIT_SCOPE(Scalar,GlobalOrdinal,Node);
     out << "version: " << MueLu::Version() << std::endl;
 
     RCP<Matrix> A = TestHelpers::TestFactory<SC, LO, GO, NO>::Build1DPoisson(36);
     RCP<const Map> rowmap = A->getRowMap();
     RCP<AmalgamationInfo> amalgInfo;
-    RCP<Aggregates> aggregates = gimmeUncoupledAggregates(A, amalgInfo);
+    RCP<Aggregates> aggregates = AggregateGenerator<SC,LO,GO,NO>::gimmeUncoupledAggregates(A, amalgInfo);
     GO numAggs = aggregates->GetNumAggregates();
     RCP<const Teuchos::Comm<int> > comm = TestHelpers::Parameters::getDefaultComm();
 
@@ -305,14 +331,17 @@ namespace MueLuTests {
 
   } //GetNumAggregates
 
-  TEUCHOS_UNIT_TEST(Aggregates, UncoupledPhase1)
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL(Aggregates, UncoupledPhase1, Scalar, LocalOrdinal, GlobalOrdinal, Node)
   {
+#   include <MueLu_UseShortNames.hpp>
+    MUELU_TESTING_SET_OSTREAM;
+    MUELU_TESTING_LIMIT_SCOPE(Scalar,GlobalOrdinal,Node);
     out << "version: " << MueLu::Version() << std::endl;
 
     RCP<Matrix> A = TestHelpers::TestFactory<SC, LO, GO, NO>::Build1DPoisson(36);
     RCP<const Map> rowmap = A->getRowMap();
     RCP<AmalgamationInfo> amalgInfo;
-    RCP<Aggregates> aggregates = gimmeUncoupledAggregates(A, amalgInfo,true,false,false,false);
+    RCP<Aggregates> aggregates = AggregateGenerator<SC,LO,GO,NO>::gimmeUncoupledAggregates(A, amalgInfo,true,false,false,false);
     GO numAggs = aggregates->GetNumAggregates();
     RCP<const Teuchos::Comm<int> > comm = TestHelpers::Parameters::getDefaultComm();
 
@@ -381,28 +410,33 @@ namespace MueLuTests {
 
   } //UncoupledPhase1
 
-  TEUCHOS_UNIT_TEST(Aggregates, UncoupledPhase2)
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL(Aggregates, UncoupledPhase2, Scalar, LocalOrdinal, GlobalOrdinal, Node)
   {
+#   include <MueLu_UseShortNames.hpp>
+    MUELU_TESTING_SET_OSTREAM;
+    MUELU_TESTING_LIMIT_SCOPE(Scalar,GlobalOrdinal,Node);
     out << "version: " << MueLu::Version() << std::endl;
 
     RCP<Matrix> A = TestHelpers::TestFactory<SC, LO, GO, NO>::Build1DPoisson(36);
     RCP<const Map> rowmap = A->getRowMap();
     RCP<AmalgamationInfo> amalgInfo;
     bool bSuccess = true;
-    TEUCHOS_TEST_THROW(gimmeUncoupledAggregates(A, amalgInfo,false,true,true,false),
-      MueLu::Exceptions::RuntimeError, out, bSuccess);
+    TEUCHOS_TEST_THROW((AggregateGenerator<SC,LO,GO,NO>::gimmeUncoupledAggregates(A, amalgInfo,false,true,true,false)),
+                        MueLu::Exceptions::RuntimeError, out, bSuccess);
     TEST_EQUALITY(bSuccess, true);
   } //UncoupledPhase2
 
-  TEUCHOS_UNIT_TEST(Aggregates, UncoupledPhase3)
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL(Aggregates, UncoupledPhase3, Scalar, LocalOrdinal, GlobalOrdinal, Node)
   {
+#   include <MueLu_UseShortNames.hpp>
+    MUELU_TESTING_SET_OSTREAM;
+    MUELU_TESTING_LIMIT_SCOPE(Scalar,GlobalOrdinal,Node);
     out << "version: " << MueLu::Version() << std::endl;
 
     RCP<Matrix> A = TestHelpers::TestFactory<SC, LO, GO, NO>::Build1DPoisson(36);
     RCP<const Map> rowmap = A->getRowMap();
     RCP<AmalgamationInfo> amalgInfo;
-    RCP<Aggregates> aggregates = gimmeUncoupledAggregates(A, amalgInfo,false,false,false,true);
-    GO numAggs = aggregates->GetNumAggregates();
+    RCP<Aggregates> aggregates = AggregateGenerator<SC,LO,GO,NO>::gimmeUncoupledAggregates(A, amalgInfo,false,false,false,true); GO numAggs = aggregates->GetNumAggregates();
     RCP<const Teuchos::Comm<int> > comm = TestHelpers::Parameters::getDefaultComm();
 
     TEST_EQUALITY(aggregates->AggregatesCrossProcessors(),false);
@@ -469,4 +503,17 @@ namespace MueLuTests {
     out.setOutputToRootOnly(root);
 
   } //UncoupledPhase3
+
+#define MUELU_ETI_GROUP(Scalar,LO,GO,Node) \
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(Aggregates,JustAggregation,Scalar,LO,GO,Node) \
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(Aggregates,GetNumAggregates,Scalar,LO,GO,Node) \
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(Aggregates,JustUncoupledAggregation,Scalar,LO,GO,Node) \
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(Aggregates,GetNumUncoupledAggregates,Scalar,LO,GO,Node) \
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(Aggregates,UncoupledPhase1,Scalar,LO,GO,Node) \
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(Aggregates,UncoupledPhase2,Scalar,LO,GO,Node) \
+  TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT(Aggregates,UncoupledPhase3,Scalar,LO,GO,Node)
+
+#include <MueLu_ETI_4arg.hpp>
+
+
 } // namespace MueLuTests
