@@ -59,9 +59,6 @@ void Density<EvalT, Traits>::
 postRegistrationSetup(typename Traits::SetupData d,
 		      PHX::FieldManager<Traits>& vm)
 {
-  this->utils.setFieldData(density,vm);
-  this->utils.setFieldData(temp,vm);
-
   cell_data_size = density.size() / density.dimension(0);
 }
 
@@ -74,10 +71,32 @@ void Density<EvalT, Traits>:: operator () (const int i) const
     density(i,ip) =  temp(i,ip) * temp(i,ip);  
 }
 
+//**********************************************************************
+// template<typename EvalT, typename Traits>
+// KOKKOS_INLINE_FUNCTION
+// void Density<EvalT, Traits>:: operator () (const DensityTag, const int i) const
+// {
+//   for (PHX::index_size_type ip=0; ip< static_cast<PHX::index_size_type>(density.dimension_1()); ip++)
+//     density(i,ip) =  temp(i,ip) * temp(i,ip);  
+// }
+
+//**********************************************************************
+// template<typename EvalT, typename Traits>
+// KOKKOS_INLINE_FUNCTION
+// void Density<EvalT, Traits>:: operator () (const DensityTag, typename Kokkos::TeamPolicy<>::member_type & team) const
+// {
+//   for (PHX::index_size_type ip=0; ip< static_cast<PHX::index_size_type>(density.dimension_1()); ip++)
+//     density(0,ip) =  temp(0,ip) * temp(0,ip);
+// }
+
 //*********************************************************************
 template<typename EvalT, typename Traits>
 void Density<EvalT, Traits>::evaluateFields(typename Traits::EvalData d)
 {
+  // typedef Kokkos::TeamPolicy<DensityTag> team_policy ;
+  // team_policy policy(d.num_cells,2);
+  // Kokkos::parallel_for(policy, *this);
+
   Kokkos::parallel_for(d.num_cells, *this);
 }
 
