@@ -600,6 +600,46 @@ dimension(size_type ordinal) const
   return m_dim_size[ordinal];
 }
 
+//**********************************************************************
+template<typename Tag0, typename Tag1, typename Tag2, typename Tag3,
+	 typename Tag4, typename Tag5, typename Tag6, typename Tag7>
+PHX::Device::size_type 
+PHX::MDALayout<Tag0,Tag1,Tag2,Tag3,Tag4,Tag5,Tag6,Tag7>::
+extent(size_type ordinal) const
+{ 
+  if (ordinal > Rank-1 || ordinal < 0) {
+    std::ostringstream os;
+    os << "Requested Ordinal " << ordinal 
+       << " is outside the valid range of 0 - " << Rank - 1
+       << " in DataLayout object:\n"
+       << m_identifier << std::endl;
+    TEUCHOS_TEST_FOR_EXCEPTION(ordinal > Rank-1 || ordinal < 0, 
+                               std::logic_error, os.str());
+  }
+  
+  return m_dim_size[ordinal];
+}
+
+//**********************************************************************
+template<typename Tag0, typename Tag1, typename Tag2, typename Tag3,
+	 typename Tag4, typename Tag5, typename Tag6, typename Tag7>
+int 
+PHX::MDALayout<Tag0,Tag1,Tag2,Tag3,Tag4,Tag5,Tag6,Tag7>::
+extent_int(size_type ordinal) const
+{ 
+  if (ordinal > Rank-1 || ordinal < 0) {
+    std::ostringstream os;
+    os << "Requested Ordinal " << ordinal 
+       << " is outside the valid range of 0 - " << Rank - 1
+       << " in DataLayout object:\n"
+       << m_identifier << std::endl;
+    TEUCHOS_TEST_FOR_EXCEPTION(ordinal > Rank-1 || ordinal < 0, 
+		       std::logic_error, os.str());
+  }
+  
+  return static_cast<int>(m_dim_size[ordinal]);
+}
+
 //**********************************************************************
 template<typename Tag0, typename Tag1, typename Tag2, typename Tag3,
 	 typename Tag4, typename Tag5, typename Tag6, typename Tag7>
