@@ -78,9 +78,8 @@ namespace PHX {
 
     PHX::any getFieldData(const PHX::FieldTag& f);
 
-    //! Set the memory pointer for an unmanaged field
-    void setUnmanagedField(const PHX::FieldTag& f, 
-                           const PHX::any& a);
+    //! Bind the memory pointer for a field in all evaluators
+    void bindField(const PHX::FieldTag& f, const PHX::any& a);
 
     void postRegistrationSetup(typename Traits::SetupData d,
 			       PHX::FieldManager<Traits>& fm);
@@ -90,12 +89,10 @@ namespace PHX {
 #ifdef PHX_ENABLE_KOKKOS_AMT
     /*! \brief Evaluate the fields using hybrid functional (asynchronous multi-tasking) and data parallelism.
 
-      @param threads_per_task The number of threads used for data parallelism within a single task.
       @param work_size The number of work units to parallelize over.
       @param d User defined data.
      */
-    void evaluateFieldsTaskParallel(const int& threads_per_task,
-				    const int& work_size,
+    void evaluateFieldsTaskParallel(const int& work_size,
 				    typename Traits::EvalData d);
 #endif
 
