@@ -85,6 +85,18 @@ namespace PHX {
     void addEvaluatedField(const PHX::MDField<DataT,Tag0,Tag1,Tag2,Tag3,
 			   Tag4,Tag5,Tag6,Tag7>& f);
 
+
+    virtual void addContributedField(const PHX::FieldTag& ft);
+
+    template<typename DataT>
+    void addContributedField(const PHX::Field<DataT>& f);
+
+    template<typename DataT,
+	     typename Tag0, typename Tag1, typename Tag2, typename Tag3,
+	     typename Tag4, typename Tag5, typename Tag6, typename Tag7>
+    void addContributedField(const PHX::MDField<DataT,Tag0,Tag1,Tag2,Tag3,
+                             Tag4,Tag5,Tag6,Tag7>& f);
+
     virtual void addDependentField(const PHX::FieldTag& ft);
 
     // DEPRECATED: use new const version below
@@ -117,16 +129,19 @@ namespace PHX {
     virtual const std::vector< Teuchos::RCP<FieldTag> >& 
     evaluatedFields() const override;
 
+    virtual const std::vector< Teuchos::RCP<FieldTag> >& 
+    contributedFields() const override;
+
     virtual const std::vector< Teuchos::RCP<FieldTag> >& 
     dependentFields() const override;
 
     virtual void evaluateFields(typename Traits::EvalData d) = 0;
 
 #ifdef PHX_ENABLE_KOKKOS_AMT
-    virtual Kokkos::Experimental::Future<void,PHX::Device::execution_space>
-    createTask(Kokkos::Experimental::TaskPolicy<PHX::Device::execution_space>& policy,
-	       const std::size_t& num_adjacencies,
+    virtual Kokkos::Future<void,PHX::Device::execution_space>
+    createTask(Kokkos::TaskPolicy<PHX::Device::execution_space>& policy,
 	       const int& work_size,
+               const std::vector<Kokkos::Future<void,PHX::Device::execution_space>>& dependent_futures,
 	       typename Traits::EvalData d);
 
     virtual unsigned taskSize() const;
@@ -138,19 +153,20 @@ namespace PHX {
 
     virtual const std::string& getName() const override;
 
-    virtual void bindUnmanagedField(const PHX::FieldTag& ft,
-                                    const PHX::any& f) override;
+    virtual void bindField(const PHX::FieldTag& ft, const PHX::any& f) override;
 
   private:
 
     std::vector< Teuchos::RCP<FieldTag> > evaluated_;
 
+    std::vector< Teuchos::RCP<FieldTag> > contributed_;
+
     std::vector< Teuchos::RCP<FieldTag> > required_;
 
     std::string name_;
 
-    //! binds memory for an unmanaged field
-    std::unordered_map<std::string,std::function<void(const PHX::any& f)>> unmanaged_field_binders_;
+    //! functors that bind memory for evaluator fields
+    std::unordered_map<std::string,std::function<void(const PHX::any& f)>> field_binders_;
   };
 
 }
