@@ -53,19 +53,32 @@
 #include "Phalanx_FieldTag_STL_Functors.hpp"
 
 namespace PHX {
+
   //! Functor to bind unmanaged memory to a field.
   template <typename FieldType>
-  class UnmanagedMemoryBinder {
+  class MemoryBinder {
     FieldType* ptr_;
   public:
-    //UnmanagedMemoryBinder() : ptr_(nullptr) {}
-    UnmanagedMemoryBinder(FieldType* f) : ptr_(f) {}
-    UnmanagedMemoryBinder(const UnmanagedMemoryBinder& ) = default;
-    UnmanagedMemoryBinder& operator=(const UnmanagedMemoryBinder& ) = default;
-    UnmanagedMemoryBinder(UnmanagedMemoryBinder&& ) = default;
-    UnmanagedMemoryBinder& operator=(UnmanagedMemoryBinder&& ) = default;
+    //MemoryBinder() : ptr_(nullptr) {}
+    MemoryBinder(FieldType* f) : ptr_(f) {}
+    MemoryBinder(const MemoryBinder& ) = default;
+    MemoryBinder& operator=(const MemoryBinder& ) = default;
+    MemoryBinder(MemoryBinder&& ) = default;
+    MemoryBinder& operator=(MemoryBinder&& ) = default;
     void operator()(const PHX::any& f) { ptr_->setFieldData(f); }
   };
+
+  //! Dummy functor to satisfy binding to dummy field tags.
+  class DummyMemoryBinder {
+  public:
+    DummyMemoryBinder() {}
+    DummyMemoryBinder(const DummyMemoryBinder& ) = default;
+    DummyMemoryBinder& operator=(const DummyMemoryBinder& ) = default;
+    DummyMemoryBinder(DummyMemoryBinder&& ) = default;
+    DummyMemoryBinder& operator=(DummyMemoryBinder&& ) = default;
+    void operator()(const PHX::any& f) { /* DO NOTHING! */ }
+  };
+
 } // namespace PHX
 
 //**********************************************************************
@@ -97,6 +110,9 @@ addEvaluatedField(const PHX::FieldTag& ft)
   
   if ( test == evaluated_.end() )
     evaluated_.push_back(ft.clone());
+
+  // This may be overwritten if a MDField object is registered
+  (this->field_binders_)[ft.identifier()] = PHX::DummyMemoryBinder();
 }
 
 //**********************************************************************
@@ -108,8 +124,8 @@ addEvaluatedField(const PHX::Field<DataT>& f)
   this->addEvaluatedField(f.fieldTag());
 
   using NCF = PHX::MDField<DataT>;
-  (this->unmanaged_field_binders_)[f.fieldTag().identifier()] = 
-    PHX::UnmanagedMemoryBinder<NCF>(const_cast<NCF*>(&f));
+  (this->field_binders_)[f.fieldTag().identifier()] = 
+    PHX::MemoryBinder<NCF>(const_cast<NCF*>(&f));
 }
 
 //**********************************************************************
@@ -124,8 +140,53 @@ addEvaluatedField(const PHX::MDField<DataT,Tag0,Tag1,Tag2,Tag3,
   this->addEvaluatedField(f.fieldTag());
 
   using NCF = PHX::MDField<DataT,Tag0,Tag1,Tag2,Tag3,Tag4,Tag5,Tag6,Tag7>;
-  (this->unmanaged_field_binders_)[f.fieldTag().identifier()] = 
-    PHX::UnmanagedMemoryBinder<NCF>(const_cast<NCF*>(&f));
+  (this->field_binders_)[f.fieldTag().identifier()] = 
+    PHX::MemoryBinder<NCF>(const_cast<NCF*>(&f));
+}
+
+//**********************************************************************
+template<typename Traits>
+void PHX::EvaluatorWithBaseImpl<Traits>::
+addContributedField(const PHX::FieldTag& ft)
+{ 
+  PHX::FTPredRef pred(ft);
+  std::vector< Teuchos::RCP<FieldTag> >::iterator test = 
+    std::find_if(contributed_.begin(), contributed_.end(), pred);
+  
+  if ( test == contributed_.end() )
+    contributed_.push_back(ft.clone());
+
+  // This may be overwritten if a MDField object is registered
+  (this->field_binders_)[ft.identifier()] = PHX::DummyMemoryBinder();
+}
+
+//**********************************************************************
+template<typename Traits>
+template<typename DataT>
+void PHX::EvaluatorWithBaseImpl<Traits>::
+addContributedField(const PHX::Field<DataT>& f)
+{ 
+  this->addContributedField(f.fieldTag());
+
+  using NCF = PHX::MDField<DataT>;
+  (this->field_binders_)[f.fieldTag().identifier()] = 
+    PHX::MemoryBinder<NCF>(const_cast<NCF*>(&f));
+}
+
+//**********************************************************************
+template<typename Traits>
+template<typename DataT,
+	 typename Tag0, typename Tag1, typename Tag2, typename Tag3,
+	 typename Tag4, typename Tag5, typename Tag6, typename Tag7>
+void PHX::EvaluatorWithBaseImpl<Traits>::
+addContributedField(const PHX::MDField<DataT,Tag0,Tag1,Tag2,Tag3,
+                    Tag4,Tag5,Tag6,Tag7>& f)
+{ 
+  this->addContributedField(f.fieldTag());
+
+  using NCF = PHX::MDField<DataT,Tag0,Tag1,Tag2,Tag3,Tag4,Tag5,Tag6,Tag7>;
+  (this->field_binders_)[f.fieldTag().identifier()] = 
+    PHX::MemoryBinder<NCF>(const_cast<NCF*>(&f));
 }
 
 //**********************************************************************
@@ -139,6 +200,9 @@ addDependentField(const PHX::FieldTag& ft)
   
   if ( test == required_.end() )
     required_.push_back(ft.clone());
+
+  // This may be overwritten if a MDField object is registered
+  (this->field_binders_)[ft.identifier()] = PHX::DummyMemoryBinder();
 }
 
 //**********************************************************************
@@ -151,8 +215,8 @@ addDependentField(const PHX::Field<DataT>& f)
   this->addDependentField(f.fieldTag());
 
   using NCF = PHX::MDField<DataT>;
-  (this->unmanaged_field_binders_)[f.fieldTag().identifier()] = 
-    PHX::UnmanagedMemoryBinder<NCF>(const_cast<NCF*>(&f));
+  (this->field_binders_)[f.fieldTag().identifier()] = 
+    PHX::MemoryBinder<NCF>(const_cast<NCF*>(&f));
 }
 
 //**********************************************************************
@@ -164,8 +228,8 @@ addDependentField(const PHX::Field<const DataT>& f)
   this->addDependentField(f.fieldTag());
 
   using NCF = PHX::MDField<DataT>;
-  (this->unmanaged_field_binders_)[f.fieldTag().identifier()] = 
-    PHX::UnmanagedMemoryBinder<NCF>(const_cast<NCF*>(&f));
+  (this->field_binders_)[f.fieldTag().identifier()] = 
+    PHX::MemoryBinder<NCF>(const_cast<NCF*>(&f));
 }
 
 //**********************************************************************
@@ -181,8 +245,8 @@ addDependentField(const PHX::MDField<DataT,Tag0,Tag1,Tag2,Tag3,
   this->addDependentField(f.fieldTag());
 
   using NCF = PHX::MDField<typename std::remove_const<DataT>::type,Tag0,Tag1,Tag2,Tag3,Tag4,Tag5,Tag6,Tag7>;
-  (this->unmanaged_field_binders_)[f.fieldTag().identifier()] = 
-    PHX::UnmanagedMemoryBinder<NCF>(const_cast<NCF*>(&f));
+  (this->field_binders_)[f.fieldTag().identifier()] = 
+    PHX::MemoryBinder<NCF>(const_cast<NCF*>(&f));
 }
 
 //**********************************************************************
@@ -197,8 +261,8 @@ addDependentField(const PHX::MDField<const DataT,Tag0,Tag1,Tag2,Tag3,
   this->addDependentField(f.fieldTag());
 
   using NCF = PHX::MDField<const DataT,Tag0,Tag1,Tag2,Tag3,Tag4,Tag5,Tag6,Tag7>;
-  (this->unmanaged_field_binders_)[f.fieldTag().identifier()] = 
-    PHX::UnmanagedMemoryBinder<NCF>(const_cast<NCF*>(&f));
+  (this->field_binders_)[f.fieldTag().identifier()] = 
+    PHX::MemoryBinder<NCF>(const_cast<NCF*>(&f));
 }
 
 //**********************************************************************
@@ -213,6 +277,12 @@ const std::vector< Teuchos::RCP<PHX::FieldTag> >&
 PHX::EvaluatorWithBaseImpl<Traits>::evaluatedFields() const
 { return evaluated_; }
 
+//**********************************************************************
+template<typename Traits>
+const std::vector< Teuchos::RCP<PHX::FieldTag> >&
+PHX::EvaluatorWithBaseImpl<Traits>::contributedFields() const
+{ return contributed_; }
+
 //**********************************************************************
 template<typename Traits>
 const std::vector< Teuchos::RCP<PHX::FieldTag> >&
@@ -222,11 +292,11 @@ PHX::EvaluatorWithBaseImpl<Traits>::dependentFields() const
 //**********************************************************************
 #ifdef PHX_ENABLE_KOKKOS_AMT
 template<typename Traits>
-Kokkos::Experimental::Future<void,PHX::Device::execution_space>
+Kokkos::Future<void,PHX::Device::execution_space>
 PHX::EvaluatorWithBaseImpl<Traits>::
-createTask(Kokkos::Experimental::TaskPolicy<PHX::Device::execution_space>& ,
-	   const std::size_t& ,
+createTask(Kokkos::TaskPolicy<PHX::Device::execution_space>& ,
 	   const int& ,
+           const std::vector<Kokkos::Future<void,PHX::Device::execution_space>>& dependent_futures,
 	   typename Traits::EvalData )
 {
   TEUCHOS_TEST_FOR_EXCEPTION(true,std::runtime_error,
@@ -266,9 +336,9 @@ getName() const
 //**********************************************************************
 template<typename Traits>
 void PHX::EvaluatorWithBaseImpl<Traits>::
-bindUnmanagedField(const PHX::FieldTag& ft, const PHX::any& f)
+bindField(const PHX::FieldTag& ft, const PHX::any& f)
 {
-  unmanaged_field_binders_[ft.identifier()](f);
+  field_binders_[ft.identifier()](f);
 }
 
 //**********************************************************************
