@@ -47,6 +47,7 @@
 #include "Phalanx_DimTag.hpp"
 #include "Phalanx_KokkosUtilities.hpp"
 #include "Phalanx_KokkosViewFactory.hpp"
+#include "Phalanx_MDField_UnmanagedAllocator.hpp"
 #include "Phalanx_KokkosDeviceTypes.hpp"
 
 #include "Teuchos_RCP.hpp"
@@ -216,63 +217,48 @@ TEUCHOS_UNIT_TEST(mdfield, CompileTimeChecked)
     
     // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     // rank()
-    cout << "Testing rank() method...";
-    TEUCHOS_TEST_FOR_EXCEPTION(a.rank() != 2, std::logic_error,
-			       "Rank in a is wrong!");
-    TEUCHOS_TEST_FOR_EXCEPTION(b.rank() != 3, std::logic_error,
-			       "Rank in b is wrong!");
-    cout << "passed!" << endl;
+    TEST_EQUALITY(a.rank(),2);
+    TEST_EQUALITY(b.rank(),3);
+    TEST_EQUALITY(c.rank(),2);
+    TEST_EQUALITY(d.rank(),3);
+    TEST_EQUALITY(e.rank(),2);
+    TEST_EQUALITY(f.rank(),3);
+
+    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+    // extent()
+    TEST_EQUALITY(b.extent(0), num_cells);
+    TEST_EQUALITY(b.extent(1), 4);
+    TEST_EQUALITY(b.extent(2), 3);
+
+    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+    // extent_int()
+    TEST_EQUALITY(b.extent_int(0), static_cast<int>(num_cells));
+    TEST_EQUALITY(b.extent_int(1), static_cast<int>(4));
+    TEST_EQUALITY(b.extent_int(2), static_cast<int>(3));
     
     // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     // dimension()
-    cout << "Testing dimension() method...";
-    TEUCHOS_TEST_FOR_EXCEPTION(b.dimension(0) != num_cells, std::logic_error,
-			       "Cell dimesion is wrong!");
-    TEUCHOS_TEST_FOR_EXCEPTION(b.dimension(1) != 4, std::logic_error,
-			       "Quadrature dimesion is wrong!");
-    TEUCHOS_TEST_FOR_EXCEPTION(b.dimension(2) != 3, std::logic_error,
-			       "Dim dimesion is wrong!");
-    cout << "passed!" << endl;
+    TEST_EQUALITY(b.dimension(0), num_cells);
+    TEST_EQUALITY(b.dimension(1), 4);
+    TEST_EQUALITY(b.dimension(2), 3);
     
     // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     // dimensions()
-    cout << "Testing dimensions() method...";
     std::vector<size_type> dims;
     b.dimensions(dims);
-    TEUCHOS_TEST_FOR_EXCEPTION(dims.size() != 3, std::logic_error,
-			       "Number of dimesions is wrong!");
-    TEUCHOS_TEST_FOR_EXCEPTION(dims[0] != 100, std::logic_error,
-			       "Number of dimesions is wrong!");
-    TEUCHOS_TEST_FOR_EXCEPTION(dims[1] != 4, std::logic_error,
-			       "Number of dimesions is wrong!");
-    TEUCHOS_TEST_FOR_EXCEPTION(dims[2] != 3, std::logic_error,
-			       "Number of dimesions is wrong!");
-    cout << "passed!" << endl;
+    TEST_EQUALITY(dims.size(), 3);
+    TEST_EQUALITY(dims[0], 100);
+    TEST_EQUALITY(dims[1], 4);
+    TEST_EQUALITY(dims[2], 3);
     
     // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     // size()
-    cout << "Testing size() method...";
-
-    TEUCHOS_TEST_FOR_EXCEPTION(a.size() != node_scalar->size(), 
-			       std::logic_error, 
-			       "Size of array a is not equal to requested size.");
-    TEUCHOS_TEST_FOR_EXCEPTION(b.size() != quad_vector->size(), 
-			       std::logic_error, 
-			       "Size of array b is not equal to requested size.");
-    TEUCHOS_TEST_FOR_EXCEPTION(c.size() != node_scalar->size(), 
-			       std::logic_error, 
-			       "Size of array c is not equal to requested size.");
-    TEUCHOS_TEST_FOR_EXCEPTION(d.size() != quad_vector->size() , 
-			       std::logic_error, 
-			       "Size of array d is not equal to requested size.");
-    TEUCHOS_TEST_FOR_EXCEPTION(e.size() != node_scalar->size(),
-			       std::logic_error,
-			       "Size of array e is not equal to requested size.");
-    TEUCHOS_TEST_FOR_EXCEPTION(f.size() != quad_vector->size() ,
-			       std::logic_error,
-			       "Size of array f is not equal to requested size.");
-    cout << "passed!" << endl;
-    
+    TEST_EQUALITY(a.size(), node_scalar->size());
+    TEST_EQUALITY(b.size(), quad_vector->size());
+    TEST_EQUALITY(c.size(), node_scalar->size());
+    TEST_EQUALITY(d.size(), quad_vector->size());
+    TEST_EQUALITY(e.size(), node_scalar->size());
+    TEST_EQUALITY(f.size(), quad_vector->size());    
     
     // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     // operator()
@@ -285,22 +271,33 @@ TEUCHOS_UNIT_TEST(mdfield, CompileTimeChecked)
     RCP<DataLayout> d5 = rcp(new MDALayout<Cell,Dim,Dim,Dim,Dim>(num_cells,1,2,3,4));
     RCP<DataLayout> d6 = rcp(new MDALayout<Cell,Dim,Dim,Dim,Dim,Dim>(num_cells,1,2,3,4,5));
     RCP<DataLayout> d7 = rcp(new MDALayout<Cell,Dim,Dim,Dim,Dim,Dim,Dim>(num_cells,1,2,3,4,5,6));
+
+    // Use Unmanaged allocator
+    MDField<double,Cell> f1 = PHX::allocateUnmanagedMDField<double,Cell>("Test1",d1);
+    MDField<double,Cell,Dim> f2 = PHX::allocateUnmanagedMDField<double,Cell,Dim>("Test2",d2);
+    MDField<double,Cell,Dim,Dim> f3 = PHX::allocateUnmanagedMDField<double,Cell,Dim,Dim>("Test3",d3);
+    MDField<double,Cell,Dim,Dim,Dim> f4 = PHX::allocateUnmanagedMDField<double,Cell,Dim,Dim,Dim>("Test4",d4);
+    MDField<double,Cell,Dim,Dim,Dim,Dim> f5 = PHX::allocateUnmanagedMDField<double,Cell,Dim,Dim,Dim,Dim>("Test5",d5);
+    MDField<double,Cell,Dim,Dim,Dim,Dim,Dim> f6 = PHX::allocateUnmanagedMDField<double,Cell,Dim,Dim,Dim,Dim,Dim>("Test6",d6);
+    MDField<double,Cell,Dim,Dim,Dim,Dim,Dim,Dim> f7 = PHX::allocateUnmanagedMDField<double,Cell,Dim,Dim,Dim,Dim,Dim,Dim>("Test7",d7);
+
+    // Pre-Unmanaged allocator
+
+    // MDField<double,Cell> f1("Test1",d1);
+    // MDField<double,Cell,Dim> f2("Test2",d2);
+    // MDField<double,Cell,Dim,Dim> f3("Test3",d3);
+    // MDField<double,Cell,Dim,Dim,Dim> f4("Test4",d4);
+    // MDField<double,Cell,Dim,Dim,Dim,Dim> f5("Test5",d5);
+    // MDField<double,Cell,Dim,Dim,Dim,Dim,Dim> f6("Test6",d6);
+    // MDField<double,Cell,Dim,Dim,Dim,Dim,Dim,Dim> f7("Test7",d7);
     
-    MDField<double,Cell> f1("Test1",d1);
-    MDField<double,Cell,Dim> f2("Test2",d2);
-    MDField<double,Cell,Dim,Dim> f3("Test3",d3);
-    MDField<double,Cell,Dim,Dim,Dim> f4("Test4",d4);
-    MDField<double,Cell,Dim,Dim,Dim,Dim> f5("Test5",d5);
-    MDField<double,Cell,Dim,Dim,Dim,Dim,Dim> f6("Test6",d6);
-    MDField<double,Cell,Dim,Dim,Dim,Dim,Dim,Dim> f7("Test7",d7);
-    
-    f1.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f1.fieldTag()));
-    f2.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f2.fieldTag()));
-    f3.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f3.fieldTag()));
-    f4.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f4.fieldTag()));
-    f5.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f5.fieldTag()));
-    f6.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f6.fieldTag()));
-    f7.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f7.fieldTag()));
+    // f1.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f1.fieldTag()));
+    // f2.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f2.fieldTag()));
+    // f3.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f3.fieldTag()));
+    // f4.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f4.fieldTag()));
+    // f5.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f5.fieldTag()));
+    // f6.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f6.fieldTag()));
+    // f7.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(f7.fieldTag()));
 
     // Access last entry in contiguous array
     f1(99) = 1.0;
@@ -367,8 +364,33 @@ TEUCHOS_UNIT_TEST(mdfield, CompileTimeChecked)
       c_f7.setFieldData(PHX::KokkosViewFactory<double,PHX::Device>::buildView(c_f7.fieldTag()));
     }
 
-
-
+    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+    // Kokkos static View accessors
+    {
+      // non-const view
+      auto kva = a.get_static_view(); 
+      kva(0,0) = 1.0;
+      auto kvc = c.get_static_view(); 
+      kvc(0,0) = MyTraits::FadType(1.0);
+      // const view (view const, not const data)
+      const auto const_kva = a.get_static_view(); 
+      const_kva(0,0) = 1.0;
+      const auto const_kvc = c.get_static_view(); 
+      const_kvc(0,0) = MyTraits::FadType(1.0);
+    }
+    // Kokkos DynRankView accessors
+    {
+      // non-const view
+      auto kva = a.get_view(); 
+      kva(0,0) = 1.0;
+      auto kvc = c.get_view(); 
+      kvc(0,0) = MyTraits::FadType(1.0);
+      // const view (view const, not const data)
+      const auto const_kva = a.get_view(); 
+      const_kva(0,0) = 1.0;
+      const auto const_kvc = c.get_view(); 
+      const_kvc(0,0) = MyTraits::FadType(1.0);
+    }
 
     // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     // check for debug build array rank enforcement
@@ -376,12 +398,9 @@ TEUCHOS_UNIT_TEST(mdfield, CompileTimeChecked)
 
     // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     // ostream
-    cout << "Testing operator<<()...";
     ostringstream output;
     output << a;
-    TEUCHOS_TEST_FOR_EXCEPTION(output.str() != "MDField<Cell,Node>(100,4): Tag: density, double, DataLayout: <Cell,Node>(100,4)", std::logic_error, "String match failed!"); 
-    cout << "passed!" << endl;
-    cout << output.str() << endl;
+    TEST_EQUALITY(output.str(), "MDField<Cell,Node>(100,4): Tag: density, double, DataLayout: <Cell,Node>(100,4)"); 
   }
 
   PHX::FinalizeKokkosDevice();  
