@@ -68,7 +68,7 @@ public:
     int nB = this->numBatches();
     std::vector<Real> receiveBuffer(nB);
     comm_->GatherAll(input,&receiveBuffer[0],1);
-    output[0] = 0;
+    output[0] = r.initialValue();
     for (int i = 0; i < nB; i++) {
       r.reduce(receiveBuffer[i],output[0]);
     }
