@@ -49,8 +49,8 @@
 #include "ROL_Types.hpp"
 #include "ROL_Vector.hpp"
 #include "ROL_BoundConstraint.hpp"
-#include "ROL_ParametrizedEqualityConstraint_SimOpt.hpp"
-#include "ROL_ParametrizedObjective_SimOpt.hpp"
+#include "ROL_EqualityConstraint_SimOpt.hpp"
+#include "ROL_Objective_SimOpt.hpp"
 #include "ROL_TeuchosBatchManager.hpp"
 
 #include "Teuchos_LAPACK.hpp"
@@ -866,7 +866,7 @@ public:
 };
 
 template<class Real>
-class EqualityConstraint_BurgersControl : public ROL::ParametrizedEqualityConstraint_SimOpt<Real> {
+class EqualityConstraint_BurgersControl : public ROL::EqualityConstraint_SimOpt<Real> {
 private:
 
   typedef H1VectorPrimal<Real> PrimalStateVector;
@@ -897,7 +897,7 @@ public:
       (Teuchos::dyn_cast<PrimalControlVector>(const_cast<ROL::Vector<Real> &>(z))).getVector();
 
     const std::vector<Real> param
-      = ROL::ParametrizedEqualityConstraint_SimOpt<Real>::getParameter();
+      = ROL::EqualityConstraint_SimOpt<Real>::getParameter();
     fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 
     fem_->compute_residual(*cp,*up,*zp);
@@ -915,7 +915,7 @@ public:
       (Teuchos::dyn_cast<PrimalControlVector>(const_cast<ROL::Vector<Real> &>(z))).getVector();
 
     const std::vector<Real> param
-      = ROL::ParametrizedEqualityConstraint_SimOpt<Real>::getParameter();
+      = ROL::EqualityConstraint_SimOpt<Real>::getParameter();
     fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 
     fem_->apply_pde_jacobian(*jvp,*vp,*up,*zp);
@@ -933,7 +933,7 @@ public:
       (Teuchos::dyn_cast<PrimalControlVector>(const_cast<ROL::Vector<Real> &>(z))).getVector();
 
     const std::vector<Real> param
-      = ROL::ParametrizedEqualityConstraint_SimOpt<Real>::getParameter();
+      = ROL::EqualityConstraint_SimOpt<Real>::getParameter();
     fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 
     fem_->apply_control_jacobian(*jvp,*vp,*up,*zp);
@@ -951,7 +951,7 @@ public:
       (Teuchos::dyn_cast<PrimalControlVector>(const_cast<ROL::Vector<Real> &>(z))).getVector();
 
     const std::vector<Real> param
-      = ROL::ParametrizedEqualityConstraint_SimOpt<Real>::getParameter();
+      = ROL::EqualityConstraint_SimOpt<Real>::getParameter();
     fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 
     fem_->apply_inverse_pde_jacobian(*ijvp,*vp,*up,*zp);
@@ -969,7 +969,7 @@ public:
       (Teuchos::dyn_cast<PrimalControlVector>(const_cast<ROL::Vector<Real> &>(z))).getVector();
 
     const std::vector<Real> param
-      = ROL::ParametrizedEqualityConstraint_SimOpt<Real>::getParameter();
+      = ROL::EqualityConstraint_SimOpt<Real>::getParameter();
     fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 
     fem_->apply_adjoint_pde_jacobian(*jvp,*vp,*up,*zp);
@@ -987,7 +987,7 @@ public:
       (Teuchos::dyn_cast<PrimalControlVector>(const_cast<ROL::Vector<Real> &>(z))).getVector();
 
     const std::vector<Real> param
-      = ROL::ParametrizedEqualityConstraint_SimOpt<Real>::getParameter();
+      = ROL::EqualityConstraint_SimOpt<Real>::getParameter();
     fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 
     fem_->apply_adjoint_control_jacobian(*jvp,*vp,*up,*zp);
@@ -1005,7 +1005,7 @@ public:
       (Teuchos::dyn_cast<PrimalControlVector>(const_cast<ROL::Vector<Real> &>(z))).getVector();
 
     const std::vector<Real> param
-      = ROL::ParametrizedEqualityConstraint_SimOpt<Real>::getParameter();
+      = ROL::EqualityConstraint_SimOpt<Real>::getParameter();
     fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 
     fem_->apply_inverse_adjoint_pde_jacobian(*iajvp,*vp,*up,*zp);
@@ -1026,7 +1026,7 @@ public:
         (Teuchos::dyn_cast<PrimalControlVector>(const_cast<ROL::Vector<Real> &>(z))).getVector();
 
       const std::vector<Real> param
-        = ROL::ParametrizedEqualityConstraint_SimOpt<Real>::getParameter();
+        = ROL::EqualityConstraint_SimOpt<Real>::getParameter();
       fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 
       fem_->apply_adjoint_pde_hessian(*ahwvp,*wp,*vp,*up,*zp);
@@ -1051,7 +1051,7 @@ public:
         (Teuchos::dyn_cast<PrimalControlVector>(const_cast<ROL::Vector<Real> &>(z))).getVector();
 
       const std::vector<Real> param
-        = ROL::ParametrizedEqualityConstraint_SimOpt<Real>::getParameter();
+        = ROL::EqualityConstraint_SimOpt<Real>::getParameter();
       fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 
       fem_->apply_adjoint_control_pde_hessian(*ahwvp,*wp,*vp,*up,*zp);
@@ -1075,7 +1075,7 @@ public:
         (Teuchos::dyn_cast<PrimalControlVector>(const_cast<ROL::Vector<Real> &>(z))).getVector();
 
       const std::vector<Real> param
-        = ROL::ParametrizedEqualityConstraint_SimOpt<Real>::getParameter();
+        = ROL::EqualityConstraint_SimOpt<Real>::getParameter();
       fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 
       fem_->apply_adjoint_pde_control_hessian(*ahwvp,*wp,*vp,*up,*zp);
@@ -1099,7 +1099,7 @@ public:
         (Teuchos::dyn_cast<PrimalControlVector>(const_cast<ROL::Vector<Real> &>(z))).getVector();
 
       const std::vector<Real> param
-        = ROL::ParametrizedEqualityConstraint_SimOpt<Real>::getParameter();
+        = ROL::EqualityConstraint_SimOpt<Real>::getParameter();
       fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 
       fem_->apply_adjoint_control_hessian(*ahwvp,*wp,*vp,*up,*zp);
@@ -1111,7 +1111,7 @@ public:
 };
 
 template<class Real>
-class Objective_BurgersControl : public ROL::ParametrizedObjective_SimOpt<Real> {
+class Objective_BurgersControl : public ROL::Objective_SimOpt<Real> {
 private:
 
   typedef H1VectorPrimal<Real> PrimalStateVector;
@@ -1135,14 +1135,14 @@ public:
     }
   }
 
-  using ROL::ParametrizedObjective_SimOpt<Real>::value;
+  using ROL::Objective_SimOpt<Real>::value;
 
   Real value( const ROL::Vector<Real> &u, const ROL::Vector<Real> &z, Real &tol ) {
     Teuchos::RCP<const std::vector<Real> > up =
       (Teuchos::dyn_cast<PrimalStateVector>(const_cast<ROL::Vector<Real> &>(u))).getVector();
 
 //    const std::vector<Real> param
-//      = ROL::ParametrizedObjective_SimOpt<Real>::getParameter();
+//      = ROL::Objective_SimOpt<Real>::getParameter();
 //    fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 //    Real nu = fem_->get_viscosity();
 //
@@ -1164,7 +1164,7 @@ public:
       (Teuchos::dyn_cast<PrimalStateVector>(const_cast<ROL::Vector<Real> &>(u))).getVector();
 
 //    const std::vector<Real> param
-//      = ROL::ParametrizedObjective_SimOpt<Real>::getParameter();
+//      = ROL::Objective_SimOpt<Real>::getParameter();
 //    fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 //    Real nu = fem_->get_viscosity();
 //
@@ -1194,7 +1194,7 @@ public:
 //      (Teuchos::dyn_cast<PrimalStateVector>(const_cast<ROL::Vector<Real> &>(v))).getVector();
 //
 //    const std::vector<Real> param
-//      = ROL::ParametrizedObjective_SimOpt<Real>::getParameter();
+//      = ROL::Objective_SimOpt<Real>::getParameter();
 //    fem_->set_problem_data(param[0],param[1],param[2],param[3]);
 //    Real nu = fem_->get_viscosity();
 //
