@@ -92,13 +92,14 @@
 namespace ROL {
 
 template <class Real>
-class EqualityConstraint_SimOpt : public EqualityConstraint<Real> {
+class EqualityConstraint_SimOpt : public virtual EqualityConstraint<Real> {
 private:
   // Additional vector storage for solve
   Teuchos::RCP<Vector<Real> > unew_;
   Teuchos::RCP<Vector<Real> > jv_;
 
   // Default parameters for solve (backtracking Newton)
+  const Real DEFAULT_atol_;
   const Real DEFAULT_rtol_;
   const Real DEFAULT_stol_;
   const Real DEFAULT_factor_;
@@ -107,6 +108,7 @@ private:
   const bool DEFAULT_print_;
 
   // User-set parameters for solve (backtracking Newton)
+  Real atol_;
   Real rtol_;
   Real stol_;
   Real factor_;
@@ -121,13 +123,14 @@ public:
   EqualityConstraint_SimOpt()
     : EqualityConstraint<Real>(),
       unew_(Teuchos::null), jv_(Teuchos::null),
-      DEFAULT_rtol_(1.e-4*std::sqrt(ROL_EPSILON<Real>())),
+      DEFAULT_atol_(1.e-4*std::sqrt(ROL_EPSILON<Real>())),
+      DEFAULT_rtol_(1.e0),
       DEFAULT_stol_(std::sqrt(ROL_EPSILON<Real>())),
       DEFAULT_factor_(0.5),
       DEFAULT_decr_(1.e-4),
       DEFAULT_maxit_(500),
       DEFAULT_print_(false),
-      rtol_(DEFAULT_rtol_), stol_(DEFAULT_stol_), factor_(DEFAULT_factor_),
+      atol_(DEFAULT_atol_), rtol_(DEFAULT_rtol_), stol_(DEFAULT_stol_), factor_(DEFAULT_factor_),
       decr_(DEFAULT_decr_), maxit_(DEFAULT_maxit_), print_(DEFAULT_print_),
       firstSolve_(true) {}
 
@@ -194,9 +197,10 @@ public:
       jv_   = u.clone();
       firstSolve_ = false;
     }
-    update(u,z);
+    update(u,z,true);
     value(c,u,z,tol);
     Real cnorm = c.norm(), alpha = 1.0, tmp = 0.0;
+    const Real ctol = std::min(atol_, rtol_*cnorm);
     int cnt = 0;
     if ( print_ ) {
       std::cout << "\n     Default EqualityConstraint_SimOpt::solve\n";
@@ -206,7 +210,7 @@ public:
       std::cout << std::setw(15) << std::left << "alpha";
       std::cout << "\n";
     }
-    while ( cnorm > rtol_ && cnt < maxit_) {
+    for (cnt = 0; cnt < maxit_; ++cnt) {
       // Compute Newton step
       applyInverseJacobian_1(*jv_,c,u,z,tol);
       unew_->set(u);
@@ -236,8 +240,11 @@ public:
       // Update iterate
       cnorm = tmp;
       u.set(*unew_);
+      if (cnorm < ctol) {
+        break;
+      }
+      update(u,z,true);
       alpha = 1.0;
-      cnt++;
     }
   }
 
@@ -261,7 +268,8 @@ public:
   */
   virtual void setSolveParameters(Teuchos::ParameterList &parlist) {
     Teuchos::ParameterList & list = parlist.sublist("SimOpt").sublist("Solve");
-    rtol_   = list.get("Residual Tolerance",            DEFAULT_rtol_);
+    atol_   = list.get("Absolute Residual Tolerance",   DEFAULT_atol_);
+    rtol_   = list.get("Relative Residual Tolerance",   DEFAULT_rtol_);
     maxit_  = list.get("Iteration Limit",               DEFAULT_maxit_);
     decr_   = list.get("Sufficient Decrease Tolerance", DEFAULT_decr_);
     stol_   = list.get("Step Tolerance",                DEFAULT_stol_);
@@ -543,19 +551,20 @@ public:
       "The method applyInverseAdjointJacobian_1 is used but not implemented!\n");
   };
 
-  /** \brief Apply the adjoint of the partial constraint Hessian at \f$(u,z)\f$,
-             \f$c_{uu}(u,z)^* \in L(L(\mathcal{C}^*, \mathcal{U}^*), \mathcal{U}^*)\f$,
-             to vector \f$v\f$ in direction \f$w\f$.
+  /** \brief Apply the simulation-space derivative of the adjoint of the constraint
+             simulation-space Jacobian at \f$(u,z)\f$ to the vector \f$w\f$ in the
+             direction \f$v\f$, according to \f$v\mapsto c_{uu}(u,z)(v,\cdot)^*w\f$.
 
-             @param[out]      ahwv is the result of applying the adjoint of the constraint Hessian to @b v at @b \f$(u,z)\f$ in direction @b w; a dual simulation-space vector
+             @param[out]      ahwv is the result of applying the simulation-space derivative of the adjoint of the constraint simulation-space Jacobian at @b \f$(u,z)\f$ to the vector @b \f$w\f$ in direction @b \f$w\f$; a dual simulation-space vector
              @param[in]       w    is the direction vector; a dual constraint-space vector
-             @param[in]       v    is a dual simulation-space vector
+             @param[in]       v    is a simulation-space vector
              @param[in]       u    is the constraint argument; a simulation-space vector
              @param[in]       z    is the constraint argument; an optimization-space vector
              @param[in,out]   tol  is a tolerance for inexact evaluations; currently unused
 
-             On return, \f$ \mathsf{ahwv} = c_{uu}(u,z)^*(w,v) \f$, where
-             \f$w \in \mathcal{C}^*\f$, \f$v \in \mathcal{U}^*\f$, and \f$\mathsf{ahwv} \in \mathcal{U}^*\f$.
+             On return, \f$\mathsf{ahwv} = c_{uu}(u,z)(v,\cdot)^*w\f$, where
+             \f$w \in \mathcal{C}^*\f$, \f$v \in \mathcal{U}\f$, and
+             \f$\mathsf{ahwv} \in \mathcal{U}^*\f$.
 
              ---
   */
@@ -587,19 +596,20 @@ public:
   }
 
 
-  /** \brief Apply the adjoint of the partial constraint Hessian at \f$(u,z)\f$,
-             \f$c_{uz}(u,z)^* \in L(L(\mathcal{C}^*, \mathcal{U}^*), \mathcal{Z}^*)\f$,
-             to vector \f$v\f$ in direction \f$w\f$.
+  /** \brief Apply the optimization-space derivative of the adjoint of the constraint
+             simulation-space Jacobian at \f$(u,z)\f$ to the vector \f$w\f$ in the
+             direction \f$v\f$, according to \f$v\mapsto c_{uz}(u,z)(v,\cdot)^*w\f$.
 
-             @param[out]      ahwv is the result of applying the adjoint of the constraint Hessian to @b v at @b \f$(u,z)\f$ in direction @b w; a dual optimization-space vector
+             @param[out]      ahwv is the result of applying the optimization-space derivative of the adjoint of the constraint simulation-space Jacobian at @b \f$(u,z)\f$ to the vector @b \f$w\f$ in direction @b \f$w\f$; a dual optimization-space vector
              @param[in]       w    is the direction vector; a dual constraint-space vector
-             @param[in]       v    is a dual simulation-space vector
+             @param[in]       v    is a simulation-space vector
              @param[in]       u    is the constraint argument; a simulation-space vector
              @param[in]       z    is the constraint argument; an optimization-space vector
              @param[in,out]   tol  is a tolerance for inexact evaluations; currently unused
 
-             On return, \f$ \mathsf{ahwv} = c_{uz}(u,z)^*(w,v) \f$, where
-             \f$w \in \mathcal{C}^*\f$, \f$v \in \mathcal{U}^*\f$, and \f$\mathsf{ahwv} \in \mathcal{Z}^*\f$.
+             On return, \f$\mathsf{ahwv} = c_{uz}(u,z)(v,\cdot)^*w\f$, where
+             \f$w \in \mathcal{C}^*\f$, \f$v \in \mathcal{U}\f$, and
+             \f$\mathsf{ahwv} \in \mathcal{Z}^*\f$.
 
              ---
   */
@@ -631,19 +641,20 @@ public:
   }
 
 
-  /** \brief Apply the adjoint of the partial constraint Hessian at \f$(u,z)\f$,
-             \f$c_{zu}(u,z)^* \in L(L(\mathcal{C}^*, \mathcal{Z}^*), \mathcal{U}^*)\f$,
-             to vector \f$v\f$ in direction \f$w\f$.
+  /** \brief Apply the simulation-space derivative of the adjoint of the constraint
+             optimization-space Jacobian at \f$(u,z)\f$ to the vector \f$w\f$ in the
+             direction \f$v\f$, according to \f$v\mapsto c_{zu}(u,z)(v,\cdot)^*w\f$.
 
-             @param[out]      ahwv is the result of applying the adjoint of the constraint Hessian to @b v at @b \f$(u,z)\f$ in direction @b w; a dual simulation-space vector
+             @param[out]      ahwv is the result of applying the simulation-space derivative of the adjoint of the constraint optimization-space Jacobian at @b \f$(u,z)\f$ to the vector @b \f$w\f$ in direction @b \f$w\f$; a dual simulation-space vector
              @param[in]       w    is the direction vector; a dual constraint-space vector
-             @param[in]       v    is a dual optimization-space vector
+             @param[in]       v    is a optimization-space vector
              @param[in]       u    is the constraint argument; a simulation-space vector
              @param[in]       z    is the constraint argument; an optimization-space vector
              @param[in,out]   tol  is a tolerance for inexact evaluations; currently unused
 
-             On return, \f$ \mathsf{ahwv} = c_{zu}(u,z)^*(w,v) \f$, where
-             \f$w \in \mathcal{C}^*\f$, \f$v \in \mathcal{Z}^*\f$, and \f$\mathsf{ahwv} \in \mathcal{U}^*\f$.
+             On return, \f$\mathsf{ahwv} = c_{zu}(u,z)(v,\cdot)^*w\f$, where
+             \f$w \in \mathcal{C}^*\f$, \f$v \in \mathcal{Z}\f$, and
+             \f$\mathsf{ahwv} \in \mathcal{U}^*\f$.
 
              ---
   */
@@ -674,19 +685,20 @@ public:
     ahwv.scale(1.0/h);
   }
 
-  /** \brief Apply the adjoint of the partial constraint Hessian at \f$(u,z)\f$,
-             \f$c_{zz}(u,z)^* \in L(L(\mathcal{C}^*, \mathcal{Z}^*), \mathcal{Z}^*)\f$,
-             to vector \f$v\f$ in direction \f$w\f$.
+  /** \brief Apply the optimization-space derivative of the adjoint of the constraint
+             optimization-space Jacobian at \f$(u,z)\f$ to the vector \f$w\f$ in the
+             direction \f$v\f$, according to \f$v\mapsto c_{zz}(u,z)(v,\cdot)^*w\f$.
 
-             @param[out]      ahwv is the result of applying the adjoint of the constraint Hessian to @b v at @b \f$(u,z)\f$ in direction @b w; a dual optimization-space vector
+             @param[out]      ahwv is the result of applying the optimization-space derivative of the adjoint of the constraint optimization-space Jacobian at @b \f$(u,z)\f$ to the vector @b \f$w\f$ in direction @b \f$w\f$; a dual optimization-space vector
              @param[in]       w    is the direction vector; a dual constraint-space vector
-             @param[in]       v    is a dual optimization-space vector
+             @param[in]       v    is a optimization-space vector
              @param[in]       u    is the constraint argument; a simulation-space vector
              @param[in]       z    is the constraint argument; an optimization-space vector
              @param[in,out]   tol  is a tolerance for inexact evaluations; currently unused
 
-             On return, \f$ \mathsf{ahwv} = c_{zz}(u,z)^*(w,v) \f$, where
-             \f$w \in \mathcal{C}^*\f$, \f$v \in \mathcal{Z}^*\f$, and \f$\mathsf{ahwv} \in \mathcal{Z}^*\f$. 
+             On return, \f$\mathsf{ahwv} = c_{zz}(u,z)(v,\cdot)^*w\f$, where
+             \f$w \in \mathcal{C}^*\f$, \f$v \in \mathcal{Z}\f$, and
+             \f$\mathsf{ahwv} \in \mathcal{Z}^*\f$.
 
              ---
   */
@@ -1117,13 +1129,746 @@ public:
       hist << std::scientific << std::setprecision(8);
       hist << "\nTest SimOpt consistency of inverse adjoint Jacobian_1: \n  ||v-inv(adj(J))adj(J)v|| = "
            << dnorm << "\n";
-      hist << "  ||v||                   = " << vnorm << "\n";
-      hist << "  Relative Error          = " << dnorm / (vnorm+ROL_UNDERFLOW<Real>()) << "\n";
+      hist << "  ||v||                    = " << vnorm << "\n";
+      hist << "  Relative Error           = " << dnorm / (vnorm+ROL_UNDERFLOW<Real>()) << "\n";
       outStream << hist.str();
     }
     return dnorm;
   }
 
+
+
+  std::vector<std::vector<Real> > checkApplyJacobian_1(const Vector<Real> &u,
+                                                       const Vector<Real> &z,
+                                                       const Vector<Real> &v,
+                                                       const Vector<Real> &jv,
+                                                       const bool printToStream = true,
+                                                       std::ostream & outStream = std::cout,
+                                                       const int numSteps = ROL_NUM_CHECKDERIV_STEPS,
+                                                       const int order = 1) {
+    std::vector<Real> steps(numSteps);
+    for(int i=0;i<numSteps;++i) {
+      steps[i] = pow(10,-i);
+    }
+   
+    return checkApplyJacobian_1(u,z,v,jv,steps,printToStream,outStream,order);
+  }
+  
+  
+  
+  
+  std::vector<std::vector<Real> > checkApplyJacobian_1(const Vector<Real> &u,
+                                                       const Vector<Real> &z,
+                                                       const Vector<Real> &v,
+                                                       const Vector<Real> &jv,
+                                                       const std::vector<Real> &steps, 
+                                                       const bool printToStream = true,
+                                                       std::ostream & outStream = std::cout,
+                                                       const int order = 1) {
+ 
+    TEUCHOS_TEST_FOR_EXCEPTION( order<1 || order>4, std::invalid_argument, 
+                                "Error: finite difference order must be 1,2,3, or 4" );
+ 
+    Real one(1.0);
+ 
+    using Finite_Difference_Arrays::shifts;
+    using Finite_Difference_Arrays::weights;
+ 
+    Real tol = std::sqrt(ROL_EPSILON<Real>());
+ 
+    int numSteps = steps.size();
+    int numVals = 4;
+    std::vector<Real> tmp(numVals);
+    std::vector<std::vector<Real> > jvCheck(numSteps, tmp);
+ 
+    // Save the format state of the original outStream.
+    Teuchos::oblackholestream oldFormatState;
+    oldFormatState.copyfmt(outStream);
+ 
+    // Compute constraint value at x.
+    Teuchos::RCP<Vector<Real> > c = jv.clone();
+    this->update(u,z);
+    this->value(*c, u, z, tol);
+ 
+    // Compute (Jacobian at x) times (vector v).
+    Teuchos::RCP<Vector<Real> > Jv = jv.clone();
+    this->applyJacobian_1(*Jv, v, u, z, tol);
+    Real normJv = Jv->norm();
+ 
+    // Temporary vectors.
+    Teuchos::RCP<Vector<Real> > cdif = jv.clone();
+    Teuchos::RCP<Vector<Real> > cnew = jv.clone();
+    Teuchos::RCP<Vector<Real> > unew = u.clone();
+ 
+    for (int i=0; i<numSteps; i++) {
+ 
+      Real eta = steps[i];
+ 
+      unew->set(u);
+ 
+      cdif->set(*c);
+      cdif->scale(weights[order-1][0]);
+ 
+      for(int j=0; j<order; ++j) {
+ 
+         unew->axpy(eta*shifts[order-1][j], v);
+
+         if( weights[order-1][j+1] != 0 ) {
+             this->update(*unew,z);
+             this->value(*cnew,*unew,z,tol);
+             cdif->axpy(weights[order-1][j+1],*cnew);
+         }
+
+      }
+ 
+      cdif->scale(one/eta);
+ 
+      // Compute norms of Jacobian-vector products, finite-difference approximations, and error.
+      jvCheck[i][0] = eta;
+      jvCheck[i][1] = normJv;
+      jvCheck[i][2] = cdif->norm();
+      cdif->axpy(-one, *Jv);
+      jvCheck[i][3] = cdif->norm();
+ 
+      if (printToStream) {
+        std::stringstream hist;
+        if (i==0) {
+        hist << std::right
+             << std::setw(20) << "Step size"
+             << std::setw(20) << "norm(Jac*vec)"
+             << std::setw(20) << "norm(FD approx)"
+             << std::setw(20) << "norm(abs error)"
+             << "\n"
+             << std::setw(20) << "---------"
+             << std::setw(20) << "-------------"
+             << std::setw(20) << "---------------"
+             << std::setw(20) << "---------------"
+             << "\n";
+        }
+        hist << std::scientific << std::setprecision(11) << std::right
+             << std::setw(20) << jvCheck[i][0]
+             << std::setw(20) << jvCheck[i][1]
+             << std::setw(20) << jvCheck[i][2]
+             << std::setw(20) << jvCheck[i][3]
+             << "\n";
+        outStream << hist.str();
+      }
+ 
+    }
+ 
+    // Reset format state of outStream.
+    outStream.copyfmt(oldFormatState);
+ 
+    return jvCheck;
+  } // checkApplyJacobian
+
+
+  std::vector<std::vector<Real> > checkApplyJacobian_2(const Vector<Real> &u,
+                                                       const Vector<Real> &z,
+                                                       const Vector<Real> &v,
+                                                       const Vector<Real> &jv,
+                                                       const bool printToStream = true,
+                                                       std::ostream & outStream = std::cout,
+                                                       const int numSteps = ROL_NUM_CHECKDERIV_STEPS,
+                                                       const int order = 1) {
+    std::vector<Real> steps(numSteps);
+    for(int i=0;i<numSteps;++i) {
+      steps[i] = pow(10,-i);
+    }
+   
+    return checkApplyJacobian_2(u,z,v,jv,steps,printToStream,outStream,order);
+  }
+  
+  
+  
+  
+  std::vector<std::vector<Real> > checkApplyJacobian_2(const Vector<Real> &u,
+                                                       const Vector<Real> &z,
+                                                       const Vector<Real> &v,
+                                                       const Vector<Real> &jv,
+                                                       const std::vector<Real> &steps, 
+                                                       const bool printToStream = true,
+                                                       std::ostream & outStream = std::cout,
+                                                       const int order = 1) {
+ 
+    TEUCHOS_TEST_FOR_EXCEPTION( order<1 || order>4, std::invalid_argument, 
+                                "Error: finite difference order must be 1,2,3, or 4" );
+ 
+    Real one(1.0);
+ 
+    using Finite_Difference_Arrays::shifts;
+    using Finite_Difference_Arrays::weights;
+ 
+    Real tol = std::sqrt(ROL_EPSILON<Real>());
+ 
+    int numSteps = steps.size();
+    int numVals = 4;
+    std::vector<Real> tmp(numVals);
+    std::vector<std::vector<Real> > jvCheck(numSteps, tmp);
+ 
+    // Save the format state of the original outStream.
+    Teuchos::oblackholestream oldFormatState;
+    oldFormatState.copyfmt(outStream);
+ 
+    // Compute constraint value at x.
+    Teuchos::RCP<Vector<Real> > c = jv.clone();
+    this->update(u,z);
+    this->value(*c, u, z, tol);
+ 
+    // Compute (Jacobian at x) times (vector v).
+    Teuchos::RCP<Vector<Real> > Jv = jv.clone();
+    this->applyJacobian_2(*Jv, v, u, z, tol);
+    Real normJv = Jv->norm();
+ 
+    // Temporary vectors.
+    Teuchos::RCP<Vector<Real> > cdif = jv.clone();
+    Teuchos::RCP<Vector<Real> > cnew = jv.clone();
+    Teuchos::RCP<Vector<Real> > znew = z.clone();
+ 
+    for (int i=0; i<numSteps; i++) {
+ 
+      Real eta = steps[i];
+ 
+      znew->set(z);
+ 
+      cdif->set(*c);
+      cdif->scale(weights[order-1][0]);
+ 
+      for(int j=0; j<order; ++j) {
+ 
+         znew->axpy(eta*shifts[order-1][j], v);
+
+         if( weights[order-1][j+1] != 0 ) {
+             this->update(u,*znew);
+             this->value(*cnew,u,*znew,tol);
+             cdif->axpy(weights[order-1][j+1],*cnew);
+         }
+
+      }
+ 
+      cdif->scale(one/eta);
+ 
+      // Compute norms of Jacobian-vector products, finite-difference approximations, and error.
+      jvCheck[i][0] = eta;
+      jvCheck[i][1] = normJv;
+      jvCheck[i][2] = cdif->norm();
+      cdif->axpy(-one, *Jv);
+      jvCheck[i][3] = cdif->norm();
+ 
+      if (printToStream) {
+        std::stringstream hist;
+        if (i==0) {
+        hist << std::right
+             << std::setw(20) << "Step size"
+             << std::setw(20) << "norm(Jac*vec)"
+             << std::setw(20) << "norm(FD approx)"
+             << std::setw(20) << "norm(abs error)"
+             << "\n"
+             << std::setw(20) << "---------"
+             << std::setw(20) << "-------------"
+             << std::setw(20) << "---------------"
+             << std::setw(20) << "---------------"
+             << "\n";
+        }
+        hist << std::scientific << std::setprecision(11) << std::right
+             << std::setw(20) << jvCheck[i][0]
+             << std::setw(20) << jvCheck[i][1]
+             << std::setw(20) << jvCheck[i][2]
+             << std::setw(20) << jvCheck[i][3]
+             << "\n";
+        outStream << hist.str();
+      }
+ 
+    }
+ 
+    // Reset format state of outStream.
+    outStream.copyfmt(oldFormatState);
+ 
+    return jvCheck;
+  } // checkApplyJacobian
+
+
+
+  std::vector<std::vector<Real> > checkApplyAdjointHessian_11(const Vector<Real> &u,
+                                                              const Vector<Real> &z,
+                                                              const Vector<Real> &p,
+                                                              const Vector<Real> &v,
+                                                              const Vector<Real> &hv,
+                                                              const bool printToStream = true,
+                                                              std::ostream & outStream = std::cout,
+                                                              const int numSteps = ROL_NUM_CHECKDERIV_STEPS,
+                                                              const int order = 1 ) {
+    std::vector<Real> steps(numSteps);
+    for(int i=0;i<numSteps;++i) {
+      steps[i] = pow(10,-i);
+    }
+   
+    return checkApplyAdjointHessian_11(u,z,p,v,hv,steps,printToStream,outStream,order);
+  
+  }
+  
+  std::vector<std::vector<Real> > checkApplyAdjointHessian_11(const Vector<Real> &u,
+                                                              const Vector<Real> &z,
+                                                              const Vector<Real> &p,
+                                                              const Vector<Real> &v,
+                                                              const Vector<Real> &hv,
+                                                              const std::vector<Real> &steps,  
+                                                              const bool printToStream = true,
+                                                              std::ostream & outStream = std::cout,
+                                                              const int order = 1 ) {
+    using Finite_Difference_Arrays::shifts;
+    using Finite_Difference_Arrays::weights;
+  
+    Real one(1.0);
+  
+    Real tol = std::sqrt(ROL_EPSILON<Real>());
+  
+    int numSteps = steps.size();
+    int numVals = 4;
+    std::vector<Real> tmp(numVals);
+    std::vector<std::vector<Real> > ahpvCheck(numSteps, tmp);
+  
+    // Temporary vectors.
+    Teuchos::RCP<Vector<Real> > AJdif = hv.clone();
+    Teuchos::RCP<Vector<Real> > AJp = hv.clone();
+    Teuchos::RCP<Vector<Real> > AHpv = hv.clone();
+    Teuchos::RCP<Vector<Real> > AJnew = hv.clone();
+    Teuchos::RCP<Vector<Real> > unew = u.clone();
+  
+    // Save the format state of the original outStream.
+    Teuchos::oblackholestream oldFormatState;
+    oldFormatState.copyfmt(outStream);
+  
+    // Apply adjoint Jacobian to p.
+    this->update(u,z);
+    this->applyAdjointJacobian_1(*AJp, p, u, z, tol);
+  
+    // Apply adjoint Hessian at (u,z), in direction v, to p.
+    this->applyAdjointHessian_11(*AHpv, p, v, u, z, tol);
+    Real normAHpv = AHpv->norm();
+  
+    for (int i=0; i<numSteps; i++) {
+  
+      Real eta = steps[i];
+  
+      // Apply adjoint Jacobian to p at (u+eta*v,z).
+      unew->set(u);
+  
+      AJdif->set(*AJp);
+      AJdif->scale(weights[order-1][0]);     
+  
+      for(int j=0; j<order; ++j) {
+  
+          unew->axpy(eta*shifts[order-1][j],v); 
+  
+          if( weights[order-1][j+1] != 0 ) {    
+              this->update(*unew,z);
+              this->applyAdjointJacobian_1(*AJnew, p, *unew, z, tol);
+              AJdif->axpy(weights[order-1][j+1],*AJnew);
+          }
+      }
+  
+      AJdif->scale(one/eta);
+  
+      // Compute norms of Jacobian-vector products, finite-difference approximations, and error.
+      ahpvCheck[i][0] = eta;
+      ahpvCheck[i][1] = normAHpv;
+      ahpvCheck[i][2] = AJdif->norm();
+      AJdif->axpy(-one, *AHpv);
+      ahpvCheck[i][3] = AJdif->norm();
+  
+      if (printToStream) {
+        std::stringstream hist;
+        if (i==0) {
+        hist << std::right
+             << std::setw(20) << "Step size"
+             << std::setw(20) << "norm(adj(H)(u,v))"
+             << std::setw(20) << "norm(FD approx)"
+             << std::setw(20) << "norm(abs error)"
+             << "\n"
+             << std::setw(20) << "---------"
+             << std::setw(20) << "-----------------"
+             << std::setw(20) << "---------------"
+             << std::setw(20) << "---------------"
+             << "\n";
+        }
+        hist << std::scientific << std::setprecision(11) << std::right
+             << std::setw(20) << ahpvCheck[i][0]
+             << std::setw(20) << ahpvCheck[i][1]
+             << std::setw(20) << ahpvCheck[i][2]
+             << std::setw(20) << ahpvCheck[i][3]
+             << "\n";
+        outStream << hist.str();
+      }
+  
+    }
+  
+    // Reset format state of outStream.
+    outStream.copyfmt(oldFormatState);
+  
+    return ahpvCheck;
+  } // checkApplyAdjointHessian_11
+
+  std::vector<std::vector<Real> > checkApplyAdjointHessian_21(const Vector<Real> &u,
+                                                              const Vector<Real> &z,
+                                                              const Vector<Real> &p,
+                                                              const Vector<Real> &v,
+                                                              const Vector<Real> &hv,
+                                                              const bool printToStream = true,
+                                                              std::ostream & outStream = std::cout,
+                                                              const int numSteps = ROL_NUM_CHECKDERIV_STEPS,
+                                                              const int order = 1 ) {
+    std::vector<Real> steps(numSteps);
+    for(int i=0;i<numSteps;++i) {
+      steps[i] = pow(10,-i);
+    }
+   
+    return checkApplyAdjointHessian_21(u,z,p,v,hv,steps,printToStream,outStream,order);
+  
+  }
+  
+  std::vector<std::vector<Real> > checkApplyAdjointHessian_21(const Vector<Real> &u,
+                                                              const Vector<Real> &z,
+                                                              const Vector<Real> &p,
+                                                              const Vector<Real> &v,
+                                                              const Vector<Real> &hv,
+                                                              const std::vector<Real> &steps,  
+                                                              const bool printToStream = true,
+                                                              std::ostream & outStream = std::cout,
+                                                              const int order = 1 ) {
+    using Finite_Difference_Arrays::shifts;
+    using Finite_Difference_Arrays::weights;
+  
+    Real one(1.0);
+  
+    Real tol = std::sqrt(ROL_EPSILON<Real>());
+  
+    int numSteps = steps.size();
+    int numVals = 4;
+    std::vector<Real> tmp(numVals);
+    std::vector<std::vector<Real> > ahpvCheck(numSteps, tmp);
+  
+    // Temporary vectors.
+    Teuchos::RCP<Vector<Real> > AJdif = hv.clone();
+    Teuchos::RCP<Vector<Real> > AJp = hv.clone();
+    Teuchos::RCP<Vector<Real> > AHpv = hv.clone();
+    Teuchos::RCP<Vector<Real> > AJnew = hv.clone();
+    Teuchos::RCP<Vector<Real> > znew = z.clone();
+  
+    // Save the format state of the original outStream.
+    Teuchos::oblackholestream oldFormatState;
+    oldFormatState.copyfmt(outStream);
+  
+    // Apply adjoint Jacobian to p.
+    this->update(u,z);
+    this->applyAdjointJacobian_1(*AJp, p, u, z, tol);
+  
+    // Apply adjoint Hessian at (u,z), in direction v, to p.
+    this->applyAdjointHessian_21(*AHpv, p, v, u, z, tol);
+    Real normAHpv = AHpv->norm();
+  
+    for (int i=0; i<numSteps; i++) {
+  
+      Real eta = steps[i];
+  
+      // Apply adjoint Jacobian to p at (u,z+eta*v).
+      znew->set(z);
+  
+      AJdif->set(*AJp);
+      AJdif->scale(weights[order-1][0]);     
+  
+      for(int j=0; j<order; ++j) {
+  
+          znew->axpy(eta*shifts[order-1][j],v); 
+  
+          if( weights[order-1][j+1] != 0 ) {    
+              this->update(u,*znew);
+              this->applyAdjointJacobian_1(*AJnew, p, u, *znew, tol);
+              AJdif->axpy(weights[order-1][j+1],*AJnew);
+          }
+      }
+  
+      AJdif->scale(one/eta);
+  
+      // Compute norms of Jacobian-vector products, finite-difference approximations, and error.
+      ahpvCheck[i][0] = eta;
+      ahpvCheck[i][1] = normAHpv;
+      ahpvCheck[i][2] = AJdif->norm();
+      AJdif->axpy(-one, *AHpv);
+      ahpvCheck[i][3] = AJdif->norm();
+  
+      if (printToStream) {
+        std::stringstream hist;
+        if (i==0) {
+        hist << std::right
+             << std::setw(20) << "Step size"
+             << std::setw(20) << "norm(adj(H)(u,v))"
+             << std::setw(20) << "norm(FD approx)"
+             << std::setw(20) << "norm(abs error)"
+             << "\n"
+             << std::setw(20) << "---------"
+             << std::setw(20) << "-----------------"
+             << std::setw(20) << "---------------"
+             << std::setw(20) << "---------------"
+             << "\n";
+        }
+        hist << std::scientific << std::setprecision(11) << std::right
+             << std::setw(20) << ahpvCheck[i][0]
+             << std::setw(20) << ahpvCheck[i][1]
+             << std::setw(20) << ahpvCheck[i][2]
+             << std::setw(20) << ahpvCheck[i][3]
+             << "\n";
+        outStream << hist.str();
+      }
+  
+    }
+  
+    // Reset format state of outStream.
+    outStream.copyfmt(oldFormatState);
+  
+    return ahpvCheck;
+  } // checkApplyAdjointHessian_21
+
+  std::vector<std::vector<Real> > checkApplyAdjointHessian_12(const Vector<Real> &u,
+                                                              const Vector<Real> &z,
+                                                              const Vector<Real> &p,
+                                                              const Vector<Real> &v,
+                                                              const Vector<Real> &hv,
+                                                              const bool printToStream = true,
+                                                              std::ostream & outStream = std::cout,
+                                                              const int numSteps = ROL_NUM_CHECKDERIV_STEPS,
+                                                              const int order = 1 ) {
+    std::vector<Real> steps(numSteps);
+    for(int i=0;i<numSteps;++i) {
+      steps[i] = pow(10,-i);
+    }
+   
+    return checkApplyAdjointHessian_12(u,z,p,v,hv,steps,printToStream,outStream,order);
+  
+  }
+  
+  std::vector<std::vector<Real> > checkApplyAdjointHessian_12(const Vector<Real> &u,
+                                                              const Vector<Real> &z,
+                                                              const Vector<Real> &p,
+                                                              const Vector<Real> &v,
+                                                              const Vector<Real> &hv,
+                                                              const std::vector<Real> &steps,  
+                                                              const bool printToStream = true,
+                                                              std::ostream & outStream = std::cout,
+                                                              const int order = 1 ) {
+    using Finite_Difference_Arrays::shifts;
+    using Finite_Difference_Arrays::weights;
+  
+    Real one(1.0);
+  
+    Real tol = std::sqrt(ROL_EPSILON<Real>());
+  
+    int numSteps = steps.size();
+    int numVals = 4;
+    std::vector<Real> tmp(numVals);
+    std::vector<std::vector<Real> > ahpvCheck(numSteps, tmp);
+  
+    // Temporary vectors.
+    Teuchos::RCP<Vector<Real> > AJdif = hv.clone();
+    Teuchos::RCP<Vector<Real> > AJp = hv.clone();
+    Teuchos::RCP<Vector<Real> > AHpv = hv.clone();
+    Teuchos::RCP<Vector<Real> > AJnew = hv.clone();
+    Teuchos::RCP<Vector<Real> > unew = u.clone();
+  
+    // Save the format state of the original outStream.
+    Teuchos::oblackholestream oldFormatState;
+    oldFormatState.copyfmt(outStream);
+  
+    // Apply adjoint Jacobian to p.
+    this->update(u,z);
+    this->applyAdjointJacobian_2(*AJp, p, u, z, tol);
+  
+    // Apply adjoint Hessian at (u,z), in direction v, to p.
+    this->applyAdjointHessian_12(*AHpv, p, v, u, z, tol);
+    Real normAHpv = AHpv->norm();
+  
+    for (int i=0; i<numSteps; i++) {
+  
+      Real eta = steps[i];
+  
+      // Apply adjoint Jacobian to p at (u+eta*v,z).
+      unew->set(u);
+  
+      AJdif->set(*AJp);
+      AJdif->scale(weights[order-1][0]);     
+  
+      for(int j=0; j<order; ++j) {
+  
+          unew->axpy(eta*shifts[order-1][j],v); 
+  
+          if( weights[order-1][j+1] != 0 ) {    
+              this->update(*unew,z);
+              this->applyAdjointJacobian_2(*AJnew, p, *unew, z, tol);
+              AJdif->axpy(weights[order-1][j+1],*AJnew);
+          }
+      }
+  
+      AJdif->scale(one/eta);
+  
+      // Compute norms of Jacobian-vector products, finite-difference approximations, and error.
+      ahpvCheck[i][0] = eta;
+      ahpvCheck[i][1] = normAHpv;
+      ahpvCheck[i][2] = AJdif->norm();
+      AJdif->axpy(-one, *AHpv);
+      ahpvCheck[i][3] = AJdif->norm();
+  
+      if (printToStream) {
+        std::stringstream hist;
+        if (i==0) {
+        hist << std::right
+             << std::setw(20) << "Step size"
+             << std::setw(20) << "norm(adj(H)(u,v))"
+             << std::setw(20) << "norm(FD approx)"
+             << std::setw(20) << "norm(abs error)"
+             << "\n"
+             << std::setw(20) << "---------"
+             << std::setw(20) << "-----------------"
+             << std::setw(20) << "---------------"
+             << std::setw(20) << "---------------"
+             << "\n";
+        }
+        hist << std::scientific << std::setprecision(11) << std::right
+             << std::setw(20) << ahpvCheck[i][0]
+             << std::setw(20) << ahpvCheck[i][1]
+             << std::setw(20) << ahpvCheck[i][2]
+             << std::setw(20) << ahpvCheck[i][3]
+             << "\n";
+        outStream << hist.str();
+      }
+  
+    }
+  
+    // Reset format state of outStream.
+    outStream.copyfmt(oldFormatState);
+  
+    return ahpvCheck;
+  } // checkApplyAdjointHessian_12
+
+  std::vector<std::vector<Real> > checkApplyAdjointHessian_22(const Vector<Real> &u,
+                                                              const Vector<Real> &z,
+                                                              const Vector<Real> &p,
+                                                              const Vector<Real> &v,
+                                                              const Vector<Real> &hv,
+                                                              const bool printToStream = true,
+                                                              std::ostream & outStream = std::cout,
+                                                              const int numSteps = ROL_NUM_CHECKDERIV_STEPS,
+                                                              const int order = 1 ) {
+    std::vector<Real> steps(numSteps);
+    for(int i=0;i<numSteps;++i) {
+      steps[i] = pow(10,-i);
+    }
+   
+    return checkApplyAdjointHessian_22(u,z,p,v,hv,steps,printToStream,outStream,order);
+  
+  }
+  
+  std::vector<std::vector<Real> > checkApplyAdjointHessian_22(const Vector<Real> &u,
+                                                              const Vector<Real> &z,
+                                                              const Vector<Real> &p,
+                                                              const Vector<Real> &v,
+                                                              const Vector<Real> &hv,
+                                                              const std::vector<Real> &steps,  
+                                                              const bool printToStream = true,
+                                                              std::ostream & outStream = std::cout,
+                                                              const int order = 1 ) {
+    using Finite_Difference_Arrays::shifts;
+    using Finite_Difference_Arrays::weights;
+  
+    Real one(1.0);
+  
+    Real tol = std::sqrt(ROL_EPSILON<Real>());
+  
+    int numSteps = steps.size();
+    int numVals = 4;
+    std::vector<Real> tmp(numVals);
+    std::vector<std::vector<Real> > ahpvCheck(numSteps, tmp);
+  
+    // Temporary vectors.
+    Teuchos::RCP<Vector<Real> > AJdif = hv.clone();
+    Teuchos::RCP<Vector<Real> > AJp = hv.clone();
+    Teuchos::RCP<Vector<Real> > AHpv = hv.clone();
+    Teuchos::RCP<Vector<Real> > AJnew = hv.clone();
+    Teuchos::RCP<Vector<Real> > znew = z.clone();
+  
+    // Save the format state of the original outStream.
+    Teuchos::oblackholestream oldFormatState;
+    oldFormatState.copyfmt(outStream);
+  
+    // Apply adjoint Jacobian to p.
+    this->update(u,z);
+    this->applyAdjointJacobian_2(*AJp, p, u, z, tol);
+  
+    // Apply adjoint Hessian at (u,z), in direction v, to p.
+    this->applyAdjointHessian_22(*AHpv, p, v, u, z, tol);
+    Real normAHpv = AHpv->norm();
+  
+    for (int i=0; i<numSteps; i++) {
+  
+      Real eta = steps[i];
+  
+      // Apply adjoint Jacobian to p at (u,z+eta*v).
+      znew->set(z);
+  
+      AJdif->set(*AJp);
+      AJdif->scale(weights[order-1][0]);     
+  
+      for(int j=0; j<order; ++j) {
+  
+          znew->axpy(eta*shifts[order-1][j],v); 
+  
+          if( weights[order-1][j+1] != 0 ) {    
+              this->update(u,*znew);
+              this->applyAdjointJacobian_2(*AJnew, p, u, *znew, tol);
+              AJdif->axpy(weights[order-1][j+1],*AJnew);
+          }
+      }
+  
+      AJdif->scale(one/eta);
+  
+      // Compute norms of Jacobian-vector products, finite-difference approximations, and error.
+      ahpvCheck[i][0] = eta;
+      ahpvCheck[i][1] = normAHpv;
+      ahpvCheck[i][2] = AJdif->norm();
+      AJdif->axpy(-one, *AHpv);
+      ahpvCheck[i][3] = AJdif->norm();
+  
+      if (printToStream) {
+        std::stringstream hist;
+        if (i==0) {
+        hist << std::right
+             << std::setw(20) << "Step size"
+             << std::setw(20) << "norm(adj(H)(u,v))"
+             << std::setw(20) << "norm(FD approx)"
+             << std::setw(20) << "norm(abs error)"
+             << "\n"
+             << std::setw(20) << "---------"
+             << std::setw(20) << "-----------------"
+             << std::setw(20) << "---------------"
+             << std::setw(20) << "---------------"
+             << "\n";
+        }
+        hist << std::scientific << std::setprecision(11) << std::right
+             << std::setw(20) << ahpvCheck[i][0]
+             << std::setw(20) << ahpvCheck[i][1]
+             << std::setw(20) << ahpvCheck[i][2]
+             << std::setw(20) << ahpvCheck[i][3]
+             << "\n";
+        outStream << hist.str();
+      }
+  
+    }
+  
+    // Reset format state of outStream.
+    outStream.copyfmt(oldFormatState);
+  
+    return ahpvCheck;
+  } // checkApplyAdjointHessian_22
+
 }; // class EqualityConstraint_SimOpt
 
 } // namespace ROL
