@@ -65,9 +65,8 @@
 #include "ROL_MeanVarianceQuadrangle.hpp"
 #include "ROL_MixedQuantileQuadrangle.hpp"
 #include "ROL_SuperQuantileQuadrangle.hpp"
-#include "ROL_Chebyshev1Kusuoka.hpp"
-#include "ROL_Chebyshev2Kusuoka.hpp"
-#include "ROL_Chebyshev3Kusuoka.hpp"
+#include "ROL_ChebyshevKusuoka.hpp"
+#include "ROL_SpectralRisk.hpp"
 #include "ROL_QuantileQuadrangle.hpp"
 #include "ROL_QuantileRadiusQuadrangle.hpp"
 #include "ROL_SmoothedWorstCaseQuadrangle.hpp"
@@ -97,9 +96,8 @@ namespace ROL {
     RISKMEASURE_QUANTILERADIUSQUADRANGLE,
     RISKMEASURE_SMOOTHEDWORSTCASEQUADRANGLE,
     RISKMEASURE_SUPERQUANTILEQUADRANGLE,
-    RISKMEASURE_CHEBYSHEV1KUSUOKA,
-    RISKMEASURE_CHEBYSHEV2KUSUOKA,
-    RISKMEASURE_CHEBYSHEV3KUSUOKA,
+    RISKMEASURE_CHEBYSHEVKUSUOKA,
+    RISKMEASURE_SPECTRALRISK,
     RISKMEASURE_TRUNCATEDMEANQUADRANGLE,
     RISKMEASURE_CHI2DIVERGENCE,
     RISKMEASURE_KLDIVERGENCE,
@@ -137,12 +135,10 @@ namespace ROL {
              retString = "Mixed-Quantile Quadrangle";               break;
       case RISKMEASURE_SUPERQUANTILEQUADRANGLE:
              retString = "Super Quantile Quadrangle";               break;
-      case RISKMEASURE_CHEBYSHEV1KUSUOKA:
-             retString = "Chebyshev 1 Kusuoka";                     break;
-      case RISKMEASURE_CHEBYSHEV2KUSUOKA:
-             retString = "Chebyshev 2 Kusuoka";                     break;
-      case RISKMEASURE_CHEBYSHEV3KUSUOKA:
-             retString = "Chebyshev 3 Kusuoka";                     break;
+      case RISKMEASURE_CHEBYSHEVKUSUOKA:
+             retString = "Chebyshev-Kusuoka";                       break;
+      case RISKMEASURE_SPECTRALRISK:
+             retString = "Spectral Risk";                           break;
       case RISKMEASURE_QUANTILEQUADRANGLE:
              retString = "Quantile-Based Quadrangle";               break;
       case RISKMEASURE_QUANTILERADIUSQUADRANGLE:
@@ -178,9 +174,8 @@ namespace ROL {
             (ed == RISKMEASURE_MEANVARIANCEQUADRANGLE) ||
             (ed == RISKMEASURE_MIXEDQUANTILEQUADRANGLE) ||
             (ed == RISKMEASURE_SUPERQUANTILEQUADRANGLE) ||
-            (ed == RISKMEASURE_CHEBYSHEV1KUSUOKA) ||
-            (ed == RISKMEASURE_CHEBYSHEV2KUSUOKA) ||
-            (ed == RISKMEASURE_CHEBYSHEV3KUSUOKA) ||
+            (ed == RISKMEASURE_CHEBYSHEVKUSUOKA) ||
+            (ed == RISKMEASURE_SPECTRALRISK) ||
             (ed == RISKMEASURE_QUANTILEQUADRANGLE) ||
             (ed == RISKMEASURE_QUANTILERADIUSQUADRANGLE) ||
             (ed == RISKMEASURE_SMOOTHEDWORSTCASEQUADRANGLE) ||
@@ -252,12 +247,10 @@ namespace ROL {
              return Teuchos::rcp(new MixedQuantileQuadrangle<Real>(parlist));
       case RISKMEASURE_SUPERQUANTILEQUADRANGLE:
              return Teuchos::rcp(new SuperQuantileQuadrangle<Real>(parlist));
-      case RISKMEASURE_CHEBYSHEV1KUSUOKA:
-             return Teuchos::rcp(new Chebyshev1Kusuoka<Real>(parlist));
-      case RISKMEASURE_CHEBYSHEV2KUSUOKA:
-             return Teuchos::rcp(new Chebyshev2Kusuoka<Real>(parlist));
-      case RISKMEASURE_CHEBYSHEV3KUSUOKA:
-             return Teuchos::rcp(new Chebyshev3Kusuoka<Real>(parlist));
+      case RISKMEASURE_CHEBYSHEVKUSUOKA:
+             return Teuchos::rcp(new ChebyshevKusuoka<Real>(parlist));
+      case RISKMEASURE_SPECTRALRISK:
+             return Teuchos::rcp(new SpectralRisk<Real>(parlist));
       case RISKMEASURE_QUANTILEQUADRANGLE:
              return Teuchos::rcp(new QuantileQuadrangle<Real>(parlist));
       case RISKMEASURE_QUANTILERADIUSQUADRANGLE:
