@@ -94,44 +94,37 @@ public:
     Hp_ = g.clone();
   }
 
-  void run( Vector<Real> &s, Real &snorm, Real &del, int &iflag, int &iter, const Vector<Real> &x,
-            const Vector<Real> &grad, const Real &gnorm, ProjectedObjective<Real> &pObj ) { 
-    Real tol = std::sqrt(ROL_EPSILON<Real>()), zero(0), one(1), two(2), half(0.5);
-    const Real gtol = std::min(tol1_,tol2_*gnorm);
-
-    // Gradient Vector
-    g_->set(grad);
-    Real normg = gnorm;
-    if ( pObj.isConActivated() ) {
-      primalVector_->set(grad.dual());
-      pObj.pruneActive(*primalVector_,grad.dual(),x);
-      g_->set(primalVector_->dual());
-      normg = g_->norm();
-    }
-
-    // Old and New Step Vectors
+  void run( Vector<Real>           &s,
+            Real                   &snorm,
+            int                    &iflag,
+            int                    &iter,
+            const Real              del,
+            TrustRegionModel<Real> &model ) {
+    Real tol = std::sqrt(ROL_EPSILON<Real>());
+    const Real zero(0), one(1), two(2), half(0.5);
+    // Initialize step
     s.zero(); s_->zero();
     snorm = zero;
     Real snorm2(0), s1norm2(0);
-
-    // Preconditioned Gradient Vector
-    //pObj.precond(*v,*g,x,tol);
-    pObj.reducedPrecond(*v_,*g_,x,grad.dual(),x,tol);
-
-    // Basis Vector
-    p_->set(*v_);
-    p_->scale(-one);
+    // Compute (projected) gradient
+    model.dualTransform(*g_,*model.getGradient());
+    Real gnorm = g_->norm(), normg = gnorm;
+    const Real gtol = std::min(tol1_,tol2_*gnorm);
+    // Preconditioned (projected) gradient vector
+    model.precond(*v_,*g_,s,tol);
+    // Initialize basis vector
+    p_->set(*v_); p_->scale(-one);
     Real pnorm2 = v_->dot(g_->dual());
-
+    // Initialize scalar storage
     iter = 0; iflag = 0;
     Real kappa(0), beta(0), sigma(0), alpha(0), tmp(0), sMp(0);
     Real gv = v_->dot(g_->dual());
     pRed_ = zero;
-
+    // Iterate CG
     for (iter = 0; iter < maxit_; iter++) {
-      //pObj.hessVec(*Hp,*p,x,tol);
-      pObj.reducedHessVec(*Hp_,*p_,x,grad.dual(),x,tol);
-
+      // Apply Hessian to direction p
+      model.hessVec(*Hp_,*p_,s,tol);
+      // Check positivity of Hessian
       kappa = p_->dot(Hp_->dual());
       if (kappa <= zero) {
         sigma = (-sMp+sqrt(sMp*sMp+pnorm2*(del*del-snorm2)))/pnorm2;
@@ -139,52 +132,54 @@ public:
         iflag = 2; 
         break;
       }
-
+      // Update step
       alpha = gv/kappa;
       s_->set(s); 
       s_->axpy(alpha,*p_);
       s1norm2 = snorm2 + two*alpha*sMp + alpha*alpha*pnorm2;
-
+      // Check if step exceeds trust region radius
       if (s1norm2 >= del*del) {
         sigma = (-sMp+sqrt(sMp*sMp+pnorm2*(del*del-snorm2)))/pnorm2;
         s.axpy(sigma,*p_);
         iflag = 3; 
         break;
       }
-
+      // Update model predicted reduction
       pRed_ += half*alpha*gv;
-
+      // Set step to temporary step and store norm
       s.set(*s_);
       snorm2 = s1norm2;  
-
+      // Check for convergence
       g_->axpy(alpha,*Hp_);
       normg = g_->norm();
       if (normg < gtol) {
         break;
       }
-
-      //pObj.precond(*v,*g,x,tol);
-      pObj.reducedPrecond(*v_,*g_,x,grad.dual(),x,tol);
+      // Preconditioned updated (projected) gradient vector
+      model.precond(*v_,*g_,s,tol);
       tmp   = gv; 
       gv    = v_->dot(g_->dual());
       beta  = gv/tmp;    
-
+      // Update basis vector
       p_->scale(beta);
       p_->axpy(-one,*v_);
       sMp    = beta*(sMp+alpha*pnorm2);
       pnorm2 = gv + beta*beta*pnorm2; 
     }
+    // Update model predicted reduction
     if (iflag > 0) {
       pRed_ += sigma*(gv-half*sigma*kappa);
     }
-
+    // Check iteration count
     if (iter == maxit_) {
       iflag = 1;
     }
     if (iflag != 1) { 
       iter++;
     }
-
+    // Update norm of step and update model predicted reduction
+    model.primalTransform(*s_,s);
+    s.set(*s_);
     snorm = s.norm();
     TrustRegion<Real>::setPredictedReduction(pRed_);
   }
