@@ -65,6 +65,7 @@
 #include <Teuchos_ScalarTraits.hpp>
 #include <Teuchos_TestForException.hpp>
 #include <ROL_Vector.hpp>
+#include <ROL_config.h>
 
 /** \def    ROL_NUM_CHECKDERIV_STEPS
     \brief  Number of steps for derivative checks.
@@ -121,8 +122,14 @@ std::string NumberToString( T Number )
     Teuchos::RCP<Vector<Real> > gradientVec;
     Teuchos::RCP<Vector<Real> > descentVec;
     Teuchos::RCP<Vector<Real> > constraintVec;
+    int nfval;
+    int ngrad;
     Real searchSize; // line search parameter (alpha) or trust-region radius (delta)
-    StepState(void) : gradientVec(Teuchos::null), descentVec(Teuchos::null), constraintVec(Teuchos::null),
+    StepState(void) : gradientVec(Teuchos::null),
+                      descentVec(Teuchos::null),
+                      constraintVec(Teuchos::null),
+                      nfval(0),
+                      ngrad(0),
                       searchSize(0) {}
   };  
       
@@ -804,85 +811,6 @@ std::string NumberToString( T Number )
     return CURVATURECONDITION_WOLFE;
   }
 
-  /** \enum   ROL::ETrustRegion
-      \brief  Enumeration of trust-region solver types.
-
-      \arg    CAUCHYPOINT     describe
-      \arg    TRUNCATEDCG     describe
-      \arg    DOGLEG          describe
-      \arg    DOUBLEDOGLEG    describe
-   */
-  enum ETrustRegion{
-    TRUSTREGION_CAUCHYPOINT = 0,
-    TRUSTREGION_TRUNCATEDCG,
-    TRUSTREGION_DOGLEG,
-    TRUSTREGION_DOUBLEDOGLEG,
-    TRUSTREGION_LAST
-  };
-
-  inline std::string ETrustRegionToString(ETrustRegion tr) {
-    std::string retString;
-    switch(tr) {
-      case TRUSTREGION_CAUCHYPOINT:   retString = "Cauchy Point";        break;
-      case TRUSTREGION_TRUNCATEDCG:   retString = "Truncated CG";        break;
-      case TRUSTREGION_DOGLEG:        retString = "Dogleg";              break;
-      case TRUSTREGION_DOUBLEDOGLEG:  retString = "Double Dogleg";       break;
-      case TRUSTREGION_LAST:          retString = "Last Type (Dummy)";   break;
-      default:                        retString = "INVALID ETrustRegion";
-    }
-    return retString;
-  }
-  
-  /** \enum  ROL::ETrustRegionFlag 
-      \brief Enumation of flags used by trust-region solvers.
-
-      \arg TRUSTREGION_FLAG_SUCCESS        Actual and predicted reductions are positive 
-      \arg TRUSTREGION_FLAG_POSPREDNEG     Reduction is positive, predicted negative (impossible)
-      \arg TRUSTREGION_FLAG_NPOSPREDPOS    Reduction is nonpositive, predicted positive
-      \arg TRUSTREGION_FLAG_NPOSPREDNEG    Reduction is nonpositive, predicted negative (impossible)
-      \arg TRUSTREGION_FLAG_QMINSUFDEC     Insufficient decrease of the quadratic model (bound constraint only)
-      \arg TRUSTREGION_FLAG_NAN            Actual and/or predicted reduction is NaN
-
-  */
-  enum ETrustRegionFlag {
-    TRUSTREGION_FLAG_SUCCESS = 0,
-    TRUSTREGION_FLAG_POSPREDNEG,
-    TRUSTREGION_FLAG_NPOSPREDPOS,
-    TRUSTREGION_FLAG_NPOSPREDNEG,
-    TRUSTREGION_FLAG_QMINSUFDEC,
-    TRUSTREGION_FLAG_NAN,
-    TRUSTREGION_FLAG_UNDEFINED 
-  };
- 
-
-  inline std::string ETrustRegionFlagToString(ETrustRegionFlag trf) {
-    std::string retString;
-    switch(trf) {
-      case TRUSTREGION_FLAG_SUCCESS:  
-        retString = "Both actual and predicted reductions are positive (success)";
-        break;
-      case TRUSTREGION_FLAG_POSPREDNEG: 
-        retString = "Actual reduction is positive and predicted reduction is negative (impossible)";
-        break;
-      case TRUSTREGION_FLAG_NPOSPREDPOS: 
-        retString = "Actual reduction is nonpositive and predicted reduction is positive";
-        break;
-      case TRUSTREGION_FLAG_NPOSPREDNEG:
-        retString = "Actual reduction is nonpositive and predicted reduction is negative (impossible)";
-        break;
-      case TRUSTREGION_FLAG_QMINSUFDEC:
-        retString = "Sufficient decrease of the quadratic model not met (bound constraints only)";
-        break;
-      case TRUSTREGION_FLAG_NAN:
-        retString = "Actual and/or predicted reduction is a NaN";
-        break;
-      default:
-        retString = "INVALID ETrustRegionFlag";       
-    }
-    return retString;
-  }
-
-
   /** \enum  ROL::ECGFlag 
       \brief Enumation of flags used by conjugate gradient methods.
 
@@ -921,55 +849,6 @@ std::string NumberToString( T Number )
     }
     return retString;
   }
-  
-
-
-
-
-  /** \brief  Verifies validity of a TrustRegion enum.
-    
-      \param  tr  [in]  - enum of the TrustRegion
-      \return 1 if the argument is a valid TrustRegion; 0 otherwise.
-    */
-  inline int isValidTrustRegion(ETrustRegion ls){
-    return( (ls == TRUSTREGION_CAUCHYPOINT)  ||
-            (ls == TRUSTREGION_TRUNCATEDCG)  ||
-            (ls == TRUSTREGION_DOGLEG)       ||
-            (ls == TRUSTREGION_DOUBLEDOGLEG)
-          );
-  }
-
-  inline ETrustRegion & operator++(ETrustRegion &type) {
-    return type = static_cast<ETrustRegion>(type+1);
-  }
-
-  inline ETrustRegion operator++(ETrustRegion &type, int) {
-    ETrustRegion oldval = type;
-    ++type;
-    return oldval;
-  }
-
-  inline ETrustRegion & operator--(ETrustRegion &type) {
-    return type = static_cast<ETrustRegion>(type-1);
-  }
-
-  inline ETrustRegion operator--(ETrustRegion &type, int) {
-    ETrustRegion oldval = type;
-    --type;
-    return oldval;
-  }
-
-  inline ETrustRegion StringToETrustRegion(std::string s) {
-    s = removeStringFormat(s);
-    for ( ETrustRegion tr = TRUSTREGION_CAUCHYPOINT; tr < TRUSTREGION_LAST; tr++ ) {
-      if ( !s.compare(removeStringFormat(ETrustRegionToString(tr))) ) {
-        return tr;
-      }
-    }
-    return TRUSTREGION_CAUCHYPOINT;
-  }
-
-
 
 
   /** \enum   ROL::ETestObjectives
