@@ -615,24 +615,24 @@ FAD_BINARYOP_MACRO(pow,
                    expr1.val() == value_type(0) ? value_type(0.0) : value_type(c.val()*expr1.fastAccessDx(i)/expr1.val()*std::pow(expr1.val(),c.val())))
 FAD_BINARYOP_MACRO(max,
                    MaxOp,
-                   std::max(expr1.val(), expr2.val()),
+                   expr1.val() >= expr2.val() ? expr1.val() : expr2.val(),
                    expr1.val() >= expr2.val() ? expr1.dx(i) : expr2.dx(i),
                    expr1.val() >= expr2.val() ? expr1.fastAccessDx(i) :
                                                 expr2.fastAccessDx(i),
-                   std::max(c.val(), expr2.val()),
-                   std::max(expr1.val(), c.val()),
+                   c.val() >= expr2.val() ? c.val() : expr2.val(),
+                   expr1.val() >= c.val() ? expr1.val() : c.val(),
                    c.val() >= expr2.val() ? value_type(0) : expr2.dx(i),
                    expr1.val() >= c.val() ? expr1.dx(i) : value_type(0),
                    c.val() >= expr2.val() ? value_type(0) : expr2.fastAccessDx(i),
                    expr1.val() >= c.val() ? expr1.fastAccessDx(i) : value_type(0))
 FAD_BINARYOP_MACRO(min,
                    MinOp,
-                   std::min(expr1.val(), expr2.val()),
+                   expr1.val() <= expr2.val() ? expr1.val() : expr2.val(),
                    expr1.val() <= expr2.val() ? expr1.dx(i) : expr2.dx(i),
                    expr1.val() <= expr2.val() ? expr1.fastAccessDx(i) :
                                                 expr2.fastAccessDx(i),
-                   std::min(c.val(), expr2.val()),
-                   std::min(expr1.val(), c.val()),
+                   c.val() <= expr2.val() ? c.val() : expr2.val(),
+                   expr1.val() <= c.val() ? expr1.val() : c.val(),
                    c.val() <= expr2.val() ? value_type(0) : expr2.dx(i),
                    expr1.val() <= c.val() ? expr1.dx(i) : value_type(0),
                    c.val() <= expr2.val() ? value_type(0) : expr2.fastAccessDx(i),
