@@ -34,7 +34,7 @@
  */
 #ifndef CHACO_H
 #define CHACO_H
-/*
+/**
  * Calling Chaco from other programs. Throughout this document we have
  * assumed that Chaco is being used as a stand{alone program. However,
  * this needn't be the case. We designed version 2.0 to allow for easy
@@ -74,17 +74,17 @@
  *         adjacency array. The values in adjacency from start i ; 1] to start i];1 are the
  *         vertices adjacent to vertex i in the graph. (Note that C arrays begin at zero,
  *         so in Fortran, the relevant range would be start i] to start(i + 1) ; 1.)
- *      3. adjacency. Type int *. As indicated in the description of start, this array con-
- *         tains a list of edges for all vertices in the graph. Note that if the FREE GRAPH
- *         parameter from x6.8 is set to TRUE, then after converting to a new data struc-
- *         ture, both start and adjacency are freed. If this is inappropriate for your ap-
- *         plication (e.g. you want to keep the graph, or you didn't dynamically allocate
+ *      3. adjacency. Type int *. As indicated in the description of start, this array contains
+ *         a list of edges for all vertices in the graph. Note that if the FREE GRAPH
+ *         parameter from x6.8 is set to TRUE, then after converting to a new data structure,
+ *         both start and adjacency are freed. If this is inappropriate for your application
+ *         (e.g. you want to keep the graph, or you didn't dynamically allocate
  *         these arrays), then you should set FREE GRAPH to FALSE.
- *      4. vwgts. Type int *. This array of length nvtxs speci es weights for all the
+ *      4. vwgts. Type int *. This array of length nvtxs specifies weights for all the
  *         vertices. If you pass in a NULL pointer, then all vertices are given unit weight.
  *         Vertex weights should be positive.
  *
- *      5. ewgts. Type oat * (Fortran type real*4). This array
+ *      5. ewgts. Type float * (Fortran type real*4). This array
  *         specifies weights for all the edges. It is of the same length
  *         as adjacency and is indexed in the same way. If you use
  *         Kernighan-Lin or the multilevel partitioner, these values
@@ -108,10 +108,10 @@
  * B. Output le names.
  *
  *      9. outassignname. Type char *. If you desire the final
- *         assignment to be written * to a file, this argument gives the
- *         name of that file. If this argument is NULL * or if the
+ *         assignment to be written to a file, this argument gives the
+ *         name of that file. If this argument is NULL or if the
  *         parameter OUTPUT ASSIGN is 0, then the assignment is not
- *         written to * a file.
+ *         written to a file.
  *
  *     10. out filename. Type char *. This is the name of a file in
  *         which the results of the run are printed. If it is NULL or if
@@ -155,30 +155,30 @@
  * E. Partitioning options.
  *      16. global method. Type int. This argument speci es the global partitioning
  *         method and should have a value from 1 and 7. These values are the same
- *         as those on the \Global method" menu when running Chaco in stand{alone
+ *         as those on the "Global method" menu when running Chaco in standalone
  *         method, as reviewed in x5.4.
  *
  *      17. local method. Type int. This argument speci es the local
  *         partitioning method and should have a value of 1 or 2. These
- *         values are the same as those on the \Local method" menu when
+ *         values are the same as those on the "Local method" menu when
  *         running Chaco in stand{alone method, as reviewed in x5.4.
  *
- *      18. rqi ag. Type int. If you requested spectral partitioning and wish to use the
+ *      18. rqi flag. Type int. If you requested spectral partitioning and wish to use the
  *          multilevel RQI/Symmlq eigensolver, this argument should be set to 1. If you
  *          wish instead to use Lanczos, it should be set to 0.
  *
- *      19. vmax. Type int. If you are using either the multilevel{KL partitioner, or
+ *      19. vmax. Type int. If you are using either the multilevel-KL partitioner, or
  *         the multilevel RQI/Symmlq eigensolver, you need to specify when the coarsest
  *         graph is small enough. When a coarse graph has no more than vmax vertices,
- *         the recursive coarsening is nished.
+ *         the recursive coarsening is finished.
  *      20. ndims. Type int. This argument should have a value of 1, 2 or 3 indicating
  *         partitioning by bisection, quadrisection or octasection.
- *      21. eigtol. Type double. If you are using a spectral method or multilevel{
- *         KL, this argument specifies the tolerance you request for the eigensolver. A
+ *      21. eigtol. Type double. If you are using a spectral method or multilevel-KL,
+ *         this argument specifies the tolerance you request for the eigensolver. A
  *         discussion of an appropriate choice can be found in the description of the
  *         EIGEN TOLERANCE parameter in x6.2. Note that this argument overrides the
  *         value of the EIGEN TOLERANCE parameter.
- *      22. seed. Type long. This is a seed for the random number generator \rand()".
+ *      22. seed. Type long. This is a seed for the random number generator "rand()".
  *         Note that it overrides the RANDOM SEED parameter.
  */
 #ifdef __cplusplus
@@ -187,26 +187,26 @@ extern "C" {
 
 #include <stdio.h>
 
-extern int interface(int    nvtxs,                 /* number of vertices in full graph */
-                     int *  start,                 /* start of edge list for each vertex */
-                     int *  adjacency,             /* edge list data */
-                     int *  vwgts,                 /* weights for all vertices */
-                     float *ewgts,                 /* weights for all edges */
-                     float *x, float *y, float *z, /* coordinates for inertial method */
-                     char *  outassignname,        /* name of assignment output file */
-                     char *  outfilename,          /* output file name */
-                     int *   assignment,           /* set number of each vtx (length n) */
-                     int     architecture,         /* 0 => hypercube, d => d-dimensional mesh */
-                     int     ndims_tot,            /* total number of cube dimensions to divide */
-                     int     mesh_dims[3],         /* dimensions of mesh of processors */
-                     double *goal,                 /* desired set sizes for each set */
-                     int     global_method,        /* global partitioning algorithm */
-                     int     local_method,         /* local partitioning algorithm */
-                     int     rqi_flag,             /* should I use RQI/Symmlq eigensolver? */
-                     int     vmax,                 /* how many vertices to coarsen down to? */
-                     int     ndims,                /* number of eigenvectors (2^d sets) */
-                     double  eigtol,               /* tolerance on eigenvectors */
-                     long    seed);                /* for random graph mutations */
+extern int interface(int    nvtxs,                 /**< number of vertices in full graph */
+                     int *  start,                 /**< start of edge list for each vertex */
+                     int *  adjacency,             /**< edge list data */
+                     int *  vwgts,                 /**< weights for all vertices */
+                     float *ewgts,                 /**< weights for all edges */
+                     float *x, float *y, float *z, /**< coordinates for inertial method */
+                     char *  outassignname,        /**< name of assignment output file */
+                     char *  outfilename,          /**< output file name */
+                     int *   assignment,           /**< set number of each vtx (length n) */
+                     int     architecture,         /**< 0 => hypercube, d => d-dimensional mesh */
+                     int     ndims_tot,            /**< total number of cube dimensions to divide */
+                     int     mesh_dims[3],         /**< dimensions of mesh of processors */
+                     double *goal,                 /**< desired set sizes for each set */
+                     int     global_method,        /**< global partitioning algorithm */
+                     int     local_method,         /**< local partitioning algorithm */
+                     int     rqi_flag,             /**< should I use RQI/Symmlq eigensolver? */
+                     int     vmax,                 /**< how many vertices to coarsen down to? */
+                     int     ndims,                /**< number of eigenvectors (2^d sets) */
+                     double  eigtol,               /**< tolerance on eigenvectors */
+                     long    seed);                /**< for random graph mutations */
 
 /* Chaco interface to read assignment vector from file */
 extern int input_assign(FILE *, char *, int, int *);
