@@ -42,7 +42,7 @@ The function ex_create() creates a new exodus file and returns an ID
 that can subsequently be used to refer to the file.
 
 All floating point values in an exodus file are stored as either
-4-byte (\c float) or 8-byte (\c double) numbers; no mixing of 4- and
+4-byte (float) or 8-byte (double) numbers; no mixing of 4- and
 8-byte numbers in a single file is allowed. An application code can
 compute either 4- or 8-byte values and can designate that the values
 be stored in the exodus file as either 4- or 8-byte numbers;
@@ -66,17 +66,17 @@ causes of errors include:
             path name (from the current directory).
 
 \param cmode Mode. Use one of the following predefined constants:
-\arg \c EX_NOCLOBBER  To create the new file only if the given file name does
+\arg EX_NOCLOBBER  To create the new file only if the given file name does
 not refer to a
                       file that already exists.
 
-\arg \c EX_CLOBBER    To create the new file, regardless of whether a file with
+\arg EX_CLOBBER    To create the new file, regardless of whether a file with
 the same
                       name already exists. If a file with the same name does
 exist, its
                       contents will be erased.
 
-\arg \c EX_64BIT_OFFSET To create a model that can store individual datasets
+\arg EX_64BIT_OFFSET To create a model that can store individual datasets
 larger than
                         2 gigabytes. This modifies the internal storage used by
 exodusII and
@@ -84,27 +84,27 @@ exodusII and
 offset'
                         mode. See largemodel for more details on this
                         mode. A large model file will also be created if the
-                        environment variable \c EXODUS_LARGE_MODEL is defined
+                        environment variable EXODUS_LARGE_MODEL is defined
                         in the users environment. A message will be printed to
 standard output
                         if this environment variable is found. EX_LARGE_MODEL is
 alias.
 
-\arg \c EX_NORMAL_MODEL Create a standard model.
+\arg EX_NORMAL_MODEL Create a standard model.
 
-\arg \c EX_NETCDF4	To create a model using the HDF5-based NetCDF-4
+\arg EX_NETCDF4	To create a model using the HDF5-based NetCDF-4
                         output. An HDF5-based NetCDF-4 file will also be created
 if the
-                        environment variable \c EXODUS_NETCDF4 is defined in the
+                        environment variable EXODUS_NETCDF4 is defined in the
                         users environment. A message will be printed to standard
 output if
                         this environment variable is found.
 
-\arg \c EX_NOSHARE	Do not open the underlying NetCDF file in \e share
+\arg EX_NOSHARE	Do not open the underlying NetCDF file in \e share
 mode. See the
                         NetCDF documentation for more details.
 
-\arg \c EX_SHARE	Do open the underlying NetCDF file in \e share mode. See
+\arg EX_SHARE	Do open the underlying NetCDF file in \e share mode. See
 the NetCDF
                         documentation for more details.
 
@@ -126,7 +126,7 @@ and include files.
 
 The following code segment creates an exodus file called \file{test.exo}:
 
-\code
+~~~{.c}
 #include "exodusII.h"
 int CPU_word_size, IO_word_size, exoid;
 CPU_word_size = sizeof(float);      \comment{use float or double}
@@ -137,7 +137,7 @@ exoid = ex_create ("test.exo"       \comment{filename path}
                     EX_CLOBBER,     \comment{create mode}
                     &CPU_word_size, \comment{CPU float word size in bytes}
                     &IO_word_size); \comment{I/O float word size in bytes}
-\endcode
+~~~
 
 */
 #include "exodusII.h"
@@ -290,10 +290,10 @@ int ex_create_int(const char *path, int cmode, int *comp_ws, int *io_ws, int run
       option = getenv("EXODUS_NETCDF4");
       if (option != NULL) {
         netcdf4_mode = NC_NETCDF4;
-	if (option[0] != 'q') {
-	  fprintf(stderr, "EXODUS: Using netcdf version 4 selected via "
-		  "EXODUS_NETCDF4 environment variable\n");
-	}
+        if (option[0] != 'q') {
+          fprintf(stderr, "EXODUS: Using netcdf version 4 selected via "
+                          "EXODUS_NETCDF4 environment variable\n");
+        }
       }
       else {
         netcdf4_mode = 0;
