@@ -180,7 +180,7 @@ int ex_put_concat_all_blocks(int exoid, const ex_block_params *param)
       snprintf(errmsg, MAX_ERR_LENGTH,                                                             \
                "ERROR: failed to locate " TNAME " block status in file id %d", exoid);             \
       ex_err("ex_put_concat_all_blocks", errmsg, exerrval);                                        \
-      ex_safe_free(GSTAT);                                                                         \
+      free(GSTAT);                                                                                 \
       return (EX_FATAL);                                                                           \
     }                                                                                              \
                                                                                                    \
@@ -191,7 +191,7 @@ int ex_put_concat_all_blocks(int exoid, const ex_block_params *param)
       snprintf(errmsg, MAX_ERR_LENGTH,                                                             \
                "ERROR: failed to store " TNAME " block status array to file id %d", exoid);        \
       ex_err("ex_put_concat_all_blocks", errmsg, exerrval);                                        \
-      ex_safe_free(GSTAT);                                                                         \
+      free(GSTAT);                                                                                 \
       return (EX_FATAL);                                                                           \
     }                                                                                              \
                                                                                                    \
