@@ -347,8 +347,8 @@ int main(int argc, char **argv)
     ebids[5] = 15;
     ebids[6] = 16;
 
-    error =
-        ex_put_elem_block(exoid, ebids[0], "quad", num_elem_in_block[0], num_nodes_per_elem[0], 1);
+    error = ex_put_block(exoid, EX_ELEM_BLOCK, ebids[0], "quad", num_elem_in_block[0],
+                         num_nodes_per_elem[0], 1);
     printf("after ex_put_elem_block, error = %d\n", error);
 
     if (error) {
@@ -363,8 +363,8 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error =
-        ex_put_elem_block(exoid, ebids[1], "quad", num_elem_in_block[1], num_nodes_per_elem[1], 1);
+    error = ex_put_block(exoid, EX_ELEM_BLOCK, ebids[1], "quad", num_elem_in_block[1],
+                         num_nodes_per_elem[1], 1);
     printf("after ex_put_elem_block, error = %d\n", error);
 
     if (error) {
@@ -372,8 +372,8 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error =
-        ex_put_elem_block(exoid, ebids[2], "hex", num_elem_in_block[2], num_nodes_per_elem[2], 1);
+    error = ex_put_block(exoid, EX_ELEM_BLOCK, ebids[2], "hex", num_elem_in_block[2],
+                         num_nodes_per_elem[2], 1);
     printf("after ex_put_elem_block, error = %d\n", error);
 
     if (error) {
@@ -381,8 +381,8 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error =
-        ex_put_elem_block(exoid, ebids[3], "tetra", num_elem_in_block[3], num_nodes_per_elem[3], 1);
+    error = ex_put_block(exoid, EX_ELEM_BLOCK, ebids[3], "tetra", num_elem_in_block[3],
+                         num_nodes_per_elem[3], 1);
     printf("after ex_put_elem_block, error = %d\n", error);
 
     if (error) {
@@ -390,8 +390,8 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error =
-        ex_put_elem_block(exoid, ebids[4], "wedge", num_elem_in_block[4], num_nodes_per_elem[4], 1);
+    error = ex_put_block(exoid, EX_ELEM_BLOCK, ebids[4], "wedge", num_elem_in_block[4],
+                         num_nodes_per_elem[4], 1);
     printf("after ex_put_elem_block, error = %d\n", error);
 
     if (error) {
@@ -399,8 +399,8 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error =
-        ex_put_elem_block(exoid, ebids[5], "tetra", num_elem_in_block[5], num_nodes_per_elem[5], 1);
+    error = ex_put_block(exoid, EX_ELEM_BLOCK, ebids[5], "tetra", num_elem_in_block[5],
+                         num_nodes_per_elem[5], 1);
     printf("after ex_put_elem_block, error = %d\n", error);
 
     if (error) {
@@ -408,8 +408,8 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error =
-        ex_put_elem_block(exoid, ebids[6], "tri", num_elem_in_block[6], num_nodes_per_elem[6], 1);
+    error = ex_put_block(exoid, EX_ELEM_BLOCK, ebids[6], "tri", num_elem_in_block[6],
+                         num_nodes_per_elem[6], 1);
     printf("after ex_put_elem_block, error = %d\n", error);
 
     if (error) {
@@ -498,7 +498,7 @@ int main(int argc, char **argv)
     connect[2] = 3;
     connect[3] = 4;
 
-    error = ex_put_elem_conn(exoid, ebids[0], connect);
+    error = ex_put_conn(exoid, EX_ELEM_BLOCK, ebids[0], connect);
     printf("after ex_put_elem_conn, error = %d\n", error);
     if (error) {
       ex_close(rootid);
@@ -510,7 +510,7 @@ int main(int argc, char **argv)
     connect[2] = 7;
     connect[3] = 8;
 
-    error = ex_put_elem_conn(exoid, ebids[1], connect);
+    error = ex_put_conn(exoid, EX_ELEM_BLOCK, ebids[1], connect);
     printf("after ex_put_elem_conn, error = %d\n", error);
 
     if (error) {
@@ -527,7 +527,7 @@ int main(int argc, char **argv)
     connect[6] = 15;
     connect[7] = 16;
 
-    error = ex_put_elem_conn(exoid, ebids[2], connect);
+    error = ex_put_conn(exoid, EX_ELEM_BLOCK, ebids[2], connect);
     printf("after ex_put_elem_conn, error = %d\n", error);
 
     if (error) {
@@ -540,7 +540,7 @@ int main(int argc, char **argv)
     connect[2] = 19;
     connect[3] = 20;
 
-    error = ex_put_elem_conn(exoid, ebids[3], connect);
+    error = ex_put_conn(exoid, EX_ELEM_BLOCK, ebids[3], connect);
     printf("after ex_put_elem_conn, error = %d\n", error);
 
     if (error) {
@@ -555,7 +555,7 @@ int main(int argc, char **argv)
     connect[4] = 25;
     connect[5] = 26;
 
-    error = ex_put_elem_conn(exoid, ebids[4], connect);
+    error = ex_put_conn(exoid, EX_ELEM_BLOCK, ebids[4], connect);
     printf("after ex_put_elem_conn, error = %d\n", error);
 
     if (error) {
@@ -572,7 +572,7 @@ int main(int argc, char **argv)
     connect[6] = 30;
     connect[7] = 29;
 
-    error = ex_put_elem_conn(exoid, ebids[5], connect);
+    error = ex_put_conn(exoid, EX_ELEM_BLOCK, ebids[5], connect);
     printf("after ex_put_elem_conn, error = %d\n", error);
 
     if (error) {
@@ -584,7 +584,7 @@ int main(int argc, char **argv)
     connect[1] = 32;
     connect[2] = 33;
 
-    error = ex_put_elem_conn(exoid, ebids[6], connect);
+    error = ex_put_conn(exoid, EX_ELEM_BLOCK, ebids[6], connect);
     printf("after ex_put_elem_conn, error = %d\n", error);
 
     if (error) {
@@ -597,14 +597,14 @@ int main(int argc, char **argv)
     /* write element block attributes */
 
     attrib[0] = 3.14159;
-    error     = ex_put_elem_attr(exoid, ebids[0], attrib);
+    error     = ex_put_attr(exoid, EX_ELEM_BLOCK, ebids[0], attrib);
     printf("after ex_put_elem_attr, error = %d\n", error);
     if (error) {
       ex_close(rootid);
       exit(-1);
     }
 
-    error = ex_put_elem_attr(exoid, ebids[0], attrib);
+    error = ex_put_attr(exoid, EX_ELEM_BLOCK, ebids[0], attrib);
     printf("after ex_put_elem_attr, error = %d\n", error);
     if (error) {
       ex_close(rootid);
@@ -612,7 +612,7 @@ int main(int argc, char **argv)
     }
 
     attrib[0] = 6.14159;
-    error     = ex_put_elem_attr(exoid, ebids[1], attrib);
+    error     = ex_put_attr(exoid, EX_ELEM_BLOCK, ebids[1], attrib);
     printf("after ex_put_elem_attr, error = %d\n", error);
 
     if (error) {
@@ -620,7 +620,7 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error = ex_put_elem_attr(exoid, ebids[2], attrib);
+    error = ex_put_attr(exoid, EX_ELEM_BLOCK, ebids[2], attrib);
     printf("after ex_put_elem_attr, error = %d\n", error);
 
     if (error) {
@@ -628,7 +628,7 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error = ex_put_elem_attr(exoid, ebids[3], attrib);
+    error = ex_put_attr(exoid, EX_ELEM_BLOCK, ebids[3], attrib);
     printf("after ex_put_elem_attr, error = %d\n", error);
 
     if (error) {
@@ -636,7 +636,7 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error = ex_put_elem_attr(exoid, ebids[4], attrib);
+    error = ex_put_attr(exoid, EX_ELEM_BLOCK, ebids[4], attrib);
     printf("after ex_put_elem_attr, error = %d\n", error);
 
     if (error) {
@@ -644,7 +644,7 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error = ex_put_elem_attr(exoid, ebids[5], attrib);
+    error = ex_put_attr(exoid, EX_ELEM_BLOCK, ebids[5], attrib);
     printf("after ex_put_elem_attr, error = %d\n", error);
 
     if (error) {
@@ -652,7 +652,7 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error = ex_put_elem_attr(exoid, ebids[6], attrib);
+    error = ex_put_attr(exoid, EX_ELEM_BLOCK, ebids[6], attrib);
     printf("after ex_put_elem_attr, error = %d\n", error);
 
     if (error) {
@@ -662,7 +662,7 @@ int main(int argc, char **argv)
 
     attrib_names[0] = "THICKNESS";
     for (i = 0; i < 7; i++) {
-      error = ex_put_elem_attr_names(exoid, ebids[i], attrib_names);
+      error = ex_put_attr_names(exoid, EX_ELEM_BLOCK, ebids[i], attrib_names);
       printf("after ex_put_elem_attr_names, error = %d\n", error);
       if (error) {
         ex_close(rootid);
@@ -678,7 +678,7 @@ int main(int argc, char **argv)
     nsids[0] = 20;
     nsids[1] = 21;
 
-    error = ex_put_node_set_param(exoid, nsids[0], 5, 5);
+    error = ex_put_set_param(exoid, EX_NODE_SET, nsids[0], 5, 5);
     printf("after ex_put_node_set_param, error = %d\n", error);
 
     if (error) {
@@ -698,21 +698,21 @@ int main(int argc, char **argv)
     dist_fact[3] = 4.0;
     dist_fact[4] = 5.0;
 
-    error = ex_put_node_set(exoid, nsids[0], node_list);
+    error = ex_put_set(exoid, EX_NODE_SET, nsids[0], node_list);
     printf("after ex_put_node_set, error = %d\n", error);
     if (error) {
       ex_close(rootid);
       exit(-1);
     }
 
-    error = ex_put_node_set_dist_fact(exoid, nsids[0], dist_fact);
+    error = ex_put_set_dist_fact(exoid, EX_NODE_SET, nsids[0], dist_fact);
     printf("after ex_put_node_set_dist_fact, error = %d\n", error);
     if (error) {
       ex_close(rootid);
       exit(-1);
     }
 
-    error = ex_put_node_set_param(exoid, nsids[1], 3, 3);
+    error = ex_put_set_param(exoid, EX_NODE_SET, nsids[1], 3, 3);
     printf("after ex_put_node_set_param, error = %d\n", error);
 
     if (error) {
@@ -728,14 +728,14 @@ int main(int argc, char **argv)
     dist_fact[1] = 2.1;
     dist_fact[2] = 3.1;
 
-    error = ex_put_node_set(exoid, nsids[1], node_list);
+    error = ex_put_set(exoid, EX_NODE_SET, nsids[1], node_list);
     printf("after ex_put_node_set, error = %d\n", error);
     if (error) {
       ex_close(rootid);
       exit(-1);
     }
 
-    error = ex_put_node_set_dist_fact(exoid, nsids[1], dist_fact);
+    error = ex_put_set_dist_fact(exoid, EX_NODE_SET, nsids[1], dist_fact);
     printf("after ex_put_node_set_dist_fact, error = %d\n", error);
     if (error) {
       ex_close(rootid);
@@ -818,7 +818,7 @@ int main(int argc, char **argv)
 
     /* side set #1  - quad */
 
-    error = ex_put_side_set_param(exoid, ssids[0], 2, 4);
+    error = ex_put_set_param(exoid, EX_SIDE_SET, ssids[0], 2, 4);
     printf("after ex_put_side_set_param, error = %d\n", error);
 
     if (error) {
@@ -837,7 +837,7 @@ int main(int argc, char **argv)
     dist_fact[2] = 30.2;
     dist_fact[3] = 30.3;
 
-    error = ex_put_side_set(exoid, 30, elem_list, side_list);
+    error = ex_put_set(exoid, EX_SIDE_SET, 30, elem_list, side_list);
     printf("after ex_put_side_set, error = %d\n", error);
 
     if (error) {
@@ -845,7 +845,7 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error = ex_put_side_set_dist_fact(exoid, 30, dist_fact);
+    error = ex_put_set_dist_fact(exoid, EX_SIDE_SET, 30, dist_fact);
     printf("after ex_put_side_set_dist_fact, error = %d\n", error);
 
     if (error) {
@@ -855,7 +855,7 @@ int main(int argc, char **argv)
 
     /* side set #2  - quad, spanning 2 elements  */
 
-    error = ex_put_side_set_param(exoid, 31, 2, 4);
+    error = ex_put_set_param(exoid, EX_SIDE_SET, 31, 2, 4);
     printf("after ex_put_side_set_param, error = %d\n", error);
 
     if (error) {
@@ -874,7 +874,7 @@ int main(int argc, char **argv)
     dist_fact[2] = 31.2;
     dist_fact[3] = 31.3;
 
-    error = ex_put_side_set(exoid, 31, elem_list, side_list);
+    error = ex_put_set(exoid, EX_SIDE_SET, 31, elem_list, side_list);
     printf("after ex_put_side_set, error = %d\n", error);
 
     if (error) {
@@ -882,7 +882,7 @@ int main(int argc, char **argv)
       exit(-1);
     }
 
-    error = ex_put_side_set_dist_fact(exoid, 31, dist_fact);
+    error = ex_put_set_dist_fact(exoid, EX_SIDE_SET, 31, dist_fact);
     printf("after ex_put_side_set_dist_fact, error = %d\n", error);
 
     if (error) {
@@ -892,7 +892,7 @@ int main(int argc, char **argv)
 
     /* side set #3  - hex */
 
-    error = ex_put_side_set_param(exoid, 32, 7, 0);
+    error = ex_put_set_param(exoid, EX_SIDE_SET, 32, 7, 0);
     printf("after ex_put_side_set_param, error = %d\n", error);
 
     if (error) {
@@ -916,7 +916,7 @@ int main(int argc, char **argv)
     side_list[5] = 1;
     side_list[6] = 6;
 
-    error = ex_put_side_set(exoid, 32, elem_list, side_list);
+    error = ex_put_set(exoid, EX_SIDE_SET, 32, elem_list, side_list);
     printf("after ex_put_side_set, error = %d\n", error);
 
     if (error) {
@@ -926,7 +926,7 @@ int main(int argc, char **argv)
 
     /* side set #4  - tetras */
 
-    error = ex_put_side_set_param(exoid, 33, 8, 0);
+    error = ex_put_set_param(exoid, EX_SIDE_SET, 33, 8, 0);
     printf("after ex_put_side_set_param, error = %d\n", error);
 
     if (error) {
@@ -952,7 +952,7 @@ int main(int argc, char **argv)
     side_list[6] = 3;
     side_list[7] = 4;
 
-    error = ex_put_side_set(exoid, 33, elem_list, side_list);
+    error = ex_put_set(exoid, EX_SIDE_SET, 33, elem_list, side_list);
     printf("after ex_put_side_set, error = %d\n", error);
 
     if (error) {
@@ -962,7 +962,7 @@ int main(int argc, char **argv)
 
     /* side set #5  - wedges and tris */
 
-    error = ex_put_side_set_param(exoid, 34, 10, 0);
+    error = ex_put_set_param(exoid, EX_SIDE_SET, 34, 10, 0);
     printf("after ex_put_side_set_param, error = %d\n", error);
 
     if (error) {
@@ -992,7 +992,7 @@ int main(int argc, char **argv)
     side_list[8] = 4;
     side_list[9] = 5;
 
-    error = ex_put_side_set(exoid, 34, elem_list, side_list);
+    error = ex_put_set(exoid, EX_SIDE_SET, 34, elem_list, side_list);
     printf("after ex_put_side_set, error = %d\n", error);
 
     if (error) {
@@ -1074,15 +1074,15 @@ int main(int argc, char **argv)
 
     var_names[0] = "glo_vars";
 
-    error = ex_put_var_param(exoid, "g", num_glo_vars);
-    printf("after ex_put_var_param, error = %d\n", error);
+    error = ex_put_variable_param(exoid, EX_GLOBAL, num_glo_vars);
+    printf("after ex_put_variable_param, error = %d\n", error);
     if (error) {
       ex_close(rootid);
       exit(-1);
     }
 
-    error = ex_put_var_names(exoid, "g", num_glo_vars, var_names);
-    printf("after ex_put_var_names, error = %d\n", error);
+    error = ex_put_variable_names(exoid, EX_GLOBAL, num_glo_vars, var_names);
+    printf("after ex_put_variable_names, error = %d\n", error);
     if (error) {
       ex_close(rootid);
       exit(-1);
@@ -1093,15 +1093,15 @@ int main(int argc, char **argv)
     var_names[0] = "node_variable_a_very_long_name_0";
     var_names[1] = "nod_var1";
 
-    error = ex_put_var_param(exoid, "n", num_nod_vars);
-    printf("after ex_put_var_param, error = %d\n", error);
+    error = ex_put_variable_param(exoid, EX_NODAL, num_nod_vars);
+    printf("after ex_put_variable_param, error = %d\n", error);
     if (error) {
       ex_close(rootid);
       exit(-1);
     }
 
-    error = ex_put_var_names(exoid, "n", num_nod_vars, var_names);
-    printf("after ex_put_var_names, error = %d\n", error);
+    error = ex_put_variable_names(exoid, EX_NODAL, num_nod_vars, var_names);
+    printf("after ex_put_variable_names, error = %d\n", error);
     if (error) {
       ex_close(rootid);
       exit(-1);
@@ -1114,15 +1114,15 @@ int main(int argc, char **argv)
     var_names[1] = "this_variable_name_is_just_right";
     var_names[2] = "this_variable_name_is_tooooo_long";
 
-    error = ex_put_var_param(exoid, "e", num_ele_vars);
-    printf("after ex_put_var_param, error = %d\n", error);
+    error = ex_put_variable_param(exoid, EX_ELEM_BLOCK, num_ele_vars);
+    printf("after ex_put_variable_param, error = %d\n", error);
     if (error) {
       ex_close(rootid);
       exit(-1);
     }
 
-    error = ex_put_var_names(exoid, "e", num_ele_vars, var_names);
-    printf("after ex_put_var_names, error = %d\n", error);
+    error = ex_put_variable_names(exoid, EX_ELEM_BLOCK, num_ele_vars, var_names);
+    printf("after ex_put_variable_names, error = %d\n", error);
     if (error) {
       ex_close(rootid);
       exit(-1);
@@ -1135,15 +1135,15 @@ int main(int argc, char **argv)
       var_names[1] = "ns_var1";
       var_names[2] = "ns_var2";
 
-      error = ex_put_var_param(exoid, "m", num_nset_vars);
-      printf("after ex_put_var_param, error = %d\n", error);
+      error = ex_put_variable_param(exoid, "m", num_nset_vars);
+      printf("after ex_put_variable_param, error = %d\n", error);
       if (error) {
         ex_close(rootid);
         exit(-1);
       }
 
-      error = ex_put_var_names(exoid, "m", num_nset_vars, var_names);
-      printf("after ex_put_var_names, error = %d\n", error);
+      error = ex_put_variable_names(exoid, "m", num_nset_vars, var_names);
+      printf("after ex_put_variable_names, error = %d\n", error);
       if (error) {
         ex_close(rootid);
         exit(-1);
@@ -1157,15 +1157,15 @@ int main(int argc, char **argv)
       var_names[1] = "ss_var1";
       var_names[2] = "ss_var2";
 
-      error = ex_put_var_param(exoid, "s", num_sset_vars);
-      printf("after ex_put_var_param, error = %d\n", error);
+      error = ex_put_variable_param(exoid, EX_SIDE_SET, num_sset_vars);
+      printf("after ex_put_variable_param, error = %d\n", error);
       if (error) {
         ex_close(rootid);
         exit(-1);
       }
 
-      error = ex_put_var_names(exoid, "s", num_sset_vars, var_names);
-      printf("after ex_put_var_names, error = %d\n", error);
+      error = ex_put_variable_names(exoid, EX_SIDE_SET, num_sset_vars, var_names);
+      printf("after ex_put_variable_names, error = %d\n", error);
       if (error) {
         ex_close(rootid);
         exit(-1);
@@ -1183,7 +1183,7 @@ int main(int argc, char **argv)
       }
     }
 
-    error = ex_put_elem_var_tab(exoid, num_elem_blk, num_ele_vars, truth_tab);
+    error = ex_put_truth_table(exoid, EX_ELEM_BLOCK, num_elem_blk, num_ele_vars, truth_tab);
     printf("after ex_put_elem_var_tab, error = %d\n", error);
 
     if (error) {
@@ -1227,7 +1227,7 @@ int main(int argc, char **argv)
         glob_var_vals[j] = (float)(j + 2) * time_value;
       }
 
-      error = ex_put_glob_vars(exoid, whole_time_step, num_glo_vars, glob_var_vals);
+      error = ex_put_var(exoid, whole_time_step, EX_GLOBAL, 1, 1, num_glo_vars, glob_var_vals);
       printf("after ex_put_glob_vars, error = %d\n", error);
 
       if (error) {
