@@ -41,11 +41,16 @@
 #include <Ioss_Property.h>        // for Property
 #include <Ioss_PropertyManager.h> // for PropertyManager
 #include <Ioss_State.h>           // for State
+#include <Ioss_VariableType.h>    // for component_count()
 #include <stddef.h>               // for size_t, nullptr
 #include <stdint.h>               // for int64_t
 #include <string>                 // for string
 #include <vector>                 // for vector
 
+#ifdef SEACAS_HAVE_KOKKOS
+#include <Kokkos_Core.hpp> // for Kokkos::View
+#endif
+
 namespace Ioss {
 
   class EntityBlock;
@@ -112,7 +117,7 @@ namespace Ioss {
 
     /** \brief Set the name of the entity.
      *
-     *  \param[in] name The new name of the entity.
+     *  \param[in] new_name The new name of the entity.
      */
     void set_name(const std::string &new_name) { entityName = new_name; }
 
@@ -211,13 +216,32 @@ namespace Ioss {
     int put_field_data(const std::string &field_name, std::vector<int64_t> &data) const;
     int put_field_data(const std::string &field_name, std::vector<Complex> &data) const;
 
+#ifdef SEACAS_HAVE_KOKKOS
+    // Get and put this field's data into the specified Kokkos::View.
+    // Returns the number of entities for which the field was read.
+    // Resizes 'data' to size needed to hold all values;
+    // however, any Views that were previously created referencing the same
+    // underlying memory allocation as 'data' will remain the original size.
+    template <typename T>
+    int get_field_data(const std::string &field_name, Kokkos::View<T *> &data) const;
+
+    template <typename T>
+    int get_field_data(const std::string &field_name, Kokkos::View<T **> &data) const;
+
+    template <typename T>
+    int put_field_data(const std::string &field_name, Kokkos::View<T *> &data) const;
+
+    template <typename T>
+    int put_field_data(const std::string &field_name, Kokkos::View<T **> &data) const;
+#endif
+
     /** Get the number of bytes used to store the INT data type
      *
      *  \returns The number of bytes.
      */
     Ioss::Field::BasicType field_int_type() const
     {
-      if (get_database() == nullptr || get_database()->int_byte_size_api() == 4) {
+      if (database_ == nullptr || get_database()->int_byte_size_api() == 4) {
         return Ioss::Field::INT32;
       }
       else {
@@ -405,4 +429,215 @@ inline int Ioss::GroupingEntity::field_describe(Ioss::Field::RoleType role, Name
  */
 inline size_t Ioss::GroupingEntity::field_count() const { return fields.count(); }
 
+#ifdef SEACAS_HAVE_KOKKOS
+
+// Will want to template on Memory space (with a default template value of the default memory space)
+// Will also want to template on static vs. dynamic data, and other View template parameters,
+// with defaults.
+
+/** \brief Read field data from the database file into memory using a 1-D Kokkos:::View.
+ *
+ *  \tparam T The data type
+ *  \param[in] field_name The name of the field to read.
+ *  \param[out] data The data.
+ *  \returns The number of values read.
+ *
+ */
+template <typename T>
+int Ioss::GroupingEntity::get_field_data(const std::string &field_name,
+                                         Kokkos::View<T *> &data) const
+{
+  verify_field_exists(field_name, "input");
+
+  Ioss::Field field = get_field(field_name);
+
+  // Resize the view
+  int new_view_size = field.raw_count() * field.raw_storage()->component_count();
+  Kokkos::resize(data, new_view_size);
+  size_t data_size = new_view_size * sizeof(T);
+
+  // Create a host mirror view. (No memory allocation if data is in HostSpace.)
+  typename Kokkos::View<T *>::HostMirror host_data = Kokkos::create_mirror_view(data);
+
+  // Extract a pointer to the underlying allocated memory of the host view.
+  // Kokkos::View::ptr_on_device() will soon be changed to Kokkos::View::data(),
+  // in which case, TOPTR(data) will work.
+  T *host_data_ptr = host_data.ptr_on_device();
+
+  // Extract the data from disk to the underlying memory pointed to by host_data_ptr.
+  int retval = internal_get_field_data(field, host_data_ptr, data_size);
+
+  // At this point, transform the field if specified...
+  if (retval >= 0)
+    field.transform(host_data_ptr);
+
+  // Copy the data to the device. (No op if data is in HostSpace.)
+  Kokkos::deep_copy(data, host_data);
+
+  return retval;
+}
+
+/** \brief Read field data from the database file into memory using a 2-D Kokkos:::View.
+ *
+ *  \tparam T The data type
+ *  \param[in] field_name The name of the field to read.
+ *  \param[out] data The data.
+ *  \returns The number of values read.
+ *
+ */
+template <typename T>
+int Ioss::GroupingEntity::get_field_data(const std::string & field_name,
+                                         Kokkos::View<T **> &data) const
+{
+  verify_field_exists(field_name, "input");
+
+  Ioss::Field field = get_field(field_name);
+
+  // Resize the view
+  int new_view_size_left  = field.raw_count();
+  int new_view_size_right = field.raw_storage()->component_count();
+  Kokkos::resize(data, new_view_size_left, new_view_size_right);
+  size_t data_size = new_view_size_left * new_view_size_right * sizeof(T);
+
+  // Create and allocate an array to hold the data temporarily.
+  // This is necessary to ensure the data is placed in the correct
+  // location in the 2-D array, avoiding incorrect placement due
+  // to Views with padded dimensions.
+  T *data_array = new T[data_size];
+
+  // Create a host mirror view. (No memory allocation if data is in HostSpace.)
+  typename Kokkos::View<T **>::HostMirror host_data = Kokkos::create_mirror_view(data);
+
+  // Extract the data from disk to the underlying memory pointed to by host_data_ptr.
+  int retval = internal_get_field_data(field, data_array, data_size);
+
+  // At this point, transform the field if specified...
+  if (retval >= 0)
+    field.transform(data_array);
+
+  // Copy the data to the host Mirror view.
+  // The host mirror view has the same layout as the device view.
+  // For CUDA, this will be LayoutLeft. In this case, the loop order
+  // chosen here will be slower than the reversed loop order.
+  // However, The time for this extra in-memory copy is small
+  // compared with the time to copy from disk into memory.
+  for (int i = 0; i < new_view_size_left; ++i) {
+    for (int j = 0; j < new_view_size_right; ++j) {
+      host_data(i, j) = data_array[new_view_size_right * i + j];
+    }
+  }
+
+  // Delete the temporary array
+  delete[] data_array;
+
+  // Copy the data to the device. (No op if data is in HostSpace.)
+  Kokkos::deep_copy(data, host_data);
+
+  return retval;
+}
+
+// Will want to template on Memory space (with a default template value of the default memory space)
+// Will also want to template on static vs. dynamic data, and other View template parameters,
+// with defaults.
+
+/** \brief Write field data from memory into the database file using a 1-D Kokkos::View.
+ *
+ *  \tparam T The data type
+ *  \param[in] field_name The name of the field to write.
+ *  \param[in] data The data.
+ *  \returns The number of values written.
+ *
+ */
+template <typename T>
+int Ioss::GroupingEntity::put_field_data(const std::string &field_name,
+                                         Kokkos::View<T *> &data) const
+{
+  verify_field_exists(field_name, "output");
+
+  Ioss::Field field     = get_field(field_name);
+  size_t      data_size = field.raw_count() * field.raw_storage()->component_count() * sizeof(T);
+
+  // Create a host mirror view. (No memory allocation if data is in HostSpace.)
+  typename Kokkos::View<T *>::HostMirror host_data = Kokkos::create_mirror_view(data);
+
+  // Copy the data to the host. (No op if data is in HostSpace.)
+  Kokkos::deep_copy(host_data, data);
+
+  // Extract a pointer to the underlying allocated memory of the host view.
+  // Kokkos::View::ptr_on_device() will soon be changed to Kokkos::View::data(),
+  // in which case, TOPTR(data) will work.
+  T *host_data_ptr = host_data.ptr_on_device();
+
+  // Transform the field
+  field.transform(host_data_ptr);
+
+  // Copy the data to disk from the underlying memory pointed to by host_data_ptr.
+  return internal_put_field_data(field, host_data_ptr, data_size);
+}
+
+/** \brief Write field data from memory into the database file using a 2-D Kokkos::View.
+ *
+ *  \tparam T The data type
+ *  \param[in] field_name The name of the field to write.
+ *  \param[in] data The data.
+ *  \returns The number of values written.
+ *
+ */
+template <typename T>
+int Ioss::GroupingEntity::put_field_data(const std::string & field_name,
+                                         Kokkos::View<T **> &data) const
+{
+  verify_field_exists(field_name, "output");
+
+  Ioss::Field field = get_field(field_name);
+
+  int    view_size_left  = data.dimension_0();
+  int    view_size_right = data.dimension_1();
+  size_t data_size       = field.raw_count() * field.raw_storage()->component_count() * sizeof(T);
+
+  if (view_size_left * view_size_right * sizeof(T) != data_size) {
+    std::ostringstream errmsg;
+    errmsg << "\nERROR: View dimensions are inconsistent with field raw count or raw storage "
+              "component count"
+           << "for field" << field_name << "\n\n";
+    IOSS_ERROR(errmsg);
+  }
+
+  // Create a host mirror view. (No memory allocation if data is in HostSpace.)
+  typename Kokkos::View<T **>::HostMirror host_data = Kokkos::create_mirror_view(data);
+
+  // Copy the data to the host. (No op if data is in HostSpace.)
+  Kokkos::deep_copy(host_data, data);
+
+  // Create and allocate an array to hold the data temporarily.
+  // This is necessary to ensure the data is taken from the correct
+  // location in the 2-D array, avoiding incorrect location due
+  // to Views with padded dimensions.
+  T *data_array = new T[data_size];
+
+  // Copy the data from the host Mirror view.
+  // The host mirror view has the same layout as the device view.
+  // For CUDA, this will be LayoutLeft. In this case, the loop order
+  // chosen here will be slower than the reversed loop order.
+  // However, The time for this extra in-memory copy is small
+  // compared with the time to copy to disk from memory.
+  for (int i = 0; i < view_size_left; ++i) {
+    for (int j = 0; j < view_size_right; ++j) {
+      data_array[view_size_right * i + j] = host_data(i, j);
+    }
+  }
+
+  // Transform the field
+  field.transform(data_array);
+
+  // Copy the data to disk from the underlying memory pointed to by data_array.
+  int retval = internal_put_field_data(field, data_array, data_size);
+
+  // Delete the temporary array
+  delete[] data_array;
+
+  return retval;
+}
+#endif
+
 #endif
