@@ -76,6 +76,7 @@ namespace Ioss {
   class Region;
   class SideBlock;
   class SideSet;
+  class StructuredBlock;
 }
 
 /** \brief A namespace for the decompose-on-the-fly version of the
@@ -125,6 +126,11 @@ namespace Iopx {
                                size_t data_size) const override;
     int64_t get_field_internal(const Ioss::ElementBlock *eb, const Ioss::Field &field, void *data,
                                size_t data_size) const override;
+    int64_t get_field_internal(const Ioss::StructuredBlock *sb, const Ioss::Field &field,
+                               void *data, size_t data_size) const
+    {
+      return -1;
+    }
     int64_t get_field_internal(const Ioss::SideBlock *fb, const Ioss::Field &field, void *data,
                                size_t data_size) const override;
     int64_t get_field_internal(const Ioss::NodeSet *ns, const Ioss::Field &field, void *data,
@@ -164,6 +170,11 @@ namespace Iopx {
                                size_t data_size) const override;
     int64_t put_field_internal(const Ioss::CommSet *cs, const Ioss::Field &field, void *data,
                                size_t data_size) const override;
+    int64_t put_field_internal(const Ioss::StructuredBlock *sb, const Ioss::Field &field,
+                               void *data, size_t data_size) const
+    {
+      return -1;
+    }
 
     int64_t put_Xset_field_internal(ex_entity_type type, const Ioss::EntitySet *ns,
                                     const Ioss::Field &field, void *data, size_t data_size) const;
@@ -171,7 +182,7 @@ namespace Iopx {
                                     const Ioss::Field &field, void *data, size_t data_size) const;
 
     int get_file_pointer() const override; // Open file and set exodusFilePtr.
-    int free_file_pointer() const override; 
+    int free_file_pointer() const override;
 
     int64_t read_nodal_coordinates();
     void read_elements(const Ioss::ElementBlock &block);
