@@ -108,7 +108,7 @@ C
     3   CONTINUE
     2 CONTINUE
 C
-      CALL EXINQ (NTP2EX,EXTIMS,NUMTIM,FDUM,CDUM,IERR)
+      NUMTIM = EXINQI (NTP2EX,EXTIMS)
       CALL EXGATM (NTP2EX,TIMES,IERR)
       OUTTIM = -1.
       ISTEP = NUMTIM
