@@ -2,23 +2,23 @@
  * Copyright (C) 2009 Sandia Corporation.  Under the terms of Contract
  * DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains
  * certain rights in this software
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
  * met:
- * 
+ *
  *     * Redistributions of source code must retain the above copyright
  *       notice, this list of conditions and the following disclaimer.
- * 
+ *
  *     * Redistributions in binary form must reproduce the above
  *       copyright notice, this list of conditions and the following
  *       disclaimer in the documentation and/or other materials provided
  *       with the distribution.
- * 
+ *
  *     * Neither the name of Sandia Corporation nor the names of its
  *       contributors may be used to endorse or promote products derived
  *       from this software without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -30,26 +30,22 @@
  * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- * 
+ *
  */
 /* cgispst.c - linker specifiable driver routine for driver Postscript B/W (pst) */
 
 #include "ifdefx.h"
 #include "mdcgi.h"
 
-void    cgipst_();      /* tell linker to load driver */
+void cgipst_(); /* tell linker to load driver */
 
-void
-cgispst(void)               /* make name external so linker will load file*/
-{}
+void cgispst(void) /* make name external so linker will load file*/ {}
 
-
-void
-cgi_def_ini(void)
+void cgi_def_ini(void)
 {
-   anything*          devid;
+  anything *devid;
 
-   xcact_ (cgipst_, &devid);
-   xcoon_ ( &devid );
+  xcact_(cgipst_, &devid);
+  xcoon_(&devid);
 
 } /* end cgi_def_ini */
