@@ -11,8 +11,6 @@
 
 #include "basker_nfactor_blk.hpp"
 #include "basker_nfactor_col.hpp"
-#include "basker_nfactor_blk_inc.hpp"
-#include "basker_nfactor_col_inc.hpp"
 
 #ifdef BASKER_KOKKOS
 #include <Kokkos_Core.hpp>
@@ -110,8 +108,8 @@ namespace BaskerNS
     const Int U_col =  S(lvl)(kid);
     Int U_row       =  0;
 	  
-    const Int scol = LU(U_col)(U_row).scol;
-    const Int ecol = LU(U_col)(U_row).ecol;
+    //const Int scol = LU(U_col)(U_row).scol; //Not used
+    //const Int ecol = LU(U_col)(U_row).ecol; //Not used
 
 
     #ifdef BASKER_DEBUG_NFACTOR_COL2
@@ -143,6 +141,8 @@ namespace BaskerNS
     printf("\n\n\n done with UPPER, kid: %d \n\n\n", kid);
     #endif
 
+    
+    // return;
 
 
     //------Need because extend does not 
@@ -201,7 +201,7 @@ namespace BaskerNS
 
       }//for - over all sublevel 1...lvl-2
     
-
+    
 
     //---------Lower Factor (old sublevel lvl-1)-------
     
@@ -216,7 +216,7 @@ namespace BaskerNS
 		     b_size, 3, LU(U_col)(U_row).scol, 0);
 
 
-
+    
 
 
     //printf("\n\n======= LOWER, KID: %d ======= \n\n", kid);
@@ -228,7 +228,7 @@ namespace BaskerNS
     //if(lvl < 2)
       {
 	//for(Int k=0; k < 1; ++k)
-      for(Int k = 0; k < LU(U_col)(U_row).ncol; ++k)
+	for(Int k = 0; k < LU(U_col)(U_row).ncol; ++k)
       {
 
 	#ifdef BASKER_DEBUG_NFACTOR_COL2
@@ -390,20 +390,12 @@ namespace BaskerNS
     const Int L_col     = S(sl)(my_leader);
     Int L_row           = l-sl+1; //Might have to think about th
     const Int U_col     = S(lvl)(kid);
-    Int U_row     =
-      (lvl==1)?(kid%2):S(sl)(kid)%LU_size(U_col);
 
-
-    //printf("test \n");
-    if((S(sl)(kid) > 14) &&
-       (S(sl)(kid) > LU_size(U_col)) &&
-       (lvl != 1))
-      {
-	//printf("lower offdiag new num, %d %d \n",
-	//     S(sl)(kid), LU_size(U_col));
-	Int tm = (S(sl)(kid)+1)/16;
-	U_row = ((S(sl)(kid)+1) - (tm*16))%LU_size(U_col);
-      }
+    Int my_row_leader  = S(0)(find_leader(kid,lvl-1));
+    //Int my_new_row = 
+    // L_col - my_row_leader;
+    Int U_row     = L_col-my_row_leader;
+   
 
     const Int X_col     = S(0)(my_leader);
     Int X_row     = l+1; //this will change for us 
@@ -413,7 +405,7 @@ namespace BaskerNS
     Int col_idx_offset  = 0;
     
     BASKER_MATRIX     &U   = LU(U_col)(U_row);
-    const Int         bcol = U.scol;
+    //const Int         bcol = U.scol;
          
     #ifdef BASKER_DEBUG_NFACTOR_COL2
     if(L_row >= LL_size(L_col))
@@ -432,17 +424,17 @@ namespace BaskerNS
     printf("Upper_fact_offdiag, kid: %d leader: %d l: %d lvl: %d works_size: %d X: %d %d L: %d %d U: %d %d k: %d \n",
 	   kid, my_leader, l, lvl, LL_size[X_col], X_col, X_row, L_col, L_row, U_col, U_row,  k+U.scol);
       }
-    #endif
+     #endif
 
-    #ifdef BASKER_DEBUG_NFACTOR_COL2
+     #ifdef BASKER_DEBUG_NFACTOR_COL2
     //if(lower == BASKER_TRUE)
       {
     printf("OFF-DIAG, kid: %d, l: %d  X: %d %d L: %d %d \n",
 	   kid, l,X_col, X_row, L_col, L_row);
       }
-    #endif
+     #endif
 
-    const BASKER_BOOL A_option = BASKER_FALSE;
+      //const BASKER_BOOL A_option = BASKER_FALSE;
 	    
     t_dense_back_solve_offdiag(kid,
 			       L_col, L_row,
@@ -513,7 +505,7 @@ namespace BaskerNS
     //should remove either as a paramter or here
     Int team_leader      = find_leader(kid, sl);
     const Int leader_idx = S(0)(team_leader);
-    Int loop_col_idx     = S(l)(kid);
+    //Int loop_col_idx     = S(l)(kid);
 
     //#ifdef BASKER_DEBUG_NFACTOR_COL2
     if(lower == BASKER_TRUE)
@@ -551,12 +543,12 @@ namespace BaskerNS
 	Int      p_sizeL  = LL(leader_idx)(blk).p_size;
 	ENTRY_1DARRAY &X  = LL(my_idx)(blk).ews;
 	INT_1DARRAY   &ws = LL(my_idx)(blk).iws;
-	const Int ws_size = LL(my_idx)(blk).iws_size;
-	Int       p_size  = LL(my_idx)(blk).p_size;
+	//const Int ws_size = LL(my_idx)(blk).iws_size;
+	//Int       p_size  = LL(my_idx)(blk).p_size;
 	Int       *color  = &(ws[0]);
-	Int     *pattern  = &(color[ws_size]); 
-	Int      brow     = LL(my_idx)(blk).srow;
-	Int      browL    = LL(leader_idx)(blk).srow;
+	//Int     *pattern  = &(color[ws_size]); 
+	//Int      brow     = LL(my_idx)(blk).srow;
+	//Int      browL    = LL(leader_idx)(blk).srow;
 
 	#ifdef BASKER_DEBUG_NFACTOR_COL2
 	if(lower == BASKER_TRUE)
@@ -566,8 +558,8 @@ namespace BaskerNS
 	  }
 	#endif
 
-	Int *colorL   = &(wsL(0));
-	Int *patternL = &(colorL[ws_size]);
+	//Int *colorL   = &(wsL(0));
+	//Int *patternL = &(colorL[ws_size]);
 
         #ifdef BASKER_DEBUG_NFACTOR_COL2
 	if(lower == BASKER_TRUE)
@@ -641,7 +633,7 @@ namespace BaskerNS
 		   p_sizeL, leader_idx, blk, kid);
 	    #endif
 	    LL(leader_idx)(blk).p_size = p_sizeL;
-	    p_size = 0;
+	    //p_size = 0; //not needed
 	  }//over all blks
 	  }
       }//if not team_leader
@@ -683,22 +675,15 @@ namespace BaskerNS
     //Copy B -> C
     Int bl = l+1;
     Int A_col = S(lvl)(kid);
-    Int A_row = (lvl==1)?(2):S(bl)(kid)%(LU_size(A_col));
 
-    if((S(bl)(kid) > 14) &&
-       (S(bl)(kid) > LU_size(A_col)) &&
-       (lvl != 1))
-      {
-	//printf("test cm %d %d %d \n",
-	//     kid, S(bl)(kid), LU_size(A_col));
-
-	Int tm = (S(bl)(kid)+1)/16;
-	A_row  = ((S(bl)(kid)+1) - (tm*16))%LU_size(A_col);
-      } 
+    Int my_row_leader = S(0)(find_leader(kid,lvl-1));
+    //Int my_new_row = 
+    // S(bl)(kid) - my_row_leader;
+    Int A_row = S(bl)(kid) - my_row_leader;
+    
 
 
-
-    Int CM_idx = kid;
+    //Int CM_idx = kid;
     
     BASKER_MATRIX  *Bp;
     if(A_row != (LU_size(A_col)-1))
@@ -718,15 +703,15 @@ namespace BaskerNS
     //B.info();
     //B.print();
     
-    Int team_leader   = find_leader(kid, l);
+    //Int team_leader   = find_leader(kid, l);
     ENTRY_1DARRAY   X = LL(leader_idx)(bl).ews;
     INT_1DARRAY    ws = LL(leader_idx)(bl).iws;
-    const Int brow    = LL(leader_idx)(bl).srow;
-    const Int nrow    = LL(leader_idx)(bl).nrow;
-    Int p_size        = LL(leader_idx)(bl).p_size;
-    const Int ws_size = LL(leader_idx)(bl).iws_size;
-    Int *color        = &(ws(0));
-    Int *pattern      = &(color[ws_size]);
+    //const Int brow    = LL(leader_idx)(bl).srow;
+    //const Int nrow    = LL(leader_idx)(bl).nrow;
+    //Int p_size        = LL(leader_idx)(bl).p_size;
+    //const Int ws_size = LL(leader_idx)(bl).iws_size;
+    //Int *color        = &(ws(0));
+    //Int *pattern      = &(color[ws_size]);
 
 
     #ifdef BASKER_DEBUG_NFACTOR_COL2
@@ -735,13 +720,13 @@ namespace BaskerNS
 	   kid, bl, A_col, A_row);
     #endif
 
-    const Int bbcol = B.scol;
+    //const Int bbcol = B.scol;
    
     for(Int i = B.col_ptr(k);
 	i < B.col_ptr(k+1); ++i)
       {
 	Int B_row = B.row_idx(i);	
-	Int j = gperm(B_row+B.srow);
+	//Int j = gperm(B_row+B.srow);
 
         #ifdef BASKER_DEBUG_NFACTOR_COL22
 	printf("Scanning_2 A: %d %d lvl %d l: %d bl:%d brow: % d %d K: %d \n",
@@ -767,10 +752,10 @@ namespace BaskerNS
       {
 
 	Int bl = l+1;
-    Int A_col = S(lvl)(kid);
+        //Int A_col = S(lvl)(kid);
+  
+    /*
     Int A_row = (lvl==1)?(2):S(bl)(kid)%(LU_size(A_col));
-
-
     //maybe no???
      if((S(bl)(kid) > 14) &&
        (S(bl)(kid) > LU_size(A_col)) &&
@@ -785,22 +770,22 @@ namespace BaskerNS
 
      // printf("kid: %d leader_idx: %d bl: %d \n",
      //	    kid, leader_idx, bl);
+     */
 
-
-    Int CM_idx = kid;
+    //Int CM_idx = kid;
     ENTRY_1DARRAY   X   = LL(leader_idx)(bl).ews;
     INT_1DARRAY    ws   = LL(leader_idx)(bl).iws;
-    const Int   ws_size = LL(leader_idx)(bl).ews_size;
-    const Int      brow = LL(leader_idx)(bl).srow;
+    //const Int   ws_size = LL(leader_idx)(bl).ews_size;
+    //const Int      brow = LL(leader_idx)(bl).srow;
     const Int      nrow = LL(leader_idx)(bl).nrow;
-    Int p_size          = LL[leader_idx][bl].p_size;
+    //Int p_size          = LL[leader_idx][bl].p_size;
 
     //For recounting patterns in dense blk
     //Need better sparse update
-    Int p_count  =0 ; 
+    //Int p_count  =0 ; 
     
     Int *color   = &(ws(0));
-    Int *pattern = &(color[ws_size]);
+    //Int *pattern = &(color[ws_size]);
     
     #ifdef BASKER_DEBUG_NFACTOR_COL2
     printf("moving, kid: %d  A: %d %d %d %d p_size: %d \n", 
@@ -889,11 +874,11 @@ namespace BaskerNS
     BASKER_MATRIX        &U = LU(U_col)(U_row); 
     
     INT_1DARRAY     ws = LL(X_col)(X_row).iws;
-    const Int  ws_size = LL(X_col)(X_row).iws_size;
+    //const Int  ws_size = LL(X_col)(X_row).iws_size;
     ENTRY_1DARRAY    X = LL(X_col)(X_row).ews;
 
-    const Int brow     = U.srow;
-    const Int bcol     = U.scol;
+    //const Int brow     = U.srow;
+    //const Int bcol     = U.scol;
 
     pivot = U.tpivot;
     
