@@ -47,7 +47,7 @@ namespace BaskerNS
     Int t_size            = 0;
     Int t_loc             = 0;
     btf_schedule(0) = btf_tabs_offset;
-    BASKER_BOOL  move_fwd = BASKER_TRUE;
+    //BASKER_BOOL  move_fwd = BASKER_TRUE; //NU
 
     for(Int b = btf_tabs_offset; b < btf_nblks;
 	b++)
@@ -127,10 +127,10 @@ namespace BaskerNS
 
     btf_nblks = nblks;
 
-    //#ifdef BASKER_DEBUG_ORDER_BTF
+    #ifdef BASKER_DEBUG_ORDER_BTF
     printf("------------BTF CUT: %d --------------\n", 
 	   btf_tabs(btf_tabs_offset));
-    //#endif
+    #endif
 
     return 0;
   }//end find BTF
@@ -145,15 +145,18 @@ namespace BaskerNS
     Int          nblks = 0;
 
     strong_component(M,nblks,order_btf_array,btf_tabs);
-
     btf_nblks = nblks;
-
     btf_flag = BASKER_TRUE;
 
-    //#ifdef BASKER_DEBUG_ORDER_BTF
+    if(Options.verbose == BASKER_TRUE)
+      {
+	printf("BTF nblks returned: %d \n", nblks);
+      }
+
+    #ifdef BASKER_DEBUG_ORDER_BTF
     printf("BTF nblks returned: %d \n", nblks);
-    BASKER_ASSERT(nblks>1, "NOT ENOUGH BTF BLOCKS");
-    //#endif
+    //BASKER_ASSERT(nblks>1, "NOT ENOUGH BTF BLOCKS");
+    #endif
 
     #ifdef BASKER_DEBUG_ORDER_BTF
     if(nblks<2)
@@ -181,6 +184,30 @@ namespace BaskerNS
     printf("\n");
     #endif
 
+    if (Options.verbose == BASKER_TRUE)
+      {
+	//Print first 3 and last 3 
+	printf("BTF tabs: \n");
+	if (nblks < 10)
+	  {
+	    for(Int i=0; i <nblks+1; i++)
+	      {
+		printf("%d, ", btf_tabs(i));
+	      }
+	  }
+	else
+	  {
+	    printf("%d, %d, %d, ...., ",
+		   btf_tabs(0), btf_tabs(1), btf_tabs(2));
+	    printf("%d, %d, %d",
+		   btf_tabs(nblks-3), btf_tabs(nblks-2),
+		   btf_tabs(nblks-1));
+	  }
+	printf("\n");
+      }//if verbose
+ 
+
+
     permute_col(M, order_btf_array);
     permute_row(M, order_btf_array);
 
@@ -224,7 +251,7 @@ namespace BaskerNS
 
     //changed col to row, error.
     //print to see issue
-    //printMTX("A_AMD.mtx", M);
+    //printMTX("A_TOTAL.mtx", M);
     
        
     break_into_parts2(M, nblks, btf_tabs);
@@ -233,6 +260,12 @@ namespace BaskerNS
     find_btf_schedule(M, nblks, btf_tabs);
 
 
+    if (Options.verbose == BASKER_TRUE)
+      {
+	printf("BTF Cut: %d \n",
+	       btf_tabs(btf_tabs_offset));
+      }
+
     #ifdef BASKER_DEBUG_ORDER_BTF
     printf("------------BTF CUT: %d --------------\n", 
 	   btf_tabs(btf_tabs_offset));
@@ -566,15 +599,25 @@ namespace BaskerNS
     //If nblks  == 1, than only BTF_A exists
     if(nblks == 1)
       {
-        //#ifdef BASKER_DEBUG_ORDER_BTF
+        #ifdef BASKER_DEBUG_ORDER_BTF
 	printf("Short Circuit part_call \n");
-	//#endif
+	#endif
 	BTF_A = A;
 	//Options.btf = BASKER_FALSE;
 	btf_tabs_offset = 1;
 	return 0;
       }
 
+    //Short circuit for incomplete
+    if(Options.incomplete == BASKER_TRUE)
+      {
+	//printf("Short Ciruit incomplete\n");
+	BTF_A = A;
+	btf_nblks = 1;
+	btf_tabs_offset = 1;
+	return 0;
+      }
+
     //Step 1.
     //Find total work estimate
     Int total_work_estimate = 0;
@@ -594,8 +637,10 @@ namespace BaskerNS
 		((double)1/num_threads) + 
 		((double)BASKER_BTF_IMBALANCE)));
 
+    #ifdef BASKER_DEBUG_ORDER_BTF
     printf("Break size: %d \n", break_size);
-
+    #endif
+    
     Int t_size            = 0;
     Int scol              = M.ncol;
     Int blk_idx           = nblks;
@@ -641,13 +686,15 @@ namespace BaskerNS
 	//break due to size
 	else if(blk_work >= break_size)
 	  {
+	    #ifdef BASKER_DEBUG_ORDER_BTF
 	    printf("break due to size\n");
+	    #endif
 	    move_fwd = BASKER_FALSE;
 	  }
 	//break due to end
 	else if(blk_idx == 1)
 	  {
-	    printf("break last blk\n");
+	    //printf("break last blk\n");
 	    blk_idx = 0;
 	    t_size = t_size + blk_size;
 	    scol = btf_tabs[blk_idx];	
@@ -662,15 +709,15 @@ namespace BaskerNS
 	  }
       }//end while(move_fwd)
 
-    //#ifdef BASKER_DEBUG_ORDER_BTF
+    #ifdef BASKER_DEBUG_ORDER_BTF
     printf("Done finding BTF2 cut.  Cut size: %d scol: %d \n",
 	   t_size, scol);
     printf("Done finding BTF2 cut. blk_idx: %d \n", 
 	   blk_idx);
     //BASKER_ASSERT(t_size > 0, "BTF CUT SIZE NOT BIG ENOUGH\n");
     
-    BASKER_ASSERT((scol >= 0) && (scol < M.ncol), "SCOL\n");
-    //#endif
+    BASKER_ASSERT((scol >= 0) && (scol <= M.ncol), "SCOL\n");
+    #endif
     
     //Comeback and change
     btf_tabs_offset = blk_idx;
@@ -740,6 +787,12 @@ namespace BaskerNS
 	   BTF_C.srow, BTF_C.nrow,
 	   BTF_C.scol, BTF_C.nrow);
     #endif
+
+    //Added check
+    if((BTF_C.nrow == 0)||(BTF_C.ncol == 0))
+      {
+	return 0;
+      }
     
     //Scan and find nnz
     //We can do this much better!!!!
@@ -794,6 +847,7 @@ namespace BaskerNS
       }
     if(BTF_C.v_fill == BASKER_FALSE)
       {
+	
 	BASKER_ASSERT(BTF_C.ncol >= 0, "BTF_C.ncol");
 	MALLOC_INT_1DARRAY(BTF_C.col_ptr, BTF_C.ncol+1);
 	BASKER_ASSERT(BTF_C.nnz > 0, "BTF_C.nnz");
@@ -878,6 +932,9 @@ namespace BaskerNS
    )
   {
 
+
+    //printf("===this strong comp called====");
+
     typedef long int   l_int;
     
     INT_1DARRAY perm_in;
@@ -889,6 +946,18 @@ namespace BaskerNS
       {
 	perm_in(i) = i;
       }
+    if(Options.incomplete == BASKER_TRUE)
+      {
+	for(Int i = 0; i < M.ncol; i++)
+	  {
+	    perm(i) = i;
+	  }
+	nblks = 1;
+	CC(0) = 0;
+	CC(1) = M.ncol;
+	return 0;
+      }
+
     //printf("SC one \n");
     //my_strong_component(M,nblks,perm,perm_in, CC);
     BaskerSSWrapper<Int>::my_strong_component(M.ncol,
