@@ -46,6 +46,7 @@
 #include <sstream>
 #include <iomanip>
 #include <string>
+#include <cstring>
 #include <stdexcept>
 #include <cstdlib>
 #include <cctype>
@@ -66,8 +67,21 @@ struct expression_evaluation_exception : public virtual std::exception
   virtual const char* what() const throw() {
     return "Error evaluating expressions";
   }
+
 };
 
+struct expression_undefined_exception : public virtual std::exception
+{
+  virtual const char* what() const throw() {
+    std::string rtnMsg = "Found undefined function with name: " + m_msg + " and " + std::to_string(m_numArgs)  + " argument(s)";
+    return rtnMsg.c_str();
+  }
+
+  expression_undefined_exception(const char *msg, std::size_t numArgs) : m_msg(msg), m_numArgs(numArgs) {}
+
+  std::string m_msg;
+  std::size_t m_numArgs = 0;
+};
 }
 
 namespace stk {
@@ -124,13 +138,19 @@ class Node
   //
 public:
   enum { MAXIMUM_NUMBER_OF_OVERLOADED_FUNCTION_NAMES = 5 };
+  enum { MAXIMUM_FUNCTION_NAME_LENGTH = 32 };
 
-  explicit Node(Opcode opcode)
+  explicit Node(Opcode opcode, Eval* owner)
     : m_opcode(opcode),
-      m_left(0),
-      m_right(0),
-      m_other(0)
-  {m_data.function.undefinedFunction = false;}
+      m_left(nullptr),
+      m_right(nullptr),
+      m_other(nullptr),
+      m_owner(owner)
+  {
+      m_data.function.undefinedFunction = false;
+      for(unsigned i=0; i<MAXIMUM_NUMBER_OF_OVERLOADED_FUNCTION_NAMES; ++i)
+          m_data.function.function[i] = nullptr;
+  }
 
 private:
   explicit Node(const Node &);
@@ -160,12 +180,14 @@ public:
     {
       CFunctionBase* function[MAXIMUM_NUMBER_OF_OVERLOADED_FUNCTION_NAMES];
       bool undefinedFunction;
+      char functionName[MAXIMUM_FUNCTION_NAME_LENGTH];
     } function;
   } m_data;
 
-  Node *m_left;
-  Node *m_right;
-  Node *m_other;
+  Node* m_left;
+  Node* m_right;
+  Node* m_other;
+  Eval* m_owner;
 };
 
 
@@ -186,10 +208,11 @@ Node::eval() const
 
   case OPCODE_RVALUE:
     /* Directly access the variable */
-    if (m_left)
-      return (*m_data.variable.variable)[m_left->eval()];
-    else
+    if (m_left) {
+      return m_data.variable.variable->getArrayValue(m_left->eval(), m_owner->getArrayOffsetType());
+    } else {
       return m_data.variable.variable->getValue();
+    }
 
   case OPCODE_MULTIPLY:
     return m_left->eval()*m_right->eval();
@@ -249,7 +272,7 @@ Node::eval() const
 
   case OPCODE_ASSIGN:
     if (m_left)
-      return (*m_data.variable.variable)[m_left->eval()] = m_right->eval();
+      return m_data.variable.variable->getArrayValue(m_left->eval(),  m_owner->getArrayOffsetType()) = m_right->eval();
     else {
       *m_data.variable.variable = m_right->eval();
       return m_data.variable.variable->getValue();
@@ -267,6 +290,11 @@ Node::eval() const
 
       for(unsigned int i=0; i<Node::MAXIMUM_NUMBER_OF_OVERLOADED_FUNCTION_NAMES; ++i)
       {
+         if(nullptr == m_data.function.function[i])
+         {
+             throw expression_undefined_exception(m_data.function.functionName, argc);
+         }
+
         // Linear search to match the function name and number of arguments.
         if( m_data.function.function[i]->getArgCount() == argc) {
           return (*m_data.function.function[i])(argc, argv);
@@ -756,6 +784,9 @@ parseFunction(
       c_function = (*it).second;
     }
     function->m_data.function.function[0] = c_function;
+    std::strncpy(function->m_data.function.functionName,
+                 function_name.c_str(),
+                 function_name.length() < Node::MAXIMUM_FUNCTION_NAME_LENGTH-1 ? function_name.length() : Node::MAXIMUM_FUNCTION_NAME_LENGTH-1);
 
     if (!c_function)
       eval.getUndefinedFunctionSet().insert(function_name);
@@ -770,9 +801,13 @@ parseFunction(
         ++it2,
         ++iCount)
     {
-      //cout << "  [" << (*it2).first << ", " << (*it2).second->getArgCount() << "]" << endl;
+//      cout << "  [" << (*it2).first << ", " << (*it2).second->getArgCount() << "]" << endl;
       c_function = (*it2).second;
       function->m_data.function.function[iCount] = c_function;
+      std::strncpy(function->m_data.function.functionName,
+                   function_name.c_str(),
+                   function_name.length() < Node::MAXIMUM_FUNCTION_NAME_LENGTH-1 ? function_name.length() : Node::MAXIMUM_FUNCTION_NAME_LENGTH-1);
+
 
       if (!c_function)
         eval.getUndefinedFunctionSet().insert(function_name);
@@ -787,6 +822,7 @@ parseFunction(
 
   if (!c_function) {
     function->m_data.function.undefinedFunction = true;
+    eval.getUndefinedFunctionSet().insert(function_name);
   }
 
   return function;
@@ -911,12 +947,14 @@ parseRValue(
 
 Eval::Eval(
   VariableMap::Resolver & resolver,
-  const std::string & expression)
+  const std::string & expression,
+  Variable::ArrayOffset arrayOffsetType)
   : m_variableMap(resolver),
     m_expression(expression),
     m_syntaxStatus(false),
     m_parseStatus(false),
-    m_headNode(nullptr)
+    m_headNode(nullptr),
+    m_arrayOffsetType(arrayOffsetType)
 {}
 
 Eval::~Eval()
@@ -929,7 +967,7 @@ Node *
 Eval::newNode(
   int           opcode)
 {
-  Node *new_node = new Node(static_cast<Opcode>(opcode));
+  Node *new_node = new Node(static_cast<Opcode>(opcode), this);
   m_nodes.push_back(new_node);
   return new_node;
 }
