@@ -43,6 +43,8 @@
 #include <set>
 #include <stdexcept>
 #include <cctype>
+#include <sstream>
+
 
 #include <stk_util/util/string_case_compare.hpp>
 
@@ -72,8 +74,9 @@ public:
    * <b>int</b> are currently supported.
    *
    */
-  enum Type {DOUBLE, INTEGER};
-  enum Use {DEPENDENT, INDEPENDENT};
+  enum Type        {DOUBLE, INTEGER};
+  enum Use         {DEPENDENT, INDEPENDENT};
+  enum ArrayOffset {ZERO_BASED_INDEX, ONE_BASED_INDEX};
   
   /**
    * Creates a new <b>Variable</b> instance.
@@ -82,9 +85,11 @@ public:
   Variable()
     : m_type(DOUBLE),
       m_use(INDEPENDENT),
+      m_size(1),
       m_doublePtr(&m_doubleValue),
       m_doubleValue(0.0)
-  {}
+  {
+  }
 
   /**
    * Creates a new <b>Variable</b> instance.  The new variable will be local and
@@ -96,7 +101,8 @@ public:
    */
   explicit Variable(Type type)
     : m_type(type),
-      m_use(INDEPENDENT)
+      m_use(INDEPENDENT),
+      m_size(1)
   {
     switch (type) {
     case DOUBLE:
@@ -120,12 +126,14 @@ public:
    *				variable.
    *
    */
-  explicit Variable(double &address)
+  explicit Variable(double &address, unsigned definedLength=std::numeric_limits<int>::max())
     : m_type(DOUBLE),
       m_use(INDEPENDENT),
+      m_size(definedLength),
       m_doublePtr(&address),
       m_doubleValue(0.0)
-  {}
+  {
+  }
 
   /**
    * Creates a new <b>Variable</b> instance.  The new variable will use the
@@ -137,12 +145,14 @@ public:
    *				variable.
    *
    */
-  explicit Variable(int &address)
+  explicit Variable(int &address, unsigned definedLength=std::numeric_limits<int>::max())
     : m_type(INTEGER),
       m_use(INDEPENDENT),
+      m_size(definedLength),
       m_intPtr(&address),
       m_intValue(0)
-  {}
+  {
+  }
 
   /**
    * @brief Member function <b>operator=</b> assigns a new value to the variable.
@@ -157,6 +167,9 @@ public:
   Variable &operator=(const double &value) {
     m_type = this->m_type;
     m_use = this->m_use;
+    if(m_size != 1 && m_size != std::numeric_limits<int>::max()) {
+      throw std::runtime_error("In analytic expression evaluator, invalid use of equal on multi-component variable");
+    }
     if (m_type == INTEGER)
       *m_intPtr = static_cast<int>(value);
     else if (m_type == DOUBLE)
@@ -176,6 +189,9 @@ public:
   Variable &operator=(const int &value) {
     m_type = this->m_type;
     m_use = this->m_use;
+    if(m_size != 1 && m_size != std::numeric_limits<int>::max()) {
+      throw std::runtime_error("In analytic expression evaluator, invalid use of equal on multi-component variable");
+    }
     if (m_type == INTEGER)
       *m_intPtr = value;
     else if (m_type == DOUBLE)
@@ -199,23 +215,41 @@ public:
 
   /**
    * @brief Member function <b>operator[]</b> returns a value from an array of
-   * double values.  No bounds checkin is performed.  Not even if the variable is and
-   * array is checked.
+   * double values.  
    *
    * @param index		a <b>int</b> value of the zero based index into the
    *				array to retrieve the value.
    *
    * @return			a <b>double</b> reference to the value.
    */
-  inline double &operator[](int index) {
-    if (m_type != DOUBLE)
+  inline double& getArrayValue(int index, ArrayOffset offsetType) const {
+    if (m_type != DOUBLE) {
       throw std::runtime_error("Only double arrays allowed");
+    }
 
-    if (m_doublePtr == 0)
+    if (m_doublePtr == nullptr) {
       throw std::runtime_error("Unbound variable");
+    }
 
-    return m_doublePtr[index];
-  }
+    if(offsetType == ZERO_BASED_INDEX) {
+      if(index < 0 || (index+1) > m_size) {
+        std::stringstream error;
+        error << "Attempting to access invalid component '"<<index<<"' in analytic function.  Valid components are 0 to '"<<m_size-1<<"'.  ";
+        throw std::runtime_error(error.str());
+      }
+      return m_doublePtr[index];
+    } else if (offsetType == ONE_BASED_INDEX) {
+      if(index < 1 || (index) > m_size) {
+        std::stringstream error;
+        error << "Attempting to access invalid component '"<<index<<"' in analytic function.  Valid components are 1 to '"<<m_size<<"'.  ";
+        throw std::runtime_error(error.str());
+      }
+      return m_doublePtr[index-1];
+    } else {
+      throw std::runtime_error("Invalid internal state of expression evalutor");
+      return m_doublePtr[0];
+    }
+  } 
 
   /**
    * @brief Member function <b>bind</b> binds variable to the address of the
@@ -226,9 +260,13 @@ public:
    *
    * @return			a <b>Variable</b> reference to the variable.
    */
-  inline Variable &bind(double &value_ref) {
+  inline Variable &bind(double &value_ref, int definedLength=std::numeric_limits<int>::max()) {
+
+    //std::cout<<"Bound variable of length: "<<definedLength<<std::endl;
+
     m_type = DOUBLE;
     m_doublePtr = &value_ref;
+    m_size = definedLength;
     return *this;
   }
 
@@ -241,9 +279,10 @@ public:
    *
    * @return			a <b>Variable</b> reference to the variable.
    */
-  inline Variable &bind(int &value_ref) {
+  inline Variable &bind(int &value_ref, int definedLength=std::numeric_limits<int>::max()) {
     m_type = INTEGER;
     m_intPtr = &value_ref;
+    m_size = definedLength;
     return *this;
   }
 
@@ -257,26 +296,40 @@ public:
     switch (m_type) {
     case DOUBLE:
       m_doublePtr = &m_doubleValue;
+      m_size = 1;
       m_doubleValue = 0.0;
       break;
     case INTEGER:
       m_intPtr = &m_intValue;
+      m_size = 1;
       m_intValue = 0;
       break;
     }
     return *this;
   }
 
-  double *getAddress() const {
+  //
+  //  Get the variable pointer and its defined length
+  //
+  double* getAddress() const {
     return m_doublePtr;
   }
-  
+  int getLength() const {
+    return m_size;
+  }
+
   /**
    * @brief Member function <b>getValue</b> returns the variable value as a double.
    *
    * @return			a <b>double</b> value of the value of the variable.
    */
   inline double getValue() const {
+
+    if(m_size != 1 && m_size != std::numeric_limits<int>::max()) {
+      throw std::runtime_error("Invalid direct access of array variable, must access by index");
+    }
+
+
     switch (m_type) {
     case DOUBLE:
       return *m_doublePtr;
@@ -289,7 +342,8 @@ public:
 private:
   Type	        m_type;                 ///< Variable data type
   Use           m_use;                  ///< Variable is dependent or independent
-  
+  int           m_size;                 ///< Size of defined values in the double or int pointer  
+
   union {
     double *	m_doublePtr;		///< Pointer to value as double
     int *	m_intPtr;		///< Pointer to value as integer
