@@ -433,15 +433,13 @@ const Ioss::GroupingEntity *get_associated_ioss_entity(const mesh::Part &part);
 
 size_t db_api_int_size(const Ioss::GroupingEntity *entity);
 
-mesh::EntityRank part_primary_entity_rank(const mesh::Part &part);
-
 void initialize_spatial_dimension(mesh::MetaData &meta, size_t spatial_dimension, const std::vector<std::string> &entity_rank_names);
 
 void get_io_field_type(const stk::mesh::FieldBase *field,
                        const stk::mesh::FieldRestriction &res,
                        std::pair<std::string, Ioss::Field::BasicType> *result);
 /**
- * \}
+ * \---}
  */
 
 }//namespace io
