@@ -7,6 +7,8 @@
 #include <stk_mesh/base/Field.hpp>
 #include <stk_mesh/baseImpl/elementGraph/ElemElemGraph.hpp>
 #include <stk_mesh/base/FEMHelpers.hpp>
+#include <stk_mesh/base/ExodusTranslator.hpp>
+#include <stk_mesh/baseImpl/EquivalentEntityBlocks.hpp>
 
 #include <stk_util/parallel/ParallelReduce.hpp>
 
@@ -17,7 +19,8 @@
 #include <stk_unit_test_utils/MeshFixture.hpp>
 #include <stk_unit_test_utils/getOption.h>
 
-#include "../../stk_io/stk_io/StkMeshIoBroker.hpp"
+#include "stk_io/StkMeshIoBroker.hpp"
+#include <stdlib.h>
 
 class DisconnectMesh : public DGTetFixture {};
 
@@ -50,7 +53,7 @@ TEST_F(DisconnectMesh, tet)
     stk::mesh::BulkData newBulkData(newMetaData, get_bulk().parallel());
     disconnectMesh(get_meta(), get_bulk(), get_meta().universal_part(), newMetaData, newBulkData);
 
-    stk::unit_test_util::write_mesh_using_stk_io("mike_new.g", newBulkData);
+    stk::io::write_mesh("mike_new.g", newBulkData);
 }
 
 class TOSDTWD : public stk::unit_test_util::MeshFixture
@@ -62,59 +65,30 @@ protected:
 
 TEST_F(TOSDTWD, disconnect_mesh)
 {
-    std::string exodusFileName = unitTestUtils::getOption("-i", "generated:10x10x10");
-    std::string mesh_name = unitTestUtils::getOption("-o", "blown_up.g");
+    std::string exodusFileName = stk::unit_test_util::get_option("-i", "generated:10x10x10");
+    std::string mesh_name = stk::unit_test_util::get_option("-o", "blown_up.g");
 
     {
         setup_mesh(exodusFileName, stk::mesh::BulkData::NO_AUTO_AURA);
         stk::mesh::MetaData newMetaData(get_meta().spatial_dimension());
         stk::mesh::BulkData newBulkData(newMetaData, get_bulk().parallel());
         disconnectMesh(get_meta(), get_bulk(), get_meta().universal_part(), newMetaData, newBulkData);
-        stk::unit_test_util::write_mesh_using_stk_io(mesh_name, newBulkData);
+        stk::io::write_mesh(mesh_name, newBulkData);
     }
 }
 
 TEST_F(TOSDTWD, birds)
 {
-    std::string velocityx = unitTestUtils::getOption("-vx", "10");
-    std::string velocityy = unitTestUtils::getOption("-vy", "10");
-    std::string steps = unitTestUtils::getOption("-steps", "100");
-    std::string ax = unitTestUtils::getOption("-ax", "0");
-    std::string ay = unitTestUtils::getOption("-ay", "9.8");
-    std::string exodusFileName = unitTestUtils::getOption("-i", "generated:10x10x10");
+    std::string exodusFileName = stk::unit_test_util::get_option("-i", "generated:10x10x10");
     std::string results_name = "output_results.g";
 
-    double velx = 10;
-    {
-        std::istringstream is(velocityx);
-        is >> velx;
-    }
+    double velx = stk::unit_test_util::get_command_line_option("-vx", 10.0);
+    double vely = stk::unit_test_util::get_command_line_option("-vy", 10.0);
+    unsigned int num_steps = stk::unit_test_util::get_command_line_option("-steps", 100u);
+    double accelx = stk::unit_test_util::get_command_line_option("-ax", 0.0);
+    double accely = stk::unit_test_util::get_command_line_option("-ay", 9.8);
 
-    double vely = 10;
-    {
-        std::istringstream is(velocityy);
-        is >> vely;
-    }
-
-    unsigned int num_steps;
-    {
-        std::istringstream is(steps);
-        is >> num_steps;
-    }
-
-    double accelx = 0;
-    {
-        std::istringstream is(ax);
-        is >> accelx;
-    }
-
-    double accely = 9.8;
-    {
-        std::istringstream is(ay);
-        is >> accely;
-    }
-
-    std::cerr << "Using (vx,vy) and (gx,gy) of (" << velx << ", " << vely << ") and (" << ax << ", " << ay << ")" << std::endl;
+    std::cerr << "Using (vx,vy) and (gx,gy) of (" << velx << ", " << vely << ") and (" << accelx << ", " << accely << ")" << std::endl;
 
     {
            stk::io::StkMeshIoBroker stkIo(get_comm());
@@ -299,6 +273,194 @@ TEST_F(TOSDTWD, birds)
          }
 }
 
+TEST_F(TOSDTWD, expand_mesh)
+{
+    std::string temp_name = "temp_abc.g";
+
+    {
+        std::string exodusFileName = stk::unit_test_util::get_option("-i", "generated:10x10x10");
+        setup_mesh(exodusFileName, stk::mesh::BulkData::NO_AUTO_AURA);
+        stk::mesh::MetaData newMetaData(get_meta().spatial_dimension());
+        stk::mesh::BulkData newBulkData(newMetaData, get_bulk().parallel());
+        disconnectMesh(get_meta(), get_bulk(), get_meta().universal_part(), newMetaData, newBulkData);
+        stk::io::write_mesh(temp_name, newBulkData);
+    }
+
+    {
+        stk::io::StkMeshIoBroker stkIo(get_comm());
+        stkIo.add_mesh_database(temp_name, stk::io::READ_MESH);
+        stkIo.create_input_mesh();
+
+        typedef stk::mesh::Field<double, stk::mesh::Cartesian3d> CoordFieldType;
+
+        CoordFieldType& field = stkIo.meta_data().declare_field<CoordFieldType>(stk::topology::NODE_RANK, "disp");
+
+        stk::mesh::put_field(field, stkIo.meta_data().universal_part());
+        stkIo.populate_bulk_data();
+
+        std::string results_name = stk::unit_test_util::get_option("-o", "output_results.g");
+        size_t fh = stkIo.create_output_mesh(results_name, stk::io::WRITE_RESULTS);
+
+        stkIo.add_field(fh, field); /*@\label{io:results:add_field}*/
+
+        std::vector<stk::mesh::Entity> elements;
+        stk::mesh::get_entities(stkIo.bulk_data(), stk::topology::ELEM_RANK, elements);
+
+        CoordFieldType *coords = stkIo.meta_data().get_field<CoordFieldType>(stk::topology::NODE_RANK, "coordinates");
+
+        stk::mesh::PartVector elementBlocks;
+        const stk::mesh::PartVector parts = stkIo.meta_data().get_parts();
+        for(size_t i=0;i<parts.size();++i)
+        {
+            if(stk::mesh::is_element_block(*parts[i]))
+            {
+                elementBlocks.push_back(parts[i]);
+            }
+        }
+
+        double xc = 0, yc = 0, zc = 0;
+        std::vector<std::vector<double> > element_centroid(elements.size());
+        std::vector<std::vector<double> > part_centroid(elementBlocks.size());
+        for(size_t i=0;i<part_centroid.size();++i)
+            part_centroid[i].resize(3);
+
+        std::vector<size_t> num_elements_in_part(elementBlocks.size(), 0);
+
+        for(size_t i = 0; i < elements.size(); i++)
+        {
+            unsigned num_nodes = stkIo.bulk_data().num_nodes(elements[i]);
+            const stk::mesh::Entity *nodes = stkIo.bulk_data().begin_nodes(elements[i]);
+            double exc = 0, eyc = 0, ezc = 0;
+            for(unsigned j = 0; j < num_nodes; ++j)
+            {
+                double *node_data = stk::mesh::field_data(*coords, nodes[j]);
+                exc += node_data[0];
+                eyc += node_data[1];
+                ezc += node_data[2];
+            }
+
+            exc /= num_nodes;
+            eyc /= num_nodes;
+            ezc /= num_nodes;
+            xc += exc;
+            yc += eyc;
+            zc += ezc;
+            element_centroid[i].resize(3);
+            element_centroid[i][0] = exc;
+            element_centroid[i][1] = eyc;
+            element_centroid[i][2] = ezc;
+
+            for(size_t j=0;j<elementBlocks.size();++j)
+            {
+                stk::mesh::Selector sel = *elementBlocks[j];
+                if(sel(stkIo.bulk_data().bucket(elements[i])))
+                {
+                    num_elements_in_part[j]++;
+                    part_centroid[j][0] += exc;
+                    part_centroid[j][1] += eyc;
+                    part_centroid[j][2] += ezc;
+                }
+            }
+        }
+
+        for(size_t j=0;j<elementBlocks.size();++j)
+        {
+            if(num_elements_in_part[j]>0)
+            {
+                part_centroid[j][0] = part_centroid[j][0] / num_elements_in_part[j] - xc;
+                part_centroid[j][1] = part_centroid[j][1] / num_elements_in_part[j] - yc;
+                part_centroid[j][2] = part_centroid[j][2] / num_elements_in_part[j] - zc;
+            }
+        }
+
+        xc /= elements.size();
+        yc /= elements.size();
+        zc /= elements.size();
+
+        for(size_t i = 0; i < element_centroid.size(); ++i)
+        {
+            element_centroid[i][0] -= xc;
+            element_centroid[i][1] -= yc;
+            element_centroid[i][2] -= zc;
+        }
+
+        std::cerr << "(x,y,z) centroid = (" << xc << ", " << yc << ", " << zc << ")\n";
+
+        double time = 0;
+        double scalar = 10;
+
+        std::vector<std::vector<double>> direct(elementBlocks.size());
+        for(size_t i=0;i<direct.size();++i)
+        {
+            direct[i].resize(3);
+            direct[i][0] = 2.0*((double) rand() / (RAND_MAX))-1.0;
+            direct[i][1] = 2.0*((double) rand() / (RAND_MAX))-1.0;
+            direct[i][2] = 2.0*((double) rand() / (RAND_MAX))-1.0;
+            double mag = sqrt(direct[i][0]*direct[i][0] + direct[i][1]*direct[i][1] + direct[i][2]*direct[i][2]);
+            direct[i][0] /= mag;
+            direct[i][1] /= mag;
+            direct[i][2] /= mag;
+        }
+
+        for(size_t i = 0; i < elements.size(); i++)
+        {
+            unsigned num_nodes = stkIo.bulk_data().num_nodes(elements[i]);
+            const stk::mesh::Entity *nodes = stkIo.bulk_data().begin_nodes(elements[i]);
+
+            std::vector<double> direction = {0, 0, 0};
+            for(unsigned j = 0; j < num_nodes; ++j)
+            {
+                double *disp = stk::mesh::field_data(field, nodes[j]);
+                for(size_t k=0;k<elementBlocks.size();++k)
+                {
+                    stk::mesh::Selector sel = *elementBlocks[k];
+                    if(sel(stkIo.bulk_data().bucket(elements[i])))
+                    {
+                        disp[0] = scalar * direct[k][0];
+                        disp[1] = scalar * direct[k][1];
+                        disp[2] = scalar * direct[k][2];
+                    }
+                }
+//                    disp[0] = scalar * element_centroid[i][0];
+//                    disp[1] = scalar * element_centroid[i][1];
+//                    disp[2] = scalar * element_centroid[i][2];
+            }
+        }
+
+        stkIo.begin_output_step(fh, time);
+        stkIo.write_defined_output_fields(fh);
+        stkIo.end_output_step(fh);
+
+        // choose x direction
+//        std::vector<double> goWhere = { 1, 0, 0 };
+//        double scale = 10;
+//
+//        for(size_t i = 0; i < elements.size(); i++)
+//        {
+//            if(!sel(stkIo.bulk_data().bucket(elements[i])))
+//            {
+//                unsigned num_nodes = stkIo.bulk_data().num_nodes(elements[i]);
+//                const stk::mesh::Entity *nodes = stkIo.bulk_data().begin_nodes(elements[i]);
+//
+//                std::vector<double> direction = {0, 0, 0};
+//                for(unsigned j = 0; j < num_nodes; ++j)
+//                {
+//                    double *disp = stk::mesh::field_data(field, nodes[j]);
+//
+//                    disp[0] += scale * goWhere[0];
+//                    disp[1] += scale * goWhere[1];
+//                    disp[2] += scale * goWhere[2];
+//                }
+//            }
+//        }
+//
+//        time += delta_t;
+//        stkIo.begin_output_step(fh, time);
+//        stkIo.write_defined_output_fields(fh);
+//        stkIo.end_output_step(fh);
+    }
+}
+
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 void copyPartsToNewMesh(const stk::mesh::MetaData &oldMeta, stk::mesh::MetaData &newMeta)
@@ -350,62 +512,6 @@ void copyFieldsToNewMesh(const stk::mesh::MetaData &oldMeta, stk::mesh::MetaData
     }
 }
 
-void copySelectedEntitiesToNewMesh(const stk::mesh::BulkData& oldBulkData,
-                                   const stk::mesh::Selector subMeshSelector,
-                                   std::map<stk::mesh::Entity, stk::mesh::Entity> &oldToNewEntityMap,
-                                   stk::mesh::MetaData &newMeta,
-                                   stk::mesh::BulkData &newBulkData)
-{
-    for(stk::mesh::EntityRank rank = stk::topology::NODE_RANK; rank <= stk::topology::ELEMENT_RANK; rank++)
-    {
-        const stk::mesh::BucketVector &buckets = oldBulkData.get_buckets(rank, subMeshSelector);
-        for(size_t i = 0; i < buckets.size(); i++)
-        {
-            stk::mesh::Bucket &bucket = *buckets[i];
-            for(size_t j = 0; j < bucket.size(); j++)
-            {
-                const stk::mesh::PartVector &oldParts = bucket.supersets();
-                stk::mesh::PartVector newParts(oldParts.size(), 0);
-                for(size_t k = 0; k < oldParts.size(); k++)
-                {
-                    newParts[k] = &newMeta.get_part(oldParts[k]->mesh_meta_data_ordinal());
-                }
-                stk::mesh::Entity newEntity = newBulkData.declare_entity(rank, oldBulkData.identifier(bucket[j]), newParts);
-                oldToNewEntityMap[bucket[j]] = newEntity;
-            }
-        }
-    }
-}
-
-void copyRelationsToNewMesh(const stk::mesh::BulkData& oldBulkData,
-                            const stk::mesh::Selector subMeshSelector,
-                            const std::map<stk::mesh::Entity, stk::mesh::Entity> &oldToNewEntityMap,
-                            stk::mesh::BulkData &newBulkData)
-{
-    for(stk::mesh::EntityRank rank = stk::topology::NODE_RANK; rank <= stk::topology::ELEMENT_RANK; rank++)
-    {
-        const stk::mesh::BucketVector &buckets = oldBulkData.get_buckets(rank, subMeshSelector);
-        for(size_t i = 0; i < buckets.size(); i++)
-        {
-            stk::mesh::Bucket &bucket = *buckets[i];
-            for(size_t j = 0; j < bucket.size(); j++)
-            {
-                stk::mesh::Entity newFromEntity = oldToNewEntityMap.find(bucket[j])->second;
-                for(stk::mesh::EntityRank downRank = stk::topology::NODE_RANK; downRank < rank; downRank++)
-                {
-                    unsigned numConnectedEntities = oldBulkData.num_connectivity(bucket[j], downRank);
-                    const stk::mesh::Entity *connectedEntities = oldBulkData.begin(bucket[j], downRank);
-                    for(unsigned connectOrder = 0; connectOrder < numConnectedEntities; connectOrder++)
-                    {
-                        stk::mesh::Entity newToEntity = oldToNewEntityMap.find(connectedEntities[connectOrder])->second;
-                        newBulkData.declare_relation(newFromEntity, newToEntity, connectOrder);
-                    }
-                }
-            }
-        }
-    }
-}
-
 void copyFieldDataToNewMesh(const stk::mesh::MetaData &oldMeta,
                             const stk::mesh::BulkData& oldBulkData,
                             const stk::mesh::Selector subMeshSelector,
@@ -429,7 +535,9 @@ void copyFieldDataToNewMesh(const stk::mesh::MetaData &oldMeta,
                     for(size_t j = 0; j < bucket.size(); j++)
                     {
                         stk::mesh::Entity oldEntity = bucket[j];
-                        stk::mesh::Entity newEntity = oldToNewEntityMap.find(oldEntity)->second;
+                        const stk::mesh::EntityKey oldKey = oldBulkData.entity_key(oldEntity);
+                        // stk::mesh::Entity newEntity = oldToNewEntityMap.find(oldEntity)->second;
+                        stk::mesh::Entity newEntity = newBulkData.get_entity(oldKey);
                         void *oldData = stk::mesh::field_data(*oldFields[k], oldEntity);
                         void *newData = stk::mesh::field_data(*newFields[k], newEntity);
                         memcpy(newData, oldData, stk::mesh::field_bytes_per_entity(*oldFields[k], oldEntity));
@@ -537,5 +645,37 @@ void disconnectMesh(const stk::mesh::MetaData &oldMeta,
     }
     newBulkData.modification_end();
 
+    // ===============================
+
+    stk::mesh::EntityVector sides;
+    stk::mesh::get_selected_entities(oldMeta.locally_owned_part(), oldBulkData.buckets(oldMeta.side_rank()), sides);
+
+    std::vector<std::pair<stk::mesh::EntityId, unsigned> > elemSides(sides.size());
+
+    for(size_t i=0;i<sides.size();++i)
+    {
+        const stk::mesh::Entity* elementA = oldBulkData.begin_elements(sides[i]);
+        const stk::mesh::ConnectivityOrdinal* ords = oldBulkData.begin_element_ordinals(sides[i]);
+        elemSides[i] = std::make_pair(oldBulkData.identifier(elementA[0]), static_cast<unsigned>(ords[0]));
+    }
+
+    newBulkData.modification_begin();
+
+    for(size_t i=0;i<elemSides.size();++i)
+    {
+        stk::mesh::Entity newElement = newBulkData.get_entity(stk::topology::ELEM_RANK, elemSides[i].first);
+        stk::mesh::OrdinalVector oldPartOrds;
+        oldBulkData.bucket(sides[i]).supersets(oldPartOrds);
+        stk::mesh::PartVector newParts;
+        for(size_t j=0; j<oldPartOrds.size(); j++)
+        {
+            stk::mesh::Part &newPart = newMeta.get_part(oldPartOrds[j]);
+            if(!stk::mesh::is_auto_declared_part(newPart))
+                newParts.push_back(&newPart);
+        }
+        newBulkData.declare_element_side(oldBulkData.identifier(sides[i]), newElement, elemSides[i].second, newParts);
+    }
+    newBulkData.modification_end();
+
     copyFieldDataToNewMesh(oldMeta, oldBulkData, subMeshSelector, oldToNewEntityMap, newMeta, newBulkData, node_in_old_mesh);
 }
