@@ -45,7 +45,6 @@
 #include <stk_mesh/base/GetEntities.hpp>
 
 #include <stk_mesh/baseImpl/MeshImplUtils.hpp>
-#include <stk_util/parallel/ParallelComm.hpp>  // for CommBuffer, CommAll
 #include <vector>                       // for vector, etc
 #include <unordered_map>
 #include "stk_mesh/base/Bucket.hpp"     // for Bucket
@@ -167,7 +166,7 @@ struct create_single_edge_impl
     else {
       edge = iedge->second;
     }
-    perm = mesh.find_permutation(elem_topo, &elem_nodes[0], edge_topo, &edge_nodes[0], m_edge_ordinal);
+    perm = mesh.find_permutation(elem_topo, elem_nodes.data(), edge_topo, edge_nodes.data(), m_edge_ordinal);
     ThrowRequireMsg(perm != INVALID_PERMUTATION, "CreateEdges:  could not find valid permutation to connect face to element");
     mesh.declare_relation(ielem, edge, m_edge_ordinal, perm, ordinal_scratch, part_scratch);
   }
@@ -279,7 +278,7 @@ struct create_edge_impl
         else {
           edge = iedge->second;
         }
-        perm = mesh.find_permutation(elem_topo, &elem_nodes[0], edge_topo, &edge_nodes[0], e);
+        perm = mesh.find_permutation(elem_topo, elem_nodes.data(), edge_topo, edge_nodes.data(), e);
         ThrowRequireMsg(perm != INVALID_PERMUTATION, "CreateEdges:  could not find valid permutation to connect face to element");
         mesh.declare_relation(m_bucket[ielem], edge, e, perm, ordinal_scratch, part_scratch);
       }
@@ -361,7 +360,7 @@ struct connect_face_impl
         //which is fine
         if (iedge != m_edge_map.end()) {
           Entity edge = iedge->second;
-          Permutation perm = mesh.find_permutation(face_topo, &face_nodes[0], edge_topo, &edge_nodes[0], e);
+          Permutation perm = mesh.find_permutation(face_topo, face_nodes.data(), edge_topo, edge_nodes.data(), e);
           ThrowRequireMsg(perm != INVALID_PERMUTATION, "CreateEdges:  could not find valid permutation to connect face to element");
           mesh.declare_relation(m_bucket[iface], edge, e, perm, ordinal_scratch, part_scratch);
         }
@@ -399,7 +398,7 @@ namespace impl {
 
 void create_edges( BulkData & mesh )
 {
-  create_edges(mesh, mesh.mesh_meta_data().universal_part(), 0 );
+  create_edges(mesh, mesh.mesh_meta_data().universal_part(), nullptr );
 }
 
 void create_edges( BulkData & mesh, const Selector & element_selector, Part * part_to_insert_new_edges )
