@@ -6,14 +6,15 @@
 #include <string>
 #include "BulkData.hpp"
 #include "MetaData.hpp"
-#include "../baseImpl/elementGraph/ElemElemGraph.hpp"
-#include "../baseImpl/elementGraph/MeshDiagnosticObserver.hpp"
-#include "../baseImpl/EquivalentEntityBlocks.hpp"
-#include "../../../stk_util/stk_util/parallel/DistributedIndex.hpp"
+#include <stk_mesh/base/ExodusTranslator.hpp>
+#include "stk_mesh/baseImpl/elementGraph/ElemElemGraph.hpp"
+#include "stk_mesh/baseImpl/elementGraph/MeshDiagnosticObserver.hpp"
+#include "stk_mesh/baseImpl/EquivalentEntityBlocks.hpp"
+#include "stk_util/parallel/DistributedIndex.hpp"
 #include "GetEntities.hpp"
-#include "../baseImpl/MeshImplUtils.hpp"
-#include "../baseImpl/elementGraph/BulkDataIdMapper.hpp"
-#include "../baseImpl/elementGraph/ElemGraphCoincidentElems.hpp"
+#include "stk_mesh/baseImpl/MeshImplUtils.hpp"
+#include "stk_mesh/baseImpl/elementGraph/BulkDataIdMapper.hpp"
+#include "stk_mesh/baseImpl/elementGraph/ElemGraphCoincidentElems.hpp"
 #include "stk_util/parallel/DebugTool.hpp"
 
 namespace stk { namespace mesh {
@@ -100,7 +101,7 @@ std::vector<std::string> get_messages_for_split_coincident_elements(const stk::m
         std::string blockNames;
         blockNames = "{";
         for (const stk::mesh::Part* part : elementParts) {
-            if (stk::mesh::impl::is_element_block(*part)) {
+            if (stk::mesh::is_element_block(*part)) {
                 blockNames += " " + part->name();
             }
         }
