@@ -57,7 +57,10 @@
 #include "stk_mesh/base/Part.hpp"       // for Part
 #include "stk_mesh/base/Selector.hpp"   // for Selector, operator|, etc
 #include "stk_mesh/base/Types.hpp"      // for PartVector, EntityProc, etc
+#include "stk_mesh/base/FEMHelpers.hpp"
 #include "stk_topology/topology.hpp"    // for topology, etc
+#include "stk_mesh/baseImpl/MeshImplUtils.hpp"
+
 namespace stk { namespace mesh { class BulkData; } }
 namespace stk { namespace mesh { namespace fixtures { class BoxFixture; } } }
 namespace stk { namespace mesh { namespace fixtures { class RingFixture; } } }
@@ -76,7 +79,6 @@ using stk::mesh::MetaData;
 using stk::mesh::BulkData;
 using stk::mesh::Selector;
 using stk::mesh::PartVector;
-using stk::mesh::BaseEntityRank;
 using stk::mesh::PairIterRelation;
 using stk::mesh::EntityProc;
 using stk::mesh::Entity;
@@ -776,25 +778,65 @@ void copySelectedEntitiesToNewMesh(const stk::unit_test_util::BulkDataTester& ol
 {
     for(stk::mesh::EntityRank rank = stk::topology::NODE_RANK; rank <= stk::topology::ELEMENT_RANK; rank++)
     {
-        const stk::mesh::BucketVector &buckets = oldBulkData.get_buckets(rank, subMeshSelector);
-        for(size_t i = 0; i < buckets.size(); i++)
+        // Skip over creating all faces since those will be managed by declare_element_side() later on
+        if (rank != stk::topology::FACE_RANK)
         {
-            stk::mesh::Bucket &bucket = *buckets[i];
-            for(size_t j = 0; j < bucket.size(); j++)
+            const stk::mesh::BucketVector &buckets = oldBulkData.get_buckets(rank, subMeshSelector);
+            for(size_t i = 0; i < buckets.size(); i++)
             {
-                const stk::mesh::PartVector &oldParts = bucket.supersets();
-                stk::mesh::PartVector newParts(oldParts.size(), 0);
-                for(size_t k = 0; k < oldParts.size(); k++)
+                stk::mesh::Bucket &bucket = *buckets[i];
+                for(size_t j = 0; j < bucket.size(); j++)
                 {
-                    newParts[k] = &newMeta.get_part(oldParts[k]->mesh_meta_data_ordinal());
+                    const stk::mesh::PartVector &oldParts = bucket.supersets();
+                    stk::mesh::PartVector newParts(oldParts.size(), 0);
+                    for(size_t k = 0; k < oldParts.size(); k++)
+                    {
+                        newParts[k] = &newMeta.get_part(oldParts[k]->mesh_meta_data_ordinal());
+                    }
+                    stk::mesh::Entity newEntity = newBulkData.declare_entity(rank, oldBulkData.identifier(bucket[j]), newParts);
+                    oldToNewEntityMap[bucket[j]] = newEntity;
                 }
-                stk::mesh::Entity newEntity = newBulkData.declare_entity(rank, oldBulkData.identifier(bucket[j]), newParts);
-                oldToNewEntityMap[bucket[j]] = newEntity;
             }
         }
     }
 }
 
+void copySelectedFacesToNewMesh(const stk::unit_test_util::BulkDataTester& oldBulkData,
+                                   const stk::mesh::Selector subMeshSelector,
+                                   std::map<stk::mesh::Entity, stk::mesh::Entity> &oldToNewEntityMap,
+                                   stk::mesh::MetaData &newMeta,
+                                   stk::unit_test_util::BulkDataTester &newBulkData)
+{
+    const stk::mesh::BucketVector &buckets = oldBulkData.get_buckets(stk::topology::FACE_RANK, subMeshSelector);
+    for(size_t i = 0; i < buckets.size(); i++)
+    {
+        stk::mesh::Bucket &bucket = *buckets[i];
+        for(size_t j = 0; j < bucket.size(); j++)
+        {
+            const stk::mesh::Entity oldSide = bucket[j];
+            const stk::mesh::PartVector &oldParts = bucket.supersets();
+            stk::mesh::PartVector newParts;
+            for(size_t k = 0; k < oldParts.size(); k++)
+            {
+                if (!stk::mesh::is_auto_declared_part(*oldParts[k]))
+                {
+                    newParts.push_back( &newMeta.get_part(oldParts[k]->mesh_meta_data_ordinal()) );
+                }
+            }
+
+            const stk::mesh::Entity * connected_elements = oldBulkData.begin_elements(oldSide);
+            const Entity oldFirstElement = connected_elements[0];
+            ThrowRequire(oldBulkData.is_valid(oldFirstElement));
+
+            stk::mesh::ConnectivityOrdinal faceOrdinal = stk::mesh::impl::get_ordinal_for_element_side_pair(oldBulkData, oldFirstElement, oldSide);
+            stk::mesh::Entity newFirstElement = newBulkData.get_entity(oldBulkData.entity_key(oldFirstElement));
+
+            stk::mesh::Entity newSide = newBulkData.declare_element_side(newFirstElement, faceOrdinal, newParts);
+            oldToNewEntityMap[oldSide] = newSide;
+        }
+    }
+}
+
 void copyRelationsToNewMesh(const stk::unit_test_util::BulkDataTester& oldBulkData,
                             const stk::mesh::Selector subMeshSelector,
                             const std::map<stk::mesh::Entity, stk::mesh::Entity> &oldToNewEntityMap,
@@ -802,21 +844,28 @@ void copyRelationsToNewMesh(const stk::unit_test_util::BulkDataTester& oldBulkDa
 {
     for(stk::mesh::EntityRank rank = stk::topology::NODE_RANK; rank <= stk::topology::ELEMENT_RANK; rank++)
     {
-        const stk::mesh::BucketVector &buckets = oldBulkData.get_buckets(rank, subMeshSelector);
-        for(size_t i = 0; i < buckets.size(); i++)
+        // Skip over all face relations since those will be managed by declare_element_side() later on
+        if (rank != stk::topology::FACE_RANK)
         {
-            stk::mesh::Bucket &bucket = *buckets[i];
-            for(size_t j = 0; j < bucket.size(); j++)
+            const stk::mesh::BucketVector &buckets = oldBulkData.get_buckets(rank, subMeshSelector);
+            for(size_t i = 0; i < buckets.size(); i++)
             {
-                stk::mesh::Entity newFromEntity = oldToNewEntityMap.find(bucket[j])->second;
-                for(stk::mesh::EntityRank downRank = stk::topology::NODE_RANK; downRank < rank; downRank++)
+                stk::mesh::Bucket &bucket = *buckets[i];
+                for(size_t j = 0; j < bucket.size(); j++)
                 {
-                    unsigned numConnectedEntities = oldBulkData.num_connectivity(bucket[j], downRank);
-                    const stk::mesh::Entity *connectedEntities = oldBulkData.begin(bucket[j], downRank);
-                    for(unsigned connectOrder = 0; connectOrder < numConnectedEntities; connectOrder++)
+                    stk::mesh::Entity newFromEntity = oldToNewEntityMap.find(bucket[j])->second;
+                    for(stk::mesh::EntityRank downRank = stk::topology::NODE_RANK; downRank < rank; downRank++)
                     {
-                        stk::mesh::Entity newToEntity = oldToNewEntityMap.find(connectedEntities[connectOrder])->second;
-                        newBulkData.declare_relation(newFromEntity, newToEntity, connectOrder);
+                        if (downRank != stk::topology::FACE_RANK)
+                        {
+                            unsigned numConnectedEntities = oldBulkData.num_connectivity(bucket[j], downRank);
+                            const stk::mesh::Entity *connectedEntities = oldBulkData.begin(bucket[j], downRank);
+                            for(unsigned connectOrder = 0; connectOrder < numConnectedEntities; connectOrder++)
+                            {
+                                stk::mesh::Entity newToEntity = oldToNewEntityMap.find(connectedEntities[connectOrder])->second;
+                                newBulkData.declare_relation(newFromEntity, newToEntity, connectOrder);
+                            }
+                        }
                     }
                 }
             }
@@ -872,6 +921,7 @@ void createSerialSubMesh(const stk::mesh::MetaData &oldMeta,
     newBulkData.modification_begin();
     copySelectedEntitiesToNewMesh(oldBulkData, subMeshSelector, oldToNewEntityMap, newMeta, newBulkData);
     copyRelationsToNewMesh(oldBulkData, subMeshSelector, oldToNewEntityMap, newBulkData);
+    copySelectedFacesToNewMesh(oldBulkData, subMeshSelector, oldToNewEntityMap, newMeta, newBulkData);
     newBulkData.modification_end();
 
     copyFieldDataToNewMesh(oldMeta, oldBulkData, subMeshSelector, oldToNewEntityMap, newMeta);
