@@ -40,8 +40,6 @@
 #include <unit_tests/UnitTestUtils.hpp>
 #include <unit_tests/MeshUtilsForBoundingVolumes.hpp>
 
-#include <exodusMeshInterface.h>
-
 #include <stk_util/parallel/Parallel.hpp>  // for ParallelMachine, etc
 #include <stk_unit_test_utils/getOption.h>
 
@@ -49,11 +47,11 @@ namespace
 {
 
 void runStkSearchTestUsingStkAABoxes(stk::search::SearchMethod search);
-void runStkSearchTestUsingGtkAABoxes(stk::search::SearchMethod search);
+void runStkSearchTestUsingFloatAABoxes(stk::search::SearchMethod search);
 void testPerformanceOfAxisAlignedBoundingBoxes(stk::search::SearchMethod searchMethod, MPI_Comm comm);
-void testStkSearchUsingStkAABoxes(MPI_Comm comm, std::vector<GtkBox> &domainBoxes,
+void testStkSearchUsingStkAABoxes(MPI_Comm comm, std::vector<FloatBox> &domainBoxes,
         stk::search::SearchMethod searchMethod, SearchResults boxIdPairResults);
-void testStkSearchUsingGtkAABoxes(MPI_Comm comm, std::vector<GtkBox> &domainBoxes,
+void testStkSearchUsingFloatAABoxes(MPI_Comm comm, std::vector<FloatBox> &domainBoxes,
                 stk::search::SearchMethod searchMethod, SearchResults boxIdPairResults);
 
 TEST(Performance, ofAxisAlignedBoundingBoxesUsingOctTree)
@@ -151,36 +149,40 @@ TEST(Performance, stkSearchUsingOcttreeUsingStkAABoxes)
     runStkSearchTestUsingStkAABoxes(stk::search::OCTREE);
 }
 
-TEST(Performance, stkSearchUsingBoostUsingGtkAABoxes)
+TEST(Performance, stkSearchUsingKdtreeUsingStkAABoxes)
 {
-    runStkSearchTestUsingGtkAABoxes(stk::search::BOOST_RTREE);
+    runStkSearchTestUsingStkAABoxes(stk::search::KDTREE);
 }
 
-TEST(Performance, stkSearchUsingOcttreeUsingGtkAABoxes)
+TEST(Performance, stkSearchUsingBoostUsingFloatAABoxes)
 {
-    runStkSearchTestUsingGtkAABoxes(stk::search::OCTREE);
+    runStkSearchTestUsingFloatAABoxes(stk::search::BOOST_RTREE);
 }
 
-TEST(Performance, gtkSearchUsingOcttreeUsingGtkAABoxes)
+TEST(Performance, stkSearchUsingOcttreeUsingFloatAABoxes)
 {
-    runStkSearchTestUsingGtkAABoxes(stk::search::OCTREE);
+    runStkSearchTestUsingFloatAABoxes(stk::search::OCTREE);
 }
 
+TEST(Performance, stkSearchUsingKdtreeUsingFloatAABoxes)
+{
+    runStkSearchTestUsingFloatAABoxes(stk::search::KDTREE);
+}
 
-void runStkSearchTestUsingGtkAABoxes(stk::search::SearchMethod searchMethod)
+void runStkSearchTestUsingFloatAABoxes(stk::search::SearchMethod searchMethod)
 {
     MPI_Comm comm = MPI_COMM_WORLD;
-    std::vector<GtkBox> domainBoxes;
+    std::vector<FloatBox> domainBoxes;
     fillDomainBoxes(comm, domainBoxes);
 
     SearchResults boxIdPairResults;
-    testStkSearchUsingGtkAABoxes(comm, domainBoxes, searchMethod, boxIdPairResults);
+    testStkSearchUsingFloatAABoxes(comm, domainBoxes, searchMethod, boxIdPairResults);
 }
 
 void runStkSearchTestUsingStkAABoxes(stk::search::SearchMethod searchMethod)
 {
     MPI_Comm comm = MPI_COMM_WORLD;
-    std::vector<GtkBox> domainBoxes;
+    std::vector<FloatBox> domainBoxes;
     fillDomainBoxes(comm, domainBoxes);
 
     SearchResults boxIdPairResults;
@@ -188,15 +190,15 @@ void runStkSearchTestUsingStkAABoxes(stk::search::SearchMethod searchMethod)
 }
 
 
-void testStkSearchUsingStkAABoxes(MPI_Comm comm, std::vector<GtkBox> &domainBoxes,
+void testStkSearchUsingStkAABoxes(MPI_Comm comm, std::vector<FloatBox> &domainBoxes,
         stk::search::SearchMethod searchMethod, SearchResults boxIdPairResults)
 {
     int procId = stk::parallel_machine_rank(comm);
 
     StkBoxVector stkBoxes(domainBoxes.size());
-    fillStkBoxesUsingGtkBoxes(domainBoxes, procId, stkBoxes);
+    fillStkBoxesUsingFloatBoxes(domainBoxes, procId, stkBoxes);
 
-    std::string rangeBoxComm = unitTestUtils::getOption("-rangeBoxComm", "yes");
+    std::string rangeBoxComm = stk::unit_test_util::get_option("-rangeBoxComm", "yes");
     bool rangeResultsCommunicated = ( rangeBoxComm == "yes" );
 
     double startTime = stk::wall_time();
@@ -220,19 +222,19 @@ void testStkSearchUsingStkAABoxes(MPI_Comm comm, std::vector<GtkBox> &domainBoxe
     }
 }
 
-void testStkSearchUsingGtkAABoxes(MPI_Comm comm, std::vector<GtkBox> &domainBoxes,
+void testStkSearchUsingFloatAABoxes(MPI_Comm comm, std::vector<FloatBox> &domainBoxes,
         stk::search::SearchMethod searchMethod, SearchResults boxIdPairResults)
 {
     int procId = stk::parallel_machine_rank(comm);
 
-    GtkBoxVector searchBoxPairs(domainBoxes.size());
+    FlaotBoxVector searchBoxPairs(domainBoxes.size());
     for(size_t i=0;i<domainBoxes.size();i++)
     {
         Ident domainBoxId(i, procId);
         searchBoxPairs[i] = std::make_pair(domainBoxes[i], domainBoxId);
     }
 
-    std::string rangeBoxComm = unitTestUtils::getOption("-rangeBoxComm", "yes");
+    std::string rangeBoxComm = stk::unit_test_util::get_option("-rangeBoxComm", "yes");
     bool rangeResultsCommunicated = ( rangeBoxComm == "yes" );
 
     double startTime = stk::wall_time();
@@ -261,13 +263,13 @@ TEST(Performance, getGoldResults)
     MPI_Comm comm = MPI_COMM_WORLD;
     int procId = stk::parallel_machine_rank(comm);
 
-    std::vector<GtkBox> domainBoxes;
+    std::vector<FloatBox> domainBoxes;
     fillDomainBoxes(comm, domainBoxes);
 
     SearchResults boxIdPairResults;
 
     StkBoxVector stkBoxes(domainBoxes.size());
-    fillStkBoxesUsingGtkBoxes(domainBoxes, procId, stkBoxes);
+    fillStkBoxesUsingFloatBoxes(domainBoxes, procId, stkBoxes);
 
     double startTime = stk::wall_time();
     for (size_t i=0;i<stkBoxes.size();++i)
