@@ -39,6 +39,7 @@
 #include <stk_search/CoarseSearchBoostRTree.hpp>
 #include <stk_search/OctTreeOps.hpp>
 #include <stk_search/SearchMethod.hpp>
+#include <stk_search/KDTree_stk_interface.hpp>
 
 #include <vector>
 #include <utility>
@@ -52,7 +53,7 @@ std::ostream& operator<<(std::ostream &out, SearchMethod method)
   switch( method )   {
   case BOOST_RTREE:            out << "BOOST_RTREE"; break;
   case OCTREE:                 out << "OCTREE"; break;
-  case GTK_SEARCH:             out << "GTK_SEARCH"; break;
+  case KDTREE:                 out << "KDTREE"; break;
   case MORTON_LINEARIZED_BVH:  out << "MORTON_LINEARIZED_BVH"; break;
   }
   return out;
@@ -129,6 +130,9 @@ void coarse_search( std::vector<std::pair<DomainBox,DomainIdent> > const& domain
   case OCTREE:
     coarse_search_octree(domain,range,comm,intersections,communicateRangeBoxInfo);
     break;
+  case KDTREE:
+    kdtree_search(domain,range,comm,intersections,communicateRangeBoxInfo);
+    break;
   default:
     std::cerr << "coarse_search(..) interface used does not support SearchMethod " << method << std::endl;
     abort();
