@@ -34,13 +34,7 @@
 #ifndef STKTOPOLOGY_TOPOLOGY_HPP
 #define STKTOPOLOGY_TOPOLOGY_HPP
 
-#include <boost/config.hpp>
-//
-// Set some default values GPU support
-//
-#  ifndef BOOST_GPU_ENABLED
-#  define BOOST_GPU_ENABLED
-#  endif
+#include <stk_util/stk_config.h>
 
 #include <string>
 
@@ -117,7 +111,11 @@ struct topology
     , HEX_27, HEXAHEDRON_27 = HEX_27
     , END_TOPOLOGY
     , NUM_TOPOLOGIES = END_TOPOLOGY - BEGIN_TOPOLOGY
-    , SUPERELEMENT_START = END_TOPOLOGY+1
+    , SUPEREDGE_START = END_TOPOLOGY+1
+    , SUPEREDGE_END = SUPEREDGE_START + 1000
+    , SUPERFACE_START = SUPEREDGE_END+1
+    , SUPERFACE_END = SUPERFACE_START + 1000
+    , SUPERELEMENT_START = SUPERFACE_END+1
     , FORCE_TOPOLOGY_TO_UNSIGNED = ~0U // max unsigned int
   };
 
@@ -126,126 +124,101 @@ struct topology
   //***************************************************************************
 
   /// is this topology valid
-  BOOST_GPU_ENABLED
   bool is_valid() const { return m_value != INVALID_TOPOLOGY; }
 
   /// get the name of this topology
   std::string name() const;
 
   /// does this topology have homogeneous faces
-  BOOST_GPU_ENABLED
   bool has_homogeneous_faces() const;
 
   /// is this topology a shell topology (i.e. an element with only two sides)
-  BOOST_GPU_ENABLED
   bool is_shell() const;
 
   /// what is the rank of this topology
-  BOOST_GPU_ENABLED
   rank_t rank() const;
 
   /// what is the side rank of this topology
-  BOOST_GPU_ENABLED
   rank_t side_rank() const;
 
   /// what is the topological dimension of this topology
-  BOOST_GPU_ENABLED
   unsigned dimension() const;
 
   /// how many nodes define this topology
-  BOOST_GPU_ENABLED
+  STK_FUNCTION
   unsigned num_nodes() const;
 
   /// how many nodes are vertices
-  BOOST_GPU_ENABLED
   unsigned num_vertices() const;
 
   /// how many edges does this topology have
-  BOOST_GPU_ENABLED
   unsigned num_edges() const;
 
   /// how many faces does this topology have
-  BOOST_GPU_ENABLED
   unsigned num_faces() const;
 
   /// how many different node permutations does this topology have
-  BOOST_GPU_ENABLED
   unsigned num_permutations() const;
 
   /// how many different positive node permutations does this topology have
-  BOOST_GPU_ENABLED
   unsigned num_positive_permutations() const;
 
-  BOOST_GPU_ENABLED
   bool is_positive_polarity(unsigned permutation_ordinal) const {
       return (permutation_ordinal < num_positive_permutations());
   }
 
   /// is this topology defined on the given spatial dimension
-  BOOST_GPU_ENABLED
   bool defined_on_spatial_dimension(unsigned spatial_dimension) const;
 
   /// what is the base topology (i.e. topology where num_nodes == num_vertices)
-  BOOST_GPU_ENABLED
   topology base() const;
 
   /// what is the topology of the given edge
-  BOOST_GPU_ENABLED
   topology edge_topology() const;
 
   /// what is the topology of the given face
-  BOOST_GPU_ENABLED
   topology face_topology(unsigned face_ordinal = 0) const;
 
   /// fill the output ordinals with the ordinals that make up the given edge
   template <typename OrdinalOutputIterator>
-  BOOST_GPU_ENABLED
   void edge_node_ordinals(unsigned edge_ordinal, OrdinalOutputIterator output_ordinals) const;
 
   /// fill the output ordinals with the ordinals that make up the given face
   template <typename OrdinalOutputIterator>
-  BOOST_GPU_ENABLED
   void face_node_ordinals(unsigned face_ordinal, OrdinalOutputIterator output_ordinals) const;
 
   /// fill the output ordinals with the ordinals that make up the given permutation
   template <typename OrdinalOutputIterator>
-  BOOST_GPU_ENABLED
   void permutation_node_ordinals(unsigned permutation_ordinal, OrdinalOutputIterator output_ordinals) const;
 
   /// fill the output nodes with the nodes that make up the given edge
   /// input 'nodes' is expected to be of length num_nodes.
   template <typename NodeArray, typename NodeOutputIterator>
-  BOOST_GPU_ENABLED
   void edge_nodes(const NodeArray & nodes, unsigned edge_ordinal, NodeOutputIterator output_nodes) const;
 
   /// fill the output nodes with the nodes that make up the given face
   /// input 'nodes' is expected to be of length num_nodes.
  template <typename NodeArray, typename NodeOutputIterator>
-  BOOST_GPU_ENABLED
   void face_nodes(const NodeArray & nodes, unsigned face_ordinal, NodeOutputIterator output_nodes) const;
 
   /// fill the output nodes with the nodes that make up the given permutation
   /// input 'nodes' is expected to be of length num_nodes.
   template <typename NodeArray, typename NodeOutputIterator>
-  BOOST_GPU_ENABLED
   void permutation_nodes(const NodeArray & nodes, unsigned permutation_ordinal, NodeOutputIterator output_nodes) const;
 
   /// do the two arrays define equivalent entities (same nodes, but maybe a different permutation)
   /// return a pair<bool, permutation_number> bool and permutation number from a to b
   template <typename NodeArrayA, typename NodeArrayB>
-  BOOST_GPU_ENABLED
-  std::pair<bool,unsigned> equivalent(const NodeArrayA & a, const NodeArrayB & b) const;
+  std::pair<bool,unsigned> equivalent(const NodeArrayA & a, const NodeArrayB & possible_permutation_of_a) const;
 
   /// return the permutation index which gives the lowest lexicographical ordering of the nodes
   /// input 'nodes' is expected to be of length num_nodes.
   template <typename NodeArray>
-  BOOST_GPU_ENABLED
   unsigned lexicographical_smallest_permutation(const NodeArray &nodes, bool only_positive_permutations = false) const;
 
   /// return the permutation index which gives the lowest lexicographical ordering of the nodes that preserves polarity
   /// input 'nodes' is expected to be of length num_nodes.
   template <typename NodeArray>
-  BOOST_GPU_ENABLED
   unsigned lexicographical_smallest_permutation_preserve_polarity(const NodeArray &nodes, const NodeArray &element_nodes) const;
 
   /// fill the output ordinals with the ordinals that make up the given sub topology
@@ -253,7 +226,6 @@ struct topology
 #pragma hd_warning_disable
 #endif
   template <typename OrdinalOutputIterator>
-  BOOST_GPU_ENABLED
   void sub_topology_node_ordinals(unsigned sub_rank, unsigned sub_ordinal, OrdinalOutputIterator output_ordinals) const
   {
     switch(sub_rank)
@@ -271,7 +243,6 @@ struct topology
 #pragma hd_warning_disable
 #endif
   template <typename NodeArray, typename NodeOutputIterator>
-  BOOST_GPU_ENABLED
   void sub_topology_nodes(const NodeArray & nodes, unsigned sub_rank, unsigned sub_ordinal, NodeOutputIterator output_nodes) const
   {
     switch(sub_rank)
@@ -284,7 +255,6 @@ struct topology
   }
 
   /// how many 'sub topologies' does this topology have
-  BOOST_GPU_ENABLED
   unsigned num_sub_topology(unsigned sub_rank) const
   {
     switch(sub_rank)
@@ -299,7 +269,6 @@ struct topology
   }
 
   /// what is the topology of the given sub topology
-  BOOST_GPU_ENABLED
   topology sub_topology(unsigned sub_rank, unsigned sub_ordinal = 0) const
   {
     switch(sub_rank)
@@ -316,7 +285,6 @@ struct topology
 
   /// fill the output ordinals with the ordinals that make up the given side topology
   template <typename OrdinalOutputIterator>
-  BOOST_GPU_ENABLED
   void side_node_ordinals(unsigned side_ordinal, OrdinalOutputIterator output_ordinals) const
   {
     sub_topology_node_ordinals( side_rank(), side_ordinal, output_ordinals);
@@ -325,14 +293,12 @@ struct topology
   /// fill the output nodes with the nodes that make up the given side topology
   /// input 'nodes' is expected to be of length num_nodes.
   template <typename NodeArray, typename NodeOutputIterator>
-  BOOST_GPU_ENABLED
   void side_nodes(const NodeArray & nodes, unsigned side_ordinal, NodeOutputIterator output_nodes) const
   {
     sub_topology_nodes( nodes, side_rank(), side_ordinal, output_nodes);
   }
 
   /// how many 'side topologies' does this topology have
-  BOOST_GPU_ENABLED
   unsigned num_sides() const
   {
     unsigned num_sides_out = 0u;
@@ -344,34 +310,46 @@ struct topology
 
 
   /// what is the topology of the given side topology
-  BOOST_GPU_ENABLED
   topology side_topology(unsigned side_ordinal = 0) const
   {
     return sub_topology(side_rank(), side_ordinal);
   }
 
-  BOOST_GPU_ENABLED
   bool is_superelement() const
   {
     return m_value > SUPERELEMENT_START;
   }
 
+  STK_FUNCTION
+  bool is_superface() const
+  {
+    return m_value > SUPERFACE_START && m_value < SUPERFACE_END;
+  }
+
+  STK_FUNCTION
+  bool is_superedge() const
+  {
+    return m_value > SUPEREDGE_START && m_value < SUPEREDGE_END;
+  }
+
+  bool is_super_topology() const
+  {
+    return is_superelement() || is_superface() || is_superedge();
+  }
+
   //***************************************************************************
   //cast to integer type
   //***************************************************************************
 
   /// implicit cast to topology_t enum type
-  BOOST_GPU_ENABLED
   operator topology_t() const
   { return m_value; }
 
   /// return topology_t enum type
-  BOOST_GPU_ENABLED
   topology_t operator()() const
   { return m_value; }
 
   /// return topology_t enum type
-  BOOST_GPU_ENABLED
   topology_t value() const
   { return m_value; }
 
@@ -379,13 +357,13 @@ struct topology
   //constructors
   //***************************************************************************
   /// default construct to invalid
-  BOOST_GPU_ENABLED
+  STK_FUNCTION
   topology()
     : m_value(INVALID_TOPOLOGY)
   {}
 
   /// implicit construct from a topology_t
-  BOOST_GPU_ENABLED
+  STK_FUNCTION
   topology(topology_t topo)
     : m_value(topo)
   {}
@@ -417,7 +395,6 @@ struct topology
   // topology_type conversion constructors
   //***************************************************************************
   template <topology_t Topology>
-  BOOST_GPU_ENABLED
   topology(topology_type<Topology> /* t */ )
     : m_value(Topology)
   {}
@@ -432,14 +409,14 @@ struct topology
 //***************************************************************************
 //increment and decrement rank_t
 //***************************************************************************
-BOOST_GPU_ENABLED inline
+inline
 topology::rank_t operator++(stk::topology::rank_t &r)
 {
   r = static_cast<topology::rank_t>(r+1);
   return r;
 }
 
-BOOST_GPU_ENABLED inline
+inline
 topology::rank_t operator++(stk::topology::rank_t &r,int)
 {
   topology::rank_t tmp = r;
@@ -447,14 +424,14 @@ topology::rank_t operator++(stk::topology::rank_t &r,int)
   return tmp;
 }
 
-BOOST_GPU_ENABLED inline
+inline
 topology::rank_t operator--(stk::topology::rank_t &r)
 {
   r = static_cast<topology::rank_t>(r-1);
   return r;
 }
 
-BOOST_GPU_ENABLED inline
+inline
 topology::rank_t operator--(stk::topology::rank_t &r,int)
 {
   topology::rank_t tmp = r;
@@ -465,14 +442,14 @@ topology::rank_t operator--(stk::topology::rank_t &r,int)
 //***************************************************************************
 //increment and decrement topology_t
 //***************************************************************************
-BOOST_GPU_ENABLED inline
+inline
 topology::topology_t operator++(stk::topology::topology_t &t)
 {
   t = static_cast<topology::topology_t>(t+1);
   return t;
 }
 
-BOOST_GPU_ENABLED inline
+inline
 topology::topology_t operator++(stk::topology::topology_t &t,int)
 {
   topology::topology_t tmp = t;
@@ -480,14 +457,14 @@ topology::topology_t operator++(stk::topology::topology_t &t,int)
   return tmp;
 }
 
-BOOST_GPU_ENABLED inline
+inline
 topology::topology_t operator--(stk::topology::topology_t &t)
 {
   t = static_cast<topology::topology_t>(t-1);
   return t;
 }
 
-BOOST_GPU_ENABLED inline
+inline
 topology::topology_t operator--(stk::topology::topology_t &t,int)
 {
   topology::topology_t tmp = t;
@@ -498,14 +475,14 @@ topology::topology_t operator--(stk::topology::topology_t &t,int)
 //***************************************************************************
 //increment and decrement topology
 //***************************************************************************
-BOOST_GPU_ENABLED inline
+inline
 topology operator++(topology &t)
 {
   ++t.m_value;
   return t;
 }
 
-BOOST_GPU_ENABLED inline
+inline
 topology operator++(topology &t,int)
 {
   topology tmp = t;
@@ -513,14 +490,14 @@ topology operator++(topology &t,int)
   return tmp;
 }
 
-BOOST_GPU_ENABLED inline
+inline
 topology operator--(topology &t)
 {
   --t.m_value;
   return t;
 }
 
-BOOST_GPU_ENABLED inline
+inline
 topology operator--(topology &t,int)
 {
   topology tmp = t;
@@ -531,14 +508,42 @@ topology operator--(topology &t,int)
 //***************************************************************************
 //create superelement
 //***************************************************************************
-BOOST_GPU_ENABLED inline
+inline
+topology create_superedge_topology(unsigned num_nodes)
+{
+  if ( num_nodes < 1u ) return topology::INVALID_TOPOLOGY;
+  return static_cast<topology::topology_t>(num_nodes + topology::SUPEREDGE_START);
+}
+
+inline
+topology create_superedge_topology(int num_nodes)
+{
+  if ( num_nodes < 1 ) return topology::INVALID_TOPOLOGY;
+  return static_cast<topology::topology_t>(num_nodes + topology::SUPEREDGE_START);
+}
+
+inline
+topology create_superface_topology(unsigned num_nodes)
+{
+  if ( num_nodes < 1u ) return topology::INVALID_TOPOLOGY;
+  return static_cast<topology::topology_t>(num_nodes + topology::SUPERFACE_START);
+}
+
+inline
+topology create_superface_topology(int num_nodes)
+{
+  if ( num_nodes < 1 ) return topology::INVALID_TOPOLOGY;
+  return static_cast<topology::topology_t>(num_nodes + topology::SUPERFACE_START);
+}
+
+inline
 topology create_superelement_topology(unsigned num_nodes)
 {
   if ( num_nodes < 1u ) return topology::INVALID_TOPOLOGY;
   return static_cast<topology::topology_t>(num_nodes + topology::SUPERELEMENT_START);
 }
 
-BOOST_GPU_ENABLED inline
+inline
 topology create_superelement_topology(int num_nodes)
 {
   if ( num_nodes < 1 ) return topology::INVALID_TOPOLOGY;
