@@ -4,6 +4,8 @@
 #include <string>
 #include <stk_util/parallel/Parallel.hpp>  // for ParallelMachine, etc
 #include <stk_io/DatabasePurpose.hpp> // for stk::io::DatabasePurpose
+#include <stk_io/FillMesh.hpp>
+#include <stk_io/WriteMesh.hpp>
 namespace stk { namespace mesh { class BulkData; }}
 namespace stk { namespace io { class StkMeshIoBroker; } }
 
@@ -12,14 +14,6 @@ namespace stk
 namespace unit_test_util
 {
 
-void fill_mesh_using_stk_io(const std::string &meshSpec, stk::mesh::BulkData &bulkData);
-void fill_mesh_using_stk_io_with_auto_decomp(const std::string &meshSpec, stk::mesh::BulkData &bulkData);
-void fill_mesh_using_stk_io_preexisting(stk::io::StkMeshIoBroker & stkIo, const std::string& meshSpec, stk::mesh::BulkData& bulkData);
-
-void write_mesh_using_stk_io(const std::string &filename,
-                             stk::mesh::BulkData &bulkData,
-                             stk::io::DatabasePurpose databasePurpose = stk::io::WRITE_RESULTS);
-
 // Example:  meshSizeSpec = "2x2x1"
 void generated_mesh_to_file_in_serial(const std::string &meshSizeSpec, const std::string &fileName);
 
@@ -32,5 +26,4 @@ void generate_mesh_from_serial_spec_and_load_in_parallel_with_auto_decomp(const
 } // namespace unit_test_util
 } // namespace stk
 
-
 #endif
