@@ -129,7 +129,7 @@ public:
         m_mesh = new stk::mesh::BulkData(*m_meta,communicator);
         std::ostringstream oss;
         oss << "generated:" << num_x << "x" << num_y << "x" << m_mesh->parallel_size() << "|sideset:xXyYzZ";
-        std::string exodusFileName = unitTestUtils::getOption("-i", oss.str());
+        std::string exodusFileName = stk::unit_test_util::get_option("-i", oss.str());
         stk::io::StkMeshIoBroker exodus_file_reader(communicator);
         exodus_file_reader.set_bulk_data(*m_mesh);
         exodus_file_reader.add_mesh_database(exodusFileName, stk::io::READ_MESH);
