@@ -57,7 +57,6 @@
 #include "stk_mesh/base/Types.hpp"      // for EntityProc, EntityVector, etc
 #include "stk_topology/topology.hpp"    // for topology, etc
 #include "stk_util/util/PairIter.hpp"   // for PairIter
-#include <stk_util/parallel/CommSparse.hpp>
 
 #include "stk_io/StkMeshIoBroker.hpp"
 #include <stk_mesh/base/Comm.hpp>
@@ -840,7 +839,7 @@ TEST(BulkDataModificationEnd, DISABLED_create_edges_with_min_map)
 
         const int spatialDim = 3;
         stk::mesh::MetaData stkMeshMetaData(spatialDim);
-        stk::unit_test_util::BulkDataTester stkMeshBulkData(stkMeshMetaData, communicator, stk::mesh::ConnectivityMap::minimal_upward_connectivity_map());
+        stk::unit_test_util::BulkDataTester stkMeshBulkData(stkMeshMetaData, communicator, stk::mesh::ConnectivityMap::default_map());
 
         // Elements 1 and 2 on proc 0, Elements 3 and 4 on proc 1
         // Elements 2 and 3 are shared because of nodes 9, 10, 11, 12
