@@ -42,7 +42,6 @@
 #include <map>                          // for map, multimap
 #include <memory>                       //
 #include <set>                          // for multiset, set
-#include <stk_util/diag/Mapv.hpp>       // for Mapv, MapvNode, etc
 #include <stk_util/diag/String.hpp>     // for String, Identifier
 #include <stk_util/diag/StringUtil.hpp>  // for demangle
 #include <stk_util/util/Array.hpp>      // for Array, ArrayContainer
@@ -98,9 +97,7 @@ Writer &operator<<(Writer &dout, const std::type_info &t);
  */
 template <class T, class U>
 Writer &operator<<(Writer & dout, const std::pair<T, U> &pair) {
-//  dout << typeid(pair) << "(" << pair.first << ":" << pair.second << ")";
   dout << "(" << pair.first << ":" << pair.second << ")";
-
   return dout;
 }
 
@@ -463,65 +460,6 @@ dump(
   return dout;
 }
 
-// /**
-//  * @brief Template <b>dump</b> prints the object contained within a
-//  * hash_map to the diagnostic writer.
-//  *
-//  * @param dout    a <b>Writer</b> reference to the diagnostic writer to
-//  *      write the hash_map to.
-//  *
-//  * @param t    a <b>hash_map</b> of objects.
-//  *
-//  * @return    a <b>Writer</b> reference to this object
-//  */
-// template <class Key, class T>
-// Writer &
-// dump(
-//   Writer &      dout,
-//   const hash_map<Key, T> &  t)
-// {
-//   if (dout.shouldPrint()) {
-//     dout << typeid(t) << ", size " << t.size() << push << dendl;
-
-//     for (typename hash_map<Key, T>::const_iterator it = t.begin(); it != t.end(); ++it)
-//       dout <<  "[" << (*it).first << "] " << (*it).second << dendl;
-
-//     dout << pop;
-//   }
-
-//   return dout;
-// }
-
-// /**
-//  * @brief Template <b>dump</b> prints the object pointed to that are
-//  * contained within a hash_map to the diagnostic writer.
-//  *
-//  * @param dout    a <b>Writer</b> reference to the diagnostic writer to
-//  *      write the hash_map to.
-//  *
-//  * @param t    a <b>hash_map</b> of objects.
-//  *
-//  * @return    a <b>Writer</b> reference to this object
-//  */
-// template <class Key, class T>
-// Writer &
-// dump(
-//   Writer &      dout,
-
-//   const hash_map<Key, T *> &  t)
-// {
-//   if (dout.shouldPrint()) {
-//     dout << typeid(t) << ", size " << t.size() << push << dendl;
-
-//     for (typename hash_map<Key, T *>::const_iterator it = t.begin(); it != t.end(); ++it)
-//       dout << "[" << (*it).first << "] " << (*it)->second << dendl;
-
-//     dout << pop;
-//   }
-
-//   return dout;
-// }
-
 
 template <size_t n>
 Writer &operator<<(Writer &dout, const std::bitset<n> &t) {
@@ -741,7 +679,7 @@ operator<<(Writer &dout, const sierra::Array<ElementType, Tag0, Tag1, Tag2, Tag3
   if (dout.shouldPrint()) {
     std::string type = sierra::demangle(typeid(X).name());
     dout << type.substr(0, type.find(", sierra::TypeListEnd")) << ">" << push << dendl;
-    dout.getStream() << array; // ArrayVerbosePrint<X::NumDim>::dump(dout, array.dimension(), array.ptr(), array.stride());
+    dout.getStream() << array;
     dout << pop << dendl;
   }
   return dout;
@@ -765,7 +703,7 @@ operator<<(Writer &dout, const sierra::ArrayContainer<ElementType, Tag0, Tag1, T
   if (dout.shouldPrint()) {
     std::string type = sierra::demangle(typeid(X).name());
     dout << type.substr(0, type.find(", sierra::TypeListEnd")) << ">" << push << dendl;
-    dout.getStream() << array; // ArrayVerbosePrint<X::NumDim>::dump(dout, array.dimension(), array.ptr(), array.stride());
+    dout.getStream() << array;
     dout << pop << dendl;
   }
   return dout;
@@ -780,7 +718,7 @@ operator<<(Writer &dout, const sierra::FArray<ElementType, Dimension> &array)
 
   if (dout.shouldPrint()) {
     dout << sierra::demangle(typeid(X).name()) << push << dendl;
-    dout.getStream() << array; // ArrayVerbosePrint<X::NumDim>::dump(dout, array.dimension(), array.ptr(), array.stride());
+    dout.getStream() << array;
     dout << pop << dendl;
   }
   return dout;
@@ -794,42 +732,12 @@ operator<<(Writer &dout, const sierra::FArrayContainer<ElementType, Dimension> &
 
   if (dout.shouldPrint()) {
     dout << sierra::demangle(typeid(X).name()) << push << dendl;
-    dout.getStream() << array; // ArrayVerbosePrint<X::NumDim>::dump(dout, array.dimension(), array.ptr(), array.stride());
+    dout.getStream() << array;
     dout << pop << dendl;
   }
   return dout;
 }
 
-/**
- * @brief Template function <b>dump</b> writes a Mapv_no_delete object to the
- * diagnostic writer.
- *
- * @param dout		a <b>Writer</b> reference to the diagnostic writer to
- *			write the Mapv_no_delete to.
- *
- * @param t		a <b>sierra::String</b> const reference to the Mapv_no_delete to write.
- *
- * @return		a <b>Writer</b> reference to this object
- */
-template <class T>
-Writer &
-dump(
-  Writer &			dout,
-  const sierra::Mapv_no_delete<T> &	t)
-{
-  if (dout.shouldPrint()) {
-    dout << typeid(t) << ", size " << t.size() << push << dendl;
-
-    for (typename sierra::Mapv_no_delete<T>::const_iterator it = t.begin(); it != t.end(); ++it) {
-      dout << "[" << (*it).mapv_key() << "] " << (*it) << dendl;
-    }
-
-    dout << pop;
-  }
-
-  return dout;
-}
-
 /**
  * @brief Template function <b>dump</b> writes the vecmap object to the
  * diagnostic writer.
@@ -977,58 +885,6 @@ dump(
   return dout;
 }
 
-/**
- * @brief Template <b>dump</b> writes a MapvNode object to the diagnostic writer.
- *
- * @param dout		a <b>Writer</b> reference to the diagnostic writer to write the
- *			Mapvnod to.
- *
- * @param t		a <b>MapvNode</b> const reference to the MapvNode to write.
- *
- * @return		a <b>Writer</b> reference to this object
- */
-template <class T, class U>
-Writer &
-dump(
-  Writer &			dout,
-  const sierra::MapvNode<T, U> &	t)
-{
-  if (dout.shouldPrint()) {
-    dout << typeid(t) << ", " << t.mapv_key();
-  }
-
-  return dout;
-}
-
-/**
- * @brief Template function <b>dump</b> writes a Mapv object to the diagnostic writer.
- *
- * @param dout		a <b>Writer</b> reference to the diagnostic writer to write the
- *			Mapv to.
- *
- * @param t		a <b>std::vector</b> const reference to the Mapv to write.
- *
- * @return		a <b>Writer</b> reference to this object
- */
-template <class T, class U>
-Writer &
-dump(
-  Writer &		dout,
-  const sierra::Mapv<T, U> &	t)
-{
-  if (dout.shouldPrint()) {
-    dout << typeid(t) << ", size " << t.size() << push << dendl;
-
-    for (typename sierra::Mapv<T, U>::const_iterator it = t.begin(); it != t.end(); ++it) {
-      dout << "[" << (*it).mapv_key() << "] " << (*it) << dendl;
-    }
-
-    dout << pop;
-  }
-
-  return dout;
-}
-
 /**
  * @brief Member function <b>operator<<</b> writer a vecset object the
  * diagnostic writer.
@@ -1093,54 +949,6 @@ Writer &operator<<(Writer &dout, const sierra::vecmap<Key *, T *, U> &t) {
   return dump(dout, t);
 }
 
-/**
- * @brief Template function <b>operator<<</b> writes a Mpav_no_delete object to the
- * diagnostic writer.
- *
- * @param dout		a <b>Writer</b> reference to the diagnostic writer to
- *			write the Mapv_no_delete to.
- *
- * @param t		a <b>vecmap</b> const reference to the Mapv_no_delete.
- *
- * @return		a <b>Writer</b> reference to this object
- */
-template <class T>
-Writer &operator<<(Writer &dout, const sierra::Mapv_no_delete<T> &t) {
-  return dump(dout, t);
-}
-
-/**
- * @brief Member function <b>operator<<</b> writes a Mapv object to the diagnostic
- * writer.
- *
- * @param dout		a <b>Writer</b> reference to the diagnostic writer to
- *			write the Mapv to.
- *
- * @param t		a <b>vecmap</b> const reference to the Mapv.
- *
- * @return		a <b>Writer</b> reference to this object
- */
-template <class T, class U>
-Writer &operator<<(Writer &dout, const sierra::Mapv<T, U> &t) {
-  return dump(dout, t);
-}
-
-/**
- * @brief Template function <b>operator<<</b> writes a MapvNode object to the
- * diagnostic writer.
- *
- * @param dout		a <b>Writer</b> reference to the diagnostic writer to
- *			write the MapvNode to.
- *
- * @param t		a <b>MapvNode</b> const reference to the MapvNode.
- *
- * @return		a <b>Writer</b> reference to this object
- */
-template <class T, class U>
-Writer &operator<<(Writer &dout, const sierra::MapvNode<T, U> &t) {
-  return dump(dout, t);
-}
-
 /**
  * @brief Template function <b>operator<<</b> writes a vecmap of pointers object to
  * the dignostic writer.
