@@ -69,13 +69,6 @@ void reset_warning_count();
  */
 void set_max_warning_count(unsigned int max_messages);
 
-/**
- * @brief Function <b>set_max_messages</b> sets the maximum number of warning
- * and doomed messages displayed before the message is thrown as a RuntimeError exception.
- *
- */
-unsigned get_max_warning_count();
-
 /**
  * @brief Member function <b>report_warning</b> ...
  *
