@@ -175,7 +175,7 @@ DistributedIndex::DistributedIndex (
       m_key_span.assign(partition_bounds.begin(),partition_bounds.end());
     }
     if (m_comm_size > 1) {
-      MPI_Bcast( (m_key_span.empty() ? NULL : & m_key_span[0]), info[0] * sizeof(KeySpan), MPI_BYTE, 0, comm );
+      MPI_Bcast( m_key_span.data(), info[0] * sizeof(KeySpan), MPI_BYTE, 0, comm );
     }
   }
 #else
@@ -680,7 +680,7 @@ void DistributedIndex::generate_new_global_key_upper_bound(
 
 #if defined( STK_HAS_MPI )
   if (m_comm_size > 1) {
-    MPI_Allreduce( (local_counts.empty() ? NULL : & local_counts[0]) , (global_counts.empty() ? NULL : & global_counts[0]) ,
+    MPI_Allreduce( local_counts.data(), global_counts.data(),
                    m_span_count + 1 , MPI_UNSIGNED_LONG ,
                    MPI_SUM , m_comm );
   }
@@ -834,8 +834,8 @@ void DistributedIndex::generate_new_keys_global_planning(
 #if defined(__INTEL_COMPILER) && (__INTEL_COMPILER >= 1200)
       {
         // MPI doesn't do 'const' in its interface, but the send buffer is const
-        void * send_buf = const_cast<long int*>( (new_request.empty() ? NULL : & new_request[0]) );
-        void * recv_buf = (new_request_global.empty() ? NULL : & new_request_global[0]) ;
+        void * send_buf = const_cast<long int*>( new_request.data() );
+        void * recv_buf = new_request_global.data();
         for (int root = 0; root < m_comm_size; ++root)
           {
             MPI_Gather( send_buf , m_span_count , MPI_LONG ,
@@ -845,8 +845,8 @@ void DistributedIndex::generate_new_keys_global_planning(
 #else
       {
         // MPI doesn't do 'const' in its interface, but the send buffer is const
-        void * send_buf = const_cast<long int*>( (new_request.empty() ? NULL : & new_request[0]) );
-        void * recv_buf = (new_request_global.empty() ? NULL : & new_request_global[0]) ;
+        void * send_buf = const_cast<long int*>( new_request.data() );
+        void * recv_buf = new_request_global.data();
         MPI_Allgather( send_buf , m_span_count , MPI_LONG,
                        recv_buf , m_span_count , MPI_LONG , m_comm );
       }
@@ -1019,7 +1019,7 @@ void DistributedIndex::generate_new_keys(
   // Unpacking
   unpack_recv_buffer( all, m_comm_size, new_keys);
 
-  stk::util::radix_sort_unsigned((new_keys.empty() ? NULL : &new_keys[0]), new_keys.size());
+  stk::util::radix_sort_unsigned(new_keys.data(), new_keys.size());
 
   requested_keys.resize( m_span_count );
 
