@@ -538,7 +538,7 @@ public:
 		  const unsigned n2, const unsigned n3,
 		  const unsigned n4, const unsigned n5,
 		  const unsigned n6, const unsigned n7 )
-    : BaseType(NULL, n0, n1, n2, n3, n4, n5, n6, n7 ),
+    : BaseType(nullptr, n0, n1, n2, n3, n4, n5, n6, n7 ),
       m_capacity(0)
   {
     resize_memory( m_stride[NumDim] );
@@ -548,7 +548,7 @@ public:
 		  const unsigned n2, const unsigned n3,
 		  const unsigned n4, const unsigned n5,
 		  const unsigned n6 )
-    : BaseType( NULL, n0, n1, n2, n3, n4, n5, n6 ),
+    : BaseType( nullptr, n0, n1, n2, n3, n4, n5, n6 ),
       m_capacity(0)
   {
     resize_memory( m_stride[NumDim] );
@@ -557,7 +557,7 @@ public:
   ArrayContainer( const unsigned n0, const unsigned n1,
 		  const unsigned n2, const unsigned n3,
 		  const unsigned n4, const unsigned n5 )
-    : BaseType( NULL, n0, n1, n2, n3, n4, n5 ),
+    : BaseType( nullptr, n0, n1, n2, n3, n4, n5 ),
       m_capacity(0)
   {
     resize_memory( m_stride[NumDim] );
@@ -566,7 +566,7 @@ public:
   ArrayContainer( const unsigned n0, const unsigned n1,
 		  const unsigned n2, const unsigned n3,
 		  const unsigned n4 )
-    : BaseType( NULL, n0, n1, n2, n3, n4 ),
+    : BaseType( nullptr, n0, n1, n2, n3, n4 ),
       m_capacity(0)
   {
     resize_memory( m_stride[NumDim] );
@@ -574,7 +574,7 @@ public:
 
   ArrayContainer( const unsigned n0, const unsigned n1,
 		  const unsigned n2, const unsigned n3 )
-    : BaseType( NULL, n0, n1, n2, n3 ),
+    : BaseType( nullptr, n0, n1, n2, n3 ),
       m_capacity(0)
   {
     resize_memory( m_stride[NumDim] );
@@ -582,28 +582,28 @@ public:
 
   ArrayContainer( const unsigned n0, const unsigned n1,
 		  const unsigned n2 )
-    : BaseType( NULL, n0, n1, n2 ),
+    : BaseType( nullptr, n0, n1, n2 ),
       m_capacity(0)
   {
     resize_memory( m_stride[NumDim] );
   }
 
   ArrayContainer( const unsigned n0, const unsigned n1 )
-    : BaseType( NULL, n0, n1 ),
+    : BaseType( nullptr, n0, n1 ),
       m_capacity(0)
   {
     resize_memory( m_stride[NumDim] );
   }
 
   ArrayContainer( const unsigned n0 )
-    : BaseType( NULL, n0 ),
+    : BaseType( nullptr, n0 ),
       m_capacity(0)
   {
     resize_memory( m_stride[NumDim] );
   }
 
   ArrayContainer( const unsigned n[] )
-    : BaseType( NULL, n ),
+    : BaseType( nullptr, n ),
       m_capacity(0)
   {
     resize_memory( m_stride[NumDim] );
