@@ -183,7 +183,6 @@ checkConstantFadVectorView2(const ViewType& view,
   for (size_type i4=0; i4<h_view.dimension_4(); ++i4) {
   for (size_type i5=0; i5<h_view.dimension_5(); ++i5) {
   for (size_type i6=0; i6<h_view.dimension_6(); ++i6) {
-#if defined( KOKKOS_USING_EXPERIMENTAL_VIEW )
     for (size_type k=0; k<num_ensemble; ++k)
       TEUCHOS_TEST_EQUALITY(h_view(i0,i1,i2,i3,i4,i5,i6,0).val().coeff(k),
                             v.val().coeff(k), out, success);
@@ -192,16 +191,6 @@ checkConstantFadVectorView2(const ViewType& view,
         TEUCHOS_TEST_EQUALITY(h_view(i0,i1,i2,i3,i4,i5,i6,0).dx(j).coeff(k),
                               v.dx(j).coeff(k), out, success);
     }
-#else
-    for (size_type k=0; k<num_ensemble; ++k)
-      TEUCHOS_TEST_EQUALITY(h_view.at(i0,i1,i2,i3,i4,i5,i6,0).val().coeff(k),
-                            v.val().coeff(k), out, success);
-    for (size_type j=0; j<num_fad; ++j) {
-      for (size_type k=0; k<num_ensemble; ++k)
-        TEUCHOS_TEST_EQUALITY(h_view.at(i0,i1,i2,i3,i4,i5,i6,0).dx(j).coeff(k),
-                              v.dx(j).coeff(k), out, success);
-    }
-#endif
   }}}}}}}
 
   return success;
