@@ -25,7 +25,7 @@ using Teuchos::ParameterList;
 
 // hide stuff
 namespace {
-   // Simple preconditioner class that adds a counter 
+   // Simple preconditioner class that adds a counter
    class StratimikosFactoryPreconditioner :  public Thyra::DefaultPreconditioner<double>{
    public:
       StratimikosFactoryPreconditioner() : iter_(0) {}
@@ -41,14 +41,14 @@ namespace {
 
    // factory used to initialize the Teko::StratimikosFactory
    // user data
-   class TekoFactoryBuilder 
+   class TekoFactoryBuilder
          : public Teuchos::AbstractFactory<Thyra::PreconditionerFactoryBase<double> > {
    public:
       TekoFactoryBuilder(const Teuchos::RCP<Stratimikos::DefaultLinearSolverBuilder> & builder,
                          const Teuchos::RCP<Teko::RequestHandler> & rh) : builder_(builder), requestHandler_(rh) {}
       Teuchos::RCP<Thyra::PreconditionerFactoryBase<double> > create() const
       { return Teuchos::rcp(new StratimikosFactory(builder_,requestHandler_)); }
- 
+
    private:
       Teuchos::RCP<Stratimikos::DefaultLinearSolverBuilder> builder_;
       Teuchos::RCP<Teko::RequestHandler> requestHandler_;
@@ -87,7 +87,7 @@ bool StratimikosFactory::isCompatible(const Thyra::LinearOpSourceBase<double> &f
    Thyra::EApplyEpetraOpAs epetraFwdOpApplyAs;
    Thyra::EAdjointEpetraOp epetraFwdOpAdjointSupport;
    double epetraFwdOpScalar;
-  
+
    // check to make sure this is an epetra CrsMatrix
    Teuchos::RCP<const LinearOpBase<double> > fwdOp = fwdOpSrc.getOp();
    epetraFwdOpViewExtractor_->getEpetraOpView(
@@ -164,7 +164,7 @@ void StratimikosFactory::initializePrec_Thyra(
   Teuchos::RCP<const LinearOpBase<double> > fwdOp = fwdOpSrc->getOp();
 
   // Get the concrete preconditioner object
-  StratimikosFactoryPreconditioner & defaultPrec 
+  StratimikosFactoryPreconditioner & defaultPrec
         = Teuchos::dyn_cast<StratimikosFactoryPreconditioner>(*prec);
   Teuchos::RCP<LinearOpBase<double> > prec_Op = defaultPrec.getNonconstUnspecifiedPrecOp();
 
@@ -198,7 +198,7 @@ void StratimikosFactory::initializePrec_Thyra(
   // setup reordering if required
   std::string reorderType = paramList_->get<std::string>("Reorder Type");
   if(reorderType!="") {
-     
+
      Teuchos::RCP<const Thyra::BlockedLinearOpBase<double> > blkFwdOp =
          Teuchos::rcp_dynamic_cast<const Thyra::BlockedLinearOpBase<double> >(fwdOp,true);
      RCP<const Teko::BlockReorderManager> brm = Teko::blockedReorderFromString(reorderType);
@@ -268,7 +268,7 @@ void StratimikosFactory::initializePrec_Epetra(
   TEUCHOS_TEST_FOR_EXCEPT(fwdOp.get()==NULL);
   TEUCHOS_TEST_FOR_EXCEPT(prec==NULL);
 #endif
- 
+
   //
   // Unwrap and get the forward Epetra_Operator object
   //
@@ -282,11 +282,11 @@ void StratimikosFactory::initializePrec_Epetra(
     outArg(epetraFwdOpAdjointSupport),outArg(epetraFwdOpScalar)
                                              );
   // Get the concrete preconditioner object
-  StratimikosFactoryPreconditioner & defaultPrec 
+  StratimikosFactoryPreconditioner & defaultPrec
         = Teuchos::dyn_cast<StratimikosFactoryPreconditioner>(*prec);
 
   // Get the EpetraLinearOp object that is used to implement the preconditoner linear op
-  RCP<Thyra::EpetraLinearOp> epetra_precOp 
+  RCP<Thyra::EpetraLinearOp> epetra_precOp
         = rcp_dynamic_cast<Thyra::EpetraLinearOp>(defaultPrec.getNonconstUnspecifiedPrecOp(),true);
 
   // Get the embedded ML_Epetra::MultiLevelPreconditioner object if it exists
@@ -329,7 +329,7 @@ void StratimikosFactory::initializePrec_Epetra(
 
     // Create the initial preconditioner: DO NOT compute it yet
     teko_precOp = rcp( new Teko::Epetra::InverseFactoryOperator(invFactory_));
-    
+
     timer.stop();
     if(mediumVerbosity)
       Teuchos::OSTab(out).o() <<"> Creation time = "<<timer.totalElapsedTime()<<" sec\n";
@@ -355,7 +355,7 @@ void StratimikosFactory::initializePrec_Epetra(
         }
      }
      else {
-        Teuchos::RCP<Epetra_Operator> wrappedFwdOp 
+        Teuchos::RCP<Epetra_Operator> wrappedFwdOp
            = buildWrappedEpetraOperator(epetraFwdOp,teko_precOp->getNonconstForwardOp(),*out);
 
         // write out to disk
@@ -372,7 +372,7 @@ void StratimikosFactory::initializePrec_Epetra(
            }
            else TEUCHOS_ASSERT(false);
         }
-   
+
         teko_precOp->rebuildInverseOperator(wrappedFwdOp);
      }
 
@@ -472,11 +472,11 @@ StratimikosFactory::getValidParameters() const
     pl->set("Write Block Operator",false,
             "Write out the segregated operator to disk with the name \"block-?_xx\"");
     pl->set("Strided Blocking","1",
-            "Assuming that the user wants Strided blocking, break the operating into\n"
+            "Assuming that the user wants Strided blocking, break the operator into\n"
             "blocks. The syntax can be thought to be associated with the solution\n"
             "vector. For example if your variables are [u v w p T], and we want [u v w]\n"
-            "blocked together, and p and T seperate then the relevant string is \"3 1 1\".\n"
-            "Meaning put the first 3 unknowns per node together and sperate the v and w\n"
+            "blocked together, and p and T separate then the relevant string is \"3 1 1\".\n"
+            "Meaning put the first 3 unknowns per node together and separate the v and w\n"
             "components.");
     pl->set("Reorder Type","",
             "This specifies how the blocks are reordered for use in the preconditioner.\n"
@@ -487,11 +487,11 @@ StratimikosFactory::getValidParameters() const
             "velocity and pressure forming an inner two-by-two block, and then the\n"
             "temperature unknowns forming a two-by-two system with the velocity-pressure\n"
             "block.");
-    pl->set("Inverse Type","Amesos", 
+    pl->set("Inverse Type","Amesos",
             "The type of inverse operator the user wants. This can be one of the defaults\n"
             "from Stratimikos, or a Teko preconditioner defined in the\n"
             "\"Inverse Factory Library\".");
-    pl->sublist("Inverse Factory Library",false,"Definition of Teko preconditioners."); 
+    pl->sublist("Inverse Factory Library",false,"Definition of Teko preconditioners.");
 
     validPL = pl;
   }
@@ -514,12 +514,12 @@ Teuchos::RCP<Epetra_Operator> StratimikosFactory::buildWrappedEpetraOperator(
 //    if(wrappedOp==Teuchos::null)
 //    {
 //       wrappedOp = Teuchos::rcp(new Teko::Epetra::StridedEpetraOperator(decomp_,Jac));
-// 
+//
 //       // reorder the blocks if requested
 //       std::string reorderType = paramList_->get<std::string>("Reorder Type");
 //       if(reorderType!="") {
 //          RCP<const Teko::BlockReorderManager> brm = Teko::blockedReorderFromString(reorderType);
-// 
+//
 //          // out << "Teko: Reordering = " << brm->toString() << std::endl;
 //          Teuchos::rcp_dynamic_cast<Teko::Epetra::StridedEpetraOperator>(wrappedOp)->Reorder(*brm);
 //       }
@@ -527,12 +527,12 @@ Teuchos::RCP<Epetra_Operator> StratimikosFactory::buildWrappedEpetraOperator(
 //    else {
 //       Teuchos::rcp_dynamic_cast<Teko::Epetra::StridedEpetraOperator>(wrappedOp)->RebuildOps();
 //    }
-// 
+//
 //    // test blocked operator for correctness
 //    if(paramList_->get<bool>("Test Block Operator")) {
 //       bool result
 //          = Teuchos::rcp_dynamic_cast<Teko::Epetra::StridedEpetraOperator>(wrappedOp)->testAgainstFullOperator(600,1e-14);
-// 
+//
 //       out << "Teko: Tested operator correctness:  " << (result ? "passed" : "FAILED!") << std::endl;
 //    }
 
@@ -589,7 +589,7 @@ void StratimikosFactory::buildStridedVectors(const Epetra_Operator & Jac,
    // verify that the decomposition is appropriate for this matrix
    TEUCHOS_ASSERT((rangeMap.NumMyElements() % numVars)==0);
    TEUCHOS_ASSERT((rangeMap.NumGlobalElements() % numVars)==0);
-  
+
    int * globalIds = rangeMap.MyGlobalElements();
 
    vars.resize(decomp.size());
@@ -611,7 +611,7 @@ void addTekoToStratimikosBuilder(Stratimikos::DefaultLinearSolverBuilder & build
    TEUCHOS_TEST_FOR_EXCEPTION(builder.getValidParameters()->sublist("Preconditioner Types").isParameter(stratName),std::logic_error,
                       "Teko::addTekoToStratimikosBuilder cannot add \"" + stratName +"\" because it is already included in builder!");
 
-   Teuchos::RCP<Stratimikos::DefaultLinearSolverBuilder> builderCopy 
+   Teuchos::RCP<Stratimikos::DefaultLinearSolverBuilder> builderCopy
        = Teuchos::rcp(new Stratimikos::DefaultLinearSolverBuilder(builder));
 
    // use default constructor to add Teko::StratimikosFactory
@@ -626,7 +626,7 @@ void addTekoToStratimikosBuilder(Stratimikos::DefaultLinearSolverBuilder & build
    TEUCHOS_TEST_FOR_EXCEPTION(builder.getValidParameters()->sublist("Preconditioner Types").isParameter(stratName),std::logic_error,
                       "Teko::addTekoToStratimikosBuilder cannot add \"" + stratName +"\" because it is already included in builder!");
 
-   Teuchos::RCP<Stratimikos::DefaultLinearSolverBuilder> builderCopy 
+   Teuchos::RCP<Stratimikos::DefaultLinearSolverBuilder> builderCopy
        = Teuchos::rcp(new Stratimikos::DefaultLinearSolverBuilder(builder));
 
    // build an instance of a Teuchos::AbsractFactory<Thyra::PFB> so request handler is passed onto
