@@ -94,10 +94,10 @@
 #endif
 
 // Teko includes
-#include "Epetra/Teko_EpetraHelpers.hpp"
-#include "Epetra/Teko_EpetraOperatorWrapper.hpp"
-#include "Tpetra/Teko_TpetraHelpers.hpp"
-#include "Tpetra/Teko_TpetraOperatorWrapper.hpp"
+#include "Teko_EpetraHelpers.hpp"
+#include "Teko_EpetraOperatorWrapper.hpp"
+#include "Teko_TpetraHelpers.hpp"
+#include "Teko_TpetraOperatorWrapper.hpp"
 
 // Tpetra
 #include "Thyra_TpetraLinearOp.hpp"
@@ -105,6 +105,7 @@
 #include "Tpetra_Vector.hpp"
 #include "Thyra_TpetraThyraWrappers.hpp"
 #include "TpetraExt_MatrixMatrix.hpp"
+#include "Tpetra_RowMatrixTransposer.hpp"
 
 #include <cmath>
 
@@ -439,10 +440,29 @@ RCP<Tpetra::CrsMatrix<ST,LO,GO,NT> > buildGraphLaplacian(ST * x,ST * y,ST * z,GO
   */
 void applyOp(const LinearOp & A,const MultiVector & x,MultiVector & y,double alpha,double beta)
 {
-   // Thyra::apply(*A,Thyra::NONCONJ_ELE,*x,&*y,alpha,beta);
    Thyra::apply(*A,Thyra::NOTRANS,*x,y.ptr(),alpha,beta);
 }
 
+/** \brief Apply a transposed linear operator to a multivector (think of this as a matrix
+  *        vector multiply).
+  *
+  * Apply a transposed linear operator to a multivector. This also permits arbitrary scaling
+  * and addition of the result. This function gives
+  *     
+  *    \f$ y = \alpha A^T x + \beta y \f$
+  *
+  * \param[in]     A
+  * \param[in]     x
+  * \param[in,out] y
+  * \param[in]     \alpha
+  * \param[in]     \beta
+  *
+  */
+void applyTransposeOp(const LinearOp & A,const MultiVector & x,MultiVector & y,double alpha,double beta)
+{
+   Thyra::apply(*A,Thyra::TRANS,*x,y.ptr(),alpha,beta);
+}
+
 /** \brief Update the <code>y</code> vector so that \f$y = \alpha x+\beta y\f$
   */
 void update(double alpha,const MultiVector & x,double beta,MultiVector & y)
@@ -698,43 +718,40 @@ ModifiableLinearOp getAbsRowSumMatrix(const LinearOp & op)
   */
 ModifiableLinearOp getAbsRowSumInvMatrix(const LinearOp & op)
 {
-   bool isTpetra = false;
-   RCP<const Epetra_CrsMatrix> eCrsOp;
-   RCP<const Tpetra::CrsMatrix<ST,LO,GO,NT> > tCrsOp;
+   if(Teko::TpetraHelpers::isTpetraLinearOp(op)) {
+     ST scalar = 0.0;
+     bool transp = false;
+     RCP<const Tpetra::CrsMatrix<ST,LO,GO,NT> > tCrsOp = Teko::TpetraHelpers::getTpetraCrsMatrix(op, &scalar, &transp);
 
-   try {
-      // get Epetra or Tpetra Operator
-      RCP<const Thyra::EpetraLinearOp > eOp = rcp_dynamic_cast<const Thyra::EpetraLinearOp >(op);
-      RCP<const Thyra::TpetraLinearOp<ST,LO,GO,NT> > tOp = rcp_dynamic_cast<const Thyra::TpetraLinearOp<ST,LO,GO,NT> >(op);
+     // extract diagonal
+     const RCP<Tpetra::Vector<ST,LO,GO,NT> > ptrDiag = Tpetra::createVector<ST,LO,GO,NT>(tCrsOp->getRowMap());
+     Tpetra::Vector<ST,LO,GO,NT> & diag = *ptrDiag;
+
+     // compute absolute value row sum
+     diag.putScalar(0.0);
+     for(LO i=0;i<(LO) tCrsOp->getNodeNumRows();i++) {
+        LO numEntries = tCrsOp->getNumEntriesInLocalRow (i); 
+        std::vector<LO> indices(numEntries);
+        std::vector<ST> values(numEntries);
+        Teuchos::ArrayView<const LO> indices_av(indices);
+        Teuchos::ArrayView<const ST> values_av(values);
+        tCrsOp->getLocalRowView(i,indices_av,values_av);
+
+        // build abs value row sum
+        for(LO j=0;j<numEntries;j++)
+           diag.sumIntoLocalValue(i,std::abs(values_av[j]));
+     }
+     diag.scale(scalar);
+     diag.reciprocal(diag); // invert entries
+
+     // build Thyra diagonal operator
+     return Teko::TpetraHelpers::thyraDiagOp(ptrDiag,*tCrsOp->getRowMap(),"absRowSum( " + op->getObjectLabel() + " ))");
 
-      // cast it to a CrsMatrix
-      RCP<Teuchos::FancyOStream> out = Teuchos::VerboseObjectBase::getDefaultOStream();
-      if (!eOp.is_null()){
-        eCrsOp = rcp_dynamic_cast<const Epetra_CrsMatrix>(eOp->epetra_op(),true);
-      }   
-      else if (!tOp.is_null()){
-        tCrsOp = rcp_dynamic_cast<const Tpetra::CrsMatrix<ST,LO,GO,NT> >(tOp->getConstTpetraOperator(),true);
-        isTpetra = true;
-      }
-      else
-        throw std::logic_error("Neither Epetra nor Tpetra");
    }
-   catch (std::exception & e) {
-      RCP<Teuchos::FancyOStream> out = Teuchos::VerboseObjectBase::getDefaultOStream();
+   else{
+     RCP<const Thyra::EpetraLinearOp > eOp = rcp_dynamic_cast<const Thyra::EpetraLinearOp >(op,true);
+     RCP<const Epetra_CrsMatrix> eCrsOp = rcp_dynamic_cast<const Epetra_CrsMatrix>(eOp->epetra_op(),true);
 
-      *out << "Teko: getAbsRowSumInvMatrix requires an Epetra_CrsMatrix or a Tpetra::CrsMatrix\n";
-      *out << "    Could not extract an Epetra_Operator or a Tpetra_Operator from a \"" << op->description() << std::endl;
-      *out << "           OR\n";
-      *out << "    Could not cast an Epetra_Operator to a Epetra_CrsMatrix or a Tpetra_Operator to a Tpetra::CrsMatrix\n";
-      *out << std::endl;
-      *out << "*** THROWN EXCEPTION ***\n";
-      *out << e.what() << std::endl;
-      *out << "************************\n";
-      
-      throw e;
-   }
-
-   if(!isTpetra){
      // extract diagonal
      const RCP<Epetra_Vector> ptrDiag = rcp(new Epetra_Vector(eCrsOp->RowMap()));
      Epetra_Vector & diag = *ptrDiag;
@@ -755,31 +772,6 @@ ModifiableLinearOp getAbsRowSumInvMatrix(const LinearOp & op)
      // build Thyra diagonal operator
      return Teko::Epetra::thyraDiagOp(ptrDiag,eCrsOp->RowMap(),"absRowSum( " + op->getObjectLabel() + " )");
    }
-   else {
-     // extract diagonal
-     const RCP<Tpetra::Vector<ST,LO,GO,NT> > ptrDiag = Tpetra::createVector<ST,LO,GO,NT>(tCrsOp->getRowMap());
-     Tpetra::Vector<ST,LO,GO,NT> & diag = *ptrDiag;
-
-     // compute absolute value row sum
-     diag.putScalar(0.0);
-     for(LO i=0;i<(LO) tCrsOp->getNodeNumRows();i++) {
-        LO numEntries = tCrsOp->getNumEntriesInLocalRow (i); 
-        std::vector<LO> indices(numEntries);
-        std::vector<ST> values(numEntries);
-        Teuchos::ArrayView<const LO> indices_av(indices);
-        Teuchos::ArrayView<const ST> values_av(values);
-        tCrsOp->getLocalRowView(i,indices_av,values_av);
-
-        // build abs value row sum
-        for(LO j=0;j<numEntries;j++)
-           diag.sumIntoLocalValue(i,std::abs(values_av[j]));
-     }
-     diag.reciprocal(diag); // invert entries
-
-     // build Thyra diagonal operator
-     return Teko::TpetraHelpers::thyraDiagOp(ptrDiag,*tCrsOp->getRowMap(),"absRowSum( " + op->getObjectLabel() + " ))");
-
-   }
 
 }
 
@@ -979,48 +971,25 @@ const MultiVector getDiagonal(const Teko::LinearOp & A,const DiagonalType & dt)
   */
 const ModifiableLinearOp getInvDiagonalOp(const LinearOp & op)
 {
-   bool isTpetra = false;
-   RCP<const Epetra_CrsMatrix> eCrsOp;
-   RCP<const Tpetra::CrsMatrix<ST,LO,GO,NT> > tCrsOp;
+   if (Teko::TpetraHelpers::isTpetraLinearOp(op)){
+     ST scalar = 0.0;
+     bool transp = false;
+     RCP<const Tpetra::CrsMatrix<ST,LO,GO,NT> > tCrsOp = Teko::TpetraHelpers::getTpetraCrsMatrix(op, &scalar, &transp);
 
-   try {
-      // get Epetra or Tpetra Operator
-      RCP<const Thyra::EpetraLinearOp > eOp = rcp_dynamic_cast<const Thyra::EpetraLinearOp >(op);
-      RCP<const Thyra::TpetraLinearOp<ST,LO,GO,NT> > tOp = rcp_dynamic_cast<const Thyra::TpetraLinearOp<ST,LO,GO,NT> >(op);
+     // extract diagonal
+     const RCP<Tpetra::Vector<ST,LO,GO,NT> > diag = Tpetra::createVector<ST,LO,GO,NT>(tCrsOp->getRowMap());
+     diag->scale(scalar);
+     tCrsOp->getLocalDiagCopy(*diag);
+     diag->reciprocal(*diag);
+
+     // build Thyra diagonal operator
+     return Teko::TpetraHelpers::thyraDiagOp(diag,*tCrsOp->getRowMap(),"inv(diag( " + op->getObjectLabel() + " ))");
 
-      // cast it to a CrsMatrix
-      RCP<Teuchos::FancyOStream> out = Teuchos::VerboseObjectBase::getDefaultOStream();
-      if (!eOp.is_null()){
-        eCrsOp = rcp_dynamic_cast<const Epetra_CrsMatrix>(eOp->epetra_op(),true);
-      }   
-      else if (!tOp.is_null()){
-        tCrsOp = rcp_dynamic_cast<const Tpetra::CrsMatrix<ST,LO,GO,NT> >(tOp->getConstTpetraOperator(),true);
-        isTpetra = true;
-      }
-      else
-        throw std::logic_error("Neither Epetra nor Tpetra");
    }
-   catch (std::exception & e) {
-      RCP<Teuchos::FancyOStream> out = Teuchos::VerboseObjectBase::getDefaultOStream();
-  
-      RCP<const Thyra::EpetraLinearOp > eOp = rcp_dynamic_cast<const Thyra::EpetraLinearOp >(op);
-      RCP<const Thyra::TpetraLinearOp<ST,LO,GO,NT> > tOp = rcp_dynamic_cast<const Thyra::TpetraLinearOp<ST,LO,GO,NT> >(op);
-      *out << eOp;
-      *out << tOp;
+   else {
+     RCP<const Thyra::EpetraLinearOp > eOp = rcp_dynamic_cast<const Thyra::EpetraLinearOp >(op,true); 
+     RCP<const Epetra_CrsMatrix> eCrsOp = rcp_dynamic_cast<const Epetra_CrsMatrix>(eOp->epetra_op(),true);
 
-      *out << "Teko: getInvDiagonalOp requires an Epetra_CrsMatrix or a Tpetra::CrsMatrix\n";
-      *out << "    Could not extract an Epetra_Operator or a Tpetra_Operator from a \"" << op->description() << std::endl;
-      *out << "           OR\n";
-      *out << "    Could not cast an Epetra_Operator to a Epetra_CrsMatrix or a Tpetra_Operator to a Tpetra::CrsMatrix\n";
-      *out << std::endl;
-      *out << "*** THROWN EXCEPTION ***\n";
-      *out << e.what() << std::endl;
-      *out << "************************\n";
-      
-      throw e;
-   }
-
-   if(!isTpetra){
      // extract diagonal
      const RCP<Epetra_Vector> diag = rcp(new Epetra_Vector(eCrsOp->RowMap()));
      TEUCHOS_TEST_FOR_EXCEPT(eCrsOp->ExtractDiagonalCopy(*diag));
@@ -1029,16 +998,6 @@ const ModifiableLinearOp getInvDiagonalOp(const LinearOp & op)
      // build Thyra diagonal operator
      return Teko::Epetra::thyraDiagOp(diag,eCrsOp->RowMap(),"inv(diag( " + op->getObjectLabel() + " ))");
    }
-   else {
-     // extract diagonal
-     const RCP<Tpetra::Vector<ST,LO,GO,NT> > diag = Tpetra::createVector<ST,LO,GO,NT>(tCrsOp->getRowMap());
-     tCrsOp->getLocalDiagCopy(*diag);
-     diag->reciprocal(*diag);
-
-     // build Thyra diagonal operator
-     return Teko::TpetraHelpers::thyraDiagOp(diag,*tCrsOp->getRowMap(),"inv(diag( " + op->getObjectLabel() + " ))");
-
-   }
 }
 
 /** \brief Multiply three linear operators. 
@@ -1488,7 +1447,7 @@ const LinearOp explicitAdd(const LinearOp & opl,const LinearOp & opr)
 {
    bool isTpetral = Teko::TpetraHelpers::isTpetraLinearOp(opl);
    bool isTpetrar = Teko::TpetraHelpers::isTpetraLinearOp(opr);
- 
+
    if(isTpetral && isTpetrar){ // Both operators are Tpetra matrices so use the explicit Tpetra matrix-matrix add
 
       // Get left and right Tpetra crs operators
@@ -1619,24 +1578,50 @@ const ModifiableLinearOp explicitSum(const LinearOp & op,
 
 const LinearOp explicitTranspose(const LinearOp & op)
 {
-   RCP<const Epetra_Operator> eOp = Thyra::get_Epetra_Operator(*op);
-   TEUCHOS_TEST_FOR_EXCEPTION(eOp==Teuchos::null,std::logic_error,
-                             "Teko::explicitTranspose Not an Epetra_Operator");
-   RCP<const Epetra_RowMatrix> eRowMatrixOp 
-         = Teuchos::rcp_dynamic_cast<const Epetra_RowMatrix>(eOp);
-   TEUCHOS_TEST_FOR_EXCEPTION(eRowMatrixOp==Teuchos::null,std::logic_error,
-                             "Teko::explicitTranspose Not an Epetra_RowMatrix");
+   if(Teko::TpetraHelpers::isTpetraLinearOp(op)){
 
-   // we now have a delete transpose operator
-   EpetraExt::RowMatrix_Transpose tranposeOp;
-   Epetra_RowMatrix & eMat = tranposeOp(const_cast<Epetra_RowMatrix &>(*eRowMatrixOp));
+     RCP<const Thyra::TpetraLinearOp<ST,LO,GO,NT> > tOp = rcp_dynamic_cast<const Thyra::TpetraLinearOp<ST,LO,GO,NT> >(op,true);
+     RCP<const Tpetra::CrsMatrix<ST,LO,GO,NT> > tCrsOp = rcp_dynamic_cast<const Tpetra::CrsMatrix<ST,LO,GO,NT> >(tOp->getConstTpetraOperator(),true);
 
-   // this copy is because of a poor implementation of the EpetraExt::Transform 
-   // implementation
-   Teuchos::RCP<Epetra_CrsMatrix> crsMat 
-         = Teuchos::rcp(new Epetra_CrsMatrix(dynamic_cast<Epetra_CrsMatrix &>(eMat)));
+     Tpetra::RowMatrixTransposer<ST,LO,GO,NT> transposer(tCrsOp);
+     RCP<const Tpetra::CrsMatrix<ST,LO,GO,NT> > transOp = transposer.createTranspose();
 
-   return Thyra::epetraLinearOp(crsMat);
+     return Thyra::constTpetraLinearOp<ST,LO,GO,NT>(Thyra::tpetraVectorSpace<ST,LO,GO,NT>(transOp->getRangeMap()),Thyra::tpetraVectorSpace<ST,LO,GO,NT>(transOp->getDomainMap()),transOp);
+
+   } else {
+
+     RCP<const Epetra_Operator> eOp = Thyra::get_Epetra_Operator(*op);
+     TEUCHOS_TEST_FOR_EXCEPTION(eOp==Teuchos::null,std::logic_error,
+                               "Teko::explicitTranspose Not an Epetra_Operator");
+     RCP<const Epetra_RowMatrix> eRowMatrixOp 
+           = Teuchos::rcp_dynamic_cast<const Epetra_RowMatrix>(eOp);
+     TEUCHOS_TEST_FOR_EXCEPTION(eRowMatrixOp==Teuchos::null,std::logic_error,
+                               "Teko::explicitTranspose Not an Epetra_RowMatrix");
+
+     // we now have a delete transpose operator
+     EpetraExt::RowMatrix_Transpose tranposeOp;
+     Epetra_RowMatrix & eMat = tranposeOp(const_cast<Epetra_RowMatrix &>(*eRowMatrixOp));
+
+     // this copy is because of a poor implementation of the EpetraExt::Transform 
+     // implementation
+     Teuchos::RCP<Epetra_CrsMatrix> crsMat 
+           = Teuchos::rcp(new Epetra_CrsMatrix(dynamic_cast<Epetra_CrsMatrix &>(eMat)));
+
+     return Thyra::epetraLinearOp(crsMat);
+   }
+}
+
+const double frobeniusNorm(const LinearOp & op)
+{
+  if(Teko::TpetraHelpers::isTpetraLinearOp(op)){
+    const RCP<const Thyra::TpetraLinearOp<ST,LO,GO,NT> > tOp = rcp_dynamic_cast<const Thyra::TpetraLinearOp<ST,LO,GO,NT> >(op);
+    const RCP<const Tpetra::CrsMatrix<ST,LO,GO,NT> > crsOp = rcp_dynamic_cast<const Tpetra::CrsMatrix<ST,LO,GO,NT> >(tOp->getConstTpetraOperator(),true);
+    return crsOp->getFrobeniusNorm();
+  } else {
+    const RCP<const Epetra_Operator> epOp = Thyra::get_Epetra_Operator(*op);
+    const RCP<const Epetra_CrsMatrix> crsOp = rcp_dynamic_cast<const Epetra_CrsMatrix>(epOp,true);
+    return crsOp->NormFrobenius();
+  }
 }
 
 const LinearOp buildDiagonal(const MultiVector & src,const std::string & lbl)
