@@ -425,96 +425,6 @@ void reduceAll(
   const Ordinal count, const Packet sendBuffer[], Packet globalReducts[]
   );
 
-/// \brief Wrapper for MPI_Reduce_scatter; equivalent to MPI_Reduce
-///   followed by MPI_Scatterv.
-/// \relates Comm
-/// \warning This function is DEPRECATED, because its name incorrectly
-///   suggests that it does an all-reduce instead of a reduce.
-///
-/// This version of the function is for Packet types that have "value
-/// semantics."  It allows a user-defined reduction operator.
-template<typename Ordinal, typename Packet>
-TEUCHOS_DEPRECATED void
-reduceAllAndScatter (
-  const Comm<Ordinal>& comm, const ValueTypeReductionOp<Ordinal,Packet> &reductOp,
-  const Ordinal sendCount, const Packet sendBuffer[] ,
-  const Ordinal recvCounts[], Packet myGlobalReducts[]
-  );
-
-/// \brief Wrapper for MPI_Reduce_scatter; equivalent to MPI_Reduce
-///   followed by MPI_Scatterv.
-/// \relates Comm
-/// \warning This function is DEPRECATED, because its name incorrectly
-///   suggests that it does an all-reduce instead of a reduce.
-///
-/// This version of the function is for Packet types that have "value
-/// semantics."  Callers specify one of Teuchos' predefined reduction
-/// operators using an enum.
-template<typename Ordinal, typename Packet>
-TEUCHOS_DEPRECATED void
-reduceAllAndScatter(
-  const Comm<Ordinal>& comm, const EReductionType reductType,
-  const Ordinal sendCount, const Packet sendBuffer[] ,
-  const Ordinal recvCounts[], Packet myGlobalReducts[]
-  );
-
-
-/// \brief Wrapper for MPI_Reduce_scatter; equivalent to MPI_Reduce
-///   followed by MPI_Scatterv.
-/// \relates Comm
-/// \warning This function is DEPRECATED, because its name incorrectly
-///   suggests that it does an all-reduce instead of a reduce.
-///
-/// This version of the function is for Packet types that have
-/// "reference semantics."  It allows a user-defined reduction
-/// operator.
-template<typename Ordinal, typename Packet>
-TEUCHOS_DEPRECATED void
-reduceAllAndScatter(
-  const Comm<Ordinal>& comm, const Serializer<Ordinal,Packet> &serializer,
-  const ReferenceTypeReductionOp<Ordinal,Packet> &reductOp,
-  const Ordinal sendCount, const Packet*const sendBuffer[] ,
-  const Ordinal recvCounts[], Packet*const myGlobalReducts[]
-  );
-
-/// \brief Wrapper for MPI_Reduce_scatter; equivalent to MPI_Reduce
-///   followed by MPI_Scatterv.
-/// \relates Comm
-/// \warning This function is DEPRECATED, because its name incorrectly
-///   suggests that it does an all-reduce instead of a reduce.
-///
-/// This version of the function is for Packet types that have "value
-/// semantics," but require a custom serializer.  It allows a
-/// user-defined reduction operator.
-template<typename Ordinal, typename Packet, typename Serializer>
-TEUCHOS_DEPRECATED void
-reduceAllAndScatter(
-  const Comm<Ordinal>& comm,
-  const Serializer& serializer,
-  const ValueTypeReductionOp<Ordinal,Packet> &reductOp,
-  const Ordinal sendCount, const Packet sendBuffer[],
-  const Ordinal recvCounts[], Packet myGlobalReducts[]
-  );
-
-/// \brief Wrapper for MPI_Reduce_scatter; equivalent to MPI_Reduce
-///   followed by MPI_Scatterv.
-/// \relates Comm
-/// \warning This function is DEPRECATED, because its name incorrectly
-///   suggests that it does an all-reduce instead of a reduce.
-///
-/// This version of the function is for Packet types that have "value
-/// semantics," but require a custom serializer.  Callers specify one
-/// of Teuchos' predefined reduction operators using an enum.
-template<typename Ordinal, typename Packet, typename Serializer>
-TEUCHOS_DEPRECATED void
-reduceAllAndScatter(
-  const Comm<Ordinal>& comm,
-  const Serializer& serializer,
-  const EReductionType reductType,
-  const Ordinal sendCount, const Packet sendBuffer[] ,
-  const Ordinal recvCounts[], Packet myGlobalReducts[]
-  );
-
 /** \brief Scan/Reduce array of objects that use value semantics using a
  * user-defined reduction operator.
  *
@@ -2020,17 +1930,6 @@ reduceAll<int, int> (const Comm<int>& comm,
                      const int sendBuffer[],
                      int globalReducts[]);
 
-// mfh 09 Apr 2013: We provide a full specialization of
-// reduceAllAndScatter, because it is an important part of
-// Tpetra::Distributor initialization.
-template<>
-TEUCHOSCOMM_LIB_DLL_EXPORT TEUCHOS_DEPRECATED void
-reduceAllAndScatter<int, int> (const Comm<int>& comm,
-                               const EReductionType reductType,
-                               const int sendCount,
-                               const int sendBuffer[],
-                               const int recvCounts[],
-                               int myGlobalReducts[]);
 template<>
 TEUCHOSCOMM_LIB_DLL_EXPORT RCP<CommRequest<int> >
 ireceive<int, int> (const Comm<int>& comm,
@@ -2284,174 +2183,6 @@ void Teuchos::reduceAll(
 }
 
 
-template<typename Ordinal, typename Packet>
-TEUCHOS_DEPRECATED void
-Teuchos::reduceAllAndScatter(
-  const Comm<Ordinal>& comm, const ValueTypeReductionOp<Ordinal,Packet> &reductOp,
-  const Ordinal sendCount, const Packet sendBuffer[] ,
-  const Ordinal recvCounts[], Packet myGlobalReducts[]
-  )
-{
-
-  TEUCHOS_COMM_TIME_MONITOR(
-    "Teuchos::CommHelpers: reduceAllAndScatter<"
-    <<OrdinalTraits<Ordinal>::name()<<","<<TypeNameTraits<Packet>::name()
-    <<">( value type, user-defined op )"
-    );
-
-  const Ordinal size = Teuchos::size(comm);
-  const Ordinal rank = Teuchos::rank(comm);
-
-#ifdef TEUCHOS_DEBUG
-  Ordinal sumRecvCounts = 0;
-  for( Ordinal i = 0; i < size; ++i )
-    sumRecvCounts += recvCounts[static_cast<ptrdiff_t>(i)];
-  TEUCHOS_TEST_FOR_EXCEPT(!(sumRecvCounts==sendCount));
-#endif
-
-  ConstValueTypeSerializationBuffer<Ordinal,Packet>
-    charSendBuffer(sendCount,sendBuffer);
-  ValueTypeSerializationBuffer<Ordinal,Packet>
-    charMyGlobalReducts(recvCounts[static_cast<ptrdiff_t>(rank)], myGlobalReducts);
-  CharToValueTypeReductionOp<Ordinal,Packet>
-    charReductOp(rcp(&reductOp,false));
-
-  const Ordinal
-    packetSize = charSendBuffer.getBytes()/sendCount;
-
-  WorkspaceStore* wss = get_default_workspace_store().get();
-  Workspace<Ordinal> charRecvCounts(wss, size);
-  for (Ordinal k = 0; k < size; ++k) {
-    charRecvCounts[k] = as<Ordinal>(recvCounts[static_cast<ptrdiff_t>(k)] * packetSize);
-  }
-
-  comm.reduceAllAndScatter(
-    charReductOp, charSendBuffer.getBytes(), charSendBuffer.getCharBuffer(),
-    &charRecvCounts[0], charMyGlobalReducts.getCharBuffer()
-    );
-
-}
-
-
-template<typename Ordinal, typename Packet>
-TEUCHOS_DEPRECATED void
-Teuchos::reduceAllAndScatter(
-  const Comm<Ordinal>& comm, const EReductionType reductType,
-  const Ordinal sendCount, const Packet sendBuffer[] ,
-  const Ordinal recvCounts[], Packet myGlobalReducts[]
-  )
-{
-  TEUCHOS_COMM_TIME_MONITOR(
-    "Teuchos::CommHelpers: reduceAllAndScatter<"
-    <<OrdinalTraits<Ordinal>::name()<<","<<TypeNameTraits<Packet>::name()
-    <<">( value type, "<<toString(reductType)<<" )"
-    );
-  ValueTypeReductionOp<Ordinal,Packet>* reductOp =
-    createOp<Ordinal, Packet> (reductType);
-  try {
-    reduceAllAndScatter (comm, *reductOp, sendCount, sendBuffer,
-                         recvCounts, myGlobalReducts);
-  }
-  catch (std::exception& e) {
-    delete reductOp;
-    throw e;
-  }
-  delete reductOp;
-}
-
-
-template<typename Ordinal, typename Packet>
-TEUCHOS_DEPRECATED void
-Teuchos::reduceAllAndScatter(
-  const Comm<Ordinal>& comm, const Serializer<Ordinal,Packet> &serializer,
-  const ReferenceTypeReductionOp<Ordinal,Packet> &reductOp,
-  const Ordinal sendCount, const Packet*const sendBuffer[],
-  const Ordinal recvCounts[], Packet*const myGlobalReducts[]
-  )
-{
-  TEUCHOS_TEST_FOR_EXCEPT(true); // ToDo: Implement and test when needed!
-}
-
-template<typename Ordinal, typename Packet, typename Serializer>
-TEUCHOS_DEPRECATED void
-Teuchos::reduceAllAndScatter(
-  const Comm<Ordinal>& comm,
-  const Serializer& serializer,
-  const ValueTypeReductionOp<Ordinal,Packet> &reductOp,
-  const Ordinal sendCount, const Packet sendBuffer[] ,
-  const Ordinal recvCounts[], Packet myGlobalReducts[]
-  )
-{
-
-  TEUCHOS_COMM_TIME_MONITOR(
-    "Teuchos::CommHelpers: reduceAllAndScatter<"
-    <<OrdinalTraits<Ordinal>::name()<<","<<TypeNameTraits<Packet>::name()
-    <<">( value type, user-defined op )"
-    );
-
-  const Ordinal size = Teuchos::size(comm);
-  const Ordinal rank = Teuchos::rank(comm);
-
-#ifdef TEUCHOS_DEBUG
-  Ordinal sumRecvCounts = 0;
-  for( Ordinal i = 0; i < size; ++i )
-    sumRecvCounts += recvCounts[static_cast<ptrdiff_t>(i)];
-  TEUCHOS_TEST_FOR_EXCEPT(!(sumRecvCounts==sendCount));
-#endif
-
-  ConstValueTypeSerializationBuffer<Ordinal,Packet,Serializer>
-    charSendBuffer(sendCount,sendBuffer,rcp(&serializer,false));
-  ValueTypeSerializationBuffer<Ordinal,Packet,Serializer>
-    charMyGlobalReducts(recvCounts[static_cast<ptrdiff_t>(rank)], myGlobalReducts,rcp(&serializer,false));
-  CharToValueTypeReductionOp<Ordinal,Packet,Serializer>
-    charReductOp(rcp(&reductOp,false),rcp(&serializer,false));
-
-  const Ordinal
-    packetSize = charSendBuffer.getBytes()/sendCount;
-
-  WorkspaceStore* wss = get_default_workspace_store().get();
-  Workspace<Ordinal> charRecvCounts(wss, size);
-  for (Ordinal k = 0; k < size; ++k) {
-    charRecvCounts[k] = as<Ordinal>(recvCounts[static_cast<ptrdiff_t>(k)] * packetSize);
-  }
-
-  comm.reduceAllAndScatter(
-    charReductOp, charSendBuffer.getBytes(), charSendBuffer.getCharBuffer(),
-    &charRecvCounts[0], charMyGlobalReducts.getCharBuffer()
-    );
-
-}
-
-template<typename Ordinal, typename Packet, typename Serializer>
-TEUCHOS_DEPRECATED void
-Teuchos::reduceAllAndScatter(
-  const Comm<Ordinal>& comm,
-  const Serializer& serializer,
-  const EReductionType reductType,
-  const Ordinal sendCount, const Packet sendBuffer[] ,
-  const Ordinal recvCounts[], Packet myGlobalReducts[]
-  )
-{
-  TEUCHOS_COMM_TIME_MONITOR(
-    "Teuchos::CommHelpers: reduceAllAndScatter<"
-    <<OrdinalTraits<Ordinal>::name()<<","<<TypeNameTraits<Packet>::name()
-    <<">( value type, "<<toString(reductType)<<" )"
-    );
-
-  ValueTypeReductionOp<Ordinal,Packet>* reductOp =
-    createOp<Ordinal, Packet> (reductType);
-  try {
-    reduceAllAndScatter (comm, serializer, *reductOp, sendCount,
-                         sendBuffer, recvCounts, myGlobalReducts);
-  }
-  catch (std::exception& e) {
-    delete reductOp;
-    throw e;
-  }
-  delete reductOp;
-}
-
-
 template<typename Ordinal, typename Packet>
 void Teuchos::scan(
   const Comm<Ordinal>& comm, const ValueTypeReductionOp<Ordinal,Packet> &reductOp,
