@@ -367,11 +367,21 @@ public:
   static void setPrintRCPNodeStatisticsOnExit(
     bool printRCPNodeStatisticsOnExit);
 
-  /** \brief Reteurn if RCPNode usage statistics will be printed when the
+  /** \brief Return if RCPNode usage statistics will be printed when the
    * program ends or not.
    */
   static bool getPrintRCPNodeStatisticsOnExit();
 
+  /** \brief Set if printActiveRCPNodes() is called on exit from the
+   * program.
+   */
+  static void setPrintActiveRcpNodesOnExit(bool printActiveRcpNodesOnExit);
+
+  /** \brief Return if printActiveRCPNodes() is called on exit from the
+   * program.
+   */
+  static bool getPrintActiveRcpNodesOnExit();
+
   /** \brief Print the list of currently active RCP nodes.
    *
    * When the macro <tt>TEUCHOS_SHOW_ACTIVE_REFCOUNTPTR_NODE_TRACE</tt> is
@@ -380,11 +390,12 @@ public:
    * program.
    *
    * When the macro <tt>TEUCHOS_SHOW_ACTIVE_REFCOUNTPTR_NODE_TRACE</tt> is
-   * defined this function will get called automatically after the program ends
-   * and all of the local and global RCP objects have been destroyed.  If any
-   * RCP nodes are printed at that time, then this is an indication that there
-   * may be some circular references that will caused memory leaks.  You memory
-   * checking tool such as valgrind or purify should complain about this!
+   * defined this function will get called automatically after the program
+   * ends by default and all of the local and global RCP objects have been
+   * destroyed.  If any RCP nodes are printed at that time, then this is an
+   * indication that there may be some circular references that will caused
+   * memory leaks.  You memory checking tool such as valgrind or purify should
+   * complain about this!
    */
   static void printActiveRCPNodes(std::ostream &out);
 
