@@ -86,10 +86,10 @@ namespace Tpetra {
     }
   } // namespace Details
 
-  Array<std::string>
+  Teuchos::Array<std::string>
   distributorSendTypes ()
   {
-    Array<std::string> sendTypes;
+    Teuchos::Array<std::string> sendTypes;
     sendTypes.push_back ("Isend");
     sendTypes.push_back ("Rsend");
     sendTypes.push_back ("Send");
@@ -818,6 +818,7 @@ namespace Tpetra {
   Distributor::computeReceives ()
   {
     using Teuchos::Array;
+    using Teuchos::ArrayRCP;
     using Teuchos::as;
     using Teuchos::CommStatus;
     using Teuchos::CommRequest;
@@ -1426,21 +1427,21 @@ namespace Tpetra {
 
     return totalReceiveLength_;
   }
-  //  template <class Ordinal>
-  void 
-  Distributor::createFromSendsAndRecvs(
-				       const ArrayView<const int> &exportProcIDs,  
-				       const ArrayView<const int> &remoteProcIDs )
+
+  void
+  Distributor::
+  createFromSendsAndRecvs (const Teuchos::ArrayView<const int>& exportProcIDs,
+                           const Teuchos::ArrayView<const int>& remoteProcIDs)
   {
-    // note the exportProcIDs and remoteProcIDs _must_ be a list that has 
-    // an entry for each GID. If the export/remoteProcIDs is taken from 
+    // note the exportProcIDs and remoteProcIDs _must_ be a list that has
+    // an entry for each GID. If the export/remoteProcIDs is taken from
     // the getProcs{From|To} lists that are extracted from a previous distributor,
     // it will generate a wrong answer, because those lists have a unique entry
     // for each processor id. A version of this with lengthsTo and lengthsFrom
-    // should be made.  
+    // should be made.
 
     howInitialized_ = Tpetra::Details::DISTRIBUTOR_INITIALIZED_BY_CREATE_FROM_SENDS_N_RECVS;
-    
+
 
     int myProcID = comm_->getRank ();
     int numProcs = comm_->getSize();
@@ -1453,13 +1454,13 @@ namespace Tpetra {
 
     for(size_t i = 0; i < numExportIDs; i++ )
       {
-	if( needSendBuff==0 && i && (exportProcIDs[i] < exportProcIDs[i-1]) )
-	  needSendBuff = 1;
-	if( exportProcIDs[i] >= 0 )
-	  {
-	    ++starts[ exportProcIDs[i] ];
-	    ++numActive;
-	  }
+        if( needSendBuff==0 && i && (exportProcIDs[i] < exportProcIDs[i-1]) )
+          needSendBuff = 1;
+        if( exportProcIDs[i] >= 0 )
+          {
+            ++starts[ exportProcIDs[i] ];
+            ++numActive;
+          }
       }
 
     selfMessage_ = ( starts[myProcID] != 0 ) ? 1 : 0;
@@ -1468,65 +1469,65 @@ namespace Tpetra {
 
     if( needSendBuff ) //grouped by processor, no send buffer or indicesTo_ needed
       {
-	if (starts[0] == 0 ) {
-	  numSends_ = 0;
-	}
-	else {
-	  numSends_ = 1;
-	}
-	for (Teuchos::Array<size_t>::iterator i=starts.begin()+1,
-	       im1=starts.begin();
-	     i != starts.end(); ++i)
-	  {
-	    if (*i != 0) ++numSends_;
-	    *i += *im1;
-	    im1 = i;
-	  }
-	// starts[i] now contains the number of exports to procs 0 through i
+        if (starts[0] == 0 ) {
+          numSends_ = 0;
+        }
+        else {
+          numSends_ = 1;
+        }
+        for (Teuchos::Array<size_t>::iterator i=starts.begin()+1,
+               im1=starts.begin();
+             i != starts.end(); ++i)
+          {
+            if (*i != 0) ++numSends_;
+            *i += *im1;
+            im1 = i;
+          }
+        // starts[i] now contains the number of exports to procs 0 through i
 
-	for (Teuchos::Array<size_t>::reverse_iterator ip1=starts.rbegin(),
-	       i=starts.rbegin()+1;
-	     i != starts.rend(); ++i)
-	  {
-	    *ip1 = *i;
-	    ip1 = i;
-	  }
-	starts[0] = 0;
-	// starts[i] now contains the number of exports to procs 0 through
-	// i-1, i.e., all procs before proc i
+        for (Teuchos::Array<size_t>::reverse_iterator ip1=starts.rbegin(),
+               i=starts.rbegin()+1;
+             i != starts.rend(); ++i)
+          {
+            *ip1 = *i;
+            ip1 = i;
+          }
+        starts[0] = 0;
+        // starts[i] now contains the number of exports to procs 0 through
+        // i-1, i.e., all procs before proc i
 
-	indicesTo_.resize(numActive);
+        indicesTo_.resize(numActive);
 
-	for (size_t i = 0; i < numExportIDs; ++i) {
-	  if (exportProcIDs[i] >= 0) {
-	    // record the offset to the sendBuffer for this export
-	    indicesTo_[starts[exportProcIDs[i]]] = i;
-	    // now increment the offset for this proc
-	    ++starts[exportProcIDs[i]];
-	  }
-	}
-	for (int proc = numProcs-1; proc != 0; --proc) {
-	  starts[proc] = starts[proc-1];
-	}
-	starts.front() = 0;
-	starts[numProcs] = numActive;
-	procsTo_.resize(numSends_);
-	startsTo_.resize(numSends_);
-	lengthsTo_.resize(numSends_);
-	maxSendLength_ = 0;
-	size_t snd = 0;
-	for (int proc = 0; proc < numProcs; ++proc ) {
-	  if (starts[proc+1] != starts[proc]) {
-	    lengthsTo_[snd] = starts[proc+1] - starts[proc];
-	    startsTo_[snd] = starts[proc];
-	    // record max length for all off-proc sends
-	    if ((proc != myProcID) && (lengthsTo_[snd] > maxSendLength_)) {
-	      maxSendLength_ = lengthsTo_[snd];
-	    }
-	    procsTo_[snd] = proc;
-	    ++snd;
-	  }
-	}
+        for (size_t i = 0; i < numExportIDs; ++i) {
+          if (exportProcIDs[i] >= 0) {
+            // record the offset to the sendBuffer for this export
+            indicesTo_[starts[exportProcIDs[i]]] = i;
+            // now increment the offset for this proc
+            ++starts[exportProcIDs[i]];
+          }
+        }
+        for (int proc = numProcs-1; proc != 0; --proc) {
+          starts[proc] = starts[proc-1];
+        }
+        starts.front() = 0;
+        starts[numProcs] = numActive;
+        procsTo_.resize(numSends_);
+        startsTo_.resize(numSends_);
+        lengthsTo_.resize(numSends_);
+        maxSendLength_ = 0;
+        size_t snd = 0;
+        for (int proc = 0; proc < numProcs; ++proc ) {
+          if (starts[proc+1] != starts[proc]) {
+            lengthsTo_[snd] = starts[proc+1] - starts[proc];
+            startsTo_[snd] = starts[proc];
+            // record max length for all off-proc sends
+            if ((proc != myProcID) && (lengthsTo_[snd] > maxSendLength_)) {
+              maxSendLength_ = lengthsTo_[snd];
+            }
+            procsTo_[snd] = proc;
+            ++snd;
+          }
+        }
       }
     else {
       // grouped by proc, no send buffer or indicesTo_ needed
@@ -1606,8 +1607,8 @@ namespace Tpetra {
       int jlast=j;
       procsFrom_[i]  = recv_list[i];
       startsFrom_[i] = j;
-      for( ; j<(size_t)remoteProcIDs.size() && 
-	     remoteProcIDs[jlast]==remoteProcIDs[j]  ; j++){;}
+      for( ; j<(size_t)remoteProcIDs.size() &&
+             remoteProcIDs[jlast]==remoteProcIDs[j]  ; j++){;}
       lengthsFrom_[i] = j-jlast;
     }
     totalReceiveLength_ = remoteProcIDs.size();
@@ -1619,5 +1620,5 @@ namespace Tpetra {
 
     numReceives_-=selfMessage_;
   }
-  
+
 } // namespace Tpetra
