@@ -51,8 +51,6 @@
 #include <Tpetra_DefaultPlatform.hpp>
 #include <MatrixMarket_Tpetra.hpp>
 
-#include <Kokkos_DefaultNode.hpp>
-
 #include <Teuchos_Array.hpp>
 #include <Teuchos_as.hpp>
 #include <Teuchos_CommHelpers.hpp>
@@ -187,19 +185,6 @@ TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL( CrsMatrix, gaussSeidelSerial, LocalOrdinalTyp
   }
 #endif // 0
 
-  // Get a Kokkos Node instance.  It would be nice if we could pass in
-  // parameters here, but threads don't matter for this test, since
-  // the current Gauss-Seidel implementation doesn't use them.
-
-  if (myRank == 0) {
-    out << "Creating Kokkos Node of type " << TypeNameTraits<node_type>::name () << endl;
-  }
-  RCP<node_type> node;
-  {
-    ParameterList pl; // Kokkos Node types require a PL inout.
-    node = rcp (new node_type (pl));
-  }
-
   // Let's build ourselves the graph of a 2-D Laplacian (Dirichlet
   // boundary conditions) on a square numGlobalPoints x
   // numGlobalPoints mesh.  Each process gets a numLocalPoints x
@@ -224,7 +209,7 @@ TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL( CrsMatrix, gaussSeidelSerial, LocalOrdinalTyp
   }
 
   // Create a contiguous row Map, with numLocalRows rows per process.
-  RCP<const map_type> rowMap = createContigMapWithNode<LO, GO, NT> (INVALID, numLocalRows, comm, node);
+  RCP<const map_type> rowMap = createContigMapWithNode<LO, GO, NT> (INVALID, numLocalRows, comm);
 
   // The Gauss-Seidel kernel requires that the row, domain, and range
   // Maps all be the same.
@@ -577,19 +562,6 @@ TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL( CrsMatrix, reorderedGaussSeidelSerial, LocalO
   }
 #endif // 0
 
-  // Get a Kokkos Node instance.  It would be nice if we could pass in
-  // parameters here, but threads don't matter for this test, since
-  // the current Gauss-Seidel implementation doesn't use them.
-
-  if (myRank == 0) {
-    out << "Creating Kokkos Node of type " << TypeNameTraits<node_type>::name () << endl;
-  }
-  RCP<node_type> node;
-  {
-    ParameterList pl; // Kokkos Node types require a PL inout.
-    node = rcp (new node_type (pl));
-  }
-
   // Let's build ourselves the graph of a 2-D Laplacian (Dirichlet
   // boundary conditions) on a square numGlobalPoints x
   // numGlobalPoints mesh.  Each process gets a numLocalPoints x
@@ -614,7 +586,7 @@ TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL( CrsMatrix, reorderedGaussSeidelSerial, LocalO
   }
 
   // Create a contiguous row Map, with numLocalRows rows per process.
-  RCP<const map_type> rowMap = createContigMapWithNode<LO, GO, NT> (INVALID, numLocalRows, comm, node);
+  RCP<const map_type> rowMap = createContigMapWithNode<LO, GO, NT> (INVALID, numLocalRows, comm);
 
   // The Gauss-Seidel kernel requires that the row, domain, and range
   // Maps all be the same.
