@@ -43,9 +43,10 @@
 #ifndef KOKKOS_BLAS1_MV_IMPL_UPDATE_HPP_
 #define KOKKOS_BLAS1_MV_IMPL_UPDATE_HPP_
 
-#include <TpetraKernels_config.h>
-#include <Kokkos_Core.hpp>
-#include <Kokkos_InnerProductSpaceTraits.hpp>
+#include "TpetraKernels_config.h"
+#include "Kokkos_Core.hpp"
+#include "Kokkos_InnerProductSpaceTraits.hpp"
+#include "TpetraKernels_ETIHelperMacros.h"
 
 #ifdef HAVE_TPETRAKERNELS_ETI_ONLY
 #define KOKKOSBLAS_ETI_ONLY
@@ -749,37 +750,43 @@ struct Update<Kokkos::View<const SCALAR**, \
 // Their definitions go in .cpp file(s) in this source directory.
 //
 
-#ifdef KOKKOS_HAVE_SERIAL
+TPETRAKERNELS_ETI_MANGLING_TYPEDEFS()
 
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( int, Kokkos::LayoutLeft, Kokkos::Serial, Kokkos::HostSpace )
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( long, Kokkos::LayoutLeft, Kokkos::Serial, Kokkos::HostSpace )
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( double, Kokkos::LayoutLeft, Kokkos::Serial, Kokkos::HostSpace )
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_SERIAL
+#define KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_SERIAL( SCALAR ) \
+  KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( SCALAR, Kokkos::LayoutLeft, Kokkos::Serial, Kokkos::HostSpace )
 
-#endif // KOKKOS_HAVE_SERIAL
+TPETRAKERNELS_INSTANTIATE_S( KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_SERIAL )
 
-#ifdef KOKKOS_HAVE_OPENMP
+#undef KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_SERIAL
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_SERIAL
 
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( int, Kokkos::LayoutLeft, Kokkos::OpenMP, Kokkos::HostSpace )
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( long, Kokkos::LayoutLeft, Kokkos::OpenMP, Kokkos::HostSpace )
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( double, Kokkos::LayoutLeft, Kokkos::OpenMP, Kokkos::HostSpace )
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_OPENMP
+#define KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_OPENMP( SCALAR ) \
+  KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( SCALAR, Kokkos::LayoutLeft, Kokkos::OpenMP, Kokkos::HostSpace )
 
-#endif // KOKKOS_HAVE_OPENMP
+TPETRAKERNELS_INSTANTIATE_S( KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_OPENMP )
 
-#ifdef KOKKOS_HAVE_PTHREAD
+#undef KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_OPENMP
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_OPENMP
 
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( int, Kokkos::LayoutLeft, Kokkos::Threads, Kokkos::HostSpace )
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( long, Kokkos::LayoutLeft, Kokkos::Threads, Kokkos::HostSpace )
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( double, Kokkos::LayoutLeft, Kokkos::Threads, Kokkos::HostSpace )
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_PTHREAD
+#define KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_PTHREAD( SCALAR ) \
+  KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( SCALAR, Kokkos::LayoutLeft, Kokkos::Threads, Kokkos::HostSpace )
 
-#endif // KOKKOS_HAVE_PTHREAD
+TPETRAKERNELS_INSTANTIATE_S( KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_PTHREAD )
 
-#ifdef KOKKOS_HAVE_CUDA
+#undef KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_PTHREAD
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_PTHREAD
 
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( int, Kokkos::LayoutLeft, Kokkos::Cuda, Kokkos::CudaUVMSpace )
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( long, Kokkos::LayoutLeft, Kokkos::Cuda, Kokkos::CudaUVMSpace )
-KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( double, Kokkos::LayoutLeft, Kokkos::Cuda, Kokkos::CudaUVMSpace )
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_CUDA
+#define KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_CUDA( SCALAR ) \
+  KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL( SCALAR, Kokkos::LayoutLeft, Kokkos::Cuda, Kokkos::CudaUVMSpace )
 
-#endif // KOKKOS_HAVE_CUDA
+TPETRAKERNELS_INSTANTIATE_S( KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_CUDA )
+
+#undef KOKKOSBLAS_IMPL_MV_UPDATE_RANK2_DECL_CUDA
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_CUDA
 
 //
 // Macro for definition of full specialization of
@@ -906,37 +913,37 @@ struct Update<Kokkos::View<const SCALAR*, \
 // Their definitions go in .cpp file(s) in this source directory.
 //
 
-#ifdef KOKKOS_HAVE_SERIAL
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_SERIAL
 
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( int, Kokkos::LayoutLeft, Kokkos::Serial, Kokkos::HostSpace )
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( long, Kokkos::LayoutLeft, Kokkos::Serial, Kokkos::HostSpace )
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( double, Kokkos::LayoutLeft, Kokkos::Serial, Kokkos::HostSpace )
 
-#endif // KOKKOS_HAVE_SERIAL
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_SERIAL
 
-#ifdef KOKKOS_HAVE_OPENMP
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_OPENMP
 
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( int, Kokkos::LayoutLeft, Kokkos::OpenMP, Kokkos::HostSpace )
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( long, Kokkos::LayoutLeft, Kokkos::OpenMP, Kokkos::HostSpace )
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( double, Kokkos::LayoutLeft, Kokkos::OpenMP, Kokkos::HostSpace )
 
-#endif // KOKKOS_HAVE_OPENMP
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_OPENMP
 
-#ifdef KOKKOS_HAVE_PTHREAD
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_PTHREAD
 
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( int, Kokkos::LayoutLeft, Kokkos::Threads, Kokkos::HostSpace )
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( long, Kokkos::LayoutLeft, Kokkos::Threads, Kokkos::HostSpace )
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( double, Kokkos::LayoutLeft, Kokkos::Threads, Kokkos::HostSpace )
 
-#endif // KOKKOS_HAVE_PTHREAD
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_PTHREAD
 
-#ifdef KOKKOS_HAVE_CUDA
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_CUDA
 
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( int, Kokkos::LayoutLeft, Kokkos::Cuda, Kokkos::CudaUVMSpace )
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( long, Kokkos::LayoutLeft, Kokkos::Cuda, Kokkos::CudaUVMSpace )
 KOKKOSBLAS_IMPL_MV_UPDATE_RANK1_DECL( double, Kokkos::LayoutLeft, Kokkos::Cuda, Kokkos::CudaUVMSpace )
 
-#endif // KOKKOS_HAVE_CUDA
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_CUDA
 
 //
 // Macro for definition of full specialization of
