@@ -47,32 +47,32 @@
 namespace KokkosBlas {
 namespace Impl {
 
-#ifdef KOKKOS_HAVE_SERIAL
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_SERIAL
 
   KOKKOSBLAS_IMPL_MV_SCAL_RANK1_SINGLECOEFF_DEF( double, Kokkos::LayoutLeft, Kokkos::Serial, Kokkos::HostSpace )
 
-#endif // KOKKOS_HAVE_SERIAL
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_SERIAL
 
 
-#ifdef KOKKOS_HAVE_OPENMP
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_OPENMP
 
   KOKKOSBLAS_IMPL_MV_SCAL_RANK1_SINGLECOEFF_DEF( double, Kokkos::LayoutLeft, Kokkos::OpenMP, Kokkos::HostSpace )
 
-#endif // KOKKOS_HAVE_OPENMP
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_OPENMP
 
 
-#ifdef KOKKOS_HAVE_PTHREAD
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_PTHREAD
 
   KOKKOSBLAS_IMPL_MV_SCAL_RANK1_SINGLECOEFF_DEF( double, Kokkos::LayoutLeft, Kokkos::Threads, Kokkos::HostSpace )
 
-#endif // KOKKOS_HAVE_PTHREAD
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_PTHREAD
 
 
-#ifdef KOKKOS_HAVE_CUDA
+#ifdef TPETRAKERNELS_BUILD_EXECUTION_SPACE_CUDA
 
   KOKKOSBLAS_IMPL_MV_SCAL_RANK1_SINGLECOEFF_DEF( double, Kokkos::LayoutLeft, Kokkos::Cuda, Kokkos::CudaUVMSpace )
 
-#endif // KOKKOS_HAVE_CUDA
+#endif // TPETRAKERNELS_BUILD_EXECUTION_SPACE_CUDA
 
 } // namespace Impl
 } // namespace KokkosBlas
