@@ -59,28 +59,16 @@ namespace FENL {
 
 // Specialization of ResponseComputation for PCE scalar types.  We currently
 // have to do this because parallel_reduce doesn't support non-POD types
-#if defined( KOKKOS_USING_EXPERIMENTAL_VIEW )
 template< typename FixtureType ,
           typename S , typename ... P >
 class ResponseComputation< FixtureType,
                            Kokkos::View<Sacado::UQ::PCE<S>*,P...>
                          >
-#else
-template< typename FixtureType ,
-          typename S , typename L , typename D , typename M >
-class ResponseComputation< FixtureType,
-                           Kokkos::View<S,L,D,M,Kokkos::Impl::ViewPCEContiguous>
-                         >
-#endif
 {
 public:
 
   typedef FixtureType fixture_type ;
-#if defined( KOKKOS_USING_EXPERIMENTAL_VIEW )
   typedef Kokkos::View<Sacado::UQ::PCE<S>*,P...> vector_type ;
-#else
-  typedef Kokkos::View<S,L,D,M,Kokkos::Impl::ViewPCEContiguous> vector_type ;
-#endif
   typedef typename vector_type::execution_space execution_space ;
   typedef typename vector_type::value_type scalar_type ;
 
@@ -753,7 +741,8 @@ public:
                                     arg_coeff_function.m_num_rv,
                                     arg_coeff_function.m_use_exp,
                                     arg_coeff_function.m_exp_shift,
-                                    arg_coeff_function.m_exp_scale)
+                                    arg_coeff_function.m_exp_scale,
+                                    arg_coeff_function.m_use_disc_exp_scale )
     , scalar_element_computation( arg_mesh,
                                   scalar_diffusion_coefficient,
                                   arg_isotropic,
