@@ -125,7 +125,7 @@ namespace Xpetra {
         // plausibility check
         for(size_t v = 0; v < maps.size(); ++v) {
           TEUCHOS_TEST_FOR_EXCEPTION(maps[v]->getMinAllGlobalIndex() != 0, std::logic_error,
-                                             "logic error. When using Thyra-style numbering all sub-block maps must start with zero as GID.");
+                                             "logic error. When using Thyra-style numbering all sub-block maps must start with zero as GID. Map block " << v << " starts with GID " << maps[v]->getMinAllGlobalIndex());
         }
 
         // store submaps in Thyra-style ordering
@@ -279,7 +279,7 @@ namespace Xpetra {
       XPETRA_TEST_FOR_EXCEPTION(maps_[block] == null, Xpetra::Exceptions::RuntimeError,
             "ExtractVector: maps_[" << block << "] is null");
       // first extract partial vector from full vector (using xpetra style GIDs)
-      const RCP<Vector> vv = VectorFactory::Build(getMap(block,false), true);
+      const RCP<Vector> vv = VectorFactory::Build(getMap(block,false), false);
       ExtractVector(*full, block, *vv);
       if(bThyraMode == false) return vv;
       TEUCHOS_TEST_FOR_EXCEPTION(bThyraMode_ == false && bThyraMode == true, Xpetra::Exceptions::RuntimeError,
@@ -304,7 +304,7 @@ namespace Xpetra {
       XPETRA_TEST_FOR_EXCEPTION(maps_[block] == null, Xpetra::Exceptions::RuntimeError,
             "ExtractVector: maps_[" << block << "] is null");
       // first extract partial vector from full vector (using xpetra style GIDs)
-      const RCP<Vector> vv = VectorFactory::Build(getMap(block,false), true);
+      const RCP<Vector> vv = VectorFactory::Build(getMap(block,false), false);
       ExtractVector(*full, block, *vv);
       if(bThyraMode == false) return vv;
       TEUCHOS_TEST_FOR_EXCEPTION(bThyraMode_ == false && bThyraMode == true, Xpetra::Exceptions::RuntimeError,
@@ -329,7 +329,7 @@ namespace Xpetra {
       XPETRA_TEST_FOR_EXCEPTION(maps_[block] == null, Xpetra::Exceptions::RuntimeError,
             "ExtractVector: maps_[" << block << "] is null");
       // first extract partial vector from full vector (using xpetra style GIDs)
-      const RCP<MultiVector> vv = MultiVectorFactory::Build(getMap(block,false), full->getNumVectors(), true);
+      const RCP<MultiVector> vv = MultiVectorFactory::Build(getMap(block,false), full->getNumVectors(), false);
       ExtractVector(*full, block, *vv);
       if(bThyraMode == false) return vv;
       TEUCHOS_TEST_FOR_EXCEPTION(bThyraMode_ == false && bThyraMode == true, Xpetra::Exceptions::RuntimeError,
@@ -355,7 +355,7 @@ namespace Xpetra {
       XPETRA_TEST_FOR_EXCEPTION(maps_[block] == null, Xpetra::Exceptions::RuntimeError,
             "ExtractVector: maps_[" << block << "] is null");
       // first extract partial vector from full vector (using xpetra style GIDs)
-      const RCP<MultiVector> vv = MultiVectorFactory::Build(getMap(block,false), full->getNumVectors(), true);
+      const RCP<MultiVector> vv = MultiVectorFactory::Build(getMap(block,false), full->getNumVectors(), false);
       ExtractVector(*full, block, *vv);
       if(bThyraMode == false) return vv;
       TEUCHOS_TEST_FOR_EXCEPTION(bThyraMode_ == false && bThyraMode == true, Xpetra::Exceptions::RuntimeError,
@@ -483,15 +483,15 @@ namespace Xpetra {
 
     //@}
 
-    RCP<     Vector> getVector(size_t i, bool bThyraMode = false) const {
+    RCP<     Vector> getVector(size_t i, bool bThyraMode = false, bool bZero = true) const {
       XPETRA_TEST_FOR_EXCEPTION(bThyraMode_ == false && bThyraMode == true, Xpetra::Exceptions::RuntimeError,
                  "MapExtractor::getVector: getVector in Thyra-style numbering only possible if MapExtractor has been created using Thyra-style numbered submaps.");
-      return VectorFactory::Build(getMap(i, bThyraMode), true);
+      return VectorFactory::Build(getMap(i, bThyraMode), bZero);
     }
-    RCP<MultiVector> getVector(size_t i, size_t numvec, bool bThyraMode = false) const {
+    RCP<MultiVector> getVector(size_t i, size_t numvec, bool bThyraMode = false, bool bZero = true) const {
       XPETRA_TEST_FOR_EXCEPTION(bThyraMode_ == false && bThyraMode == true, Xpetra::Exceptions::RuntimeError,
                  "MapExtractor::getVector: getVector in Thyra-style numbering only possible if MapExtractor has been created using Thyra-style numbered submaps.");
-      return MultiVectorFactory::Build(getMap(i, bThyraMode), numvec, true);
+      return MultiVectorFactory::Build(getMap(i, bThyraMode), numvec, bZero);
     }
 
     /// returns true, if sub maps are stored in Thyra-style numbering
