@@ -248,7 +248,7 @@ Teuchos::RCP<Xpetra::BlockedMultiVector<Scalar, LocalOrdinal, GlobalOrdinal, Nod
   // fill multivector data (first multivector contains the GID, the second the LID as scalar)
   Teuchos::ArrayRCP< Scalar > vv1 = vv->getDataNonConst(0);
   Teuchos::ArrayRCP< Scalar > vv2 = vv->getDataNonConst(1);
-  for(LocalOrdinal i = 0; i < vv->getLocalLength(); ++i) {
+  for(LocalOrdinal i = 0; i < Teuchos::as<LocalOrdinal>(vv->getLocalLength()); ++i) {
     vv1[i] = Teuchos::as<Scalar>(vv->getMap()->getGlobalElement(i));
     vv2[i] = Teuchos::as<Scalar>(i);
   }
@@ -608,11 +608,11 @@ TEUCHOS_UNIT_TEST_TEMPLATE_6_DECL( BlockedMultiVector, InsertVector, M, MA, Scal
 
 #ifdef HAVE_XPETRA_DEBUG
   // create malicious multivector
-  Teuchos::RCP<MultiVector> part1 = me->getVector(0,23,false);
+  Teuchos::RCP<MultiVector> part1 = me->getVector(0,23,false,true);
   TEST_THROW(me->InsertVector(part1,0,bvv),Xpetra::Exceptions::RuntimeError);
-  Teuchos::RCP<MultiVector> part2 = me->getVector(0,2,false);
+  Teuchos::RCP<MultiVector> part2 = me->getVector(0,2,false,true);
   TEST_THROW(me->InsertVector(part2,1,bvv),Xpetra::Exceptions::RuntimeError);
-  TEST_THROW(Teuchos::RCP<MultiVector> part3 = me->getVector(1,2,true),Xpetra::Exceptions::RuntimeError);
+  TEST_THROW(Teuchos::RCP<MultiVector> part3 = me->getVector(1,2,true,true),Xpetra::Exceptions::RuntimeError);
 #endif
 }
 
@@ -659,14 +659,14 @@ TEUCHOS_UNIT_TEST_TEMPLATE_6_DECL( BlockedMultiVector, InsertVectorThyra, M, MA,
 
 #ifdef HAVE_XPETRA_DEBUG
   // create malicious multivector
-  Teuchos::RCP<MultiVector> part1 = me->getVector(0,23,true);
+  Teuchos::RCP<MultiVector> part1 = me->getVector(0,23,true,true);
   TEST_THROW(me->InsertVector(part1,0,bvv),Xpetra::Exceptions::RuntimeError);
   // unfortunately, in Thyra mode there is no error thrown since the vectors in
   // block 0 and 1 have the same length (and the same GIDs)
-  Teuchos::RCP<MultiVector> part2 = me->getVector(0,2,true);
+  Teuchos::RCP<MultiVector> part2 = me->getVector(0,2,true,true);
   TEST_NOTHROW(me->InsertVector(part2,1,bvv,me->getThyraMode()));
   // This should throw, thought
-  Teuchos::RCP<MultiVector> part3 = me->getVector(0,2,true);
+  Teuchos::RCP<MultiVector> part3 = me->getVector(0,2,true,true);
   TEST_THROW(me->InsertVector(part2,2,bvv,me->getThyraMode()),Xpetra::Exceptions::RuntimeError);
 #endif
 }
@@ -700,8 +700,12 @@ TEUCHOS_UNIT_TEST_TEMPLATE_6_DECL( BlockedMultiVector, UpdateVector1, M, MA, Sca
   Teuchos::Array<Magnitude> bnorms1(bvv1->getNumVectors());
   Teuchos::Array<Magnitude> bnorms2(vv->getNumVectors());
   TEST_NOTHROW( bvv1->norm1(bnorms1) );
-  TEST_EQUALITY( bnorms1[0], Teuchos::ScalarTraits<Magnitude>::zero());
-  TEST_EQUALITY( bnorms1[1], Teuchos::ScalarTraits<Magnitude>::zero());
+  // TAW: CUDA produces a "dirty zero" (not exactly zero)
+  // this might be numerical effects caused by the ordering of calculations
+  TEST_COMPARE( bnorms1[0], < , 1e-12);
+  TEST_COMPARE( bnorms1[1], < , 1e-12);
+  //TEST_EQUALITY( bnorms1[0], Teuchos::ScalarTraits<Magnitude>::zero());
+  //TEST_EQUALITY( bnorms1[1], Teuchos::ScalarTraits<Magnitude>::zero());
   TEST_NOTHROW( vv->norm1(bnorms1) );
   TEST_NOTHROW( bvv2->norm1(bnorms2) );
   TEST_COMPARE_FLOATING_ARRAYS(bnorms1,bnorms2,Teuchos::ScalarTraits<Magnitude>::zero());
