@@ -43,14 +43,10 @@
 // ***********************************************************************
 //
 // @HEADER
-/*
- * MatrixMatrix_UnitTests.cpp
- *
- *  Created on: Oct 9, 2012
- *      Author: wiesner
- */
+
 
 #include <Teuchos_UnitTestHarness.hpp>
+#include <Xpetra_UnitTestHelpers.hpp>
 #include <Teuchos_Array.hpp>
 #include <Teuchos_Tuple.hpp>
 #include <Teuchos_CommHelpers.hpp>
@@ -153,10 +149,13 @@ namespace {
   //
 
   /// unit test for matrix-matrix multiplication (both for Epetra and Tpetra)
-  TEUCHOS_UNIT_TEST_TEMPLATE_4_DECL( MatrixMatrix, Multiply, Scalar, LO, GO, Node )
+  TEUCHOS_UNIT_TEST_TEMPLATE_6_DECL( MatrixMatrix, Multiply_Epetra, M, MA, Scalar, LO, GO, Node )
   {
     typedef Xpetra::Map<LO, GO, Node> MapClass;
     typedef Xpetra::MapFactory<LO, GO, Node> MapFactoryClass;
+    typedef Xpetra::CrsMatrix<Scalar,LO,GO,Node> CrsMatrixClass;
+    typedef Xpetra::Matrix<Scalar,LO,GO,Node> MatrixClass;
+    typedef Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node> CrsMatrixWrapClass;
 
     //Teuchos::RCP<Teuchos::FancyOStream> fos = Teuchos::getFancyOStream(Teuchos::rcpFromRef(std::cout));
     //yAB->describe(*fos, Teuchos::VERB_EXTREME);
@@ -192,9 +191,7 @@ namespace {
       EpetraExt::MatrixMarketFileToCrsMatrix("AtB.mat",*epMap,*epMap,*epMap,ptrAtB);
       EpetraExt::MatrixMarketFileToCrsMatrix("ABt.mat",*epMap,*epMap,*epMap,ptrABt);
       EpetraExt::MatrixMarketFileToCrsMatrix("AtBt.mat",*epMap,*epMap,*epMap,ptrAtBt);
-      if(ptrA == NULL || ptrB == NULL || ptrAB == NULL ||
-         ptrAtB == NULL || ptrABt == NULL || ptrAtBt == NULL)
-        std::cout << "matrix not found" << std::endl;
+      TEUCHOS_TEST_FOR_EXCEPTION(ptrA == NULL || ptrB == NULL || ptrAB == NULL || ptrAtB == NULL || ptrABt == NULL || ptrAtBt == NULL,std::logic_error,"Could not open one or more matrix files");
       RCP<Epetra_CrsMatrix> epA = Teuchos::rcp(ptrA);
       RCP<Epetra_CrsMatrix> epB = Teuchos::rcp(ptrB);
       RCP<Epetra_CrsMatrix> epAB = Teuchos::rcp(ptrAB);
@@ -205,21 +202,21 @@ namespace {
       /////////////////////////////////////// transform Epetra objects to Xpetra (needed for MueLu)
 
       // build Xpetra objects from Epetra_CrsMatrix objects
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xAmat = Teuchos::rcp(new Xpetra::EpetraCrsMatrixT<GO,Node>(epA));
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xBmat = Teuchos::rcp(new Xpetra::EpetraCrsMatrixT<GO,Node>(epB));
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xABmat = Teuchos::rcp(new Xpetra::EpetraCrsMatrixT<GO,Node>(epAB));
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xAtBmat = Teuchos::rcp(new Xpetra::EpetraCrsMatrixT<GO,Node>(epAtB));
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xABtmat = Teuchos::rcp(new Xpetra::EpetraCrsMatrixT<GO,Node>(epABt));
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xAtBtmat = Teuchos::rcp(new Xpetra::EpetraCrsMatrixT<GO,Node>(epAtBt));
+      Teuchos::RCP<CrsMatrixClass> xAmat = Teuchos::rcp(new MA(epA));
+      Teuchos::RCP<CrsMatrixClass> xBmat = Teuchos::rcp(new MA(epB));
+      Teuchos::RCP<CrsMatrixClass> xABmat = Teuchos::rcp(new MA(epAB));
+      Teuchos::RCP<CrsMatrixClass> xAtBmat = Teuchos::rcp(new MA(epAtB));
+      Teuchos::RCP<CrsMatrixClass> xABtmat = Teuchos::rcp(new MA(epABt));
+      Teuchos::RCP<CrsMatrixClass> xAtBtmat = Teuchos::rcp(new MA(epAtBt));
 
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xA = Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xAmat));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xB = Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xBmat));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xAB= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xABmat));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xAtB= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xAtBmat));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xABt= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xABtmat));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xAtBt= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xAtBtmat));
+      Teuchos::RCP<MatrixClass> xA = Teuchos::rcp(new CrsMatrixWrapClass(xAmat));
+      Teuchos::RCP<MatrixClass> xB = Teuchos::rcp(new CrsMatrixWrapClass(xBmat));
+      Teuchos::RCP<MatrixClass> xAB= Teuchos::rcp(new CrsMatrixWrapClass(xABmat));
+      Teuchos::RCP<MatrixClass> xAtB= Teuchos::rcp(new CrsMatrixWrapClass(xAtBmat));
+      Teuchos::RCP<MatrixClass> xABt= Teuchos::rcp(new CrsMatrixWrapClass(xABtmat));
+      Teuchos::RCP<MatrixClass> xAtBt= Teuchos::rcp(new CrsMatrixWrapClass(xAtBtmat));
 
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > yAB= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(map, 6));
+      Teuchos::RCP<MatrixClass> yAB= Teuchos::rcp(new CrsMatrixWrapClass(map, 6));
 
       //Teuchos::RCP<Teuchos::FancyOStream> fos = Teuchos::getFancyOStream(Teuchos::rcpFromRef(std::cout));
       //yAB->describe(*fos,Teuchos::VERB_EXTREME);
@@ -263,13 +260,135 @@ namespace {
       TEUCHOS_TEST_EQUALITY(yAB->getFrobeniusNorm(), xAtBt->getFrobeniusNorm(), out, success );
     } // end Epetra test
 #endif
+  }
+
+  TEUCHOS_UNIT_TEST_TEMPLATE_6_DECL( MatrixMatrix, Multiply_Epetra64, M, MA, Scalar, LO, GO, Node )
+  {
+    typedef Xpetra::Map<LO, GO, Node> MapClass;
+    typedef Xpetra::MapFactory<LO, GO, Node> MapFactoryClass;
+    typedef Xpetra::CrsMatrix<Scalar,LO,GO,Node> CrsMatrixClass;
+    typedef Xpetra::Matrix<Scalar,LO,GO,Node> MatrixClass;
+    typedef Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node> CrsMatrixWrapClass;
+
+    //Teuchos::RCP<Teuchos::FancyOStream> fos = Teuchos::getFancyOStream(Teuchos::rcpFromRef(std::cout));
+    //yAB->describe(*fos, Teuchos::VERB_EXTREME);
+
+#if defined(HAVE_XPETRA_EPETRA) && defined(HAVE_XPETRA_EPETRAEXT)
+    { // Epetra test
+      // get a comm and node
+      RCP<const Comm<int> > comm = getDefaultComm();
+
+      Xpetra::UnderlyingLib lib = Xpetra::UseEpetra;
+
+      // generate problem
+      LO nEle = 6;
+      const RCP<const MapClass> map = MapFactoryClass::Build(lib, nEle, 0, comm);
+      const RCP<const Xpetra::EpetraMapT<GO, Node> > XepMap = Teuchos::rcp_dynamic_cast<const Xpetra::EpetraMapT<GO, Node> >(map);
+      /////////////////////////////////////// transform Xpetra::Map objects to Epetra
+      // this is needed for AztecOO
+      const Teuchos::RCP<const Epetra_Map> epMap = Teuchos::rcpFromRef(XepMap->getEpetra_Map());
+      /////////////////////////////////////// import problem matrix and RHS from files (-> Epetra)
+
+      // read in problem
+      Epetra_CrsMatrix * ptrA  = 0;
+      Epetra_CrsMatrix * ptrB  = 0;
+      Epetra_CrsMatrix * ptrAB = 0;
+      Epetra_CrsMatrix * ptrAtB = 0;
+      Epetra_CrsMatrix * ptrABt = 0;
+      Epetra_CrsMatrix * ptrAtBt = 0;
+      EpetraExt::MatrixMarketFileToCrsMatrix64("A.mat",*epMap,*epMap,*epMap,ptrA);
+      EpetraExt::MatrixMarketFileToCrsMatrix64("B.mat",*epMap,*epMap,*epMap,ptrB);
+      EpetraExt::MatrixMarketFileToCrsMatrix64("AB.mat",*epMap,*epMap,*epMap,ptrAB);
+      EpetraExt::MatrixMarketFileToCrsMatrix64("AtB.mat",*epMap,*epMap,*epMap,ptrAtB);
+      EpetraExt::MatrixMarketFileToCrsMatrix64("ABt.mat",*epMap,*epMap,*epMap,ptrABt);
+      EpetraExt::MatrixMarketFileToCrsMatrix64("AtBt.mat",*epMap,*epMap,*epMap,ptrAtBt);
+      TEUCHOS_TEST_FOR_EXCEPTION(ptrA == NULL || ptrB == NULL || ptrAB == NULL || ptrAtB == NULL || ptrABt == NULL || ptrAtBt == NULL,std::logic_error,"Could not open one or more matrix files");
+      RCP<Epetra_CrsMatrix> epA = Teuchos::rcp(ptrA);
+      RCP<Epetra_CrsMatrix> epB = Teuchos::rcp(ptrB);
+      RCP<Epetra_CrsMatrix> epAB = Teuchos::rcp(ptrAB);
+      RCP<Epetra_CrsMatrix> epAtB = Teuchos::rcp(ptrAtB);
+      RCP<Epetra_CrsMatrix> epABt = Teuchos::rcp(ptrABt);
+      RCP<Epetra_CrsMatrix> epAtBt = Teuchos::rcp(ptrAtBt);
+
+      /////////////////////////////////////// transform Epetra objects to Xpetra (needed for MueLu)
+
+      // build Xpetra objects from Epetra_CrsMatrix objects
+      Teuchos::RCP<CrsMatrixClass> xAmat = Teuchos::rcp(new MA(epA));
+      Teuchos::RCP<CrsMatrixClass> xBmat = Teuchos::rcp(new MA(epB));
+      Teuchos::RCP<CrsMatrixClass> xABmat = Teuchos::rcp(new MA(epAB));
+      Teuchos::RCP<CrsMatrixClass> xAtBmat = Teuchos::rcp(new MA(epAtB));
+      Teuchos::RCP<CrsMatrixClass> xABtmat = Teuchos::rcp(new MA(epABt));
+      Teuchos::RCP<CrsMatrixClass> xAtBtmat = Teuchos::rcp(new MA(epAtBt));
+
+      Teuchos::RCP<MatrixClass> xA = Teuchos::rcp(new CrsMatrixWrapClass(xAmat));
+      Teuchos::RCP<MatrixClass> xB = Teuchos::rcp(new CrsMatrixWrapClass(xBmat));
+      Teuchos::RCP<MatrixClass> xAB= Teuchos::rcp(new CrsMatrixWrapClass(xABmat));
+      Teuchos::RCP<MatrixClass> xAtB= Teuchos::rcp(new CrsMatrixWrapClass(xAtBmat));
+      Teuchos::RCP<MatrixClass> xABt= Teuchos::rcp(new CrsMatrixWrapClass(xABtmat));
+      Teuchos::RCP<MatrixClass> xAtBt= Teuchos::rcp(new CrsMatrixWrapClass(xAtBtmat));
+
+      Teuchos::RCP<MatrixClass> yAB= Teuchos::rcp(new CrsMatrixWrapClass(map, 6));
+
+      //Xpetra::MatrixMatrix::Multiply<Scalar, LO, GO, Node> (
+      Xpetra::MatrixMatrix<Scalar, LO, GO, Node>::Multiply (
+        *xA,
+        false,
+        *xB,
+        false,
+        *yAB);
+      //Xpetra::MatrixMatrix::Add<Scalar, LO, GO, Node>(*xAB,false,1.0,*yAB,-1.0);
+      TEUCHOS_TEST_EQUALITY(yAB->getFrobeniusNorm(), xAB->getFrobeniusNorm(), out, success );
+
+      Xpetra::MatrixMatrix<Scalar, LO, GO, Node>::Multiply (
+        *xA,
+        true,
+        *xB,
+        false,
+        *yAB);
+      //Xpetra::MatrixMatrix::Add<Scalar, LO, GO, Node>(*xAtB,false,1.0,*yAB,-1.0);
+      TEUCHOS_TEST_EQUALITY(yAB->getFrobeniusNorm(), xAtB->getFrobeniusNorm(), out, success );
+
+      Xpetra::MatrixMatrix<Scalar, LO, GO, Node>::Multiply (
+        *xA,
+        false,
+        *xB,
+        true,
+        *yAB);
+      //Xpetra::MatrixMatrix::Add<Scalar, LO, GO, Node> (*xABt,false,1.0,*yAB,-1.0);
+      TEUCHOS_TEST_EQUALITY(yAB->getFrobeniusNorm(), xABt->getFrobeniusNorm(), out, success );
+
+      Xpetra::MatrixMatrix<Scalar, LO, GO, Node>::Multiply (
+        *xA,
+        true,
+        *xB,
+        true,
+        *yAB);
+      //Xpetra::MatrixMatrix::Add<Scalar, LO, GO, Node> (*xAtBt,false,1.0,*yAB,-1.0);
+      TEUCHOS_TEST_EQUALITY(yAB->getFrobeniusNorm(), xAtBt->getFrobeniusNorm(), out, success );
+    } // end Epetra test
+#endif
+  }
+
+  TEUCHOS_UNIT_TEST_TEMPLATE_6_DECL( MatrixMatrix, Multiply_Tpetra, M, MA, Scalar, LO, GO, Node )
+  {
+#if defined(HAVE_TPETRA_INST_COMPLEX_FLOAT) || defined(HAVE_TPETRA_INST_COMPLEX_DOUBLE)
+    // The matrix reader does not work with complex scalars. Skip all tests then.
+    return;
+#endif
+    typedef Xpetra::Map<LO, GO, Node> MapClass;
+    typedef Xpetra::MapFactory<LO, GO, Node> MapFactoryClass;
+    typedef Xpetra::CrsMatrix<Scalar,LO,GO,Node> CrsMatrixClass;
+    typedef Xpetra::Matrix<Scalar,LO,GO,Node> MatrixClass;
+    typedef Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node> CrsMatrixWrapClass;
+
+    //Teuchos::RCP<Teuchos::FancyOStream> fos = Teuchos::getFancyOStream(Teuchos::rcpFromRef(std::cout));
+    //yAB->describe(*fos, Teuchos::VERB_EXTREME);
 
 #ifdef HAVE_XPETRA_TPETRA
     { // Tpetra test
 
       // get a comm and node
       RCP<const Comm<int> > comm = getDefaultComm();
-
       Xpetra::UnderlyingLib lib = Xpetra::UseTpetra;
 
       // define map
@@ -290,24 +409,24 @@ namespace {
       Teuchos::RCP<Tpetra::CrsMatrix<Scalar,LO,GO,Node> > tpAtBt = reader_type::readSparseFile("AtBt.mat",comm,pNode );
 
       // transform to Xpetra
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xAmat = Teuchos::rcp(new Xpetra::TpetraCrsMatrix<Scalar,LO,GO,Node>(tpA));
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xBmat = Teuchos::rcp(new Xpetra::TpetraCrsMatrix<Scalar,LO,GO,Node>(tpB));
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xABmat = Teuchos::rcp(new Xpetra::TpetraCrsMatrix<Scalar,LO,GO,Node>(tpAB));
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xAtBmat = Teuchos::rcp(new Xpetra::TpetraCrsMatrix<Scalar,LO,GO,Node>(tpAtB));
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xABtmat = Teuchos::rcp(new Xpetra::TpetraCrsMatrix<Scalar,LO,GO,Node>(tpABt));
-      Teuchos::RCP<Xpetra::CrsMatrix<Scalar,LO,GO,Node> > xAtBtmat = Teuchos::rcp(new Xpetra::TpetraCrsMatrix<Scalar,LO,GO,Node>(tpAtBt));
+      Teuchos::RCP<CrsMatrixClass> xAmat = Teuchos::rcp(new MA(tpA));
+      Teuchos::RCP<CrsMatrixClass> xBmat = Teuchos::rcp(new MA(tpB));
+      Teuchos::RCP<CrsMatrixClass> xABmat = Teuchos::rcp(new MA(tpAB));
+      Teuchos::RCP<CrsMatrixClass> xAtBmat = Teuchos::rcp(new MA(tpAtB));
+      Teuchos::RCP<CrsMatrixClass> xABtmat = Teuchos::rcp(new MA(tpABt));
+      Teuchos::RCP<CrsMatrixClass> xAtBtmat = Teuchos::rcp(new MA(tpAtBt));
 
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xA = Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xAmat));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xB = Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xBmat));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xAB= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xABmat));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xAtB= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xAtBmat));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xABt= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xABtmat));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > xAtBt= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(xAtBtmat));
+      Teuchos::RCP<MatrixClass> xA = Teuchos::rcp(new CrsMatrixWrapClass(xAmat));
+      Teuchos::RCP<MatrixClass> xB = Teuchos::rcp(new CrsMatrixWrapClass(xBmat));
+      Teuchos::RCP<MatrixClass> xAB= Teuchos::rcp(new CrsMatrixWrapClass(xABmat));
+      Teuchos::RCP<MatrixClass> xAtB= Teuchos::rcp(new CrsMatrixWrapClass(xAtBmat));
+      Teuchos::RCP<MatrixClass> xABt= Teuchos::rcp(new CrsMatrixWrapClass(xABtmat));
+      Teuchos::RCP<MatrixClass> xAtBt= Teuchos::rcp(new CrsMatrixWrapClass(xAtBtmat));
 
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > yAB= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(map, 10));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > yAtB= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(map, 10));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > yABt= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(map, 10));
-      Teuchos::RCP<Xpetra::Matrix<Scalar,LO,GO,Node> > yAtBt= Teuchos::rcp(new Xpetra::CrsMatrixWrap<Scalar,LO,GO,Node>(map, 10));
+      Teuchos::RCP<MatrixClass> yAB= Teuchos::rcp(new CrsMatrixWrapClass(map, 10));
+      Teuchos::RCP<MatrixClass> yAtB= Teuchos::rcp(new CrsMatrixWrapClass(map, 10));
+      Teuchos::RCP<MatrixClass> yABt= Teuchos::rcp(new CrsMatrixWrapClass(map, 10));
+      Teuchos::RCP<MatrixClass> yAtBt= Teuchos::rcp(new CrsMatrixWrapClass(map, 10));
 
       Xpetra::MatrixMatrix<Scalar, LO, GO, Node>::Multiply (
         *xA,
@@ -355,18 +474,61 @@ namespace {
   //
   // INSTANTIATIONS
   //
-#define UNIT_TEST_GROUP_ORDINAL( SC, LO, GO, Node )                     \
-  TEUCHOS_UNIT_TEST_TEMPLATE_4_INSTANT( MatrixMatrix, Multiply, SC, LO, GO, Node )
+#ifdef HAVE_XPETRA_TPETRA
 
-  typedef KokkosClassic::DefaultNode::DefaultNodeType DefaultNodeType;
+  #define XPETRA_TPETRA_TYPES( S, LO, GO, N) \
+    typedef typename Xpetra::TpetraMap<LO,GO,N> M##LO##GO##N; \
+    typedef typename Xpetra::TpetraCrsMatrix<S,LO,GO,N> MA##S##LO##GO##N;
 
-#ifndef XPETRA_TEST_USE_LONGLONG_GO
-#ifdef XPETRA_TPETRA_INST_INT_INT
-  UNIT_TEST_GROUP_ORDINAL(double, int, int, DefaultNodeType)
 #endif
-#else
-  typedef long long LongLongInt;
-  UNIT_TEST_GROUP_ORDINAL(double, int, LongLongInt, DefaultNodeType)
+
+#ifdef HAVE_XPETRA_EPETRA
+
+  #define XPETRA_EPETRA_TYPES( S, LO, GO, N) \
+    typedef typename Xpetra::EpetraMapT<GO,N> M##LO##GO##N; \
+    typedef typename Xpetra::EpetraCrsMatrixT<GO,N> MA##S##LO##GO##N;
+
 #endif
+
+  // List of tests which run only with Tpetra
+#define XP_TPETRA_MATRIX_INSTANT(S,LO,GO,N) \
+      TEUCHOS_UNIT_TEST_TEMPLATE_6_INSTANT( MatrixMatrix, Multiply_Tpetra, M##LO##GO##N , MA##S##LO##GO##N, S, LO, GO, N )
+
+  // List of tests which run only with Epetra
+#define XP_EPETRA_MATRIX_INSTANT(S,LO,GO,N) \
+      TEUCHOS_UNIT_TEST_TEMPLATE_6_INSTANT( MatrixMatrix, Multiply_Epetra, M##LO##GO##N , MA##S##LO##GO##N, S, LO, GO, N )
+
+  // List of tests which run only with Epetra64
+#define XP_EPETRA64_MATRIX_INSTANT(S,LO,GO,N) \
+      TEUCHOS_UNIT_TEST_TEMPLATE_6_INSTANT( MatrixMatrix, Multiply_Epetra64, M##LO##GO##N , MA##S##LO##GO##N, S, LO, GO, N )
+
+#if defined(HAVE_XPETRA_TPETRA)
+
+#include <TpetraCore_config.h>
+#include <TpetraCore_ETIHelperMacros.h>
+
+TPETRA_ETI_MANGLING_TYPEDEFS()
+TPETRA_INSTANTIATE_SLGN_NO_ORDINAL_SCALAR ( XPETRA_TPETRA_TYPES )
+TPETRA_INSTANTIATE_SLGN_NO_ORDINAL_SCALAR ( XP_TPETRA_MATRIX_INSTANT )
+
+#endif
+
+
+#if defined(HAVE_XPETRA_EPETRA)
+
+#include "Xpetra_Map.hpp" // defines EpetraNode
+typedef Xpetra::EpetraNode EpetraNode;
+#ifndef XPETRA_EPETRA_NO_32BIT_GLOBAL_INDICES
+XPETRA_EPETRA_TYPES(double,int,int,EpetraNode)
+XP_EPETRA_MATRIX_INSTANT(double,int,int,EpetraNode)
+#endif
+#ifndef XPETRA_EPETRA_NO_64BIT_GLOBAL_INDICES
+typedef long long LongLong;
+XPETRA_EPETRA_TYPES(double,int,LongLong,EpetraNode)
+XP_EPETRA64_MATRIX_INSTANT(double,int,LongLong,EpetraNode)
+#endif
+
+#endif
+
 } // end namespace
 
