@@ -756,15 +756,29 @@ static int split_hypergraph (int *pins[2], HGraph *ohg, HGraph *nhg,
 }
 
 
+#ifdef __INTEL_COMPILER
+#pragma optimize("",off)
+#endif
+/* KDD 3/3/16:  Release builds with Intel 15.0.2 compiler were failing nightly testing
+ * for hg_ibm03/zdrive.inp.phg.ipm.fixed07.  I traced the differences in solution to the
+ * optimization of this function.  All the input arguments were identical, yet the function
+ * returned different values for release builds than for debug builds.  The debug builds'
+ * results matched the results we get with every other compiler (release or debug), so I am
+ * disabling optimization of this function only to restore passing tests with this compiler. 
+ */
 /* UVCUVC: CHECK currently only uses 1st weight */
 static float balanceTol(PHGPartParams *hgp, int part_dim, int pno, float *ratios,
                         float tot, float pw)
 {
     float ntol=(pw==0.0) ? 0.0 : (tot*hgp->bal_tol*ratios[part_dim*pno])/pw;
-    
+
+
 /*    printf("%s: TW=%.1lf pw=%.1lf (%.3lf) old_tol=%.2f  part_s=(%.3f, %.3f) and new tol=%.2f\n", (pno==0) ? "LEFT" : "RIGHT", tot, pw, pw/tot, hgp->bal_tol, ratios[0], ratios[1], ntol);*/
     return ntol;
 }
+#ifdef __INTEL_COMPILER
+#pragma optimize("",on)
+#endif
 
 #ifdef __cplusplus
 } /* closing bracket for extern "C" */
