@@ -103,18 +103,18 @@ public:
 
   /*! \brief Constructor   
    *    \param inmatrix The users Epetra, Tpetra, or Xpetra CrsMatrix object 
-   *    \param numWeightsPerRow If row weights will be provided in setRowWeights(),
-   *        the set \c numWeightsPerRow to the number of weights per row.
+   *    \param nWeightsPerRow If row weights will be provided in setRowWeights(),
+   *        the set \c nWeightsPerRow to the number of weights per row.
    */
   XpetraCrsMatrixAdapter(const RCP<const User> &inmatrix,
-                         int numWeightsPerRow=0);
+                         int nWeightsPerRow=0);
 
   /*! \brief Specify a weight for each entity of the primaryEntityType.
    *    \param weightVal A pointer to the weights for this index.
    *    \stride          A stride to be used in reading the values.  The
    *        index \c idx weight for entity \k should be found at
    *        <tt>weightVal[k*stride]</tt>.
-   *    \param idx  A value between zero and one less that the \c numWeightsPerRow 
+   *    \param idx  A value between zero and one less that the \c nWeightsPerRow 
    *                  argument to the constructor.
    *
    * The order of weights should correspond to the order of the primary 
@@ -128,7 +128,7 @@ public:
    *    \stride          A stride to be used in reading the values.  The
    *        index \c idx weight for row \k should be found at
    *        <tt>weightVal[k*stride]</tt>.
-   *    \param idx  A value between zero and one less that the \c numWeightsPerRow 
+   *    \param idx  A value between zero and one less that the \c nWeightsPerRow 
    *                  argument to the constructor.
    *
    * The order of weights should correspond to the order of rows
@@ -193,14 +193,14 @@ public:
   }
 
 
-  int getNumWeightsPerRow() const { return numWeightsPerRow_; }
+  int getNumWeightsPerRow() const { return nWeightsPerRow_; }
 
   void getRowWeightsView(const scalar_t *&weights, int &stride,
                            int idx = 0) const
   {
     env_->localInputAssertion(__FILE__, __LINE__,
       "invalid weight index",
-      idx >= 0 && idx < numWeightsPerRow_, BASIC_ASSERTION);
+      idx >= 0 && idx < nWeightsPerRow_, BASIC_ASSERTION);
     size_t length;
     rowWeights_[idx].getStridedList(length, weights, stride);
   }
@@ -228,7 +228,7 @@ private:
   ArrayRCP<gno_t> columnIds_;  // TODO:  KDD Is it necessary to copy and store
   ArrayRCP<scalar_t> values_;  // TODO:  the matrix here?  Would prefer views.
 
-  int numWeightsPerRow_;
+  int nWeightsPerRow_;
   ArrayRCP<StridedData<lno_t, scalar_t> > rowWeights_;
   ArrayRCP<bool> numNzWeight_;
 
@@ -241,11 +241,11 @@ private:
 
 template <typename User, typename UserCoord>
   XpetraCrsMatrixAdapter<User,UserCoord>::XpetraCrsMatrixAdapter(
-    const RCP<const User> &inmatrix, int numWeightsPerRow):
+    const RCP<const User> &inmatrix, int nWeightsPerRow):
       env_(rcp(new Environment)),
       inmatrix_(inmatrix), matrix_(), rowMap_(), colMap_(), base_(),
       offset_(), columnIds_(),
-      numWeightsPerRow_(numWeightsPerRow), rowWeights_(), numNzWeight_(),
+      nWeightsPerRow_(nWeightsPerRow), rowWeights_(), numNzWeight_(),
       mayHaveDiagonalEntries(true)
 {
   typedef StridedData<lno_t,scalar_t> input_t;
@@ -283,10 +283,10 @@ template <typename User, typename UserCoord>
     offset_[i+1] = offset_[i] + nnz;
   } 
 
-  if (numWeightsPerRow_ > 0){
-    rowWeights_ = arcp(new input_t [numWeightsPerRow_], 0, numWeightsPerRow_, true);
-    numNzWeight_ = arcp(new bool [numWeightsPerRow_], 0, numWeightsPerRow_, true);
-    for (int i=0; i < numWeightsPerRow_; i++)
+  if (nWeightsPerRow_ > 0){
+    rowWeights_ = arcp(new input_t [nWeightsPerRow_], 0, nWeightsPerRow_, true);
+    numNzWeight_ = arcp(new bool [nWeightsPerRow_], 0, nWeightsPerRow_, true);
+    for (int i=0; i < nWeightsPerRow_; i++)
       numNzWeight_[i] = false;
   }
 }
@@ -317,7 +317,7 @@ template <typename User, typename UserCoord>
   typedef StridedData<lno_t,scalar_t> input_t;
   env_->localInputAssertion(__FILE__, __LINE__,
     "invalid row weight index",
-    idx >= 0 && idx < numWeightsPerRow_, BASIC_ASSERTION);
+    idx >= 0 && idx < nWeightsPerRow_, BASIC_ASSERTION);
   size_t nvtx = getLocalNumRows();
   ArrayRCP<const scalar_t> weightV(weightVal, 0, nvtx*stride, false);
   rowWeights_[idx] = input_t(weightV, stride);
@@ -347,7 +347,7 @@ template <typename User, typename UserCoord>
 {
   env_->localInputAssertion(__FILE__, __LINE__,
     "invalid row weight index",
-    idx >= 0 && idx < numWeightsPerRow_, BASIC_ASSERTION);
+    idx >= 0 && idx < nWeightsPerRow_, BASIC_ASSERTION);
 
   numNzWeight_[idx] = true;
 }
