@@ -54,7 +54,6 @@
 #include <Zoltan2_Standards.hpp>
 
 #include <Xpetra_TpetraCrsMatrix.hpp>
-#include <Xpetra_TpetraRowMatrix.hpp>
 #include <Xpetra_TpetraVector.hpp>
 #include <Tpetra_Vector.hpp>
 
@@ -591,46 +590,6 @@ struct XpetraTraits<Xpetra::CrsGraph<lno_t, gno_t, node_t> >
 };
 
 
-
-//////////////////////////////////////////////////////////////////////////////
-// Xpetra::RowMatrix
-template <typename scalar_t,
-          typename lno_t,
-          typename gno_t,
-          typename node_t>
-struct XpetraTraits<Xpetra::RowMatrix<scalar_t, lno_t, gno_t, node_t> >
-{
-  typedef Xpetra::RowMatrix<scalar_t, lno_t, gno_t, node_t> x_matrix_t;
-  typedef Xpetra::TpetraRowMatrix<scalar_t, lno_t, gno_t, node_t> xt_matrix_t;
-  typedef Tpetra::RowMatrix<scalar_t,lno_t,gno_t,node_t> t_matrix_t;
-
-  static inline RCP<x_matrix_t> convertToXpetra(const RCP<x_matrix_t > &a)
-  {
-    return a;
-  }
-
-  static RCP<x_matrix_t> doMigration(const x_matrix_t &from,
-      size_t numLocalRows, const gno_t *myNewRows)
-  {
-    Xpetra::UnderlyingLib lib = from.getRowMap()->lib();
-
-    if (lib == Xpetra::UseEpetra){
-       throw std::logic_error("compiler should have used specialization");
-    } else{
-      // Do the import with the Tpetra::CrsMatrix traits object
-      const xt_matrix_t *xtm = dynamic_cast<const xt_matrix_t *>(&from);
-      RCP<const t_matrix_t> tm = xtm->getTpetra_CrsMatrix();
-
-      RCP<t_matrix_t> tmnew = XpetraTraits<t_matrix_t>::doMigration(
-        *tm, numLocalRows, myNewRows);
-
-      RCP<x_matrix_t> xmnew = XpetraTraits<t_matrix_t>::convertToXpetra(tmnew);
-
-      return xmnew;
-    }
-  }
-};
-
 //////////////////////////////////////////////////////////////////////////////
 // Xpetra::CrsGraph specialization
 template < typename node_t>
